
--[[
    Paramvir Likhari - conkywx 2011 - 2017 - GPL3
    anemometer library for use with conkywx
--]]

local w = {true}

local TORADIANS = 3.14159265358979323846/180
local tconcat   = table.concat
local tonum     = tonumber
local floor     = math.floor
local random    = math.random
local sprintf   = string.format
local slower    = string.lower

local NavyBlue = 0x000080
local Orange   = 0xFF3300
local DarkLime = 0x32B000
local Red      = 0xff0000

local FC = require("luafunc")
local wxsplit          = FC.wxsplit
local to_rgba          = FC.to_rgba
local to_rgba_contrast = FC.to_rgba_contrast
local set_font         = FC.set_font
local make_txt         = FC.make_txt
local make_scale_txt2  = FC.make_scale_txt2
local make_bezel       = FC.make_bezel
local choose_indicator = FC.choose_indicator
local instrument_sqr   = FC.instrument_sqr

local lcollectgarbage = collectgarbage

local lcairo_fill            = cairo_fill
local lcairo_save            = cairo_save
local lcairo_restore         = cairo_restore
local lcairo_set_source_rgba = cairo_set_source_rgba
local lcairo_arc             = cairo_arc
local lcairo_set_line_width  = cairo_set_line_width
local lcairo_rectangle       = cairo_rectangle

-- for n,v in pairs(_G) do
--     n = string.lower(n)
--     if not n:match('cairo') and
--     -- if not n:match('imlib') and
--        not n:match('conky') then
--         print (n,v)
--     end
-- end

-- print(os.getenv("USER"))

local anlastVal
function w.anemometer(p)

    local cr = p.cr
    -------------------------------------------
    -- scalable version 2017 Paramvir

    ---need to fix the magnification options router
    -------------------------------------------
    local wsdial_size_actual = p.s  or 200

    local px           = p.lwx+(p.s/2)+1
    local py           = p.lwy+(p.s/2)+5
    local ws           = p.r  or 0
    local wg           = p.v  or 0
    local unit         = p.u  or 'C'
    local wvane        = p.w  or nil
    local bimage1      = p.j  or 0
    -- local tilt         = p.t  or 0
    local tilt         = p.t  or nil
    local style        = p.k  or 'all'

    local bimageT      = wxsplit(bimage1,':')
    local bimage       = bimageT[1] or 0
    local bezStyle     = {2, bimageT[2] or 1,bimageT[3] or 2.5}

    -------------------- c-lour
    local dial_colour  = p.c   or DarkLime .. ":1"
    local text_colour  = p.e   or nil -- NavyBlue .. ":1"
    local arrow_colour = p.x   or Orange  ..  ":1"

    dial_colour        = wxsplit(dial_colour,":")
    text_colour        = wxsplit(text_colour,":")
    arrow_colour       = wxsplit(arrow_colour,":")
    local astart_scale = 180
    local aend_scale   = 275
    local kmh_mph_fact = 0.62137119

    -- change this value (mph) and the rest of the dial changes
    local max_scale_mph = 50
    local max_mph_limit = 500

    if wvane and wvane == 0 then wvane = nil end

    local wxx1
    if not tonum(ws) then
        wxx1 = ws:match('([%d%.]+)')
        ws = tonum(wxx1) or 0
        -- ws = 500
        -- unit = 'C'
    end

    if not tonum(wg) then
        wxx1 = wg:match('([%d%.]+)')
        wg = tonum(wxx1) or 0
        -- wg = ws + 100
    end

    if unit == "C" then
        ws = ws * kmh_mph_fact
        wg = wg * kmh_mph_fact
    end

    local wswgFactor
    if wg > ws then
        wswgFactor = wg
    else
        wswgFactor = ws
    end

    if wswgFactor > (max_scale_mph - 10) then
        for i = (max_scale_mph - 10), max_mph_limit do
            if wswgFactor > i and ( i % 10 == 0 ) then
                max_scale_mph = i + 20
            end
        end
    end

    wsdial_size_actual = wsdial_size_actual * 1.035
    local wsdial_size = wsdial_size_actual / 2

    -- print(wsdial_size_actual,wsdial_size)

    local bimages
    if bimage  == 0 then
        bimages     = wsdial_size_actual
        wsdial_size = wsdial_size_actual * 0.48
    else
        bimages     = wsdial_size_actual
        wsdial_size = wsdial_size_actual * 0.44
    end

    if tonum(bimage) == 2 then
        lcairo_save(cr)
        -- print(tilt)
        local bzTable = { cr = cr, bezel_size = wsdial_size_actual*0.92, px = px, py = py, tilt = tilt, style = bezStyle }
        make_bezel(bzTable)
        lcairo_restore(cr)
    end

    lcairo_set_line_width(cr,1)
    lcairo_set_source_rgba(cr,to_rgba(dial_colour))
    lcairo_arc(cr,px,py,wsdial_size,TORADIANS,0)
    lcairo_fill(cr)
    if not text_colour then
        lcairo_set_source_rgba(cr, to_rgba_contrast(dial_colour))
    else
        lcairo_set_source_rgba(cr, to_rgba(text_colour))
    end

    local illumfact = {true}
    local wstxt, wstxt2, wstxt2label,txtfact1, cfact, fsizeA, pointer_size
    local spoint, epoint, radiusFactor, kntxt, kmtxt, mitxt, labelFact

    -- style = 'kn'
    if style then style = slower(style) end
    if wsdial_size < 55 then
        if style == 'mi' then
            spoint = 1; epoint = 1
            radiusFactor = 1.35
            labelFact = 0.9
            mitxt = "mi"; kntxt = ""; kmtxt = ""
        elseif style == 'km' then
            spoint = 2; epoint = 2
            radiusFactor = 0.95
            labelFact = 1
            mitxt = ""; kntxt = ""; kmtxt = "km"
        elseif style == 'kn' then
            spoint = 3; epoint = 3
            radiusFactor = 2
            labelFact = 0.85
            mitxt = ""; kntxt = "kn"; kmtxt = ""
        else
            labelFact = 1.1
            spoint = 1; epoint = 3
            radiusFactor = 1
            mitxt = "mi"; kntxt = "k"; kmtxt = "km"
        end
        if max_scale_mph < 80 then
            illumfact = {1,0.3,0.3}; cfact = 10
        elseif max_scale_mph < 100 then
            illumfact = {0.6,0.3,0.3}; cfact = 10
        elseif max_scale_mph < 170 then
            illumfact = {0.6,0.3,0.3}; cfact = 20
        elseif max_scale_mph < 290 then
            illumfact = {0.6,0.3,0.3}; cfact = 30
        else
            illumfact = {0.6,0.3,0.3}; cfact = 50
        end
        pointer_size = (wsdial_size/9)
        txtfact1 = 10
        wstxt=''; fsizeA = 10
        wstxt2 = "x10"; wstxt2label = 0.45
    elseif wsdial_size < 75 then
        spoint = 1; epoint = 2
        radiusFactor = 0.9
        if max_scale_mph < 80 then
            illumfact = {1,0.3,0.3}; cfact = 10
        elseif max_scale_mph < 100 then
            illumfact = {0.6,0.3,0.3}; cfact = 20
        elseif max_scale_mph < 170 then
            illumfact = {0.6,0.3,0.3}; cfact = 20
        elseif max_scale_mph < 290 then
            illumfact = {0.6,0.3,0.3}; cfact = 40
        else
            illumfact = {0.6,0.3,0.3}; cfact = 50
        end
        labelFact = 1
        mitxt = "mph"; kntxt = ""; kmtxt = "kmh"
        pointer_size = (wsdial_size/9)
        fsizeA = 10;  txtfact1 = 1
        wstxt='';
    else
        if max_scale_mph < 80 then
            illumfact = {1.8,0.9,0.9}; cfact = 10
        elseif max_scale_mph < 100 then
            illumfact = {1.8,0.9,0.9}; cfact = 15
        elseif max_scale_mph < 170 then
            illumfact = {0.6,0.3,0.3}; cfact = 20
        elseif max_scale_mph < 290 then
            illumfact = {0.6,0.3,0.3}; cfact = 40
        else
            illumfact = {0.6,0.3,0.3}; cfact = 50
        end
        mitxt = "mph"; kntxt = "kn"; kmtxt = "kmh"
        labelFact = 1.1
        radiusFactor = 1
        spoint = 1; epoint = 3
        fsizeA = wsdial_size / 8.0
        kntxt = "knots"; txtfact1 = 1
        pointer_size = (wsdial_size/14)
    end

    local anemo = {
        {
            maxScale    = max_scale_mph,
            radiusMarks = wsdial_size * 0.69     * radiusFactor,
            radiusText  = wsdial_size * 0.5727   * radiusFactor,
        },
        {
            maxScale    = max_scale_mph / kmh_mph_fact,
            radiusMarks = wsdial_size * -0.738   * radiusFactor,
            radiusText  = wsdial_size * 0.88     * radiusFactor,
        },
        {
            maxScale    = max_scale_mph * 0.86897624,
            radiusMarks = wsdial_size * 0.4559   * radiusFactor,
            radiusText  = wsdial_size * 0.328248 * radiusFactor,
        },
    }

    lcairo_save(cr)
    set_font({ cr, 'normal:' .. fsizeA })
    local extents   = cairo_text_extents_t:create()
    tolua.takeownership(extents)
    local lineThickness, lineLength, rin, itext
    -- for x = 1, 3 do
    for x = spoint, epoint do
        for i = 0, anemo[x].maxScale do
            if (i % (cfact / 2) == 0) and (i % cfact ~= 0) then
                lineThickness = illumfact[2]
                lineLength    = 4
                rin           = anemo[x].radiusMarks - ((lineLength-2)/2)
            elseif ( i % cfact == 0) then
                itext         = floor(i / txtfact1)
                lineThickness = illumfact[1]
                lineLength    = 5
                rin           = anemo[x].radiusMarks - ((lineLength-2)/2)
            else
                lineThickness = illumfact[3]
                lineLength    = 2
                rin           = anemo[x].radiusMarks
            end
            lcairo_save(cr)
            if rin < 0 then rin = -rin end
            local mkData = {
                cr            = cr,
                startscale    = astart_scale,
                endscale      = aend_scale,
                maxscale      = anemo[x].maxScale,
                trout         = anemo[x].radiusText,
                text          = itext,
                ext           = extents,
                lineLength    = lineLength,
                lineThickness = lineThickness,
                ifact         = cfact,
                rin           = rin,
                xval          = px,
                yval          = py,
                i             = i
            }
            make_scale_txt2(mkData)
        end
    end

    -- text output names
    if wsdial_size > 88 then
        set_font({ cr, 'bold:13' })
        wstxt = "Anemometer"
    else
        set_font({ cr, 'bold:' .. fsizeA })
        wstxt = "Anemo"
    end

    local anName = {
        cr = cr, text = wstxt, ext = extents, px = px + 10,
        py = py + (wsdial_size * 0.35), align = 'l'
    }
    make_txt(anName)
    local anTable = {true}
    if wsdial_size > 40 then
        lcairo_save(cr)
        set_font({ cr, 'normal:' .. fsizeA })
        anTable[1] = {
            cr = cr, ext = extents, text = kntxt, px = px + 7,
            py = py + anemo[3].radiusMarks * labelFact, align='l'
        }
        anTable[2] = {
            cr = cr, ext=extents, text=mitxt, px=px+7,
            py=py+anemo[1].radiusMarks * labelFact, align='l'
        }
        anTable[3] = {
            cr = cr, ext=extents, text=kmtxt, px=px+7,
            py=py+(-anemo[2].radiusMarks*1.1 * labelFact), align='l'
        }
        if wstxt2 then
            anTable[4] = {
                cr = cr, ext = extents, text = wstxt2, px = px + 20,
                py = py + (wsdial_size * 0.53), align = 'l'
            }
        end
    else
        lcairo_save(cr)
        anTable[1] = {
            cr = cr, ext=extents, text=wstxt, px=px, py=py+(-anemo[2].radiusMarks * radiusFactor)
        }
    end
    for x = 1, #anTable do
        make_txt(anTable[x])
    end


    local seed_factor_h = 0
    local seed_factor_l = 0
    if ws ~= 0 or wg > 0 then
        if wg and wg > ws then
            seed_factor_h = wg + 2
            seed_factor_l = ws - 2
        else
            seed_factor_h = ws + 3
            seed_factor_l = ws - 3
        end
    else
        seed_factor_h = nil
        seed_factor_l = nil
    end

    local wvar, wvarTxt
    if tonum(conky_info.update_interval) < 5 then
        wvar = ( tonum(conky_parse("${updates}")) % 4 ) + 1
        wvarTxt = wvar
    else
        wvar = 1
    end

    local wspd = 0
    if ws < max_scale_mph then
        local random_seed
        if wvar == 1 and seed_factor_h then
            random_seed = random( floor(seed_factor_l), floor(seed_factor_h) )
            anlastVal   = random_seed
        else
            random_seed = anlastVal or 0
            anlastVal   = nil
        end

        if random_seed <= 0 then
            random_seed = ws
        elseif random_seed > max_scale_mph then
            random_seed = max_scale_mph
        end

        wspd = random_seed
    else
        wspd = max_scale_mph
    end
    if seed_factor_h then
        wvarTxt = random(1,9) * 0.1
        wspd = tonum(sprintf('%.1f', ( wspd + wvarTxt )))
    else
        wspd = 0
    end

    -- print(anemo[3].radiusMarks*0.6,wsdial_size*0.546, wsdial_size,wsdial_size_actual )

    local indicator_range = floor(wspd * (aend_scale / max_scale_mph))
    local arrowType = tonum(p.a) or 7
    local ciData = {
        cr       = cr,
        startval = astart_scale,
        scale    = indicator_range,
        -- rout1    = anemo[1].radiusMarks*1.15,
        rout1    = wsdial_size*0.7935,
        cthick   = pointer_size*0.45,
        px       = px,
        py       = py,
        ch_indi  = arrowType,
        icolour  = arrow_colour,
    }
    choose_indicator(ciData)

    if arrowType == 7 then
        if unit == "C" then
            wvarTxt = sprintf('%.1f', wspd  / kmh_mph_fact)
        else
            wvarTxt = sprintf('%.1f', wspd )
        end
        if style == 'kn' then wvarTxt = sprintf('%.1f', wspd * 0.86897624) end

        lcairo_set_source_rgba(cr, 0,0,0,0.75)
        local recWidth = 40
        lcairo_rectangle(cr, px-(recWidth/2), py+5.5, recWidth, -14)
        lcairo_fill(cr)
        if wsdial_size>100 then
            set_font({ cr, 'bold:18' })
        else
            set_font({ cr, 'bold:12' })
        end
        lcairo_set_source_rgba(cr, 0,1,0,1)
        lcairo_save(cr)
        local anName = { cr = cr, colourContrast = nil, text=wvarTxt, ext=extents, px=px, py=py+3, align='c' }
        -- local anName = { cr = cr, colourContrast = dial_colour, text=wvarTxt, ext=extents, px=px, py=py+3, align='c' }
        make_txt(anName)
        lcairo_restore(cr)
    end
    wvar = nil

    if wvane == 1 then
        local GF = require("showcond")
        local showcond = GF.showcond

        local w_vane = {
            cr        = cr,
            lwx       = px,
            lwy       = py,
            s         = (wsdial_size*0.95),
            d         = p.d,
            i         = (p.i or "NA"),
            -- a         = (p.a or "NA"),
            l         = (p.l or "0x00ff00:1"),
            w         = 'wi',
            a         = 13,
            wvariance = p.wvariance,
            imgpath   = p.imgpath,
            nring     = 1,
        }
        showcond(w_vane)
    end

    if tonum(bimage) == 1 then
        lcairo_save(cr)
        local inData = { cr = cr, routI=bimages,  wx=px-1-(wsdial_size_actual*0.5), wy=py-(wsdial_size_actual*0.5)}
        instrument_sqr(inData)
        lcairo_restore(cr)
    end
end

lcollectgarbage()

return w
