
--[[
    Paramvir Likhari - conkywx 2011 - 2016 - GPL3
    bargraph library for use with conkywx lua implementation
    Comments at the end of this file
--]]

local w = {}

local PI        = 3.14159265358979323846
local TORADIANS = 3.14159265358979323846/180
local sprintf   = string.format
local tconcat   = table.concat
local tonum     = tonumber
local floor     = math.floor

local lcairo_move_to                     = cairo_move_to
local lcairo_line_to                     = cairo_line_to
local lcairo_rotate                      = cairo_rotate
local lcairo_stroke                      = cairo_stroke
local lcairo_fill                        = cairo_fill
local lcairo_save                        = cairo_save
local lcairo_restore                     = cairo_restore
local lcairo_set_source                  = cairo_set_source
local lcairo_transform                   = cairo_transform
local lcairo_translate                   = cairo_translate
local lcairo_matrix_init                 = cairo_matrix_init
local lcairo_set_line_width              = cairo_set_line_width
local lcairo_pattern_create_rgba         = cairo_pattern_create_rgba
local lcairo_pattern_create_radial       = cairo_pattern_create_radial
local lcairo_pattern_create_linear       = cairo_pattern_create_linear
local lcairo_pattern_add_color_stop_rgba = cairo_pattern_add_color_stop_rgba
local lcairo_pattern_destroy             = cairo_pattern_destroy
local lcairo_set_line_cap                = cairo_set_line_cap


-- package.path = package.path .. ";./lib/?.lua;"

local FC = require("luafunc")
local SS = require("sysinfo")

local wxsplit = FC.wxsplit
local to_rgba = FC.to_rgba
local msgout  = FC.msgout
local sysinfo = SS.sysinfo

----------------------------------------------------------------------
function w.bargraphMain(t)
    -- see comments at end of doc

    local cr = t.cr
    lcairo_save(cr)
    --check values
    if t.name == nil and t.arg == nil then
        print("No input values ... use parameters 'name' with 'arg' or only parameter 'arg' ")
        return
    end
    if t.max == nil then
        print("No maximum value defined, use 'max'")
        return
    end

    t.name   = t.name  or ''
    t.arg    = t.arg   or ''

    --set default values
    t.x      = t.x        or conky_window.width/2
    t.y      = t.y        or conky_window.height/2
    t.blocks = t.blocks   or 10
    t.height = t.height   or 10
    t.angle  = t.angle    or 0
    t.angle  = t.angle * TORADIANS
    --line cap style
    t.cap    = t.cap      or 'b'

    local cap="b"
    for i,v in ipairs({"s","r","b"}) do
        if v==t.cap then cap=v end
    end
    delta=0
    if t.cap=="r" or t.cap=="s" then delta = t.height end
    if cap=="s" then
        cap = CAIRO_LINE_CAP_SQUARE
    elseif cap=="r" then
        cap = CAIRO_LINE_CAP_ROUND
    elseif cap=="b" then
        cap = CAIRO_LINE_CAP_BUTT
    end
    --end line cap style

    t.width     = t.width      or 20
    t.space     = t.space      or 2
    t.radius    = t.radius     or 0
    t.angle_bar = t.angle_bar  or 0
    t.angle_bar = t.angle_bar*PI/360 --halt angle

    --colours
    t.bg_colour = t.bg_colour or {0x00FF00,0.5}
    if #t.bg_colour ~= 2 then t.bg_colour = {0x00FF00,0.5} end

    t.fg_colour = t.fg_colour or {0x00FF00,1}
    if #t.fg_colour ~= 2 then t.fg_colour = {0x00FF00,1} end

    t.alarm_colour = t.alarm_colour or t.fg_colour
    if #t.alarm_colour ~= 2 then t.alarm_colour = t.fg_colour end

    if t.mid_colour then
        for i=1, #t.mid_colour do
            if #t.mid_colour[i] ~= 3 then
                print("error in mid_color table")
                t.mid_colour[i] = {1,0xFFFFFF,1}
            end
        end
    end

    if t.bg_led and #t.bg_led~=2      then t.bg_led = t.bg_colour end
    if t.fg_led and #t.fg_led~=2      then t.fg_led = t.fg_colour end
    if t.alarm_led~= nil and #t.alarm_led~=2 then t.alarm_led = t.fg_led end

    if t.led_effect then
        if not t.bg_led     then t.bg_led = t.bg_colour end
        if not t.fg_led     then t.fg_led = t.fg_colour end
        if not t.alarm_led  then t.alarm_led = t.fg_led end
    end

    if not t.alarm  then t.alarm = t.max end --0.8*t.max end
    if not t.smooth then t.smooth = false end

    if not t.skew_x then
        t.skew_x=0
    else
        t.skew_x = t.skew_x * TORADIANS
    end
    if not t.skew_y then
        t.skew_y=0
    else
        t.skew_y = t.skew_y * TORADIANS
    end

    t.reflection_alpha  = t.reflection_alpha  or 0
    t.reflection_length = t.reflection_length or 1
    t.reflection_scale  = t.reflection_scale  or 1

    --end of default values

    --functions used to create patterns
    local barMain = {}
    local pct, pcb
    function barMain.create_smooth_linear_gradient(x0,y0,x1,y1)
        local pat = lcairo_pattern_create_linear(x0,y0,x1,y1)
        lcairo_pattern_add_color_stop_rgba(pat, 0, to_rgba(t.fg_colour))
        lcairo_pattern_add_color_stop_rgba(pat, 1, to_rgba(t.alarm_colour))
        if t.mid_colour  then
            for i=1, #t.mid_colour do
                lcairo_pattern_add_color_stop_rgba(pat, t.mid_colour[i][1], to_rgba({t.mid_colour[i][2],t.mid_colour[i][3]}))
            end
        end
        return pat
    end

    function barMain.create_smooth_radial_gradient(x0,y0,r0,x1,y1,r1)
        local pat =  lcairo_pattern_create_radial(x0,y0,r0,x1,y1,r1)
        lcairo_pattern_add_color_stop_rgba(pat, 0, to_rgba(t.fg_colour))
        lcairo_pattern_add_color_stop_rgba(pat, 1, to_rgba(t.alarm_colour))
        if t.mid_colour  then
            for i=1, #t.mid_colour do
                lcairo_pattern_add_color_stop_rgba(pat, t.mid_colour[i][1], to_rgba({t.mid_colour[i][2],t.mid_colour[i][3]}))
            end
        end
        return pat
    end

    function barMain.create_led_linear_gradient(x0,y0,x1,y1,col_alp,col_led)
        local pat = lcairo_pattern_create_linear(x0,y0,x1,y1) ---delta, 0,delta+ t.width,0)
        lcairo_pattern_add_color_stop_rgba(pat, 0.0, to_rgba(col_alp))
        lcairo_pattern_add_color_stop_rgba(pat, 0.5, to_rgba(col_led))
        lcairo_pattern_add_color_stop_rgba(pat, 1.0, to_rgba(col_alp))
        return pat
    end

    function barMain.create_led_radial_gradient(x0,y0,r0,x1,y1,r1,col_alp,col_led,mode)
        local pat = lcairo_pattern_create_radial(x0,y0,r0,x1,y1,r1)
        if mode==3 then
            lcairo_pattern_add_color_stop_rgba(pat, 0, to_rgba(col_alp))
            lcairo_pattern_add_color_stop_rgba(pat, 0.5, to_rgba(col_led))
            lcairo_pattern_add_color_stop_rgba(pat, 1, to_rgba(col_alp))
        else
            lcairo_pattern_add_color_stop_rgba(pat, 0, to_rgba(col_led))
            lcairo_pattern_add_color_stop_rgba(pat, 1, to_rgba(col_alp))
        end
        return pat
    end

    function barMain.draw_single_bar()
        --this fucntion is used for bars with a single block(blocks=1) but
        --the drawing is cut in 3 blocks : value/alarm/background
        --not zvzimzblr for circular bar
        function  barMain.create_pattern(col_alp,col_led,bg)
            local pat

            if not t.smooth then
                if t.led_effect=="e" then
                    pat = barMain.create_led_linear_gradient(-delta, 0,delta+ t.width,0,col_alp,col_led)
                elseif t.led_effect=="a" then
                    pat = barMain.create_led_linear_gradient(t.width/2, 0,t.width/2,-t.height,col_alp,col_led)
                elseif  t.led_effect=="r" then
                    pat = barMain.create_led_radial_gradient(t.width/2, -t.height/2, 0, t.width/2,-t.height/2,t.height/1.5,col_alp,col_led,2)
                else
                    pat = lcairo_pattern_create_rgba(to_rgba(col_alp))
                end
            else
                if bg then
                    pat = lcairo_pattern_create_rgba(to_rgba(t.bg_colour))
                else
                    pat = barMain.create_smooth_linear_gradient(t.width/2, 0, t.width/2,-t.height)
                end
            end
            return pat
        end

        local y1=-t.height*pct/100
        local y2=nil
        if pct>(100*t.alarm/t.max) then
            y1 = -t.height*t.alarm/100
            y2 = -t.height*pct/100
            if t.smooth then y1=y2 end
        end

        if t.angle_bar==0 then

            --block for fg value
            pat = barMain.create_pattern(t.fg_colour,t.fg_led,false)
            lcairo_set_source(cr,pat)
            lcairo_rectangle(cr,0,0,t.width,y1)
            lcairo_fill(cr)

            -- block for alarm value
            if not t.smooth and y2  then
                pat = barMain.create_pattern(t.alarm_colour,t.alarm_led,false)
                lcairo_set_source(cr,pat)
                lcairo_rectangle(cr,0,y1,t.width,y2-y1)
                lcairo_fill(cr)
                y3=y2
            else
                y2,y3=y1,y1
            end
            -- block for bg value
            lcairo_rectangle(cr,0,y2,t.width,-t.height-y3)
            pat = barMain.create_pattern(t.bg_colour,t.bg_led,true)
            lcairo_set_source(cr,pat)
            lcairo_pattern_destroy(pat)
            lcairo_fill(cr)
        end
    end  --end single bar

    function barMain.draw_multi_bar()
        --function used for bars with 2 or more blocks
        for pt = 1,t.blocks do
            --set block y
            local y1 = -(pt-1)*(t.height+t.space)
            local light_on=false

            --set colors
            local col_alp = t.bg_colour
            local col_led = t.bg_led
            if pct>=(100/t.blocks) or pct>0 then --ligth on or not the block
                if pct>=(pcb*(pt-1))  then
                    light_on = true
                    col_alp  = t.fg_colour
                    col_led  = t.fg_led
                    if pct>=(100*t.alarm/t.max) and (pcb*pt)>(100*t.alarm/t.max) then
                        col_alp = t.alarm_colour
                        col_led = t.alarm_led
                    end
                end
            end

            --set colors
            --have to try to create gradients outside the loop ?
            local pat

            if not t.smooth then
                if t.angle_bar==0 then
                    if t.led_effect == "e" then
                        pat = barMain.create_led_linear_gradient(-delta, 0,delta+ t.width,0,col_alp,col_led)
                    elseif t.led_effect == "a" then
                        pat = barMain.create_led_linear_gradient(t.width/2, -t.height/2+y1,t.width/2,0+t.height/2+y1,col_alp,col_led)
                    elseif  t.led_effect == "r" then
                        pat = barMain.create_led_radial_gradient(t.width/2, y1, 0, t.width/2,y1,t.width/1.5,col_alp,col_led,2)
                    else
                        pat = lcairo_pattern_create_rgba(to_rgba(col_alp))
                    end
                else
                     if t.led_effect == "a" then
                         pat = barMain.create_led_radial_gradient(0, 0, t.radius+(t.height+t.space)*(pt-1), 0, 0, t.radius+(t.height+t.space)*(pt), col_alp,col_led,3)
                    else
                        pat = lcairo_pattern_create_rgba(to_rgba(col_alp))
                    end

                end
            else

                if light_on then
                    if t.angle_bar == 0 then
                        pat = barMain.create_smooth_linear_gradient(t.width/2, t.height/2, t.width/2,-(t.blocks-0.5)*(t.height+t.space))
                    else
                        pat = barMain.create_smooth_radial_gradient(0, 0,(t.height+t.space),  0,0,(t.blocks+1)*(t.height+t.space),2)
                    end
                else
                    pat = lcairo_pattern_create_rgba(to_rgba(t.bg_colour))
                end
            end
            lcairo_set_source(cr, pat)
            lcairo_pattern_destroy(pat)

            --draw a block
            if t.angle_bar == 0 then
                lcairo_move_to(cr,0,y1)
                lcairo_line_to(cr,t.width,y1)
            else
                lcairo_arc( cr,0,0,
                    t.radius+(t.height+t.space)*(pt)-t.height/2,
                     -t.angle_bar -PI/2 ,
                     t.angle_bar -PI/2)
            end
            lcairo_stroke(cr)
        end
    end

    function barMain.setup_bar_graph()
        --function used to retrieve the value to display and to set the cairo structure
        if t.blocks ~= 1 then t.y = t.y - (t.height / 2) end

        local value = 0
        if t.name ~= "" then
            if t.name:match('sysinfo') then
                local sv = wxsplit(t.name,':')
                value = tonum(sysinfo(sv[2],sv[3],sv[4],sv[5]))
            else
                value = tonum(conky_parse(sprintf('${%s %s}', t.name, t.arg)))
            end
        else
            value = tonum(t.arg)
        end
        -- value = 50

        if value==nil then value =0 end

        pct = 100 * (value / t.max)
        pcb = 100 / t.blocks

        lcairo_set_line_width(cr, t.height)
        lcairo_set_line_cap(cr, cap)
        lcairo_translate(cr,t.x,t.y)
        -- -- Edit by Paramvir - this corrects the wrong start of alarm colour
        -- -- for angle = 90 and above
        -- -- There is however still an issue between 20 to 89 degrees
        lcairo_rotate(cr,(t.angle + 0.0000001))

        local matrix0 = cairo_matrix_t:create()
        tolua.takeownership(matrix0)

        lcairo_matrix_init(matrix0, 1,t.skew_y,t.skew_x,1,0,0)
        lcairo_transform(cr,matrix0)

        --call the drawing function for blocks
        if t.blocks==1 and t.angle_bar==0 then
            barMain.draw_single_bar()
            if t.reflection=="t" or t.reflection=="b" then lcairo_translate(cr,0,-t.height) end
        else
            barMain.draw_multi_bar()
        end

        --call the drawing function for reflection and prepare the mask used
        if t.reflection_alpha>0 and t.angle_bar==0 then
            local pat2
            local matrix1 = cairo_matrix_t:create()
            tolua.takeownership(matrix1)

            if t.angle_bar==0 then
                pts={-delta/2,(t.height+t.space)/2,t.width+delta,-(t.height+t.space)*(t.blocks)}
                if t.reflection=="t" then
                    lcairo_matrix_init(matrix1,1,0,0,-t.reflection_scale,0,-(t.height+t.space)*(t.blocks-0.5)*2*(t.reflection_scale+1)/2)
                    pat2 = lcairo_pattern_create_linear(t.width/2,-(t.height+t.space)*(t.blocks),t.width/2,(t.height+t.space)/2)
                elseif t.reflection=="r" then
                    lcairo_matrix_init(matrix1,-t.reflection_scale,0,0,1,delta+2*t.width,0)
                    pat2 = lcairo_pattern_create_linear(delta/2+t.width,0,-delta/2,0)
                elseif t.reflection=="l" then
                    lcairo_matrix_init(matrix1,-t.reflection_scale,0,0,1,-delta,0)
                    pat2 = lcairo_pattern_create_linear(-delta/2,0,delta/2+t.width,-0)
                else --bottom
                    lcairo_matrix_init(matrix1,1,0,0,-1*t.reflection_scale,0,(t.height+t.space)*(t.reflection_scale+1)/2)
                    pat2 = lcairo_pattern_create_linear(t.width/2,(t.height+t.space)/2,t.width/2,-(t.height+t.space)*(t.blocks))
                end
            end
            lcairo_transform(cr,matrix1)

            if t.blocks==1 and t.angle_bar==0 then
                barMain.draw_single_bar()
                lcairo_translate(cr,0,-t.height/2)
            else
                barMain.draw_multi_bar()
            end
            lcairo_set_line_width(cr,0.01)
            lcairo_pattern_add_color_stop_rgba(pat2, 0,0,0,0,1-t.reflection_alpha)
            lcairo_pattern_add_color_stop_rgba(pat2, t.reflection_length,0,0,0,1)
            if t.angle_bar==0 then
                lcairo_rectangle(cr,pts[1],pts[2],pts[3],pts[4])
            end
            lcairo_clip_preserve(cr)
            lcairo_set_operator(cr,CAIRO_OPERATOR_CLEAR)
            lcairo_stroke(cr)
            lcairo_mask(cr,pat2)
            lcairo_pattern_destroy(pat2)
            lcairo_set_operator(cr,CAIRO_OPERATOR_OVER)
        end --reflection
    end --barMain.setup_bar_graph()

    --start here !
    barMain.setup_bar_graph()
    lcairo_restore(cr)
end

function w.bargraph(p)
--  by paramvir - 2014
    -- local cr               = p.cr
    local bar_style        = tonum(p.s)
    local xval1            = p.lwx
    local yval2            = p.lwy
    local ledOn            = p.l or nil
    local bar_alarm_value  = p.x or 90
    local bar_width        = p.w or 100
    bar_width              = tonum(bar_width)
    -- --alarm colour is also the end colour for gradient colour - so leave it enabled
    local bar_alarm_colour = {0xff0000,0.80}
    local bar_fg_colour    = {0x00ff00,0.90}
    local bar_mid_colour   = {{0.38,0xffff00,0.90}}
    local bar_bg_colour    = {0xFFFFFF,0.10}
    -- local bar_alarm_colour = {Red,0.80}
    -- local bar_fg_colour    = {Lime,0.90}
    -- local bar_mid_colour   = {{0.38,Yellow,0.90}}
    -- local bar_bg_colour    = {White,0.25}

    -- -- bargraph values not likely to be changed
    local bar_max_value    = 100 ---- % value of the expected max measure
    local bar_angle_value  = 90  ---- you can vertical bars with 0 angle

    --bar_style = 5
    --ledOn = 1
    -- -- the styles match the 100 pixel value criteria - easier to place in conky
    local barVals = {
        [1] = { -- -- thin dashes
            b_height = 6,
            b_width  = 2.5,
            b_space  = 2.3,
            b_blocks = (bar_width/(6+2.3)),
        },
        [2] = { -- -- thick dashes
            b_height = 6,
            b_width  = 5,
            b_space  = 2.3,
            b_blocks = (bar_width/(6+2.3)),
            yval2    = yval2-(5/2),
        },
        [3] = { -- -- spaced vertical bars
            b_height = 2.04,
            b_width  = 6,
            b_space  = 2,
            b_blocks = (bar_width/(2+2)),
            xval1    = xval1-2,
            yval2    = yval2-(6/1.5),
            -- skew_y   = 45,
        },
        [4] = { -- -- closer spaced vertical bars
            b_height = 2,
            b_width  = 6,
            b_space  = 1,
            b_blocks = (bar_width/(2+1)),
            xval1    = xval1-2,
            yval2    = yval2-(6/1.5),
        },
        [5] = { -- -- smooth single bar without vertical slots
            b_height = bar_width-3,
            b_width  = 6,
            b_space  = 0,
            b_blocks = 1,
            xval1    = xval1-3,
            yval2    = yval2-(6/1.2),
        },
        [6] = { -- -- closer spaced vertical bars
            b_height = bar_width-1,
            b_width  = 2,
            b_space  = 0,
            b_blocks = 1,
            xval1    = xval1-2.7,
            yval2    = yval2-(6/1.25),
        },
        [7] = { -- -- closer spaced vertical bars
            b_height = 2,
            b_width  = 2,
            b_space  = 1,
            b_blocks = (bar_width/(2+1)),
            xval1    = xval1-2,
            yval2    = yval2-(6/1.5),
        },
        [8] = { -- -- wide spaced vertical bars
            b_height = 2,
            b_width  = 2,
            b_space  = 5,
            b_blocks = (bar_width/(2+5)),
            xval1    = xval1+0,
            yval2    = yval2-(6/1.5),
        },
        [9] = { -- -- wider spaced vertical bars
            b_height = 3,
            b_width  = 3,
            b_space  = 10,
            b_blocks = (bar_width/(3+10)),
            xval1    = xval1-0,
            yval2    = yval2-(6/1.5),
        },
    }
    if not ledOn then
        b_smooth = true
    else
        b_smooth = false
    end

    -- print('*** '..p.f,p.a)
    local bars_settings={
        cr           = p.cr,
        name         = p.f,
        arg          = p.a,
        x            = barVals[bar_style].xval1 or xval1,
        y            = barVals[bar_style].yval2 or yval2,
        max          = bar_max_value,
        alarm        = tonum(bar_alarm_value),
        alarm_colour = p.c or bar_alarm_colour,
        bg_colour    = bar_bg_colour,
        fg_colour    = p.e or bar_fg_colour,
        mid_colour   = p.d or bar_mid_colour,
        blocks       = barVals[bar_style].b_blocks,
        space        = barVals[bar_style].b_space,
        height       = barVals[bar_style].b_height,
        width        = barVals[bar_style].b_width,
        angle        = bar_angle_value,
        smooth       = b_smooth,
        skew_x       = barVals[bar_style].skew_x or nil,
        skew_y       = barVals[bar_style].skew_y or nil,
        -- led_effect   = bar_led_effect_alarm
        led_effect   = "r"
    }
    w.bargraphMain(bars_settings)
end

collectgarbage()

return w

--[[

==Comment==
I have left this comment here as there is lot more in this widget
than what I have exposed. The user looking at this code probably is
wanting to finger something - so this info will help.
 Paramvir

BARGRAPH WIDGET
v2.1 by wlourf(07 Jan. 2011)
this widget draws a bargraph with different effects
http://u-scripts.blogspot.com/2010/07/bargraph-widget.html

To call the script in a conky, use, before TEXT
    lua_load /path/to/the/script/bargraph.lua
    lua_draw_hook_pre main_rings
    and add one line(blank or not) after TEXT

Parameters are :
    3 parameters are mandatory
    name - the name of the conky variable to display, for example for {$cpu cpu0}, just write name="cpu"
    arg  - the argument of the above variable, for example for {$cpu cpu0}, just write arg="cpu0"
           arg can be a numerical value if name=""
    max  - the maximum value the above variable can reach, for example, for {$cpu cpu0}, just write max=100

Optional parameters:
    x,y   - coordinates of the starting point of the bar, default = middle of the conky window
    cap   - end of cap line, ossibles values are r,b,s(for round, butt, square), default="b"
            http://www.cairographics.org/samples/set_line_cap/
    angle     - angle of rotation of the bar in degress, default = 0(i.e. a vertical bar)
            set to 90 for an horizontal bar
    skew_x    - skew bar around x axis, default = 0
    skew_y    - skew bar around y axis, default = 0
    blocks    - number of blocks to display for a bar(values >0) , default= 10
    height    - height of a block, default=10 pixels
    width     - width of a block, default=20 pixels
    space     - space between 2 blocks, default=2 pixels
    angle_bar - this angle is used to draw a bar on a circular way(ok, this is no more a bar !) default=0
    radius    - for cicular bars, internal radius, default=0
            with radius, parameter width has no more effect.

Colours below are defined into braces {colour in hexadecimal, alpha}
    fg_colour    - colour of a block ON, default= {0x00FF00,1}
    bg_colour    - colour of a block OFF, default = {0x00FF00,0.5}
    alarm        - threshold, values after this threshold will use alarm_colour colour , default=max
    alarm_colour - colour of a block greater than alarm, default=fg_colour
    smooth       -(true or false), create a gradient from fg_colour to bg_colour, default=false
    mid_colour   - colours to add to gradient, with this syntax {position into the gradient(0 to1), colour hexa, alpha}
               for example, this table {{0.25,0xff0000,1},{0.5,0x00ff00,1},{0.75,0x0000ff,1}} will add
               3 colours to gradient created by fg_colour and alarm_colour, default=no mid_colour
    led_effect   - add LED effects to each block, default=no led_effect
               if smooth=true, led_effect is not used
               possibles values : "r","a","e" for radial, parallel, perdendicular to the bar(just try!)
               led_effect has to be used with theses colours :
    fg_led       - middle colour of a block ON, default = fg_colour
    bg_led       - middle colour of a block OFF, default = bg_colour
    alarm_led    - middle colour of a block > ALARM,  default = alarm_colour

    reflection parameters, not available for circular bars
    reflection_alpha  - add a reflection effect(values from 0 to 1) default = 0 = no reflection
                other values = starting opacity
    reflection_scale  - scale of the reflection(default = 1 = height of text)
    reflection_length - length of reflection, define where the opacity will be set to zero
                values from 0 to 1, default =1
    reflection    - position of reflection, relative to a vertical bar, default="b"
                possibles values are : "b","t","l","r" for bottom, top, left, right
    draw_me           - if set to false, text is not drawn(default = true or 1)
                it can be used with a conky string, if the string returns 1, the text is drawn :
                example : "${if_empty ${wireless_essid wlan0}}${else}1$endif",

    v1.0(10 Feb. 2010) original release
    v1.1(13 Feb. 2010) numeric values can be passed instead conky stats with parameters name="", arg = numeric_value
    v1.2(28 Feb. 2010) just renamed the widget to bargraph
    v1.3(03 Mar. 2010) added parameters radius & angle_bar to draw the bar in a circular way
    v2.0(12 Jul. 2010) rewrite script + add reflection effects and parameters are now set into tables
    v2.1(07 Jan. 2011) Add draw_me parameter and correct memory leaks, thanks to "Creamy Goodness"
    30 Jan. 2014 - paramvir [conkywx] fixed the alarm colour except 20 to 89 degrees
    August 2015  - paramvir parts recoded

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation version 3(GPLv3)

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.

]]

