
--[[
    Paramvir Likhari - conkywx 2011 - 2017 - GPL3
    barometer library for use with conkywx
--]]

--- find all global variables
-- for n,v in pairs(_G) do
--     n = string.lower(n)
--     -- if not n:match('cairo') and
--     if not n:match('conky') then
--         print (n,v)
--     end
-- end

--- find all local variables in scope - see second last line below
-- local getlocal  = debug.getlocal
-- local function getLocalVars()
--     local retVal = {}
--     local count = 1
--     while true do
--         local ln, lv = getlocal(2, count)
--         if ln then
--             retVal[ln] = lv
--         else
--             break
--         end
--         count = 1 + count
--     end
--     return retVal
-- end


local w = {}

local PI        = 3.14159265358979323846
local TORADIANS = 3.14159265358979323846/180
local sprintf   = string.format
local tconcat   = table.concat
local tonum     = tonumber
local supper    = string.upper
local floor     = math.floor

local lcollectgarbage = collectgarbage

local lcairo_fill            = cairo_fill
local lcairo_save            = cairo_save
local lcairo_restore         = cairo_restore
local lcairo_set_source_rgba = cairo_set_source_rgba
local lcairo_arc             = cairo_arc
local lcairo_set_line_width  = cairo_set_line_width
local lcairo_rectangle       = cairo_rectangle

local NavyBlue = 0x000080
local Orange   = 0xFF3300
local DarkLime = 0x32B000

-- package.path = package.path .. ";/lib/?.lua;"

local FC = require("luafunc")

local wxsplit          = FC.wxsplit
local to_rgba          = FC.to_rgba
local to_rgba_contrast = FC.to_rgba_contrast
local get_points       = FC.get_points
local set_font         = FC.set_font
local make_txt         = FC.make_txt
local make_scale_txt2  = FC.make_scale_txt2
local make_bezel       = FC.make_bezel
local choose_indicator = FC.choose_indicator
local circlewriting    = FC.circlewriting
local msgout           = FC.msgout

function w.barometer(p)
    local cr = p.cr
    -- barometer 27 inches to 32 inches = 5 inches
    -- 2014 - Paramvir conkywx scalable with temperature

    local px             = tonum(p.lwx)  or 200
    local py             = tonum(p.lwy)  or 200
    local baro_size_real = tonum(p.s)    or 200
    local pr             = p.r           or 0
    local unit           = p.u           or "C"
    local baro_colour    = p.c           or DarkLime .. ':1'
    local text_colour    = p.e           or nil --NavyBlue .. ':1'
    local arrow_colour   = p.x           or Orange   .. ':1'
    local tilt           = p.t           or nil
    local bimage1        = p.j           or 0
    --bimage1            = 0

    local bimageT        = wxsplit(bimage1,':')
    local bimage         = bimageT[1] or 0
    local bezStyle       = {2, bimageT[2] or 1,bimageT[3] or 2.5}

    baro_colour          = wxsplit(baro_colour,":")
    text_colour          = wxsplit(text_colour,":")
    arrow_colour         = wxsplit(arrow_colour,":")

    local prfactInToMbar = 33.8638815767
    local prfactMbarToIN = 0.02952998751
    local start_scale    = 180
    local end_scale      = 275
    local scale_diff     = 120
    local scale_factor   = 275 / 120

    local wxx1
    if not tonum(pr) then
        wxx1 = pr:match('([%d%.]+)')
        pr = tonum(wxx1) or 0
    end

    -- pr = 940
    -- unit = 'F'
    if unit == "C" then pr = pr * prfactMbarToIN end

    local baro_size=(baro_size_real/2)

    -- px = px + (baro_size_real*0.5) + 5
    px = px + (baro_size_real*0.5) + 5  -- check this
    py = py + (baro_size_real*0.5) + 1

    local bimages
    if bimage == 0 then
        bimages = baro_size_real; baro_size = baro_size_real * 0.48
    else
        bimages = baro_size_real; baro_size = baro_size_real * 0.457
    end

    lcairo_save(cr)
    if tonum(bimage) == 2 then
        local mB = {
            cr         = cr,
            px         = px,
            py         = py,
            bezel_size = baro_size * 2.091,
            tilt       = tilt,
            style      = bezStyle
        }
        make_bezel(mB)
    end
    lcairo_restore(cr)

    local bsTbl = {
        {41, 75, 999},
        [41]  = {
            fsize = 7.5, modfactor = 20, hpafact = 0.1,
            psize = (baro_size/9), brfact = {.3,.2}
        },
        [75]  = {
            fsize = 7.5, modfactor = 20, hpafact = 1,
            psize = (baro_size/12), brfact = {.7,.5}
        },
        [999] = {
            fsize = baro_size/9.5, modfactor = 20, hpafact = 1,
            psize = (baro_size/14), brfact = {2,1}
        },
    }
    local bsLook
    for i = 1, #bsTbl[1] do
        if baro_size < bsTbl[1][i] then bsLook = bsTbl[1][i]; break end
    end

    lcairo_save(cr)
    set_font({ cr, 'normal:' .. bsTbl[bsLook].fsize })

    lcairo_set_line_width(cr,1)
    lcairo_set_source_rgba(cr, to_rgba(baro_colour))
    lcairo_arc(cr, px, py, baro_size, TORADIANS, 0)
    lcairo_fill(cr)

    if not text_colour then
        lcairo_set_source_rgba(cr, to_rgba_contrast(baro_colour))
    else
        lcairo_set_source_rgba(cr, to_rgba(text_colour))
    end

    local starthpa, extents, prInchGaugeTop

    -- Inch gauge top value eg 10, 13, 16, 19 etc
    prInchGaugeTop = ( floor((pr - 1) / 3) * 3 ) + 4

    -- start value for millibars
    starthpa = floor(((((prInchGaugeTop - 3.21) * prfactInToMbar)) * 0.1) + 0.5) * 10

    local baro = {
        {
            maxScale         = 30,
            startScale       = start_scale + ((((prInchGaugeTop - 3) * prfactInToMbar) - starthpa) * scale_factor),
            endScale         = 3 * prfactInToMbar * scale_factor,
            radiusMarks      = baro_size * 0.47,
            radiusText       = baro_size * 0.35,
            lineLengthMajor  = 5,
            lineLengthMinor  = 5,
            lineLengthNormal = 2,
            iFactorMajor     = 10,
            iFactorMinor     = 5,
            textFactor       = nil,
        },
        {
            maxScale         = scale_diff,
            startScale       = start_scale,
            endScale         = end_scale,
            radiusMarks      = baro_size * -0.525,
            radiusText       = baro_size * 0.71,
            lineLengthMajor  = 5,
            lineLengthMinor  = 2,
            lineLengthNormal = 0,
            iFactorMajor     = 10,
            iFactorMinor     = 2,
            textFactor       = 1,
        },
    }

    extents  = cairo_text_extents_t:create()
    tolua.takeownership(extents)
    local cnt = 1
    for x = 1, 2 do
        for i = 0, baro[x].maxScale do
            local itext, rin, lineThickness, lineLength
            if ( i % baro[x].iFactorMinor == 0 ) and ( i % baro[x].iFactorMajor ~= 0 ) then
                lineThickness = bsTbl[bsLook].brfact[2]
                lineLength    = baro[x].lineLengthMinor
                rin           = baro[x].radiusMarks - ((lineLength-3)/2)
            elseif ( i % baro[x].iFactorMajor == 0 ) then
                if not baro[x].textFactor then
                    itext = floor(prInchGaugeTop-4 + cnt)
                    cnt   = cnt + 1
                end
                lineThickness = bsTbl[bsLook].brfact[1]
                lineLength    = baro[x].lineLengthMajor
                rin           = baro[x].radiusMarks - ((lineLength-3)/2)
            else
                lineLength    = baro[x].lineLengthNormal
                lineThickness = bsTbl[bsLook].brfact[2]
                rin           = baro[x].radiusMarks
            end
            if baro[x].textFactor then
                if ( i % bsTbl[bsLook].modfactor == 0 ) then
                    itext = ( floor( (starthpa + i) * bsTbl[bsLook].hpafact ) or 0)
                end
            end
            lcairo_save(cr)
            if rin < 0 then rin = -rin end
            local msTable = {
                cr            = cr,
                startscale    = baro[x].startScale,
                endscale      = baro[x].endScale,
                maxscale      = baro[x].maxScale,
                trout         = baro[x].radiusText,
                text          = itext,
                ext           = extents,
                rin           = rin,
                lineLength    = lineLength,
                lineThickness = lineThickness,
                xval          = px,
                yval          = py,
                i             = i,
            }
            make_scale_txt2(msTable)
        end
    end
    lcairo_set_line_width( cr, 1 )
    lcairo_save(cr)

    local itext, mtext, py1
    if baro_size < 41 then
        local mkTable = { cr = cr, ext=extents, text="x10", px=px, py=py+baro[2].radiusMarks+2 }
        make_txt(mkTable)
        itext=" "
        mtext=" "
    elseif baro_size > 70 then
        itext="inches"
        mtext="millibars"
    else
        itext="in"
        mtext="mb"
    end

    --scale labels
    lcairo_save(cr)
    local mkTable = {
        {
            cr = cr, ext=extents, text=itext, px=px+10, py=py+baro[1].radiusMarks, align='l'
        },
        {
            cr = cr, ext=extents, text=mtext, px=px+10, py=(py+(-baro[2].radiusMarks)+7), align='l'
        },
    }
    for x = 1, 2 do
        make_txt(mkTable[x])
    end

    set_font({ cr, bsTbl[bsLook].fsize })
    if baro_size > 70 then
        set_font({ cr, 'bold:' .. baro_size * 0.135 })
        mtext = "Barometer"
        py1 = 0
    else
        mtext = "Baro"
        py1 = 3
    end
    mkTable = { cr = cr, ext=extents, text=mtext, px=px+11, py=py+(baro[1].radiusMarks*0.7)+py1, align='l' }
    make_txt(mkTable)

    local arrowType = tonum(p.a) or 7
    lcairo_save(cr)
    local ciData = {
        cr       = cr,
        startval = baro[1].startScale,
        scale    = (pr - (prInchGaugeTop-3)) * prfactInToMbar * scale_factor,
        rout1    = -baro[2].radiusMarks * 1.05,
        length   = 0.65,
        cthick   = bsTbl[bsLook].psize * 0.5,
        ch_indi  = arrowType,
        px       = px,
        py       = py,
        icolour  = arrow_colour,
    }
    choose_indicator(ciData)

    if arrowType == 7 then
        if unit == "C" then pr = sprintf('%.0f', pr * prfactInToMbar) end
        if not wxx1 and not tonum(pr) then pr = 'NA' end
        lcairo_set_source_rgba(cr, 0,0,0,0.75)
        local recWidth = 40
        lcairo_rectangle(cr, px-(recWidth/2), py+6, recWidth, -14)
        lcairo_fill(cr)
        if baro_size > 100 then
            set_font({ cr, 'bold:18' })
        else
            set_font({ cr, 'bold:12' })
        end
        lcairo_set_source_rgba(cr, 0,1,0,1)
        lcairo_save(cr)
        local anName = { cr = cr, colourContrast = nil, text=pr , ext=extents, px=px, py=py+3, align='c' }
        make_txt(anName)
    end

    if not text_colour then
        lcairo_set_source_rgba(cr, to_rgba_contrast(baro_colour))
    else
        lcairo_set_source_rgba(cr, to_rgba(text_colour))
    end
    -- lcairo_set_source_rgba(cr, to_rgba(text_colour))
    lcairo_save(cr)
    local fsize=baro_size/10.45454545
    if baro_size > 65 then
        set_font({ cr, 'normal:' .. fsize })
        local bring_labels = {'STORM','RAIN','CHANGE','FAIR','VERY DRY'}
        for i = 1, 5 do
            local start = start_scale + 10
            if i > 1 then start = (start - 55) + (55 * i) end
            local cwData = {
                cr     = cr,
                text   = bring_labels[i],
                radi   = baro_size / 1.15,
                horiz  = px,
                verti  = py,
                start  = start,
                finish = start + ((string.len(bring_labels[i])) * 5 ),
                }
            circlewriting(cwData)
        end
    end

    if tonum(bimage) == 1 then
        lcairo_save(cr)
        local isData = { cr = cr, routI=bimages, wx=px-1-(baro_size_real*0.5), wy=py-(baro_size_real*0.5)}
        instrument_sqr(isData)
        lcairo_restore(cr)
    end

end

lcollectgarbage()

-- for x, v in pairs(getLocalVars()) do print(x, v) end
return w
