#!/bin/bash



# -----------------------------------------------------------------------
# Copyright GPL v3 - Paramvir Likhari 2012/2015
# -----------------------------------------------------------------------

# This defines the output of set -x debugging xtrace
if [[ -e '/tmp/wxdebug' ]]; then
	rm -rf '/tmp/bardebug'
	export PS4='+ (${LINENO}): ${FUNCNAME[0]:+${FUNCNAME[0]}(): }'
	exec 4>>'/tmp/bardebug'
	BASH_XTRACEFD=4
	set -x
fi

mpptmp="/tmp/wxmptmp"
wxfifo="${mpptmp}/mpfifo"
flist="${mpptmp}/tmpflist.m3u"
loop="${mpptmp}/loop"
lcnt="${mpptmp}/lcnt"
tpt="${mpptmp}/tpt"
shuffle="${mpptmp}/shuffle"
CurrTrack="${mpptmp}/trackno"
CurPosnFile="${mpptmp}/ANS_time_pos"
CurPause="${mpptmp}/ANS_pause"
tolua="${mpptmp}/tolua"

# Cmode= ; TimeStyle=
case $1 in
1)
	TimeStyle=1
	;;
2)
	Cmode=1
	;;
3)
	TimeStyle=1
	Cmode=1
	;;
[eEqQ])
	exit 0
	;;
esac

getVolume(){
	if [[ $(type -p amixer) != '' ]]; then
		local pvt=( "Master" "Master Surround" )
		orgifs="$IFS"; IFS=$'\n'
		for x in ${pvt[@]}; do
			pvol+=($(amixer get "$x" 2> /dev/null))
		done
		IFS="$orgifs"

		for d in "${pvol[@]}"
		do
			case $d in
			*Mono:*Playback*)
				read -a so <<<$d
				p=${so[5]%]*}; p=${p#*[}
				s=${so[3]%]*}; s=${s#*[}
				;;
			*Left:*Playback*)
				read -a so <<<$d
				p=${so[6]%]*}; p=${p#*[}
				s=${so[4]%]*}; s=${s#*[}
				;;
			esac
		done
		echo "Vol:$s=$p"
	else
		echo ""
	fi
}

# clearLastLine() {
# 	tput cuu 1 && tput el
# }

ClearHeader(){
	if [[ $4 -eq 1 ]]; then
		clear && echo -en "\e[3J"
		printf "\033c"
	else
		clear
		printf "\e]0;wx media player - track progress\a"
	fi

	vol1=$(getVolume); mute=${vol1#*=}; vol=${vol1%=*}
	if [[ $mute == 'off' ]]; then vol='Mute'; fi

	local mcol=$2
	tcol=$((mcol-3))
	local hline=$(printf "~%.0s" $(seq $((mcol-2))) )

	if [[ -e "$loop" ]]; then
		myloop=$(< "$loop" )
		myloop=${myloop//[^0-9]/}
	fi
	if [[ -e "$lcnt" ]]; then
		mylcnt=$(< "$lcnt" )
		lptxt="Loop ${mylcnt}/${myloop}. "
	else
		lptxt=
	fi

	if [[ -e "$tpt" ]]; then
		mytpt=$(< "$tpt" )
		mytpt=${mytpt#*::}
	else
		mytpt=
	fi

	if [[ -e "$shuffle" ]]; then
		shfl="Shuffle On."
	fi


	# TrackName="$1"
	# TrackName=${TrackData[4]}

	if [[ ${TrackData[2]} -ge 2 ]]; then ch='St' ; else ch='Mn' ; fi
	InfoStr="$ch/${TrackData[1]}kHz/${TrackData[3]}kbps|$vol"

	play1=$(< "$CurPause")
	if [[ $play1 == 'no' ]]; then play='Playing'; else play='Paused'; fi

	il=${#InfoStr}
	npStr=" Now ${play} Track No: $3/${mytpt} ${lptxt}${shfl}"
	nl=${#npStr}
	il=$((tcol-(il+nl)))
	local esps=$(printf " %.0s" $(seq $((il+2))) )
	if [[ $il -le -2 ]]; then
		printf "\n%.${tcol}s\n %s\n%.${tcol}s\n%s\n" "${npStr}" "${InfoStr}" " $TrackName" " $hline"
	else
		printf "\n%.$((tcol+2))s\n%.${tcol}s\n%s\n" "${npStr}${esps}${InfoStr}" " $TrackName" " $hline"
	fi
}

main(){
	while :
	do
		w_h=$(stty size)
		nrows=${w_h% *}; ncols=${w_h#* }
		if [[ $w_h == '' ]]; then
			nrows=24; ncols=80
		fi

		trackno3=$trackno1
		CurPosnRec=$CurPosn

		if [[ ! -d $mpptmp ]]; then
			echo -e "\nwx media player is not playing - exiting...\n"
			break
		else
			if [[ -s '/tmp/wxmptmp/trackno' ]]; then
				trackno1=$(< '/tmp/wxmptmp/trackno' )
				trackno=$((trackno1*2))
			fi

			# echo "pausing_keep_force get_property time_pos" > "$wxfifo"
			sleep 1

			if [[ $trackno3 -ne $trackno1 ]]; then
				orgifs="$IFS"; IFS=$'\n'
				local TrackData=($(perl -lne '
					if ( $. == '"${trackno}"' ) {
						# s/[^[:ascii:]]/+/g;
						(my @val) = $_ =~ m{^#EXTINF:([0-9]*),(.*?),(.*?),(.*?),(.*?)$};
						${val[1]} = sprintf("%d",${val[1]}/1000);
						print "$_\n" for ( @val );
						last;
					}' "${flist}" ))
				if [[ ${TrackData[4]} == '' ]]; then
					trackno=$((trackno+1))
					TrackData[4]=$(perl -lne '
						if ( $. == '"${trackno}"' ) {
							# s/[^[:ascii:]]/+/g;
							(my $val) = $_ =~ m{^.*\/(.*?)$};
							print "$val";
							last;
						}' "${flist}" )
				fi
				IFS="$orgifs"
			fi
			TrackLength=${TrackData[0]}
			TrackName=${TrackData[4]}



			TotalWidth=$((ncols-26))
			TermWidth=$ncols

			if [[ -z $TimeStyle ]]; then TimeStyle1=1 ; else TimeStyle1=2 ; fi

			if [[ -e "${CurPosnFile}" ]]; then
				ao=($(awk '{
					tl='"$TrackLength"';
					CurPosn=$1;
					CurPos=int(((CurPosn/tl) * '"$TotalWidth"'));
					Posntosh=int((CurPosn/tl) * 100 );
					ft=(tl-(CurPosn+1));
					# ft=(tl-(CurPosn));
					tt=tl;
					if ( '"$TimeStyle1"' == 1 )
						ts=1;
					else
						ts=2;

					if ( ts == 1 ) ft=(CurPosn+1);
					ttg=sprintf( "%02d:%02d:%02d",(ft/3600),((ft/60)%60),(ft%60));
					tt1=sprintf( "%02d:%02d:%02d",(tt/3600),((tt/60)%60),(tt%60));
					if ( ts == 2 ) ttg="-"ttg;
					print CurPos, Posntosh, ttg, tt1;
				}' < "${CurPosnFile}" ))
			else
				ao[0]=1; ao[1]=0; ao[2]=0; ao[3]=0;
			fi

			CurPosn=$(echo ${ao[0]})
			if [[ $CurPosn == '' ]]; then CurPosn=$CurPosnRec ; fi
			Posntosh=$(printf '%3d' ${ao[1]} )
			if [[ $CurPosn -eq 0 ]]; then
				LeftPosn=$(((TotalWidth-1)-CurPosn))
			else
				LeftPosn=$(((TotalWidth)-CurPosn))
			fi

			ttg=${ao[2]}
			ttm=${ao[3]}
			case $ttg in
			'-'*)  ttg=$ttg		;;
			*)     ttg=" $ttg"  ;;
			esac

			progtogo=$(printf '=%.0s' $(seq $CurPosn) )
			progleft=$(printf '.%.0s' $(seq $LeftPosn) )
			if [[ ${Posntosh} -ge 99 ]]; then progleft='=' ; fi
			ShowLine="${Posntosh}% [${progtogo}${progleft}]${ttg}/${ttm} "

			hdr=$(ClearHeader "$TrackName" "$ncols" "$trackno1" 1 )

			if [[ $Cmode ]]; then
				printf "%s\r" "${ShowLine}"
			else
				printf "%s\n%s\r" "${hdr}" "${ShowLine}"
				if [[ $(type -p tput) != '' ]]; then
					tput civis
				else
					setterm -cursor off
				fi
			fi
		fi
	done
}

main

exit 0

: '
Things required for music player from lua



'
