
--------- calendar by Paramvir (2017)

local w = {true}

local TORADIANS = 3.14159265358979323846/180

local tonum   = tonumber
local sprintf = string.format
local lmatch  = string.match
local tolower = string.lower
local length  = string.len
local date    = os.date
local time    = os.time

local lcollectgarbage = collectgarbage

local lcairo_move_to         = cairo_move_to
local lcairo_rotate          = cairo_rotate
local lcairo_show_text       = cairo_show_text
local lcairo_stroke          = cairo_stroke
local lcairo_fill            = cairo_fill
local lcairo_save            = cairo_save
local lcairo_restore         = cairo_restore
local lcairo_set_source_rgba = cairo_set_source_rgba
local lcairo_arc             = cairo_arc
local lcairo_set_line_width  = cairo_set_line_width

-- package.path = package.path .. ";./lib/?.lua;"

local FC = require("luafunc")

local wxsplit  = FC.wxsplit
local to_rgba  = FC.to_rgba
local set_font = FC.set_font
local msgout   = FC.msgout
local make_txt = FC.make_txt

function w.calendar(p)
    local cr       = p.cr
    local px       = p.lwx + 6
    local py       = p.lwy
    local fontInfo = p.f         or 11
    local wdateS   = p.d         or nil
    local wkStart  = p.w         or 1
    local tdayCol  = p.h         or '1:0x00ff00:1'
    local wdayCol  = p.l         or '0x00ff00:1'
    local mdayCol  = p.m         or '0xffff55:1'
    local ndayCol  = p.o         or '0xaaaaaa:1'
    local mNameCol = p.q         or '0xffa500:1'
    local mNameOrientation = p.r or 'l'


    ------ colour fixes
    local x1,x2 = lmatch(wdayCol,'^(.-)%:(.-)$')
    wdayCol = {x1,x2}
    local x1,x2 = lmatch(mdayCol,'^(.-)%:(.-)$')
    mdayCol = {x1,x2}
    local x1,x2 = lmatch(ndayCol,'^(.-)%:(.-)$')
    ndayCol = {x1,x2}
    local x1,x2 = lmatch(mNameCol,'^(.-)%:(.-)$')
    mNameCol = {x1,x2}
    local xt = wxsplit(tdayCol,':')
    local hlightLight = {(xt[2] or '0x00ff00'),(xt[3] or 1)}
    local hlightDark  = {(xt[4] or '0x000000'),(xt[5] or 1)}
    hlight = xt[1]

    local fontTable = wxsplit(fontInfo,':')
    local fontName,fontSize = 'DejaVu Sans',11
    if tonum(fontTable[1]) then
        fontSize = fontTable[1]
        fontName = fontTable[2] or 'DejaVu Sans'
    elseif tonum(fontTable[2]) then
        fontSize = fontTable[2]
        fontName = fontTable[1] or 'DejaVu Sans'
    end

    lcairo_save(cr)
    set_font({ cr, fontInfo })
    lcairo_restore(cr)

    local x1,x2
    if wdateS then x1,x2 = lmatch(wdateS,'^(.-)%:(.-)$') end
    if x1 and length(x1) < 4 then
        return msgout("Error :( option -d the year needs to be a 4 digit number.")
    end
    -- get date values if no input
    if not x1 or not x2 then
        ---- the need to get year and month from conky rather than os.date is weird
        ---- If there is no time statement before lua calendar call in conky - then
        ---- the lua os.date is not correct - as the date derived is EST not local
        -- msgout(date())
        local da = conky_parse('${time %Y:%m}')
        x1,x2    = lmatch(da,'^(.-)%:(.-)$')
        -- msgout(date())
    end
    local year  = x1
    local month = x2
    -- get last month values
    local t0 = time{year=year, month=month, day=0}
    local d0 = date("*t", t0)
    local mthDays0 = d0.day
    -- get current month values
    local t = time{year=year, month=month+1, day=0}
    local d = date("*t", t)
    local mthDays, stDay = d.day,(d.wday - d.day - wkStart + 1)%7
    if stDay == 0 then stDay = 7 end

    local cal,c = {true},1
    cal[c] = date('%B - %Y',t)
    c = c + 1

    local dtable = {true, true, true, true, true, true, true}
    if wkStart == 1 then
        dtable = {'Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'}
    elseif wkStart == 2 then
        dtable = {'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa', 'Su'}
    end
    for x = 1, #dtable do
        cal[c] = dtable[x]
        c = c + 1
    end

    -- balance empty days in calendar
    local nstDays = 42 - (mthDays + stDay)
    -- get the last month days to show
    local mthDays1 = mthDays0 - stDay

    -- days before present calendar start
    for x = mthDays1+1, mthDays0 do
        cal[c] = x
        c = c + 1
    end
    -- current calendar + nstDays and adjust them
    for x=1, mthDays + nstDays do
        if x > mthDays then x = x - mthDays end
        cal[c] = x
        c = c + 1
    end
    local mcolour, cnty, cntx = mdayCol, 1, 0   --stDay
    local orgMonth = month..year
    local nowMonth = tonum(date('%m%Y'))
    local nowDay   = tonum(date('%d'))
    -- msgout(nowMonth,month,nowDay,stDay,orgMonth)
    -- local nowDay   = 26                    ---- test date hlight
    local style
    for x = 2, #cal do
        if x > 1 and x < 9 then
            cnty = 2
            if x == 2 then cntx = 0 end
        end
        if x > 8 then
            local j = x - 9
            if j % 7 == 0 then
                cnty = cnty + 1
                cntx = 0
            end
        end
        local align
        cntx = cntx + 1

        ----- factors for hlight positioning
        local xfact,yfact = ((fontSize*2.1) * cntx),((fontSize*1.7) * cnty)
        if x > 8 and x <= stDay + 8 or x > mthDays + stDay + 8 then
            ------ Settings for last n next month
            mcolour = ndayCol
            style = 'label:'..fontName..':'..fontSize..':0'
        elseif cal[x] == nowDay and tonum(nowMonth) == tonum(orgMonth) then
        -- elseif cal[x] == nowDay then
            -- print(x,cal[x],tonum(nowMonth),tonum(month))
            style = 'label:'..fontName..':'..fontSize..':1'
            ------ finer corrections for current date placement
            local hfact = {}
            if cal[x] < 10 then
                hfact = {fontSize*0.10,fontSize*0.33}
            else
                hfact = {fontSize*0.125,fontSize*0.33}
            end
            lcairo_save(cr)
            lcairo_set_line_width(cr, 1)
            lcairo_arc(cr,
                px + xfact + hfact[1],
                py + yfact - hfact[2],
                fontSize*0.9, TORADIANS, 0)
            if tonum(hlight) == 1 then
                ------ Settings for current date hlight ring
                mcolour = hlightLight
                lcairo_set_source_rgba(cr,to_rgba(mcolour))
                lcairo_stroke(cr)
            elseif tonum(hlight) == 2 then
                ------ Settings for current date hlight filled ring
                mcolour = hlightDark
                lcairo_set_source_rgba(cr,to_rgba(hlightLight))
                lcairo_fill(cr)
            end
            lcairo_restore(cr)
        else
            if x > 1 and x < 9 then
                ----- Settings for wdays Su Mo Tu etc
                style = 'label:'..fontName..':'..fontSize..':1'
                mcolour = wdayCol
            else
                ----- Settings for days of the active month
                style = 'label:'..fontName..':'..fontSize..':0'
                mcolour = mdayCol
            end
        end
        local extents = cairo_text_extents_t:create()
        tolua.takeownership(extents)

        lcairo_save(cr)
        make_txt({
            cr      = cr,
            text    = cal[x],
            px      = px + xfact,
            py      = py + yfact,
            align   = 'c',
            icolour = mcolour,
            style   = style,
            ext     = extents,
        })
        lcairo_restore(cr)
    end
    mNameOrientation = tolower(mNameOrientation)
    if mNameOrientation ~= 'na' then
        lcairo_save(cr)
        set_font({ cr, fontName..':'..fontSize..':bold'})
        lcairo_set_source_rgba(cr,to_rgba(mNameCol))
            ------ left
        if mNameOrientation == 'l' then
            lcairo_move_to(cr, px+10, py+(fontSize*13.5))
            lcairo_rotate(cr,TORADIANS*(-90))
            ------ left flip
        elseif mNameOrientation == 'lf' then
            lcairo_move_to(cr, px, py+(fontSize*2.7))
            lcairo_rotate(cr,TORADIANS*(90))
            ------ right
        elseif mNameOrientation == 'r' then
            -- lcairo_move_to(cr, px+190, py+(fontSize*13.5))
            lcairo_move_to(cr, px+(fontSize*16.5), py+(fontSize*13.5))
            lcairo_rotate(cr,TORADIANS*(-90))
            ------ right flip
        elseif mNameOrientation == 'rf' then
            lcairo_move_to(cr, px+(fontSize*16), py+(fontSize*2.5))
            lcairo_rotate(cr,TORADIANS*(90))
            ------ top
        elseif mNameOrientation == 't' then
            lcairo_move_to(cr, px+(fontSize*1.6), py+(fontSize*2.1))
            ------ bottom
        elseif mNameOrientation == 'b' then
            lcairo_move_to(cr, px+(fontSize*1.6), py+(fontSize*15))
        end
        lcairo_show_text(cr,cal[1])
        lcairo_restore(cr)
    end

    return ''
end

lcollectgarbage()

return w

