
--[[
	Paramvir Likhari - conkywx 2011 - 2018 - GPL3
	common library for use with conkywx lua implementation
--]]

require 'cairo'
require 'imlib2'

local tconcat  = table.concat
local tonum    = tonumber
local lremove  = os.remove
-- local ssub     = string.sub
local lexecute = os.execute
local lmatch   = string.match
local lgsub    = string.gsub

local lcollectgarbage = collectgarbage

local lcairo_destroy             = cairo_destroy
local lcairo_surface_destroy     = cairo_surface_destroy
local lcairo_create              = cairo_create
local lcairo_xlib_surface_create = cairo_xlib_surface_create

local lconky_set_update_interval = conky_set_update_interval
local lconky_parse               = conky_parse
local lconky_config              = conky_config

-- package.path = package.path .. ";./lib/?.lua;"
-- myPackage.path = myPackage.path .. "./lib/?.lua;./examples/?.lua;"
local myPackage = package
myPackage.path = myPackage.path .. ";./lib/?.lua;./examples/?.lua;"
-- print(myPackage.path)

local FC = require("luafunc")

local wxsplit    = FC.wxsplit
local ExecInfo   = FC.ExecInfo
local msgout     = FC.msgout
local getopts    = FC.getopts
local FileExists = FC.FileExists

local wxf = {true}

-- remove the daynight moon angle file
-- a fresh one will get generated if daynight widget is running
lexecute( [[rm ~/.config/conkywx/conkywx_internal/wxdnmoon 2> /dev/null ]] )

function conky_startup()
    local GF = require("startup")
    local startup = GF.startup
    startup()
    return ''
end

function conky_if_running(...)
    local GF = require("if_running")
    local if_running = GF.if_running
    return if_running(...)
end

function conky_reset(val,path)
    local fname
    if path then
        fname = tconcat({lmatch(path,"(.+)/"), '/', lmatch(conky_config,"([^/]+)$")})
    end
    val = tonum(val) or nil
    if not val then
        lconky_set_update_interval(5)
        os.exit(0)
    elseif val and val == 1 then
        if not FileExists(fname) then
            lexecute(tconcat({'echo ',conky_info.update_interval,' > ',fname}))
        end
        lconky_set_update_interval(5)
    elseif val and val == 2 then
        if FileExists(fname) then
            val = ExecInfo({rfile = fname})
            lconky_set_update_interval(tonum(val))
            lremove(fname)
        end
    end
    return ''
end

function conky_resetMusic()
    local players = { 'vlc', 'audacious', 'mplayer'}
    local count = 0
    for i = 1, #players do
        if conky_if_running(players[i]) == 1 then
            count = count + 1
        end
    end
    if count == 0 then
        lremove('/tmp/audData')
        lremove('/tmp/trackPlaying')
        lconky_set_update_interval(3)
        os.exit(0)
    end
    return ''
end

function conky_if_exists(...)
    local instuff = tconcat({...},' ')
    -- local GF = require("if_running")
    -- local if_running = GF.if_running
    local if_exists  = FC.FileExists
    if if_exists(instuff) then
        return '1'
    else
        return '-1'
    end
end


function conky_pre_exec(...)
	local GF = require("pre_exec")
	local pre_exec = GF.pre_exec
	return pre_exec(...)
end

function conky_sysinfo(...)
    local GF = require("sysinfo")
    local sysinfo = GF.sysinfo
    return sysinfo(...)
end

function conky_pacman_update()
	local GF = require("pacman_update")
	local pacman_update = GF.pacman_update
	return pacman_update()
end

function wxf.showcond(p)
	local GF = require("showcond")
	local showcond = GF.showcond
	showcond(p)
end

function wxf.lexec(p)
	local GF = require("lexec")
	local lexec = GF.lexec
	lexec(p)
end

function wxf.thermometer(p)
    local GF = require("thermometer")
    local thermometer = GF.thermometer
    thermometer(p)
end

function wxf.gauge(p)
    local GF = require("gauge")
    local gauge = GF.gauge
    gauge(p)
end

function wxf.calendar(p)
    local GF = require("calendar")
    local calendar = GF.calendar
    calendar(p)
end

function wxf.wxrings(p)
	local GF = require("wxrings")
	local wxrings = GF.wxrings
	wxrings(p)
end

function wxf.wxgraph(p)
	local GF = require("wxgraph")
	local wxgraph = GF.wxgraph
	wxgraph(p)
end

function wxf.bargraph(p)
	local GF = require("bargraph")
	local bargraph = GF.bargraph
	bargraph(p)
end

function wxf.barometer(p)
	local GF = require("barometer")
	local barometer = GF.barometer
	barometer(p)
end

function wxf.anemometer(p)
	local GF = require("anemometer")
	local anemometer = GF.anemometer
	anemometer(p)
end

function wxf.clock(p)
	local GF = require("wxclock")
	local clock = GF.clock
	clock(p)
end

function wxf.daynight(p)
	local GF = require("daynight")
	local daynight = GF.daynight
	daynight(p)
end

function wxf.vumeter(p)
	local GF = require("vumeter")
	local vumeter = GF.vumeter
	vumeter(p)
end

function wxf.scroller(p)
    local GF = require("scroller")
    local scroller = GF.scroller
    scroller(p)
end

function wxf.smscroller(p)
	local GF = require("smooth_scroller")
	local smscroller = GF.smscroller
	smscroller(p)
end

function wxf.music_sort(p)
	local GF = require("music_sort")
	local music_sort = GF.music_sort
	music_sort(p)
end

function wxf.background(p)
	local background = FC.background
	background(p)
end

function wxf.showimage(p)
	local GF = require("showimage")
	local showimage = GF.showimage
	showimage(p)
end


function wxf.textformat(p)
	local GF = require("textformat")
	local textformat = GF.textformat
	textformat(p)
end


function conky_main(...)
    if not conky_window then return end
    local cs = lcairo_xlib_surface_create(conky_window.display, conky_window.drawable, conky_window.visual, conky_window.width, conky_window.height)
    local cr = lcairo_create(cs)
    lcairo_surface_destroy(cs)

    local conf_name = '/tmp/conkywx_tmp_' .. lconky_config:match("([^/]+)$")
    if (tonum(lconky_parse("${updates}")) % 10) == 3 and FileExists(conf_name)  then
        local uint = ExecInfo({ rfile = conf_name })
        if uint then
            lconky_set_update_interval(uint)
            lremove(conf_name)
        end
    end

    if (...) then
        local str = tconcat({...}, ' ')
        if lmatch(..., '\\%-') then
            str = lgsub(str, '\\%-', '-')
        end
        local p = getopts(str)
        p.cr    = cr           -- pass cr to the function called in another file
        wxf[p.n](p)
    else
        error( "\nNo lua command line options given to conkywx lua!!!\n")
    end

    lcollectgarbage()
    lcairo_destroy(cr)
    cr = nil


    return ''
end  -- end main function


--[[
    cat /sys/devices/platform/coretemp.0/hwmon/hwmon0/temp1_input
    TODO: WindVane make the cardinal ring tun instead of the compass arrow
    TODO: get a slotted ring for seconds

--]]
