#!/usr/bin/perl

=comments
    ------------------------------------------------------
    Copyright GPL v3 - Paramvir Likhari 2012/2018
    ------------------------------------------------------
=cut

my $version   = $ENV{version} ;
my $progSince = '2012/2018';

use strict;
use warnings;

sub msgout  { print STDERR "@_ \n" }
sub msgoutn { print STDERR "@_"    }

if (! defined $ARGV[0]) { die "Error :( no command line arguments found $!" }
my ( $temppath, $input_template, $cwxpath, $input_element, $config_file, $forecast_trends, $msgs_file, $clopt1, $default_wx, $default_fct, $anyUnitWx, $anyUnitFct, $solValue, $waqidata, $fctBackUrl ) = split(/\|/,"$ARGV[0]" );

use FindBin;
# use lib "$FindBin::RealBin" . "/lib";
use lib "$FindBin::RealBin";

use wxDate::wxDate;
use fixConfig;

# get the current date time information
my ($sec,$min,$hour,$day,$mon,$year,$yday,$monAbb,$monFull,$wdayAbb,$wdayFull,$nowtime) =
    split(/:/,time2str("%S:%M:%H:%-e:%m:%Y:%j:%b:%B:%a:%A:%s"));
my $ltime    = "$wdayAbb $monAbb $day ${hour}:${min}:${sec} ${year}";
my $wx_tzone = tz_offset();

my $HOME = $ENV{"HOME"};
my $config_dir_path      = "$HOME/.config/conkywx";
my $conkywx_internal_dir = "$config_dir_path/conkywx_internal";
# my $system_config_file   = "$config_dir_path/system.conf";

my ($tempname) = $temppath =~ m{/tmp/(.*?)$};

my $die_error       = "$temppath/die_error";
my $currdatam       = "$temppath/wxcurrdata.html" ;
my $currdatax       = "$temppath/wxcurrdata_2.xml";
my $temp_template   = "$temppath/wxtemp_template";
my $temp_templatesc = "$temppath/wxtemp_templateSC";
my $wund_debug_log  = "$temppath/wxwund_debug.log";
my $fallBackFct     = "$temppath/fallBackFct";
# my $stalexml        = "$temppath/stalexml";
# my $gdacsData       = "$temppath/gdacsData";
# my $newsData        = "$temppath/newsData";

# locally global variables
my ( %conf, %Mag_2day, %Mag_Nday, %TodayInfo, %output, @trends_out, %fctinfo, %fctBackup, %LMsg, $debug, $iwxpath, $iwxpathfcast, $iwindpath, %langx, %wx, %wxtrend );

my %wxsite = (
    '2' => 'wx',
    '3' => 'msn',
    'w' => 'wund',
    'm' => 'metar',
    'a' => 'accu',
    'd' => 'darkSky',
    'y' => 'yahoo',
    't' => 'timeDate',
    );

my $wxMainTitle = $wxsite{$default_wx} ;

my $wxFctTitle  = $wxsite{$default_fct} ;

my %timeFormat = (
   '1' => '%I:%M %p' ,
   '2' => '%I:%M %P' ,
   '3' => '%I:%M'    ,
   '4' => '%-I:%M%P' ,
   '5' => '%R'       ,
    );

# if ( -e $stalexml ) {
#     $wx{alertdata} .= "Using only Main Data - pws data is not reliable as it is stale. Change xml_ID and xml_ID2 in conkywx config file.";
#     unlink $stalexml
# }

### constants
# my ( %gkey, @naK, %naKeys, %dateSuffix, @suffix, $asciiMapKeyList, %asciiMap,
my ( %gkey, @naK, %naKeys, %dateSuffix, @suffix,
    %sm_m, %fd_d, %m_sm, %d_fd, %text_ref, %num_ref_main, %num_ref_l_check_main1,
    %num_ref_l_check_main2, %naList, @wxstate, @rainCond );

CONSTANTS: {

    @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);

    %text_ref = qw( Location_Name 1 Long1 1 NowImage 1 Lat1 1 Updated 1 Station_ID 1 Station_Name 1 curr_icon 1 condition 1 METAR 1 station_id 1 last_update_xml 1 Wind_From 1 wind_dir 1 CLC1 1 CLC2 1 CLC3 1 CLC4 1);

    %num_ref_main = qw(Rainfall 1 Wind_Speed 1 Wind_Gust 1 Pressure 1 Visibility 1 solarRad 1 CLH1 1 CLH2 1 CLH3 1 CLH4 1 );

    %num_ref_l_check_main1 = qw( Temp_now 1 Dew_Point 1 Humidity 1);

    %num_ref_l_check_main2 = qw( Temp_now 1 Dew_Point 1 );

    %naList = qw(-999.00 1 -999.0 1 -999 1 -9999.00 1 -9999.0 1 -9999 1 N/A 1 NA 1 null 1 "" 1 -- 1 ) ;

    @wxstate = (
        [ 'thunderstorm',   'tstorm',  'n', 'n', 'thunderstorms'  ],
        [ 'thunder',        'tstorm',  'n', 'n', 'thunderstorms'  ],
        [ 'tstorm',         'tstorm',  'n', 'n', 'thunderstorms'  ],
        [ 't-storm',        'tstorm',  'n', 'n', 'thunderstorms'  ],
        [ 'tornado',        'tstorm',  'n', 'n', 'tornado'        ],
        [ 'hurricane',      'tstorm',  'n', 'n', 'hurricane'      ],
        [ 'tropical storm', 'tstorm',  'n', 'n', 'tropical storm' ],
        [ 'blizzard',       'sleet',   'x', 'x', 'blizzard'       ],
        [ 'mixed',          'sleet',   'x', 'x', 'sleet'          ],
        [ 'hail',           'sleet',   'x', 'x', 'hail'           ],
        [ 'sleet',          'sleet',   'x', 'x', 'sleet'          ],
        [ 'flurr',          'sleet',   'x', 'x', 'flurries'       ],
        [ 'snow',           'snow',    'q', 'q', 'snow'           ],
        [ 'frig',           'pcloudy', 'x', 'x', 'very cold'      ],
        [ 'very cold',      'pcloudy', 'x', 'x', 'very cold'      ],
        [ 'spray',          'rain',    'i', 'i', 'spray'          ],
        [ 'squall',         'rain',    'i', 'i', 'squalls'        ],
        [ 'rain',           'rain',    'i', 'i', 'rain'           ],
        [ 'precip',         'rain',    'i', 'i', 'rain'           ],
        [ 'shower',         'rain',    'i', 'i', 'showers'        ],
        [ 'downpour',       'rain',    'i', 'i', 'rain'           ],
        [ 'drizzle',        'rain',    'i', 'i', 'drizzle'        ],
        [ 'fog',            'fog',     '9', '9', 'fog'            ],
        [ 'mist',           'fog',     '9', '9', 'mist'           ],
        [ 'sandstorm',      'dust',    '7', '7', 'sandstorm'      ],
        [ 'duststorm',      'dust',    '7', '7', 'duststorm'      ],
        [ 'ash',            'dust',    '7', '7', 'ash'            ],
        [ 'dust',           'dust',    '7', '7', 'dust'           ],
        [ 'sand',           'dust',    '7', '7', 'sand'           ],
        [ 'haz',            'hazy',    '9', '9', 'hazy'           ],
        [ 'smok',           'hazy',    '9', '9', 'smoke'          ],
        [ 'blust',          'windy',   '6', '6', 'wind gusts'     ],
        [ 'wind',           'windy',   '6', '6', 'windy'          ],
        [ 'partly cloudy',  'pcloudy', 'c', 'C', 'partly cloudy'  ],
        [ 'pcloudy',        'pcloudy', 'c', 'C', 'partly cloudy'  ],
        [ 'p cloudy',       'pcloudy', 'c', 'C', 'partly cloudy'  ],
        [ 'p cldy',         'pcloudy', 'c', 'C', 'partly cloudy'  ],
        [ 'mostly sunny',   'fair',    'b', 'B', 'clear'          ],
        [ 'msunny',         'fair',    'b', 'B', 'clear'          ],
        [ 'mostly clear',   'pcloudy', 'c', 'C', 'mostly clear'   ],
        [ 'mclear',         'pcloudy', 'c', 'C', 'mostly clear'   ],
        [ 'psunny',         'pcloudy', 'd', 'D', 'partly cloudy'  ],
        [ 'partly sunny',   'pcloudy', 'd', 'D', 'partly cloudy'  ],
        [ 'mcloudy',        'mcloudy', 'd', 'D', 'mostly cloudy'  ],
        [ 'm cloudy',       'mcloudy', 'd', 'D', 'mostly cloudy'  ],
        [ 'm cldy',         'mcloudy', 'd', 'D', 'mostly cloudy'  ],
        [ 'mostly cloudy',  'mcloudy', 'd', 'D', 'mostly cloudy'  ],
        [ 'sun',            'clear',   'a', 'A', 'clear'          ],
        [ 'clear',          'clear',   'a', 'A', 'clear'          ],
        [ 'pleasant',       'clear',   'a', 'A', 'clear'          ],
        [ 'breezy',         'clear',   'a', 'A', 'clear'          ],
        # [ 'warm',           'clear',   'a', 'A', 'clear'          ],
        [ 'wundy',          'clear',   'a', 'A', 'clear'          ],
        [ 'fine',           'clear',   'a', 'A', 'clear'          ],
        [ 'nice',           'clear',   'a', 'A', 'clear'          ],
        [ 'dry',            'clear',   'a', 'A', 'clear'          ],
        [ 'few',            'fair',    'b', 'B', 'clear'          ],
        [ 'fair',           'fair',    'b', 'B', 'clear'          ],
        [ 'cloud cover',    'cloudy',  'f', 'f', 'overcast'       ],
        [ 'cloudy',         'cloudy',  'f', 'f', 'overcast'       ],
        [ 'cldy',           'cloudy',  'f', 'f', 'overcast'       ],
        [ 'overcast',       'cloudy',  'f', 'f', 'overcast'       ],
        [ 'funnel cloud',   'cloudy',  'f', 'f', 'overcast'       ],
        [ 'considerable cloud',   'cloudy',  'f', 'f', 'overcast' ],
        [ 'cloud',          'pcloudy', 'd', 'D', 'partly cloudy'  ],
        [ 'ice',            'sleet',   'x', 'x', 'ice'            ],
        [ 'unknown',        'na',      '-', '-', 'na'             ],
        [ 'n/a',            'na',      '-', '-', 'na'             ],
        [ 'na',             'na',      '-', '-', 'na'             ],
        [ 'unknown precipitation','rain','i','i','rain' ],
    );

    %fd_d = (
        Today     => "Tod", Tonight   => "Ton", Monday    => "Mon",
        Tuesday   => "Tue", Wednesday => "Wed", Thursday  => "Thu",
        Friday    => "Fri", Saturday  => "Sat", Sunday    => "Sun",
        );
    %d_fd = reverse %fd_d ;

    %m_sm = (
        January   => "Jan", February  => "Feb", March     => "Mar",
        April     => "Apr", May       => "May", June      => "Jun",
        July      => "Jul", August    => "Aug", September => "Sep",
        October   => "Oct", November  => "Nov", December  => "Dec",
        );
    %sm_m = reverse %m_sm;

    @suffix = (qw(th st nd rd th th th th th th)) x 4;
    @suffix[11,12,13] = qw(th th th);
    %dateSuffix = map { $_ => $suffix[$_] } 0..$#suffix;

    %gkey =  ( D1D=>1, D2D=>1, D3D=>1, D4D=>1, D5D=>1, D6D=>1, D7D=>1, D8D=>1, D9D=>1, D10D=>1 );

    @naK = qw( PTBI PIF PIC PT FL HM BF CT LF CLH1 CLH2 CLH3 CLH4 BI UVM METAR SR SS LOV LOD LODN LODNT CTR CTS NTR NTS ATR ATS SMT MMT SDC MDC MIF MR MS MA MV MROC MROT MIC LU D1F D1I D1T D2F D2I D2T D3F D3I D3T D4F D4I D4T D5F D5I D5T D6F D6I D6T D7F D7I D7T D8F D8I D8T D9F D9I D9T D10F D10I D10T FCTPF FCTTF FCTWF FCTHF FCTUF FCTDF FCTVF FCTSRF FCTSSF FCTMRF FCTMSF FCTLLF FCTLDF FCTMIF FCTTYF GT GP GD GH GW GG GS UU TMP SOL VI BR DP );

    if ( $wxFctTitle ne "accu" ) {
        @naK = ( @naK, qw( D1P D2P D3P D4P D5P D6P D7P D8P D9P D10P ));
    }
    %naKeys = map {  $_ => undef  } @naK ;
}

if ( -e "${temppath}/dummy_debug" ) { $debug = 1 }
else { $debug = 0 }

# Astronomy module - figure out if machine is 64 or 32 bit
my $cwx_magellan;
if ( -e "${cwxpath}/lib/magellan" ) {
    $cwx_magellan = "${cwxpath}/lib/magellan";
}
else {
    my $machine_is = `uname -m`;
    chomp($machine_is);
    $cwx_magellan = (index($machine_is,'x86_64') != -1 ) ? "${cwxpath}/lib/magellan_64" :
    "${cwxpath}/lib/magellan_32";
}


sub do_debug {
    my ( $msg, @datawx1 ) = @_ ;

    open (out_data, '>>', "$wund_debug_log") or die "$LMsg{no_open_file} wund_debug_log: $!";
        print out_data "************* $msg data *************\n";
        print out_data @datawx1;
        print out_data "*****************************************************************\n";
    close (out_data);

}

sub read_msgs {

    if ( -s $msgs_file ) {

        %LMsg = &fixConf($msgs_file, 1);

        if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }
    }
}
&read_msgs;


sub sub_name { ( caller(1) )[3], $LMsg{dbg_msg_str2}, ( caller(1) )[2] ;}

sub read_config {

    if ( -s "$config_file" ) {

        %conf = &fixConf($config_file, 1);

        # Units_downloaded now hardcoded in conkywx.sh as well
        $conf{Units_downloaded}    = "IM"            ;
        # define some defaults
        $conf{today_date_format}   ||= '%A %b %d %Y' ;
        $conf{minimum_fetch_time}  ||= 900           ;
        $conf{Time_format}         ||= '%R'          ;
        $conf{show_degsym_fc_temp} ||= 5             ;
        $conf{moon_outline}        ||= 0             ;
        $conf{moon_time}           ||= '%b %d'       ;
        $conf{use_short_date}      ||= 0             ;

        # wx icons defined -----------------------------------------
        $conf{wx_iconset} ||= 2;
        my ($wp1, $wp2, $wp3, $wp4);
        $wp1 = ($conf{wx_iconset} =~ /^[1-9]$/) ? $conf{wx_iconset} : 0;

        if ( $wp1 ) {
            $iwxpath = "$cwxpath/images/wxicons-$wp1";
        }
        elsif ( -e $conf{wx_iconset} or die $LMsg{no_wx_iconset_path}) {
            $iwxpath = $conf{wx_iconset};
        }

        if ( $conf{wx_fcast_iconset} ) {
            $wp1 = ($conf{wx_fcast_iconset} =~ /^[1-9]$/) ? $conf{wx_fcast_iconset} : 0;
            if ( $wp1 ) {
                $iwxpathfcast = "$cwxpath/images/wxicons-$wp1";
            }
            elsif ( -e "$conf{wx_fcast_iconset}" or die $LMsg{no_wx_iconset_path}) {
                $iwxpathfcast = "$conf{wx_fcast_iconset}";
            }
        }
        else {
            $iwxpathfcast = $iwxpath ;
        }

        # wind icons defined -----------------------------------------
        $conf{wx_wind_iconset} ||= 1;
        $wp1 = ($conf{wx_wind_iconset} =~ /^[1-3]$/) ? $conf{wx_wind_iconset} : 0;

        if ( $wp1 ) {
            $iwindpath = "$cwxpath/images/windrose-$wp1";
        }
        elsif ( -e "$conf{wx_wind_iconset}" or die $LMsg{no_wx_iconset_path}) {
            $iwindpath = "$conf{wx_wind_iconset}";
        }

        $conf{wx_wind_fontset} ||= 2;

        ### Note: Units_to_Use is undefined if 'SI' - so only check for defined or not
        $conf{Units_to_Use} ||= 'IM';
        $conf{Units_to_Use} = uc($conf{Units_to_Use});
        my %unitsUse = ( 'IM' => 1, 'MI' => 1 );
        if ($unitsUse{$conf{Units_to_Use}}) { $conf{Units_to_Use} = 'IM' }
        else { undef $conf{Units_to_Use} }

        $conf{fallBackFct} ||= 1;
        if ( -e "$temppath/fallBackFct" ) {
            $wxFctTitle = $wxsite{$conf{fallBackFct}} ;
        }

        $conf{wundoption} = $conf{wundoption}//'pws';
        $conf{wundoption} = lc($conf{wundoption});

        # msgout($conf{wundoption});
        if ( $conf{wundoption} !~ /\bpws\b|\bair\b/ ) {
            $conf{wundoption} = 'pws'
        }
        # msgout($conf{wundoption});

    }
}
&read_config;

# show pop-up notification
sub show_noti {
    my $msg = shift;
    my $opt = shift or undef;
    if ( $conf{show_notification} or $opt ) {
        my $show_duration = ( $opt ) ? 50000 : 5000 ;
        `/usr/bin/notify-send "$msg" -i "$cwxpath/images/wxicons-2/clear.png" -t $show_duration 2> /dev/null`
    }
}


sub die_errors {
    open (out_data, '>', $die_error ) or die "$LMsg{no_open_file} die_error : $!";
        print out_data "@_ \n";
    close (out_data);
}

sub jsonParser {

    my ( $jsonToDecode ) = @_ ;

    if ( not $jsonToDecode ) { return }

    # msgout($jsonToDecode);

    require jsonPP;
    jsonPP->import( qw(decode_json) );

    my $dec = decode_json( $jsonToDecode ) ;

    return $dec

}



# ------------------------------------------------------
# conversions
# ------------------------------------------------------
# temperature
sub c_f     { return (( $_[0] * 1.8 ) + 32  ) }
sub f_c     { return (( $_[0] - 32 ) / 1.8  ) }
# distance - speed km/h to mi/h is same as km to mi
sub km_mi   { return (  $_[0] * 0.62137119  ) }
sub mi_km   { return (  $_[0] / 0.62137119  ) }
sub mm_in   { return (  $_[0] * 0.039370078 ) }
sub in_mm   { return (  $_[0] / 0.039370078 ) }
sub ft_mtr  { return (  $_[0] * 0.3048      ) }
sub mtr_ft  { return (  $_[0] / 0.3048      ) }
sub kt_mi   { return (  $_[0] * 1.1507794   ) }
# pressure
sub hPa_in  { return (  $_[0] * 0.02953     ) }
sub in_hPa  { return (  $_[0] / 0.02953     ) }
sub unity   { return (  $_[0]               ) }


my ( $d, $u_temp, $u_speed, $u_dist_big, %fn, $u_dist_mid, $u_dist_small, $u_pressure, $u_sol, $lua_wx_units, %un, );

if ( $conf{Units_to_Use} ) {
    %fn = (
        f_temp       => \&c_f,
        f_speed      => \&km_mi,
        f_dist_big   => \&km_mi,
        f_dist_mid   => \&mtr_ft,
        f_dist_small => \&mm_in,
        f_pressure   => \&hPa_in,
        f_unity      => \&unity,
    );
    $d               = '%1.2f';
    $lua_wx_units    = 'F';
}
else {
    %fn = (
        f_temp       => \&f_c,
        f_speed      => \&mi_km,
        f_dist_big   => \&mi_km,
        f_dist_mid   => \&ft_mtr,
        f_dist_small => \&in_mm,
        f_pressure   => \&in_hPa,
        f_unity      => \&unity,
    );
    $d               = '%1.0f';
    $lua_wx_units    = 'C';
}

if ($conf{Units_show}) {
    if ( $conf{Units_to_Use} ) {
        $u_temp       = '°F';
        $u_speed      = 'mph';
        $u_dist_big   = 'mi';
        $u_dist_mid   = 'ft';
        $u_dist_small = 'in';
        $u_pressure   = 'in';
        $u_sol        = 'W/m²';
    }
    else {
        $u_temp       = '°C';
        $u_speed      = 'km/h';
        $u_dist_big   = 'km';
        $u_dist_mid   = 'm';
        $u_dist_small = 'mm';
        $u_pressure   = 'hPa';
        $u_sol        = 'W/m²';
    }
}
else {
    $u_temp       = '°';
    $u_speed      = '';
    $u_dist_big   = '';
    $u_dist_mid   = '';
    $u_dist_small = '';
    $u_pressure   = '';
    $u_sol        = '';
}

sub wx_limits {
    ### $_[0] is value $_[1] is unit
    if    ($_[1] eq 'Temp_now'  and $_[0] < -130 and $_[0] > 140) {
        return( 'NA' )
    }
    elsif ($_[1] eq 'Dew_Point' and $_[0] < -100 and $_[0] > 100) {
        return( 'NA' )
    }
    elsif ($_[1] eq 'Humidity'  and $_[0] <  0   and $_[0] > 100) {
        return( 'NA' )
    }
    else {
        return($_[0])
    }
}


sub trim { $_[0] =~ s/^\s+//; $_[0] =~ s/\s+$//; return $_[0] }

sub round {
    my $intval = $_[0] - int($_[0]);
    return ( $intval >= 0.5 ) ? int( $_[0] + int(($intval)+0.5) ) : int($_[0])
}

my @fwd = qw( n nne ne ene e ese se sse s ssw sw wsw w wnw nw nnw north northnortheast northeast eastnortheast east eastsoutheast southeast southsoutheast south southsouthwest southwest westsouthwest west westnorthwest northwest northnorthwest );
my @dwf = qw(North NNE NE ENE East ESE SE SSE South SSW SW WSW West WNW NW NNW North);

my %convUnits = (
    'Depth'      => [ $fn{f_dist_big}  , "%1.1f $u_dist_big"   ],
    'Temp_now'   => [ $fn{f_temp}      , "%1.0f$u_temp"        ],
    'Dew_Point'  => [ $fn{f_temp}      , "%1.0f$u_temp"        ],
    'Wind_Speed' => [ $fn{f_speed}     , "%1.1f $u_speed"      ],
    'Wind_Gust'  => [ $fn{f_speed}     , "%1.1f $u_speed"      ],
    'Pressure'   => [ $fn{f_pressure}  , "$d $u_pressure"      ],
    'Visibility' => [ $fn{f_dist_big}  , "%1.1f $u_dist_big"   ],
    'Rainfall'   => [ $fn{f_dist_small}, "%1.1f $u_dist_small" ],
    'CLH1'       => [ $fn{f_dist_mid}  , "%1.0f $u_dist_mid"   ],
    'CLH2'       => [ $fn{f_dist_mid}  , "%1.0f $u_dist_mid"   ],
    'CLH3'       => [ $fn{f_dist_mid}  , "%1.0f $u_dist_mid"   ],
    'CLH4'       => [ $fn{f_dist_mid}  , "%1.0f $u_dist_mid"   ],
    'Tide'       => [ $fn{f_dist_mid}  , "%5.2f $u_dist_mid"   ],
    'solarRad'   => [ $fn{f_unity}     , "%1.0f $u_sol"        ],
);


# ------------------------------------------------------
# convert wind in degrees to cardinals and inter-cardinals
# ------------------------------------------------------
sub find_wind_dir {

    # if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    return $_[0] if $_[0] eq 'NA' ;

    $_[0] = lc $_[0];

    if ( $_[0] =~ m{([A-Za-z]+)} ) {
        my $findIdx = sub { 1 while $_[0] ne pop; $#_ };
        my $fidx = &$findIdx($1,@fwd);
        $fidx = ($fidx > 15) ? $fidx - 16 : ($fidx == -1) ? 0 : $fidx ;
        return ($fidx * 22.5, $dwf[$fidx])
    }
    else {
        return $dwf[sprintf("%.0f", ($_[0] % 360)/22.5)]
    }
    return $_[0]
}

# ------------------------------------------------------
# check if value is number including float -ve etc
# ------------------------------------------------------
sub check_num {

    my ($mynum, $otext) = @_;
    my ( $oval );
    if ( ! $mynum ) { $mynum = 0 }
    if ( ! $otext ) { $otext = "NA" }
    # chomp($mynum);
    $mynum = trim($mynum);
    $mynum =~ tr/\x0a\x22\x27//d;
    # $mynum =~ tr/\x0a\x20\x22\x27//d;
    my $tNum = $mynum;

    # msgout($mynum);
    if ( exists $naList{$mynum} ) { $oval = "NA" }
    elsif (lc($mynum) eq "calm" ) { $oval = "Calm" }
    else {
        if    ( exists $text_ref{$otext}    ) { $oval = $mynum }
        elsif ( $otext eq 'Clouds'   ) { $oval = ( $mynum eq "" ) ? "NA" : $mynum }
        elsif ( $otext eq 'Gmt_Time' ) { $oval = fix_tzone($mynum) }
        else {
            $tNum =~ tr/0-9.-//cd;
            # msgout($tNum);
            if ( $tNum ne '' ) {
                $mynum =~ tr/\x20//d;
                if ( exists $num_ref_main{$otext} ) {
                    $oval = ($mynum < 0) ? 'NA' : convert_unit($mynum, $otext)
                }
                elsif ( exists $num_ref_l_check_main1{$otext} ) {
                    $oval = (wx_limits($mynum,$otext) eq 'NA')
                    ? 'NA'
                    : exists $num_ref_l_check_main2{$otext}
                    ? convert_unit($mynum, $otext)
                    : $mynum;
                }
                else { $oval = "$mynum" }
            }
            else { $oval = "NA" }
        }
    }
    return ($oval);
}

sub fix_tzone { my $zone = shift ; return tz_offset($zone)/3600; }

# ------------------------------------------------------
# get present and forecast condition image and font
# ------------------------------------------------------
sub do_get_DI {

    # if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( $opt1, $cond2, $cimage, $daynite ) = @_ ;

    if ( not $cond2 ) { return '' }

    # my $trickWords = 'nice';

    $daynite ||= 2 ; # 2 is daylight time

    if ( $opt1 ) {

        chomp($cond2);
        # $cond2 =~ s/pm|am//i;
        # $cond2 = lc $cond2;
        my $cond1 = $cond2;
        if ( $opt1 != 3 ) {
            ( $cond1 ) = (lc($cond2) =~ m{^(?:nt_)?(.*?)$}) ? $1 : 'NA'
        }
        # msgout($opt1,$cond2, $cimage);

        my ( $imageout, $fday, $fnit, $cnt11, $fnd, $notFound ) ;

        foreach my $ky ( @wxstate ) {
            if ( index( $cond1, @$ky[0] ) != -1 ) {
                msgout("Original Cond: $cimage ${cond1}") if $debug;
                ($cond1, $imageout,$fday,$fnit) = (@$ky[4],@$ky[1],@$ky[2],@$ky[3]);
                msgout("wx Cond      : ${cond1}") if $debug;
                # msgout("wx Cond      : ${cond1}");
                last
            }
        }
        if ( not $imageout ) {
            $imageout = "na"; # $fday = "-"; $fnit = "-" ;
            return 'NA';
            # msgout($cond1,$imageout);
        }

        # make conditions title case
        $cond1 = join '', map { ucfirst lc } split /(\W+)/, $cond1;
        # fix clouds thing
        if ( $opt1 == 3 ) { return $cond1 }

        if ( $opt1 == 1 ) { $wx{uvwx} = $imageout }
        my ($dn_font);
        if ( $opt1 == 1 and $daynite == 1 ) {
            if ( $conf{wx_iconset} == 5 ) {
                if ( $imageout !~ /na|up|tstorm/ ) {
                    $imageout = "nt_" . ${imageout}
                }
            }
            else {
                if ( $imageout =~ /fair|mcloudy|pcloudy|clear/ ) {
                    $imageout = "nt_" . ${imageout}
                }
            }
            $dn_font  = $fnit ;
        }
        elsif ( $opt1 == 1 and $daynite == 2 ) { $dn_font = $fday }

        # copy present condition font and image
        if ( $opt1 == 1 ) {
            $TodayInfo{NowImage} = "${iwxpath}/${imageout}.png";
            return ($dn_font, $cond1)
        }
        # copy daily images and fonts are returned to calling sub
        if ( $opt1 == 2 ) {
            $fctinfo{"d_" . ${cimage} . "_cond"} = $cond1;
            $fctinfo{"d_" . ${cimage} . "_image" } = "${iwxpathfcast}/${imageout}.png" ;
            return $fday
        }
        if ( $opt1 == 5 ) {
            return "$fday;;;$cond1;;;${iwxpathfcast}/${imageout}.png"
        }
    }
}


# sub custom_sort {
#   my ($x, $y, @keys) = @_;
#   for (@keys) {
#       my $cmp = $x->{$_} cmp $y->{$_};
#       return $cmp if $cmp;
#   }
#   return 0;
# }

sub getSeismic {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    require seismic;
    seismic->import( qw(doSeismic) );

    my $retval = doSeismic( $temppath, $LMsg{no_open_file}, %conf );

    $wx{alertdata} = $retval if $retval ;

    return
}


sub rssd {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    require rssReader;
    rssReader->import( qw(rssShow) );

    rssShow( $temppath, $nowtime, $LMsg{no_open_file}, $config_file, %conf  ) ;

    return
}

# ------------------------------------------------------
# get chunks of data as string or array
# ------------------------------------------------------
sub get_line_num {

=comments
    paramvir 2016 get_line_num ver 3
    Does not match the tags - be careful with the 3rd option
    same but much faster as /pat/../pat/ range
    Usage:
    1st opts <- Input String reference \$in_string
        $str = a string reference original string as \$OriginalStringName
    2nd opts <- text to look for
        $startTag = is the start of required block
    3rd opts endTag text
        $endTag can be 1 of 3 options -
            This has to be unique block following the $startTag block
    4th opts
        $choi = arr - get from start to end text as array
        $choi = str - get from start to end text as single string [default]
    5th opts
        $extended = only used in the case of accuweather

    $extended - use 1 for looking up next index value
    $extended - use 2 added length of $startTag

    string startTag endTag returnType extended
=cut

    my ($str, $startTag, $endTag, $choi, $extended ) = @_;
    my ($returnstr, @returnarr ) ;
    $choi ||= 'str' ;
    if ( ! defined($startTag) ) { return \$_ }

    my $st1 = index($$str, $startTag);
    # msgout($st1);
    if ($st1 != -1) {
        my $offset = $st1 + 1;
        my $nd1    = index($$str, $endTag, $offset);
        $returnstr = substr($$str, $st1, $nd1 - $st1);
        if ($extended) {
            $offset = $nd1 + 1;
            $st1    = index($$str, $startTag, $offset);
            if ($st1 != -1) {
                $offset = $st1 + 1;
                $nd1    = index($$str, $endTag, $offset);
                $returnstr .= substr($$str, $st1, $nd1 - $st1);
            }
        }
        if ( $choi eq 'str' ) { return \$returnstr }
        else { @returnarr=split('\n',$returnstr); return \@returnarr }
    }
    else { return \$_ }


    return
}

# use magellan to parse METAR weather data
sub metar_parser {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    require metar;
    metar->import( qw(metarParser) );

    my ($x, $y ) = @_;

    %wx = metarParser( $cwx_magellan, $temppath, $x, $y, %wx );

    return %wx

}


sub parse_html {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    require wxparser;
    wxparser->import( qw(wxParserCurrent) );

    my ($x, $y ) = @_;

    my $userLoc = "$conf{Latitude},$conf{Longitude}" ;


    my ($rwx, $rfct, $waqi, $rbfct) = wxParserCurrent( $temppath, $currdatam, $LMsg{no_open_file}, $wxMainTitle, $wxFctTitle, $conf{darkSkyLoc}, $userLoc, $monAbb, $ltime, $year, $conf{icao_id}, $waqidata, $conf{wundoption}, $fctBackUrl );

    # msgout($$rfct);

    for my $row ( split('\n', $$rwx)) {
        my ($k1, $v1) = split('=>',$row) ;
        # if ( not $v1 ) { $v1 = 'NA' }
        $wx{$k1} = $v1;
        # msgout("current $k1=$v1")
    }
    if ( $$rfct ) {
        for my $row ( split('\n', $$rfct)) {
            my ($k1, $v1) = split('=>',$row) ;
            # if ( $v1 =~ // ) { $v1 = 'NA' }
            $fctinfo{$k1} = $v1;
            # msgout("fct $k1=$v1");
            # msgout("fct $k1=$fctinfo{$k1}")
        }
    }
    if ( $$waqi ) {
        for my $row ( split('\n', $$waqi)) {
            my ($k1, $v1) = split('=>',$row) ;
            if ( not $v1 ) { $v1 = 'NA' }
            $wx{$k1} = $v1;
            # msgout("waqi $k1 = $v1")
        }
    }

    return

}


# ------------------------------------------------------
# xml parser for wunderground weather private weather station xml data
# ------------------------------------------------------
sub parse_pws {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }
    $wx{do_xml} = 1;
    if (-e $currdatax) {
        my ( $xmlstr  ) ;
        {
        open (FILE, "<:crlf", "$currdatax") or die "$LMsg{no_open_file} main data file $!";
            local $/ = undef;
            $xmlstr = <FILE> ;
        close (FILE);
        }

        # # msgout($dsFct2);
        my $pwsData = jsonParser( $xmlstr );

        my %wundhPws = (
            name             => 'neighborhood',
            nameid           => 'stationID',
            latitude         => 'lat',
            longitude        => 'lon',
            elevation        => 'elev',
            last_update_xml  => 'obsTimeUtc',
            temperature      => 'temp',
            dewpoint         => 'dewpt',
            humidity         => 'humidity',
            pressure         => 'pressure',
            precip_today     => 'precip24Hour',
            wind_speed       => 'windSpeed',
            wind_dir_degrees => 'winddir',
            wind_gust_speed  => 'windGust',
            solarRad         => 'solarRadiation',
        );

        while ( my ($k1, $v1) = each %wundhPws ) {

            $wx{$k1} = ($$pwsData{observations}[0]{$v1}) ? $$pwsData{observations}[0]{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

            $wx{$k1} = ($$pwsData{observations}[0]{imperial}{$v1}) ? $$pwsData{observations}[0]{imperial}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        }

        $wx{wind_dir} = find_wind_dir($wx{wind_dir_degrees});

    }
}


sub verifyWX {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ($imageout2, $rpt, $corhum );
    ## fixing forecast information
    my @fd = qw( day temp cond cor font image hum );
    for ($rpt=1; $rpt<=10; $rpt++ ) {
        foreach (@fd) {
            if ( ! $fctinfo{"d_" . $rpt . "_". $_} ) {
                if ( $_ eq "image" ) {
                    $fctinfo{"d_" . $rpt . "_". $_} = "${iwxpathfcast}/na.png";
                }
            }
            else {
                if ( -e "${temppath}/na.png" ) { unlink "${temppath}/na.png" ; }
            }
        }
    }

    $wx{icon} = ( not $wx{icon} or $wx{icon} eq 'NA') ? $wx{condition} : $wx{icon};
    my $iconval = ($wx{icon}) ? $wx{icon} : 'NA' ;
    $iconval =~ s/nt_//;

    $wx{condition} = ( not $wx{condition} ) ? $iconval : $wx{condition} ;
    $wx{condition} = ($wx{condition} eq 'NA') ? $fctinfo{d_1_cond} : $wx{condition} ;

    if ( $wx{icon} =~ /clear/i or $wx{condition} =~ /clear/i ) {
        if ( $wx{visibility} and $wx{visibility} < 3.1 ) {
            $wx{condition} = 'haze';
            $wx{icon}      = 'haze';
        }
    }
    # msgout($wx{condition}, $wx{icon}, $iconval);

    if ($wx{alertdata} ) {
        $wx{alertdata} =~ s{</a>(?:,)?}{ | }g;
        $wx{alertdata} =~ s/<.*?>|\(.*?\)|&nbsp|;/ /g;
        $wx{alertdata} =~ s/\s+/ /g;
        $wx{alertdata} =~ s{\|\s*$}{} ;
    }

    $wx{humidity} = ( ($wx{humidity} ne 'NA' or $wx{humidity} ne 'null') and $wx{humidity} < 0 ) ? 'NA' : $wx{humidity} ;

    if ( $wx{humidity} eq 'NA' and $wx{temperature} and $wx{dewpoint} ) {
        # 100*(EXP((17.625*TD)/(243.04+TD))/EXP((17.625*T)/(243.04+T)))
        my $tempC = f_c($wx{temperature});
        my $dpC   = f_c($wx{dewpoint});
        $wx{humidity} = int(100*((112-(0.1*$tempC)+$dpC) / (112+(0.9*$tempC)))**8);
        $wx{humidity} = ( $wx{humidity} >= 90 ) ? 90 : $wx{humidity};
        # $wx{humidity} = int(100*((112-(0.1*$wx{temperature})+$wx{dewpoint}) / (112+(0.9*$wx{temperature})))**8)
    }
    elsif ( $wx{dewpoint} eq 'NA' and $wx{temperature} and $wx{humidity} ) {
        my $tc = f_c($wx{temperature});
        $wx{dewpoint} = c_f(((($wx{humidity} / 100)**(1/8)) * (112+(0.9*$tc)) + (0.1 * $tc)) - 112);
    }

    if ( not $wx{solarRad} or $wx{solarRad} eq 'NA' ) {
        $wx{solarRad} = ( $solValue ) ? $solValue : 'NA'
    }

    %TodayInfo = (
            Location_Name => $wx{city},
            Station_Name  => $wx{name},
            Station_ID    => $wx{nameid},
            Lat1          => $wx{latitude},
            Long1         => $wx{longitude},
            Updated       => $wx{updated},
            curr_icon     => $wx{icon},
            Temp_now      => $wx{temperature},
            condition     => $wx{condition},
            Visibility    => $wx{visibility},
            CLH1          => $wx{cloud1h},
            CLC1          => (check_num($wx{cloud1t})) ? $wx{cloud1t} : do_get_DI(3,$wx{cloud1t},0),
            Humidity      => $wx{humidity},
            Rainfall      => $wx{precip_today},
            SnowDepth     => $wx{snow},
            Pressure      => $wx{pressure},
            Dew_Point     => $wx{dewpoint},
            METAR         => $wx{metar},
            pollen        => $wx{pollen},
            Wind_Speed    => $wx{wind_speed},
            Wind_From     => $wx{wind_dir},
            Wind_Compass  => $wx{wind_dir_degrees},
            Wind_Gust     => $wx{wind_gust_speed},
            solarRad      => $wx{solarRad},
        );

    if (  ! $TodayInfo{Wind_Compass} or
            $TodayInfo{Wind_Compass} and
            $TodayInfo{Wind_Compass} eq 'null' or
            $TodayInfo{Wind_Compass} eq 'NA' or
            $TodayInfo{Wind_Compass} == 0) {
        $TodayInfo{Wind_Compass} = '0 '
    }

    my ($check1); $rpt = 1;
    while ( my ($key1, $value1) = each %TodayInfo ) {
        if ($value1) {
            $check1 = check_num( $value1, $key1 );
            $TodayInfo{$key1} = $check1;
        }
        else {
            $value1 = 'NA';
            $TodayInfo{$key1} = $value1;
        }
        # msgout( "$rpt: $key1 =>$value1", $wx{cloud1t});
        # msgout "$rpt: $key1:=>$TodayInfo{$key1}:=>$value1"; $rpt++;
    }

    if ( $TodayInfo{Updated} eq "NA" ) {
        msgout "\n\nError - Update time is NA. Use an alternate website for weather.\nPlease mail a message to the author with the Debug file.\nIn a terminal window type conkywx and choose option 4. Thanks\n\n"
    }
}


sub get_uvdata {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    require uvinfo;
    uvinfo->import( qw(getUVinfo) );

    my $uvdata = "$conkywx_internal_dir/uvdata/uvdata_$tempname";
    my $uvdate = "$day $monAbb $year";

    my @uvIn = getUVinfo( $uvdate, $nowtime, $uvdata, $LMsg{no_open_file}, $conf{elevation}, $TodayInfo{Lat1}, $TodayInfo{Long1}, %wx );

    $TodayInfo{uv_max} = $uvIn[0] ;
    $TodayInfo{uv_now} = $uvIn[1] ;

    return

}

sub get_tides {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    require tides;
    tides->import( qw(doTides) );

    my $ts = (split('/',$conf{tides_url}))[4];
    my $tide_data = "$conkywx_internal_dir/tides/tide_data_${tempname}_$ts";

    my $tForm = $timeFormat{$conf{Time_format}} // $conf{Time_format} ;
    $conf{tide_date_format} = $conf{tide_date_format} // '%a %d %B'   ;

    my $tide_days_opt = ($conf{tide_days_opt}) ? $conf{tide_days_opt} : 1;
    my $tide_days_num = ($conf{tide_days_num}) ? $conf{tide_days_num} : 7;
    my $tide_date = ($conf{tide_date}) ? $conf{tide_date} : 'monday';


    doTides( $temppath,  $tForm, $LMsg{no_open_file}, "${hour}:${min}:${sec}", ${year}, $yday, $tide_data, $conf{tide_days_opt}, $conf{tide_days_num}, $conf{tide_date}, $conf{tide_date_format} );

    return

}

# convert units imperial to SI and back
sub convert_unit {
    my ($x, $y) = @_;

    if ( $conf{xml_ID} and $conf{xml_ID} ne 'nouse' ) {
        undef($anyUnitWx)
    }

    if ( $conf{Units_to_Use} ) {
        if ( $y and $y eq 'Depth' ) {
            return(sprintf($convUnits{$y}[1], $convUnits{$y}[0]($x)))
        }
        elsif ( $anyUnitWx and index($anyUnitWx, 'C<') != -1 ) {
            return(sprintf($convUnits{$y}[1], $convUnits{$y}[0]($x)))
        }
        else {
            return(sprintf($convUnits{$y}[1], $x))
        }
    } else {
        if ( $y and $y eq 'Depth' ) {
            return(sprintf($convUnits{$y}[1], $x))
        }
        elsif ( $anyUnitWx and index($anyUnitWx, 'C<') != -1 ) {
            # msgout($x,$y);
            return(sprintf($convUnits{$y}[1], $x))
        }
        else {
            return(sprintf($convUnits{$y}[1], $convUnits{$y}[0]($x)))
        }
    }
    return
}

my %UVthing = (
    '1'  => 'Min', '6'  => 'Hi',
    '2'  => 'Low', '7'  => 'Hi',
    '3'  => 'Mod', '8'  => 'V.Hi',
    '4'  => 'Mod', '9'  => 'V.Hi',
    '5'  => 'Mod', '10' => 'V.Hi',
    '11' => 'V.Hi'
    );
my %Pthing = (
    '1'  => 'Low', '6'  => 'Mod',
    '2'  => 'Low', '7'  => 'Hi',
    '3'  => 'Low', '8'  => 'Hi',
    '4'  => 'Low', '9'  => 'Hi',
    '5'  => 'Mod', '10' => 'Hi',
    );

# ------------------------------------------------------
# handle all UV + Pollen
# ------------------------------------------------------
sub uv_pollen {
    my ($x, $y) = @_;

    if ( $y eq "UV" ) {
        if    ( $x <= 0 ) { return "NA"     }
        elsif ( $x >= 11) { return "$x:Ext" }
        else  { return "$x:$UVthing{&round($x + 0.5)}" }
    }
    if ( $y eq "pollen" )    {
        if    ( $x <= 0 ) { return "NA"     }
        elsif ( $x >= 10) { return "$x:Ext" }
        else { return "$x:$Pthing{&round($x + 0.5)}" }
    }
}

# ------------------------------------------------------
# convert forecast temps imperial to SI and back
# ------------------------------------------------------
sub ch_fcunits {
=comments
    Requirement
    [1] Hide all deg symbols (example: 95/63)
    [2] Show all deg symbols (example: 95°/63°)
    [3] Show all deg symbols, depending on Unit_to_Use (metric: 35°C/17°C) (imperial: 95°F/63°F)
    [4] Show deg symbol ONLY, after forecast pair (example: 95/63°)
    [5] Show deg symbol AND unit, after forecast pair (example: 95/63°F) [Default]
=cut

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ($x) = @_ ;
    if ( $x eq 'NA' ) {
        return ('NA');
    }
    else {
        my ($a, $b, $c, $xLeft, $xRight, $u_tempfc, $outLeft, $outRight);
        # my $degsym_at_end = 5;
        $x =~ tr/\x20\x09//d;     # get rid of ALL spaces
        if (index($x, '/') != -1) {
            ($xLeft, $xRight) = split(/\//, "$x") ;
        }

        $xLeft  = ( not exists $naList{$xLeft}  and $xLeft  += 0 ) ? $xLeft  : -999;
        $xRight = ( not exists $naList{$xRight} and $xRight += 0 ) ? $xRight : -999;

        if ( $conf{Units_to_Use} ) {
            if ( $anyUnitFct and index($anyUnitFct, 'C<') != -1 ) {
                $xLeft  = sprintf("%1.0f", c_f($xLeft));
                $xRight = sprintf("%1.0f", c_f($xRight));
                $u_tempfc = "F"
            }
            else {
                $u_tempfc = "F"
            }
        }
        else {
            if ( $anyUnitFct and index($anyUnitFct, 'C<') != -1 ) {
                $u_tempfc = "C"
            }
            else {
                $xLeft  = sprintf("%1.0f", f_c($xLeft));
                $xRight = sprintf("%1.0f", f_c($xRight));
                $u_tempfc = "C"
            }
        }

        for ($conf{show_degsym_fc_temp}) {
            if    (/1/) {$b=""          ; $c=""           }
            elsif (/2/) {$b="°"         ; $c="°"          }
            elsif (/3/) {$b="°$u_tempfc"; $c="°$u_tempfc" }
            elsif (/4/) {$b=""          ; $c="°"          }
            elsif (/5/) {$b="°"         ; $c="°$u_tempfc" }
            else        {$b=""          ; $c=""           }
        }

        if    ($xLeft  < -150 ) { $a = "--/$xRight${c}" }
        elsif ($xRight < -150 ) { $a = "$xLeft${b}/--"  }
        else {$a = "$xLeft${b}/$xRight${c}"}

        return ($a);
    }
}

# ------------------------------------------------------
# convert days long to short format - eg: Sunday to Sun
# ------------------------------------------------------
sub get_short_day {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ($x,$y) = @_ ;

    # if ( $y and $y eq 'tide' ) {}
    if (! $y) { $y = 0 }

    if ( $x eq "NA" ) {return 'NA' }
    else {
        my ($a, $b, $c );

        # $x = "Sun Dec 9";
        $x =~ s/tod.*\s+/$wdayAbb /i;
        # msgout($x);

        # msgout($wdayAbb,$wdayFull);

        $x =~ s/\s+/ /g;
        my @e1 = ( $x =~ /\s+/g ) ;
        my $f = scalar(@e1);
        if ( $f == 0 ) {
            return $x;
        }
        elsif ( $f == 2 ) {
            ($a, $b, $c) = split( " ", $x);
            # msgout($fd_d{$a}, $b, $m_sm{$c});
            if ($y == 1) { return "$fd_d{$a} $b $m_sm{$c}" }
            if ($y == 2) { return "$fd_d{$a} $b $c" }
        }
        else {
            ($a, $c) = split( " ", $x);
        }
        # if ( ! /Tonight|Today/ ) {
        # if ( length($a) > 3 or $a !~ /Tonight|Today/ ) { $a = substr($a, 0, 3) }
        if ( length($a) > 3 ) { $a = substr($a, 0, 3) }
        # msgout($f, $a, $b, $c, $x);
        # msgout( '0' x (2 - length $c) . $c);

        for ($conf{use_short_date}) {
            if    ( /0/ ) { return "$d_fd{$a} $c" if $a and $c and exists $d_fd{$a} }
            elsif ( /1/ ) {
                if ( length($c) == 1 ) { $c ='0' x (2 - length($c)) . $c }
                return "$a $c"
            }
            elsif ( /2/ ) { $c = scalar($c+0); return "$d_fd{$a} $c"       }
            elsif ( /3/ ) { $c = scalar($c+0); return "$a $c"              }
        }
    }
    #else { return ("NA"); }
}
# end if check_num function calls

# ------------------------------------------------------
# NOTE: Following functions call the full wx data
# ------------------------------------------------------

# ------------------------------------------------------
# NOTE: Apparent temp calculation based on US Navy / Australian derived Formula
# ------------------------------------------------------
sub get_apptemp {
    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( $app_temp, $a_temp, $a_hum, $a_ws );
    ($a_temp) = ($TodayInfo{Temp_now} =~ m{(-?\d+(\.\d+)?)}) ? $1 : 'NA';
    ($a_hum)  = $TodayInfo{Humidity} =~ m{(\d+)} ? $1 : 'NA';
    ($a_ws)   = $TodayInfo{Wind_Speed} =~ m{(\d+(\.\d+)?)} ? $1 : 0;

    if ( $conf{Units_to_Use} ) {
        $a_temp = ($a_temp ne 'NA') ? f_c($a_temp) : 'NA';
        $a_ws   = ($a_ws != 0)      ? mi_km($a_ws) : 0   ;
    }

    if ( $a_temp eq "NA" ) {
        $TodayInfo{app_temp} = "NA";
    }
    elsif ( $a_hum eq "NA" or $a_hum == 0 ) {
        $TodayInfo{app_temp} = $TodayInfo{Temp_now};
    }
    else {
        # conversion to wind speed m/s is built into the formula
        my $h2o_vp = ( $a_hum / 100 * 6.105 * 2.718281828 ** (( 17.27 * $a_temp ) / ( 237.7 + $a_temp )));
        $app_temp = ( $a_temp + (0.33 * $h2o_vp) - (0.7 *( $a_ws * 0.27777778)) - 4.00 );

        if ( $conf{Units_to_Use} ) {
            $TodayInfo{app_temp} = sprintf( "%1.0f$u_temp", c_f($app_temp) );
        }
        else {
            $TodayInfo{app_temp} = sprintf( "%1.0f$u_temp", $app_temp );
        }
    }
}

# ------------------------------------------------------
# Wind direction and wind - direction and Gust Handling
# ------------------------------------------------------
sub get_bearing_font {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my @brn1 = qw(  9 : ; < = > ? @ 1 2 3 4 5 6 7 8 % ! -);
    my @brn2 = qw/  I J K L M N O P A B C D E F G H % " -/;
    my @brn3 = ( qw/Y Z/, '\[', '\\', '\]', qw/^ _ ` Q R S T U V W X/, '%', '\#', '-');
    my @brn4 = ( qw/i j k l m n o p a b c d e f g h %/,'\#','-'); # uses the " character from set 3
    my @brnI = qw/  s ssw sw wsw w wnw nw nnw n nne ne ene e ese se sse un var na/;
    my @crd1 = (qw/NORTH NNE NE ENE EAST ESE SE SSE SOUTH SSW SW WSW WEST WNW NW NNW/,
        "UNKNOWN", "VAR", "NA" );
    my (%brf, %bri, @UserSet);

    my $wfn = length($conf{wx_wind_fontset});
    if ( $wfn eq 1 ) {
        $conf{wx_wind_fontset} = ($conf{wx_wind_fontset} =~ /^[1-4]$/) ? $conf{wx_wind_fontset} - 1 : 0;
    }
    else {
        @UserSet = split( " ", $conf{wx_wind_fontset} );
    }

    my @bfset = ( \@brn1, \@brn2, \@brn3, \@brn4 );
    my $xb = 0;
    foreach(@crd1) {
        if ( $wfn eq 1 ) {
            $brf{$_} = $bfset[$conf{wx_wind_fontset}][$xb];
        }
        else {
            $brf{$_} = $UserSet[$xb];
        }
        $bri{$_} = $brnI[$xb];
        $xb++;
    }
    # my ($key1, $value1);
    # while ( ($key1, $value1) = each %brf ) {
    #   msgout "$key1 =>$value1";
    # }

    my ( @bearing_font_array, @push_out, $fn1, $ic1, $windspeed_trend, %Wind_Now );

    $Wind_Now{Wind_Speed} =  $TodayInfo{Wind_Speed};
    $Wind_Now{Wind_Speed} =~ s/[ ].*//;
    # $Wind_Now{Wind_Speed} =~ s/variable/var/i;

    $Wind_Now{Wind_Gust} =  $TodayInfo{Wind_Gust};
    $Wind_Now{Wind_Gust} =~ s/[ ].*//;
    # $Wind_Now{Wind_Gust} =~ s/variable/var/i;

    $Wind_Now{Wind_From} =  $TodayInfo{Wind_From};
    $Wind_Now{Wind_From} =~ s/variable/var/i;
    # $Wind_Now{Wind_Speed} = "Calm";

    # msgout "from=$Wind_Now{Wind_From}, speed=$TodayInfo{Wind_Speed}";

    if ( $Wind_Now{Wind_Speed} eq 'NA' ) {
        # $Wind_Now{Wind_Speed} = "NA";
        $windspeed_trend = "NA";
        if (( $Wind_Now{Wind_Gust} eq "NA" or $Wind_Now{Wind_Gust} == 0 )) {
            $Wind_Now{Wind_From} = "NA";
            $Wind_Now{Wind_Gust} = "NA";
        }
        else {
            $Wind_Now{Wind_From} = "Var";
            $Wind_Now{Wind_Gust} = "$Wind_Now{Wind_Gust} $u_speed";
        }
    }
    elsif (( $Wind_Now{Wind_Speed} eq "Calm" or $Wind_Now{Wind_Speed} == 0 )) {
        $Wind_Now{Wind_Speed} = "Calm";
        $windspeed_trend = 0;
        if (( $Wind_Now{Wind_Gust} eq "NA" ) || ( $Wind_Now{Wind_Gust} == 0 )) {
            $Wind_Now{Wind_From} = "NA";
            $Wind_Now{Wind_Gust} = "NA";
        }
        else {
            $Wind_Now{Wind_From} = "Var";
            $Wind_Now{Wind_Gust} = "$Wind_Now{Wind_Gust} $u_speed";
        }
    }
    else {
        $windspeed_trend = $Wind_Now{Wind_Speed};
        $Wind_Now{Wind_Speed} = "$Wind_Now{Wind_Speed} $u_speed";
        $Wind_Now{Wind_From} = ($Wind_Now{Wind_From} eq "NA" )
        ? $Wind_Now{Wind_From} = "Var"
        : $Wind_Now{Wind_From} = $Wind_Now{Wind_From} ;
        if (( $Wind_Now{Wind_Gust} eq "NA" ) || ( $Wind_Now{Wind_Gust} == 0 )) {
            $Wind_Now{Wind_Gust} = "NA";
        }
        else {
            $Wind_Now{Wind_Gust} = "$Wind_Now{Wind_Gust} $u_speed";
        }
    }

    $fn1 = ($brf{uc($Wind_Now{Wind_From})}) ? $brf{uc($Wind_Now{Wind_From})} : "%" ;
    $ic1 = ($bri{uc($Wind_Now{Wind_From})}) ? $bri{uc($Wind_Now{Wind_From})} : "un" ;
    # msgout "from=$Wind_Now{Wind_From}, speed=$Wind_Now{Wind_Speed}";
    my $windicon_org = "${iwindpath}/${ic1}.png";
    # my $windicon_cp = "${temppath}/dailyimg_${cimage}.png";


    $TodayInfo{Wind_Speed} = $Wind_Now{Wind_Speed};

    # msgout $TodayInfo{Wind_Compass};

    if ( $Wind_Now{Wind_From} eq "Var" ) {
        $TodayInfo{Wind_Compass} = "Var";
    }
    elsif ( $Wind_Now{Wind_From} eq "NA" ) {
        $TodayInfo{Wind_Compass} = "NA";
    }

    $TodayInfo{Wind_From}         = $Wind_Now{Wind_From};
    $TodayInfo{Wind_Gust}         = $Wind_Now{Wind_Gust};
    $TodayInfo{wind_speed_trend}  = $windspeed_trend;
    $TodayInfo{wind_bearing_font} = $fn1;

    # msgout($fn1,$brf{uc($Wind_Now{Wind_From})});
    # copy($windicon_org, $nowWindDir);

    $TodayInfo{nowWindDir} = $windicon_org;


    if ($debug) {
        foreach my $wxkey ( sort keys %TodayInfo ) {
            push @push_out, "$wxkey = $TodayInfo{$wxkey}\n";
        }
        do_debug( "today (+xml)", @push_out );
        # msgout( "$LMsg{pass_today}" );
    }
}

sub get_magellan {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( $flag,$t_zone,$moonImage, $rfact, $present_hour );
    my $wx_tzone_A = $wx_tzone / 3600 ;
    if ( ! ($conf{t_zone_pick}) ) {
        $t_zone = $wx_tzone_A;
        unlink $die_error if ( -e $die_error )
    }
    else {
        $t_zone = fix_tzone($conf{t_zone_pick}) ;
        my $tz_err_msg = "Error: :(  t_zone_pick user value $conf{t_zone_pick} in config file not valid: $!" ;
        if ( $t_zone == -999 ) { die_errors($tz_err_msg) ; die $tz_err_msg }
        else { unlink $die_error if ( -e $die_error ) }
    }

    if ($conf{Latitude})  { $TodayInfo{Lat1}  = $conf{Latitude}  }
    if ($conf{Longitude}) { $TodayInfo{Long1} = $conf{Longitude} }

    if ($TodayInfo{Lat1})  { $TodayInfo{Lat1}  =~ s/"|°|\s*//g; }
    if ($TodayInfo{Long1}) { $TodayInfo{Long1} =~ s/"|°|\s*//g; }

    my $LatVal = ($TodayInfo{Lat1}  =~ m{(-?\d+(\.\d+)?)}) ? $1 : 'NA';
    my $LonVal = ($TodayInfo{Long1} =~ m{(-?\d+(\.\d+)?)}) ? $1 : 'NA';
    my $LatSign = substr( $TodayInfo{Lat1},  -1 );
    my $LonSign = substr( $TodayInfo{Long1}, -1 );

    if ( $TodayInfo{Lat1} ne "NA" || $TodayInfo{Long1} ne "NA" ) {
        if ($LatSign !~ /N|S|n|s/) {
            $TodayInfo{Lat1} = ($LatVal < 0) ? ($LatVal*-1)."S" : $LatVal."N";
        }
        else {
            if ( $LatVal < 0 and $LatSign =~ /N|n/ ) {
                die "\nError: :( \nLatitude sign mismatch: $TodayInfo{Lat1}";
            }
        }
        if ($LonSign !~ /E|W|e|w/) {
            $TodayInfo{Long1} = ($LonVal < 0) ? ($LonVal*-1)."W" : $LonVal."E";
        }
        else {
            if ( $LonVal < 0 and $LonSign =~ /E|e/ ) {
                die "\nError: :( \nLongitude sign mismatch: $TodayInfo{Long1}";
            }
        }
    }
    else {
        $TodayInfo{Lat1} = 0.1 ."N" ; $TodayInfo{Long1} = 0.1 . "E" ;
    }
    # msgout ">>>>>>>>>>$LatSign::$LonSign>>>$LatVal::$LonVal,$TodayInfo{Lat1},$TodayInfo{Long1}";
    # msgout ">>>>>>>>>>Lat:$TodayInfo{Lat1},Long:$TodayInfo{Long1}";

    my $t_format = $timeFormat{$conf{Time_format}} // $conf{Time_format};
    my %moonFormat = (
        '1' => "%b %d $t_format",
        '2' => "%b %d",
        );
    my $moon_format = $moonFormat{$conf{moon_time}} // $conf{moon_time} ;

    # $conf{moon_time} = "%b %d";
    # $conf{moon_time} =~ s/'//g;
    # $conf{moon_time} =~ tr/\"//;
    # msgout($conf{moon_time});

    my $nt_format = "%R" ; # get next day data in 24 hour format - fixes night images

    # msgout "$cwx_magellan -y $year -m $mon -d $day -t $t_zone -o $conf{moon_outline} -a $nt_format -b $moon_format -n 1 $TodayInfo{Lat1} $TodayInfo{Long1}"  ;

    my $magellan_tday = `"$cwx_magellan" -y $year -m $mon -d $day -t $t_zone -o $conf{moon_outline} -a "$t_format" -b "$moon_format" $TodayInfo{Lat1} $TodayInfo{Long1}` ;

    if ($debug) {
        msgout( "$LMsg{pass_mag}" );
        do_debug( "magellan today", "$magellan_tday");
    }

    $magellan_tday =~ tr/\x22//d;
    $magellan_tday =~ s/(?<=[\s])#.*\n/\n/;

    my $magPrintOut;
    for (split /\n/, $magellan_tday) {
        if (index($_,'[next_day_start]') != -1) { $flag = 1; next }
        if ( $flag ) {
            my ($k, $v) = split('=',$_,2);
            $Mag_Nday{$k} = $v if $k;
            if (index($_,'[next_day_end]') != -1) { undef($flag); next }
        }
        if ( ! $flag ) {
            my ($k, $v) = split('=',$_,2);
            if ($k ) {
                $Mag_2day{$k} = $v;
                $magPrintOut .= "$k=$v\n" ;
            }
        }
    }

    # $magPrintOut .= "latitude=$wx{latitude}\nlongitude=$wx{longitude}\n" ;

    open (out_data, '>', "$temppath/magellan") or
                                die "$LMsg{no_open_file} magellan: save $!";
        print out_data $magPrintOut;
    close (out_data);

    $Mag_2day{civ_twilt_S} = $Mag_2day{civ_twilt_S} // "-";
    $Mag_2day{nau_twilt_S} = $Mag_2day{nau_twilt_S} // "-";
    $Mag_2day{ast_twilt_S} = $Mag_2day{ast_twilt_S} // "-";

    my $lu1 = ($TodayInfo{Updated} or $TodayInfo{Updated} ne 'NA' ) ?
        &time2str($t_format,$TodayInfo{Updated}) :
        &time2str($t_format,$nowtime);

    my $lu2 = ($wx{last_update_xml}) ? &time2str($t_format,$wx{last_update_xml}) : "NA" ;

    $moonImage = "$cwxpath/images/moonicons/$Mag_2day{moonImage}.png";

    # moon_dec_state - 1 is if observer North of moon, 0 if observer South of moon
    if ( $conf{Dynamic_Moon} ) {
        # if ( $conf{Dynamic_Moon} == 1 ) { $present_hour = $hour }
        # else { $present_hour = &time2str("%H",$TodayInfo{Updated}) }

        # msgout($present_hour);
        # $present_hour = $hour;
        $present_hour = $hour + 10;

        my (%t0,%t1);
        $t0{$_} = 0 for (15,30); $t0{$_} = -45 for (1..14); $t0{$_} =  45 for (16..29);
        $t1{$_} = 0 for (15,30); $t1{$_} = -45 for (1..14); $t1{$_} =  45 for (16..29);

        ( $Mag_2day{moon_dec_state} == 0 )      ?
        $rfact = $t0{$Mag_2day{moonImage}} // 0 :
        $rfact = $t1{$Mag_2day{moonImage}} // 0 ;

        if ( $rfact != 0 ) {
            ( $Mag_2day{moon_dec_state} == 0 ) ?
            $TodayInfo{moon_rotate_val} = ((( $present_hour * 15 ) + $rfact) * -1) :
            $TodayInfo{moon_rotate_val} = ((( $present_hour * 15 ) + $rfact)) ;

            $TodayInfo{moonImage} = $moonImage;
        }
        else { $TodayInfo{moonImage} = $moonImage; $TodayInfo{moon_rotate_val} = 0 }
    }
    else { $TodayInfo{moonImage} = $moonImage; $TodayInfo{moon_rotate_val} = 0 }

    # msgout($TodayInfo{moon_rotate_val},$rfact );

    $TodayInfo{today_date} = &time2str($conf{today_date_format},$nowtime);

    my $dd1 = sprintf("%1.0f",(($Mag_Nday{length_day_raw} - $Mag_2day{length_day_raw}) * 3600 ));

    # msgout($Mag_Nday{length_day_raw}, $Mag_2day{length_day_raw});
    # msgout($Mag_Nday{percentilluminated},$Mag_2day{percentilluminated});

    if ( $dd1 < 0 ) {
        $TodayInfo{next_day_length_msg}   = "Shorter";
        $TodayInfo{next_day_length_trend} = $conf{fallingtrend};
        $dd1 *= -1;
    }
    elsif ( $dd1 > 0 ) {
        $TodayInfo{next_day_length_msg}   = "Longer";
        $TodayInfo{next_day_length_trend} = $conf{risingtrend};
    }
    else {
        $TodayInfo{next_day_length_msg}   = "Steady";
        $TodayInfo{next_day_length_trend} = $conf{steadytrend};
    }

    my $ds  =   $dd1 % 60;
    my $dm  =   (($dd1 / 60) % 60);
    $TodayInfo{next_day_length} =   sprintf("%02dm %02ds", $dm, $ds);

    $TodayInfo{next_day_length_trend} = ($TodayInfo{next_day_length_trend}) ? $TodayInfo{next_day_length_trend} : "~" ;

    # calculate moon rate of changes based on illumination
    $TodayInfo{moon_rate_of_change} = $Mag_Nday{percentilluminated} - $Mag_2day{percentilluminated};

    if ( -f "$temppath/old_data" ) {
        $wx{alertdata} .= "$LMsg{no_net} | ";
        $TodayInfo{fetchtime} = "$LMsg{no_net}";
    }
    else {
        my $ftime = (stat( $currdatam))[9];
        $TodayInfo{fetchtime} = &time2str($t_format,$ftime)
    }

    $TodayInfo{last_update} = ( -e "$currdatax" ) ? "$lu1 | $lu2 - $TodayInfo{Station_ID}" : "$lu1" ;

    $TodayInfo{last_update} = ( $wxFctTitle ) ? "$TodayInfo{last_update} :: ( $wxMainTitle + $wxFctTitle ) " : "$TodayInfo{last_update} :: ( $wxMainTitle )" ;

    # $TodayInfo{last_update} = ( -e "$temppath/fallBackFct" ) ? "$TodayInfo{last_update} :: ( $wxMainTitle + wund ) " : "$TodayInfo{last_update} :: ( $wxMainTitle + $wxFctTitle ) " ;

    # $TodayInfo{last_update} = ( -e "$temppath/fallBackFct" ) ? "$TodayInfo{last_update} :: ( $wxMainTitle ) " : ( $wxMainTitle == $wxFctTitle ) ? "$TodayInfo{last_update} :: ( $wxMainTitle ) " : "$TodayInfo{last_update} :: ( $wxMainTitle + $wxFctTitle ) " ;

    # msgout($TodayInfo{last_update},'<<<<<<<<<<<<');

    my $currTz = 0;
    $wx{curMin} = ( $hour * 60 ) + $min + $currTz ;
    $wx{srise}  = ($Mag_2day{sun_rise_T}      =~ m{(\d+).(\d+)}) ? (($1 * 60) + $2) : 0;
    $wx{sset}   = ($Mag_2day{sun_set_T}       =~ m{(\d+).(\d+)}) ? (($1 * 60) + $2) : 0;
    $wx{strans} = ($Mag_2day{sun_mer_trans_T} =~ m{(\d+).(\d+)}) ? (($1 * 60) + $2) : 0;

    my ($dntest);
    # we calc here the night or day icons based on civil twilight
    $dntest = ( $wx{curMin} >= ($wx{sset} + 30 ) or $wx{curMin} <= ($wx{srise} - 30) ) ? 1 : 2 ;

    ($TodayInfo{NowFont}, $TodayInfo{condition}) = do_get_DI( 1, $TodayInfo{curr_icon}, 0, $dntest );

    # msgout(">>>$TodayInfo{condition}<<<") ;
    # $ftest ||= "NA";
    # msgout "$dntest, $TodayInfo{NowFont}, $hour:$min, $wx{srise}, $wx{sset}, $TodayInfo{condition}, $t_zone, $wx_tzone, $currTz, $wx{curMin}";

}


sub wxtrends {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    require trends;
    trends->import( qw( trendGraphs trendArrows ) );

    my @ft_now = (  $wx{pressure}       ||= 0,
                    $wx{temperature}    ||= 0,
                    $wx{wind_speed}     ||= 0,
                    $wx{humidity}       ||= 0,
                    $TodayInfo{uv_now}  ||= 0,
                    $wx{dewpoint}       ||= 0,
                    $wx{visibility}     ||= 0,
                    $wx{solarRad}       ||= 0,
                );

    for (@ft_now) { $_ = ( $_ =~ m{(-?\d+(\.\d+)?)} ) ? $1 : 0 }
    my $ftNow = join( '|', @ft_now  );

    my $fcttemp = ($fctinfo{d_1_temp} =~ m{(\d+)(°)?/.*}) ? $1 : 'NA';
    my @ft_now_24 = (   $Mag_2day{sun_rise_T},
                        $Mag_2day{sun_set_T},
                        $Mag_2day{moon_rise_T},
                        $Mag_2day{moon_set_T},
                        $Mag_2day{length_visible_light_T},
                        $Mag_2day{length_day_T},
                        $Mag_2day{percentilluminated},
                        $fcttemp,
                    );

    for (@ft_now_24) { $_ = ( $_ =~ m{(-?\d+(\.\d+)?)} ) ? $1 : 0 }
    my $ftNow24 = join( '|', @ft_now_24  );

    my $confStr = "$conf{steadytrend}|$conf{fallingtrend}|$conf{risingtrend}";

    # msgout($nowtime, $temppath, $LMsg{no_open_file}, $ftNow, $wx{wind_gust_speed}, $wxMainTitle);


    %wxtrend = trendGraphs( $nowtime, $temppath, $LMsg{no_open_file}, $ftNow, $wx{wind_gust_speed}, $wxMainTitle );


    @trends_out = trendArrows( $nowtime, $forecast_trends, $LMsg{no_open_file}, $fcttemp, $TodayInfo{uv_now}, $ftNow, $ftNow24, $confStr );

    # msgout(@trends_out);

    return
}


sub present_cond_bl {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( $test_cond, $pcond, $bl_text, $lt1, $rt1, $wx_warn_sym );

    if ( %langx ) {
        $test_cond = $langx{lc($TodayInfo{condition})} if exists $langx{lc($TodayInfo{condition})}
    }
    $test_cond ||= $TodayInfo{condition};

    if ( $conf{pcondition_blacklist} ) {
        foreach(split(/_,/, $conf{pcondition_blacklist})) {
            if (index($_,':') != -1) {
                ($lt1, $rt1) = split(/:/, $_);
                $test_cond =~ s/$lt1/$rt1/i;
            }
            else {
                $test_cond =~ s/$_//i;
                $test_cond =~ s/\s+/ /;
            }
        }
        $pcond = $test_cond
    }
    else { $pcond = $test_cond }


    if ( $wx{alertdata} ) {
        $wx{alertnum} = () = ($wx{alertdata} =~ /\|/g);
        $wx{alertnum} = $wx{alertnum} + 1;
        if ( $wx{alertnum} > 1 ) { $wx_warn_sym = "!" . "$wx{alertnum} " }
        else { $wx_warn_sym = "! " }
    }
    else { $wx_warn_sym = "" }

    if ($conf{pc_what_case}) { $pcond = do_pc_what_case( $pcond ) }

    return ($wx_warn_sym . $pcond );
}

sub do_pc_what_case {
    my ( $val ) = @_;
    for ($conf{pc_what_case}) {
        if    ( /1/ ) { $val =~ s/(\w+)/\u\L$1/g }
        # [1] All UPPERCASE
        elsif ( /2/ ) { $val = uc($val) }
        # [2] all lowercase
        elsif ( /3/ ) { $val = lc($val) }
        # [3] Capitilize the first letter/first word only, in the present condition
        elsif ( /4/ ) { $val = lc($val); $val = ucfirst($val) }
    }
    return $val
}

# ------------------------------------------------------
# process pre_showtime_template template_cleanup and or language_to_use
# ------------------------------------------------------
sub do_cleanup_lang {

    my ($do_lang, $intext) = @_ ;

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    $$intext ||= undef ;
    # if ( not $$intext ) { return '' }
    $do_lang ||= 0;
    my ( $lt1, $rt1, @cleanup );

    ### do_lang 1 is from alerts do_cl_opts
    if ( $do_lang == 1 ) {
        if ( %langx ) {
            my ($key1, $value1);
            while ( ($key1, $value1) = each %langx ) {
                $wx{alertdata} =~ s/\b$key1\b/$value1/ig;
            }
        }
        return
    }

    if ( $conf{template_cleanup} ) {
        my $lt1Regex = qr/\.\*(?![?])/;
        for (split('_,', $conf{template_cleanup})) {
            next if not $_ ;
            ($lt1, $rt1) = split(/:/, $_);
            if (index($lt1,'.*') != -1) {
                $lt1 =~ s/$lt1Regex/.*?/g
            }
            if ( $lt1 and $$intext and $$intext =~ /$lt1/i ) {
                push @cleanup, [ $lt1, $rt1 ]
            }
        }

        my $tmpTest = sub {
            my ($value) = @_;
            for (@cleanup) {
                my @k = $_;
                if ( $value =~ /$k[0][0]/i ) {
                    $value =~ s/$k[0][0]/$k[0][1]/i
                }
            }
            return $value
        };

        my $tcRegex = qr/(?<=\})([\w\.]{3}[\w\-\s\.]*)(?=\$\{|\#|:_:_:_:_:|\-)/;
        $$intext =~ s/\x0a/:_:_:_:_:/g;
        $$intext =~ s/$tcRegex/&$tmpTest($1)/eg;
        $$intext =~ s/:_:_:_:_:/\x0a/g;

        return $$intext if $do_lang == 2 ;
    }
    elsif ($do_lang == 2) { return $$intext }

    my $normalStr;
    if (index($$intext,'NORMALSTART') != -1 ) {
        ($normalStr) = ($$intext =~ m{NORMALSTART(.*?)NORMALEND}s);
    }

    # if ( ! $conf{do_not_output_to_conky} ) {
    if ( not $conf{do_not_output_to_conky} ) {
        open (out_data, '>', "$temp_template") or die "$LMsg{no_open_file} temp_template: $!";
            print out_data $$intext;
        close (out_data);
    }
    elsif ( $normalStr and ! $conf{link_tmp_folder} ) {
        open (out_data, '>', "$temp_templatesc") or die "$LMsg{no_open_file} temp_templateSC: $!";
            print out_data $normalStr;
        close (out_data);
    }

    my @wxdebug;
    if ($debug) {
        do_debug( "do_cleanup_lang dump", $$intext );
        msgout "***************************************";
    }
}

sub language_hash {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    if ( $conf{language_to_use} ) {
        my $lang_to_use = lc($conf{language_to_use});
        $lang_to_use =~ s/"|'//g;
        my $lang_file = "$cwxpath/lang/$lang_to_use.lang";

        if ( -e $lang_file ) {
            my ( $langRead  ) ;
            {
            open (in_data, "<", $lang_file) or die "$LMsg{no_open_file} language file: $!";
                local $/ = undef;
                $langRead = <in_data> ;
            close (in_data);
            }
            $langRead =~ s/#.*\n//g ;
            for (split(/\n/,$langRead)) {
                next if not $_ ;
                my ($key, $value) = split(/:/);
                $langx{lc($key)} = $value;
            }
        }
    }
}

sub do_cl_opts {

    my ($whatalerts);
    my $wxalert = "$temppath/wxalert" ;

    if ( $conf{show_wx_alerts} ) {
        if ( $wx{alertdata} ) {
            do_cleanup_lang( 1 );
            open (WXALERTFILE, '>', "$wxalert") or die "$LMsg{no_open_file} wx alert file $wxalert: $!";
                print WXALERTFILE $wx{alertdata};
            close (WXALERTFILE);
            if ($debug) {
                msgout( $LMsg{dbg_msg_str1}, sub_name() );
                msgout( $wx{alertdata} );
            }
        }
        else { $whatalerts = 1 }
    }
    else { $whatalerts = 1 }

    if ( $whatalerts and -e $wxalert ) {
        unlink $wxalert or warn "Could not delete wx alert file $wxalert: $!";
    }

    %output = (
        PTBI    => "$cwxpath/images/bgrounds",
        PIF     => "$TodayInfo{NowFont}",
        PIC     => "$TodayInfo{NowImage}",
        PT      => "$TodayInfo{Temp_now}",
        FL      => "$TodayInfo{app_temp}",
        TD      => "$TodayInfo{today_date}",
        CT      => present_cond_bl,
        CLH1    => "$TodayInfo{CLH1}",
        CLC1    => "$TodayInfo{CLC1}",
        RF      => "$TodayInfo{Rainfall}",
        SD      => "$TodayInfo{SnowDepth}",
        HM      => "$TodayInfo{Humidity}" . '%',
        VI      => "$TodayInfo{Visibility}",
        WDD     => "$TodayInfo{Wind_Compass}",
        WD      => "$TodayInfo{Wind_From}",
        WS      => "$TodayInfo{Wind_Speed}",
        WG      => "$TodayInfo{Wind_Gust}",
        BF      => "$TodayInfo{wind_bearing_font}",
        BI      => "$TodayInfo{nowWindDir}",
        LN      => "$TodayInfo{Location_Name}",
        SN      => "$TodayInfo{Station_Name}",
        ID      => "$TodayInfo{Station_ID}",
        BR      => "$TodayInfo{Pressure}",
        UV      => "$TodayInfo{uv_now}",
        UVM     => "$TodayInfo{uv_max}",
        PLN     => "$TodayInfo{pollen}",
        DP      => "$TodayInfo{Dew_Point}",
        METAR   => "$TodayInfo{METAR}",
        SR      => "$Mag_2day{sun_rise}",
        SS      => "$Mag_2day{sun_set}",
        LOV     => "$Mag_2day{length_visible_light}",
        LOD     => "$Mag_2day{length_day}",
        LODN    => "$TodayInfo{next_day_length}",
        LODNM   => "$TodayInfo{next_day_length_msg}",
        LODNT   => "$TodayInfo{next_day_length_trend}",
        CTR     => "$Mag_2day{civ_twilt_R}",
        CTS     => "$Mag_2day{civ_twilt_S}",
        NTR     => "$Mag_2day{nau_twilt_R}",
        NTS     => "$Mag_2day{nau_twilt_S}",
        ATR     => "$Mag_2day{ast_twilt_R}",
        ATS     => "$Mag_2day{ast_twilt_S}",
        SMT     => "$Mag_2day{sun_mer_trans}",
        MMT     => "$Mag_2day{moon_mer_trans}",
        SDC     => "$Mag_2day{sun_dec}",
        MDC     => "$Mag_2day{moon_dec}",
        MP1     => "$Mag_2day{moonp1}",
        MP2     => "$Mag_2day{moonp2}",
        MP3     => "$Mag_2day{moonp3}",
        MP      => "$Mag_2day{moonphase}",
        MIF     => "$Mag_2day{moonfontchar}",
        MR      => "$Mag_2day{moon_rise}",
        MS      => "$Mag_2day{moon_set}",
        NM      => "$Mag_2day{Next_New_Moon}",
        FM      => "$Mag_2day{Full_Moon}",
        MFQ     => "$Mag_2day{First_Quarter}",
        MLQ     => "$Mag_2day{Last_Quarter}",
        MA      => "$Mag_2day{moon_age}",
        MV      => "$Mag_2day{percentilluminated}"."%",
        MROC    => "$TodayInfo{moon_rate_of_change}",
        MROT    => "$TodayInfo{moon_rotate_val}",
        MIC     => "$TodayInfo{moonImage}",
        LU      => "$TodayInfo{last_update}",
        LF      => "$TodayInfo{fetchtime}",
        D1D     => "$fctinfo{d_1_day}",
        D2D     => "$fctinfo{d_2_day}",
        D3D     => "$fctinfo{d_3_day}",
        D4D     => "$fctinfo{d_4_day}",
        D5D     => "$fctinfo{d_5_day}",
        D6D     => "$fctinfo{d_6_day}",
        D7D     => "$fctinfo{d_7_day}",
        D8D     => "$fctinfo{d_8_day}",
        D9D     => "$fctinfo{d_9_day}",
        D10D    => "$fctinfo{d_10_day}",
        D1T     => "$fctinfo{d_1_temp}",
        D2T     => "$fctinfo{d_2_temp}",
        D3T     => "$fctinfo{d_3_temp}",
        D4T     => "$fctinfo{d_4_temp}",
        D5T     => "$fctinfo{d_5_temp}",
        D6T     => "$fctinfo{d_6_temp}",
        D7T     => "$fctinfo{d_7_temp}",
        D8T     => "$fctinfo{d_8_temp}",
        D9T     => "$fctinfo{d_9_temp}",
        D10T    => "$fctinfo{d_10_temp}",
        D1P     => "$fctinfo{d_1_cor}%",
        D2P     => "$fctinfo{d_2_cor}%",
        D3P     => "$fctinfo{d_3_cor}%",
        D4P     => "$fctinfo{d_4_cor}%",
        D5P     => "$fctinfo{d_5_cor}%",
        D6P     => "$fctinfo{d_6_cor}%",
        D7P     => "$fctinfo{d_7_cor}%",
        D8P     => "$fctinfo{d_8_cor}%",
        D9P     => "$fctinfo{d_9_cor}%",
        D10P    => "$fctinfo{d_10_cor}%",
        D1C     => "$fctinfo{d_1_cond}",
        D2C     => "$fctinfo{d_2_cond}",
        D3C     => "$fctinfo{d_3_cond}",
        D4C     => "$fctinfo{d_4_cond}",
        D5C     => "$fctinfo{d_5_cond}",
        D6C     => "$fctinfo{d_6_cond}",
        D7C     => "$fctinfo{d_7_cond}",
        D8C     => "$fctinfo{d_8_cond}",
        D9C     => "$fctinfo{d_9_cond}",
        D10C    => "$fctinfo{d_10_cond}",
        D1F     => "$fctinfo{d_1_font}",
        D2F     => "$fctinfo{d_2_font}",
        D3F     => "$fctinfo{d_3_font}",
        D4F     => "$fctinfo{d_4_font}",
        D5F     => "$fctinfo{d_5_font}",
        D6F     => "$fctinfo{d_6_font}",
        D7F     => "$fctinfo{d_7_font}",
        D8F     => "$fctinfo{d_8_font}",
        D9F     => "$fctinfo{d_9_font}",
        D10F    => "$fctinfo{d_10_font}",
        D1I     => "$fctinfo{d_1_image}",
        D2I     => "$fctinfo{d_2_image}",
        D3I     => "$fctinfo{d_3_image}",
        D4I     => "$fctinfo{d_4_image}",
        D5I     => "$fctinfo{d_5_image}",
        D6I     => "$fctinfo{d_6_image}",
        D7I     => "$fctinfo{d_7_image}",
        D8I     => "$fctinfo{d_8_image}",
        D9I     => "$fctinfo{d_9_image}",
        D10I    => "$fctinfo{d_10_image}",
        D1H     => "$fctinfo{d_1_hum}%",
        D2H     => "$fctinfo{d_2_hum}%",
        D3H     => "$fctinfo{d_3_hum}%",
        D4H     => "$fctinfo{d_4_hum}%",
        D5H     => "$fctinfo{d_5_hum}%",
        D6H     => "$fctinfo{d_6_hum}%",
        D7H     => "$fctinfo{d_7_hum}%",
        D8H     => "$fctinfo{d_8_hum}%",
        D9H     => "$fctinfo{d_9_hum}%",
        D10H    => "$fctinfo{d_10_hum}%",
        D1PH    => "$fctinfo{d_1_corhum}%",
        D2PH    => "$fctinfo{d_2_corhum}%",
        D3PH    => "$fctinfo{d_3_corhum}%",
        D4PH    => "$fctinfo{d_4_corhum}%",
        D5PH    => "$fctinfo{d_5_corhum}%",
        D6PH    => "$fctinfo{d_6_corhum}%",
        D7PH    => "$fctinfo{d_7_corhum}%",
        D8PH    => "$fctinfo{d_8_corhum}%",
        D9PH    => "$fctinfo{d_9_corhum}%",
        D10PH   => "$fctinfo{d_10_corhum}%",
        FCTP    => "$trends_out[0]",
        FCTT    => "$trends_out[1]",
        FCTW    => "$trends_out[2]",
        FCTH    => "$trends_out[3]",
        FCTU    => "$trends_out[4]",
        FCTD    => "$trends_out[5]",
        FCTV    => "$trends_out[6]",
        FCTS    => "$trends_out[7]",
        FCTSR   => "$trends_out[8]",
        FCTSS   => "$trends_out[9]",
        FCTMR   => "$trends_out[10]",
        FCTMS   => "$trends_out[11]",
        FCTLL   => "$trends_out[12]",
        FCTLD   => "$trends_out[13]",
        FCTMI   => "$trends_out[14]",
        FCTTY   => "$trends_out[15]",
        FCTPF   => "$trends_out[16]",
        FCTTF   => "$trends_out[17]",
        FCTWF   => "$trends_out[18]",
        FCTHF   => "$trends_out[19]",
        FCTUF   => "$trends_out[20]",
        FCTDF   => "$trends_out[21]",
        FCTVF   => "$trends_out[22]",
        FCTSF   => "$trends_out[23]",
        FCTSRF  => "$trends_out[24]",
        FCTSSF  => "$trends_out[25]",
        FCTMRF  => "$trends_out[26]",
        FCTMSF  => "$trends_out[27]",
        FCTLLF  => "$trends_out[28]",
        FCTLDF  => "$trends_out[29]",
        FCTMIF  => "$trends_out[30]",
        FCTTYF  => "$trends_out[31]",
        FCTTYM  => "$trends_out[32]",
        FCTTYS  => "$trends_out[33]",
        # FCTSOLF => "$trends_out[34]",
        GT      => "$wxtrend{temp}",
        GA      => "$wxtrend{atemp}",
        GP      => "$wxtrend{press}",
        GD      => "$wxtrend{dp}",
        GH      => "$wxtrend{hum}",
        GW      => "$wxtrend{ws}",
        GG      => "$wxtrend{wsg}",
        GS      => "$wxtrend{sol}",
        GV      => "$wxtrend{vis}",
        GU      => "$wxtrend{uvi}",
        GR      => "$wxtrend{cor}",
        UU      => "$lua_wx_units",
        TMP     => "$temppath",
        SOL     => "$TodayInfo{solarRad}",
        O3      => "$wx{o3}",
        PM25    => "$wx{pm25}",
        PM10    => "$wx{pm10}",
        NO2     => "$wx{no2}",
        SO2     => "$wx{so2}",
        CO      => "$wx{co}",
        ALOC    => "$wx{loc}",
    );

    # msgout($fctinfo{d_2_cond});
    chomp($clopt1);
    if ( $clopt1 eq "-t" ) {
        if ( -e "$input_template" ) {
            my ( $tmp_copy  ) ;
            {
            open (in_data, "<", $input_template) or die "$LMsg{no_open_file} template: $!";
                local $/ = undef;
                $tmp_copy = <in_data> ;
            close (in_data);
            }
            $tmp_copy =~ s/(?<=\n)#.*\n//g;

            ## convert element to wx value
            my ($nd1, $element, $offset, $st1);
            $offset = 0;
            $st1 = index( $tmp_copy, '[' ) ;
            while ($st1 != -1) {
                $offset = $st1 + 1;
                $nd1 = index($tmp_copy, ']', $offset);
                $element = uc substr($tmp_copy, $st1+1, $nd1 - $st1-1);
                substr($tmp_copy, $st1, $nd1 - $st1+1) = ( %langx ) ? &translate($element) : ( exists $output{$element}) ? $output{$element} : 'Error' ;
                ## look for next occurrence
                $st1 = index($tmp_copy, '[', $offset);
            }

            do_cleanup_lang(3,\$tmp_copy);

            my ( @scroller );
            if ( ! $conf{do_not_output_to_conky} ) {
                print $tmp_copy;
            }
            else {
                my $normalStr;
                ($normalStr) = ($tmp_copy =~ m{NORMALSTART(.*?)NORMALEND}s);
                $tmp_copy =~ s/NORMALSTART.*NORMALEND//s;
                print $normalStr if $normalStr;

                my $conkywx_scroller = "$temppath/wxconkywx_scroller";
                open (out_data, '>', "$conkywx_scroller") or die "$LMsg{no_open_file} conkywx_scroller: $!";
                    $tmp_copy =~ s/#.*//;
                    $tmp_copy =~ s/^\s*\n//g;
                    $tmp_copy =~ s/^/@@|@@|@@|\n/g;
                    $tmp_copy =~ s/\n\s*\n//g;
                    print out_data $tmp_copy;
                close (out_data);
            }
        }
    }
    elsif ( $input_element ) {
        $input_element = uc($input_element);
        print &translate( $input_element )."\n";
    }
    my @wxdebug;
    # $debug = 1;
    if ( $debug or $ENV{debug} ) {
        foreach my $wxkey ( sort keys %output ) {
            my $out1 = &translate($wxkey);
            push @wxdebug, "$wxkey = $out1\n";
        }
        msgout "$LMsg{pass_wxel}" ;
        do_debug( "do_cli_opts dump", @wxdebug );
        msgout "******************* End of conkywx debug log ********************";
    }
}

sub translate {
    if ( $output{$_[0]} ) {
        if ( $output{$_[0]} eq 'NA' )   {
            return 'NA'
        }
        elsif ( exists $naKeys{$_[0]} ) {
            return $output{$_[0]}
        }
        else {
            $_[1] = $output{$_[0]};
            if ($_[0] eq 'UV') {
                my @val1 = split( / : /, $_[1]);
                if ( ${val1[1]} and $langx{lc(${val1[1]})} ) {
                    return( "$val1[0] : $langx{lc(${val1[1]})}" )
                }
                else {
                    return( $_[1])
                }
            }
            elsif ( $gkey{$_[0]} ) {
                my @val1 = split( / /, $_[1]);
                if (exists $langx{lc(${val1[0]})}) {
                    return( "$langx{lc(${val1[0]})} $val1[1]" )
                }
                else {
                    return($_[1])
                }
            }
            else {
                if ( $_[1] and $langx{lc $_[1]}) {
                    return( $langx{lc $_[1]} )
                }
                else {
                    $_[1] =~   s/
                                    ([a-zA-Z]{2}[a-zA-Z]*)
                                /
                                    ( exists $langx{lc $1 } ) ? $langx{lc $1} : $1
                                /gexi;
                    return( $_[1])
                }
            }
        }
    }
    ## if it gets here it is an error
    return "Error"
}

sub run_conkywx {

    my $dev = $ENV{"wxdevelopment"} // 0;
    # msgout($dev);
    parse_html;
    if ( -e "$currdatax" ) {
        parse_pws;
    }
    if ( $wx{dataIsThere} ) {
        verifyWX;
        get_apptemp;
        get_bearing_font;
        get_magellan;
        get_uvdata;
        wxtrends;
        language_hash if $conf{language_to_use};
        do_cl_opts;
        getSeismic if $conf{get_seismic} ;
        if ( $conf{tides_url} ) { &get_tides }
        else { unlink "$temppath/tides" }
        rssd if $conf{newsUrl} or $conf{gdacsUrl} ;
    }
    else {
        msgout( "$LMsg{nodata}" );
        unlink $temp_template;
        if ($debug) {
            msgout( "$LMsg{nodatadbg}" );
            do_debug( "No available" );
        }
    }

}

run_conkywx ;

#--------------------------------------------------------------
# End of file
#--------------------------------------------------------------

=comments

=cut
