
--[[
    Paramvir Likhari - conkywx 2011 - 2016 - GPL3
    bargraph library for use with conkywx lua implementation
--]]

-- local Global_Font_Size
local PI        = 3.14159265358979323846
local TORADIANS = 3.14159265358979323846/180
local TODEGREES = 180/3.14159265358979323846
local tconcat   = table.concat
local tonum     = tonumber
local atan2     = math.atan2
local abs       = math.abs
local lexecute  = os.execute

local lcairo_stroke                      = cairo_stroke
local lcairo_stroke_preserve             = cairo_stroke_preserve
local lcairo_fill                        = cairo_fill
local lcairo_save                        = cairo_save
local lcairo_restore                     = cairo_restore
local lcairo_set_source_rgba             = cairo_set_source_rgba
local lcairo_set_source                  = cairo_set_source
local lcairo_arc                         = cairo_arc
local lcairo_transform                   = cairo_transform
local lcairo_translate                   = cairo_translate
local lcairo_matrix_init                 = cairo_matrix_init
local lcairo_set_line_width              = cairo_set_line_width
local lcairo_set_dash                    = cairo_set_dash
local lcairo_clip                        = cairo_clip
local lcairo_mask                        = cairo_mask
local lcairo_pattern_create_radial       = cairo_pattern_create_radial
local lcairo_pattern_add_color_stop_rgba = cairo_pattern_add_color_stop_rgba
local lcairo_pattern_destroy             = cairo_pattern_destroy
local lcairo_set_fill_rule               = cairo_set_fill_rule
local lcairo_close_path                  = cairo_close_path

-- package.path = package.path .. ";./lib/?.lua;"
local FC = require("luafunc")
local wxsplit         = FC.wxsplit
local to_rgba         = FC.to_rgba
local get_points      = FC.get_points
local make_bezel      = FC.make_bezel
local instrument_sqr  = FC.instrument_sqr
local put_image       = FC.put_image
local ExecInfo        = FC.ExecInfo
local split_newlines  = FC.split_newlines
local FileExists      = FC.FileExists
local make_scale_txt2 = FC.make_scale_txt2

local w = {}

local magellan
-- local updates

function w.daynight(p)

    local cr          = p.cr

    -- local do_set_font     = FC.set_font
    -- local set_font     = FC.set_font

    -- local function set_font(p)
    --     Global_Font_Size = do_set_font(p) or 12
    -- end

    local function draw_daynight_sun_moon(p)

        local h_scale        = p.ds.csize * 0.7 or 0.5
        local sunDeclination = tonum(p.ds.mag.sun_dec)
        local sunAzimuth     = tonum(p.ds.mag.sun_az )
        local moonAzimuth    = tonum(p.ds.mag.moon_az)
        -- print(h_scale)

        --- Start Testing -------------------------------------------------
        -- print(sunDeclination)
        -- sunDeclination = -1
        -- sunAzimuth = 90
        if p.ds.test then sunAzimuth = p.ds.time.sec * 15 end
        --- End Testing ---------------------------------------------------

        local imageScale  = 250 / 106
        local darkEarth   = p.ds.darkness/100 or 0.72
        local sunFactor   = ( 90 - sunDeclination  ) * imageScale * h_scale
        local moonFactor  = ( 90 - p.ds.mag.moon_dec ) * imageScale * h_scale
        local xx,xy       = p.ds.x,p.ds.y
        local StartPoint  = 0

        local magicFactor = imageScale * h_scale * 0.7511
        -- local df1 = 2110
        -- local dig = 2250
        -- local dog = 2225
        local df1, dig, dog
        if abs(sunDeclination) > 5 then
            df1 = 410; dig = 550; dog = 525
        else
            df1 = 1410; dig = 1550; dog = 1525
        end

        local decFactor1, decFactor2, decFactor3, decFactorIn, decFactorOut, decInG, decOutG
        if sunDeclination >= -0.5 and sunDeclination <= 0.5 then
            decFactor1   = 10110 * magicFactor
            decInG       = 10250 * magicFactor
            decOutG      = 10225 * magicFactor
            decFactor2   = 180
            decFactor3   = 120 * magicFactor
            decFactorIn  = 210  * magicFactor
            decFactorOut = 242  * magicFactor
        elseif sunDeclination < -0.5 then
            decFactor1   = (df1 + (((df1-150) / 23.45) * sunDeclination)) * magicFactor
            decInG       = (dig + (((dig-340) / 23.45) * sunDeclination)) * magicFactor
            decOutG      = (dog + (((dog-305) / 23.45) * sunDeclination)) * magicFactor
            decFactor2   = 180
            decFactor3   = 0
            decFactorIn  = 0
            decFactorOut = 242 * magicFactor
        elseif sunDeclination > 0.5 then
            decFactor1   = (df1 - (((df1-210) / 23.45) * sunDeclination)) * magicFactor
            decInG       = (dog - (((dog-325) / 23.45) * sunDeclination)) * magicFactor
            decOutG      = (dig - (((dig-350) / 23.45) * sunDeclination)) * magicFactor
            decFactor2   = 0
            decFactor3   = 0
            decFactorIn  = (210 - (((210-200) / 23.45) * sunDeclination)) * magicFactor
            decFactorOut = (242 - (((242-210) / 23.45) * sunDeclination)) * magicFactor
        end

        local function radial_pattern(p)
            local pat = lcairo_pattern_create_radial(p.xx,p.xy,p.decIn,p.xx,p.xy,p.decOut)
            lcairo_pattern_add_color_stop_rgba(pat, 0.0, to_rgba(p.col1))
            lcairo_pattern_add_color_stop_rgba(pat, 0.1, to_rgba(p.col2))
            lcairo_pattern_add_color_stop_rgba(pat, 1.0, to_rgba(p.col3))
            return pat
        end

        -------------------------- latitude and longitudes ----------
        local tropicsSize = p.ds.line_width * 0.36
        lcairo_save(cr)
        lcairo_set_dash(cr, { 3 }, 1, 0)
        lcairo_set_source_rgba(cr, to_rgba({0x00f400,1}) )
        lcairo_set_line_width(cr, tropicsSize)
        lcairo_arc(cr, xx, xy, 2.01 * 102*(66.5/90) * magicFactor, 0, TORADIANS*360)
        lcairo_stroke(cr)
        -- lcairo_set_source_rgba(cr, to_rgba({0x00f400,1}) )
        -- lcairo_set_line_width(cr, tropicsSize)
        lcairo_arc(cr, xx, xy, 2.01 * 59.5*(66.5/90) * magicFactor, 0, TORADIANS*360)
        lcairo_stroke(cr)
        lcairo_restore(cr)
        for d = 0, 120, 10 do
            if d == 60 then
                lcairo_set_source_rgba(cr, to_rgba({0xff4b03,1}) )
                lcairo_set_line_width(cr, p.ds.line_width * 0.6)
            else
                lcairo_set_source_rgba(cr, to_rgba({0xffffff,0.25}) )
                lcairo_set_line_width(cr,p.ds.line_width * 0.2)
            end
            lcairo_arc(cr, xx, xy, 2.01 * d * magicFactor, 0, TORADIANS*360)
            lcairo_close_path(cr)
            lcairo_stroke(cr)
        end
        lcairo_save(cr)
        local outDia = 2.01 * 120 * magicFactor
        local inDia
        for i = 0, 360, 15 do
            if ( i % 30 == 0 ) then
                inDia = 0
            else
                inDia = 40 * magicFactor
            end
            local msData = {
                cr          = cr,
                startscale  = 0,
                endscale    = 360,
                maxscale    = 360,
                rout        = outDia,
                rin         = inDia,
                xval        = xx,
                yval        = xy,
                i           = i,
            }
            make_scale_txt2(msData)
        end
        lcairo_restore(cr)

        --------------------- Night part 3 -----------------------
        local kx,ky = get_points(StartPoint, sunAzimuth, decFactor2, sunFactor + decFactor1)
        local rpData = {
            xx     = xx+kx,
            xy     = xy+ky,
            decIn  = decInG,
            decOut = decOutG,
            col1   = { 0xffffff,0.00 },
            col2   = { 0x111111,0.45 },
            col3   = { 0x000000,darkEarth},
        }
        local pat1 = radial_pattern(rpData)
        lcairo_set_source_rgba(cr, to_rgba({0x111111,0}) )

        lcairo_save(cr)
        local darkAnt = (33/23.45) *sunDeclination
        if sunDeclination < 0 then
            lcairo_arc(cr, xx, xy, 242*magicFactor, 0, TORADIANS*360)
        else
            lcairo_arc(cr, xx, xy, (209.2+(33-darkAnt))*magicFactor, 0, TORADIANS*360)
        end
        lcairo_clip(cr)
        lcairo_set_source(cr, pat1)
        lcairo_mask(cr, pat1)
        lcairo_pattern_destroy(pat1)
        lcairo_restore(cr)
        if sunDeclination >= 0 then
            lcairo_save(cr)
            lcairo_set_fill_rule(cr,CAIRO_FILL_RULE_EVEN_ODD)
            lcairo_arc(cr, xx, xy, 242*magicFactor, TORADIANS, 0)
            lcairo_clip(cr)
            lcairo_arc(cr, xx, xy, (242-darkAnt)*magicFactor , 0, TORADIANS*360)
            lcairo_stroke_preserve(cr)
            lcairo_arc(cr, xx, xy, 242*magicFactor, 0, TORADIANS*360)
            lcairo_set_source_rgba(cr, to_rgba({0x111111,0.7}) )
            lcairo_fill(cr)
            lcairo_restore(cr)
        end
        --------------------- Antarctic cover -----------------------
        -- if sunDeclination > 0 then
        --     lcairo_save(cr)
        --     -- local kx,ky = get_points(StartPoint, sunAzimuth, 0 , sunFactor - decFactor2 )
        --     local kx,ky = get_points(StartPoint, sunAzimuth, 0 , sunFactor - (90*magicFactor) )
        --     local pat1 = radial_pattern({
        --         xx     = xx+kx,
        --         xy     = xy+ky,
        --         decIn  = decFactorIn,
        --         decOut = decFactorOut,
        --         col1   = { 0xffffff,0.00 },
        --         col2   = { 0x111111,0.45 },
        --         col3   = { 0x000000,0.7},
        --         })
        --     lcairo_save(cr)
        --     lcairo_arc(cr, xx, xy, 242*magicFactor, TORADIANS*(sunAzimuth-170), TORADIANS*(sunAzimuth-10))
        --     lcairo_clip(cr)
        --     lcairo_paint(cr)
        --     lcairo_set_source(cr, pat1)
        --     lcairo_mask(cr, pat1)
        --     lcairo_fill(cr)
        --     lcairo_pattern_destroy(pat1)
        --     lcairo_restore(cr)
        -- end

        --------------------- Sun -----------------------
        lcairo_save(cr)
        local decIn  = 10 * magicFactor
        local decOut = 36 * magicFactor
        local lx,ly  = get_points(StartPoint, sunAzimuth, 0 , sunFactor)
        local sx     = xx + lx
        local sy     = xy + ly
        local rpData = {
            xx     = sx,
            xy     = sy,
            decIn  = decIn,
            decOut = decOut,
            col1   = { 0xffff55,0.90 },
            col2   = { 0xff5555,0.80 },
            col3   = { 0xffaa00,0.05 },
            }
        local pat1 = radial_pattern(rpData)
        lcairo_arc(cr, sx, sy, decOut, TORADIANS, 0)
        lcairo_set_source(cr, pat1)
        lcairo_fill(cr)
        lcairo_pattern_destroy(pat1)
        lcairo_set_line_width(cr, p.ds.line_width * 0.6)
        lcairo_set_source_rgba(cr, to_rgba({0xffff00,1}))
        lcairo_arc(cr, sx, sy, decIn*1.1, TORADIANS, 0)
        lcairo_stroke(cr)
        lcairo_restore(cr)

        --------------------- Moon -----------------------
        lcairo_save(cr)
        local jx,jy = get_points(StartPoint, moonAzimuth, 0 , moonFactor)
        local mx     = xx + jx
        local my     = xy + jy

        -- print(StartPoint, moonAzimuth, moonFactor,sunAzimuth,sunFactor,xx,xy)

        local diffx  = mx - sx; local diffy = my - sy
        local rotAng = TODEGREES * (atan2(-diffx,diffy))

        lexecute([[ echo ]] .. rotAng .. [[ > ~/.config/conkywx/conkywx_internal/wxdnmoon ]])

        -- print(rotAng)

        local moonImage, moonRotate
        moonImage = tonum(p.ds.mag.moonImage)

        -- moonImage = 30
        -- p.ds.msize = 10
        local mcolour
        if p.ds.csize < 0.5 then
            if moonImage < 6 then
                moonImage = 6
            elseif moonImage > 23 and moonImage ~= 30 then
                moonImage = 23
            end
            if moonImage ~= 30 then
                mcolour = "0xffffff:1" or p.ds.mcolour
            end
            p.ds.msize = 3
        else
            mcolour = p.ds.mcolour
        end
        local moonSize = p.ds.csize * 0.113 * p.ds.msize

        if moonImage == 15 or moonImage == 30 then
            moonRotate = 0
        elseif moonImage < 4 then
            moonImage = 3
            moonRotate = -135 + rotAng
        elseif moonImage > 26 then
            moonImage = 27
            moonRotate = 135 + rotAng
        elseif moonImage < 15 then
            if (moonImage % 2) == 0 then moonImage = moonImage + 1 end
            moonRotate = -135 + rotAng
        elseif moonImage > 15 then
            if (moonImage % 2) == 0 then moonImage = moonImage + 1 end
            moonRotate = 135 + rotAng
        end
        lcairo_restore(cr)

        lcairo_set_line_width(cr,p.ds.line_width * 0.5)
        lcairo_arc(cr, mx, my, moonSize * 54, TORADIANS, 0)
        lcairo_set_source_rgba(cr, to_rgba(wxsplit(p.ds.mringcolour,':')) )
        lcairo_stroke_preserve(cr)
        lcairo_set_source_rgba(cr, to_rgba({0x000000,1}) )
        lcairo_fill(cr)
        local dial_colour
        if mcolour then mcolour = wxsplit(mcolour,':') end

        lcairo_save(cr)
        local piData = {
            cr     = cr,
            x      = mx,
            y      = my,
            file   = tconcat({moonImage,'.png'}),
            -- file   = moonImage .. '.png',
            scale  = moonSize,
            ip     = tconcat({ p.ds.ip, '/daynight/' }),
            theta  = TORADIANS * moonRotate,
            rotate = true,
            dial_colour = mcolour,
        }
        put_image(piData)
        lcairo_restore(cr)

    end     -- end of draw_daynight_sun_moon


    local function show_earth(p)
        lcairo_save(cr)
        local piData = {
            cr    = cr,
            x     = p.ds.x,
            y     = p.ds.y,
            file  = 'map.png',
            scale = p.ds.csize,
            ip    = tconcat({ p.ds.ip, '/daynight/' }),
        }
        put_image(piData)
        lcairo_restore(cr)
    end     -- end of show_earth

    local pxo                 = p.lwx       or 0
    local pyo                 = p.lwy       or 0
    local baseSize            = tonum(p.s)  or 300
    local debug               = p.d         or nil
    local darkness            = tonum(p.g)  or 82 -- 72
    local msize               = p.m         or 2.5
    local mcolour             = p.l         or nil
    local mringcolour         = p.r         or {0x00ffff,1}
    local ipath               = p.imgpath
    local test                = p.z         or nil
    local tilt                = p.t         or nil
    local BezelType           = p.j         or 0
    local time                = os.date('*t')
    local updates             = tonum(conky_parse("${updates}"))
    local updateIn            = p.u         or nil

    if tonum(updateIn) == 0 then updateIn = nil end
    local updateInterval
    if updateIn then updateInterval = 1 else updateInterval = 30 end

    -- debug = 1
    -- test = 1
    if tonum(test)  == 0 then test  = nil end
    if debug and type(debug) == 'number' then debug = tonum(debug) end
    if debug == 0 then debug = nil end


    local cwxMagellan

    local archType = ExecInfo({ cmd = 'uname -m' })
    if FileExists("./lib/magellan") then
        cwxMagellan = "./lib/magellan"
    elseif archType:match('x86_64') and FileExists("./lib/magellan_64") then
        cwxMagellan = "./lib/magellan_64"
    elseif FileExists("./lib/magellan_32") then
        cwxMagellan = "./lib/magellan_32"
    else
        error("magellan executable not found!")
    end


    local wvar
    if tonum(conky_info.update_interval) < 10 then
        wvar = ( updates % updateInterval ) + 1
    else
        wvar = updateInterval
    end
    -- print(wvar , updateInterval, ( updates % updateInterval ) + 1, updates)

    if wvar == updateInterval or not magellan then
        magellan = ExecInfo({ cmd = tconcat({ cwxMagellan,' -y ',time.year,' -m ', time.month,' -d ',time.day,' -t ',0.0,' -o ',1,' 89.999999999N',' 0.000000001E' }) })
        if debug then
            print(tconcat({'Debug update: ',time.hour,':',time.min,':',time.sec}))
        end
    end

    local mag = {}
    if magellan then
        local magval = {}
        magval = split_newlines(magellan)
        -- =================== debug ===============================
        if debug and debug == 2 then
            ExecInfo({cmd = tconcat({'echo " " > /tmp/daynight_debug'})})
        end
        for d = 1, #magval do
            local val = wxsplit(magval[d],'=')
            if val[1] and val[2] then
                mag[val[1]] = val[2]:match('"(.-)"')
                -- =================== debug ===============================
                if debug and debug == 2 and val[2] ~= '""' then
                    ExecInfo({cmd = tconcat({'echo ',val[1],' = ',val[2],' >> /tmp/daynight_debug'})})
                end
            end
        end
    end

    if baseSize <= 110 then
        lineThickness = baseSize * 0.02
    elseif baseSize <= 200 then
        lineThickness = baseSize * 0.01
    else
        lineThickness = 2
    end

    local max_disk_size = 310*2
    local disk_size    = baseSize / max_disk_size
    local px            = (pxo+(baseSize*0.52))
    local py            = (pyo+(baseSize*0.52))

    local bimageT             = wxsplit(BezelType,':')
    local bimage              = bimageT[1] or 0
    local bezStyle = {}
    if tonum(bimage) == 1 then          -- use instrument_sqr
        bezStyle[1] = 1
        disk_size = disk_size * 0.95
    elseif tonum(bimage) == 2 then      -- use bezel
        bezStyle = {2, bimageT[2] or 1, bimageT[3] or 2.5}
    end

    -- clock hands
    local secfact
    if disk_size < 0.5 then
        secfact = 4
    else
        secfact = 2
    end

    local lineThickness = secfact * disk_size * lineThickness

    if dial_colour then
        dial_colour = wxsplit(dial_colour,":")
    end

    -- common values for pointer options
    local ds = {
        x             = px,
        y             = py,
        mag           = mag,
        test          = test,
        time          = time,
        csize         = disk_size,
        mcolour       = mcolour,
        mringcolour   = mringcolour,
        msize         = msize,
        line_width    = lineThickness,
        ip            = ipath or '',
        darkness      = darkness,
    }

    -- clock bezel ##############################################
    if tonum(bezStyle[1]) == 1 then
        lcairo_save(cr)
        local isData = {
            cr    = cr,
            tilt  = tilt,
            routI = baseSize,
            wx    = px-(baseSize*0.495),
            wy    = py-(baseSize*0.495),
        }
        instrument_sqr(isData)
        lcairo_restore(cr)
    elseif tonum(bezStyle[1]) == 2 then
        lcairo_save(cr)
        local mbData = {
            cr         = cr,
            bezel_size = baseSize,
            px         = px,
            py         = py,
            tilt       = tilt,
            style      = bezStyle
        }
        make_bezel(mbData)
        lcairo_restore(cr)
    elseif tilt then
        lcairo_save(cr)
        local matrix0 = cairo_matrix_t:create()
        tolua.takeownership(matrix0)

        local t1 = wxsplit(tilt,':')
        lcairo_matrix_init(matrix0, t1[1],t1[2],t1[3],t1[4],0,0)
        lcairo_translate(cr,px,py)
        lcairo_transform(cr,matrix0)
        lcairo_translate(cr,-px,-py)
        lcairo_restore(cr)
    end

    show_earth({ ds = ds })

    lcairo_save(cr)
    draw_daynight_sun_moon({ ds = ds })
    lcairo_restore(cr)

end

collectgarbage()
return w

