#!/bin/bash


# debug='true'

if [[ $debug ]]; then
	rm -rf "/tmp/DEBUG_dompp"
	export PS4='+ (${LINENO}): ${FUNCNAME[0]:+${FUNCNAME[0]}(): }'
	exec 4>>"/tmp/DEBUG_dompp"
	BASH_XTRACEFD=4
	set -x
elif [ -t 0 ]; then
	# this is test if input from teminal then give some feedback output re file add
	myterm='true'
fi

msgout() { printf "%b\n" "$1"; }

bpath1="$( dirname "$1" )"
plpath="$( cd "$bpath1" && pwd )"

bpath2="$( dirname "$0" )"
mypath="$( cd "$bpath2" && pwd )"

mpptmp="/tmp/wxmptmp"
wxfifo="${mpptmp}/mpfifo"
plist_path="${mpptmp}/plist_path"
flist="${mpptmp}/tmpflist.m3u"
faillist="${mpptmp}/faillist.txt"
drlist="${mpptmp}/drlist.txt"
preflist="${mpptmp}/preflist"
totals="${mpptmp}/totals"
lens="${mpptmp}/lens"
ct_info="${mpptmp}/ct_info"

run_mplayer="${mypath}/rmp.sh"

if [[  $(pidof mplayer) == '' ]]; then
	if [[ -e $wxfifo ]]; then
		rm -rf $wxfifo ${mpptmp}
		killall mplayer 2> /dev/null
		mkdir ${mpptmp}
		mkfifo $wxfifo
	else
		mkdir ${mpptmp}
		mkfifo $wxfifo
	fi
fi

mplayer_test(){
	if [[ $( type -p mplayer ) == '' ]]; then
		msgout "\n\n\tPlease install mplayer.\n\twx media player uses mplayer as back end.\n\n"
		echo 'false'
		# exit 1
	fi
}

# -----------------------------------------------------------------------
# show pop-up notifocation
# -----------------------------------------------------------------------
show_noti () {

	# if [[ $show_notification -eq 1 || $2 ]]; then
		if [[ $2 ]]; then
			show_duration=50000
		else
			show_duration=5000
		fi

		if [[ $(type -p notify-send) ]]; then
			notify-send \
			"$1" \
			-i "$cwxpath/images/wxicons-2/clear.png" \
			-t $show_duration
		fi
	# fi
}

fixUnicode() {
	toFixVal="$1"

	toFixVal1=$(echo ${toFixVal%[=:]*})
	toFixVal1=$(echo ${toFixVal1,,})
	toFixVal=$(echo ${toFixVal##*[=:]})
	while read -n 1 c; do
		[[ "$c" = [aeiou] ]] && echo -n ""
		abc=${c/[a-z]/}
		def=${c/${abc}/}
		echo -n ""
		if [[ $abc != '' && $abc != [a-z] && $abc != [=\-] ]]; then
			abc=$(LC_CTYPE=C printf '%d' "'$abc")
			va=$(perl -CS -le 'print chr shift ' "$abc")
			c="${va}${def}"
		fi

		if [[ $c == '' ]]; then c=' ' ; fi
		aa="${aa}${c}"

	done < <(echo -n "$toFixVal")

    echo -n "${aa}"
}

make_playlist(){

	arrmp="$@"

	if [[ ! -e ${flist} ]]; then
		echo "#EXTM3U" > ${flist}
		msgout "$mpptmp" > $plist_path
	fi

	for ffile in "${arrmp[@]}"
	do

		case "${ffile}" in
		*::*)
			c="${ffile#*::}"
			ffile="${ffile%::*}"
			;;
		esac

		pfile="${ffile##*/}"

		c=$((c+1))

		tit=''; art=''; len=''; alb=''; aud=''

		if [[ $(type -p ffprobe) != '' ]]; then
			orgifs="$IFS"; IFS=$'\n'
			meta=( $(ffprobe -hide_banner -of flat -show_streams -select_streams a -show_format "${ffile}"  2> /dev/null) )
			IFS="$orgifs"
			for d in "${meta[@]}"
			do
				d="${d//\"/}"
				case "$d" in
				*tags.title=*)           tit="${d##*=}"                         ;;
				*tags.artist=*)          art="${d##*=}"                         ;;
				*tags.album=*)           alb="${d##*=}"                         ;;
				format.bit_rate*)        aud[5]=${d##*=}; aud[5]=${aud[5]:0:-3} ;;
				*stream.0.sample_rate*)  aud[0]=${d##*=}                        ;;
				*stream.0.channels*)     aud[2]=${d##*=}                        ;;
				format.duration*)        len="${d##*=}";len="${len%.*}"         ;;
				esac
			done
			if [[ $debug ]]; then
				printf "%s %5s  %-50.50s %s %5d\n" "Adding" "${len}s" "$ffile" "file no" "$c"
			elif [[ $myterm ]]; then
				printf "%s %5s  %-50.50s %s %5d\n" "Add FFP" "${len}s" "$pfile" "file no" "$c"
			fi
		else
			orgifs="$IFS"; IFS=$'\n'
			meta=($(mplayer -nolirc -vo null -ao null -frames 0 -identify "${ffile}"))
			IFS="$orgifs"

			for d in "${meta[@]}"
			do
				case "$d" in
				*[tT]itle:*)       tit=$(fixUnicode "${d}") ;;
				*[aA]rtist:*)      art=$(fixUnicode "${d}") ;;
				*[aA]lbum:*)       alb=$(fixUnicode "${d}") ;;
				ID_AUDIO_BITRATE*)
					aud[5]=${d##*=}
					if [[ ${aud[5]} -eq 0 ]]; then
						continue
					else
						aud[5]=${aud[5]:0:-3}
					fi
					;;
				ID_AUDIO_RATE*) aud[0]=${d##*=}; if [[ ${aud[0]} -eq 0 ]]; then continue; fi ;;
				ID_AUDIO_NCH*)  aud[2]=${d##*=}; if [[ ${aud[2]} -eq 0 ]]; then continue; fi ;;
				ID_LENGTH*)     len=${d##*=}; len=$(echo ${len%.*}) ;;
				esac
			done
			if [[ $debug ]]; then
				printf "%s %5s  %-50.50s %s %5d\n" "Adding" "${len}s" "$ffile" "file no" "$c"
			elif [[ $myterm ]]; then
				printf "%s %5s  %-50.50s %s %5d\n" "Add MPL" "${len}s" "$pfile" "file no" "$c"
			fi
		fi

		case $len in
		0|'')   msgout "\nError: :( Unknown format...for file: ${ffile}\n"
			echo "${ffile}" >> ${faillist}
			;;
		*)		# adding to the temp playlist for lua to calc new track data
				if [[ "$art" != '' ]]; then art="Artist: $art / "; fi
				if [[ "$alb" != '' ]]; then alb="Album: $alb / " ; fi
				if [[ "$tit" != '' ]]; then
					tit="Title: $tit"
				else
					tit="File name: ${ffile##*/}"
				fi
				echo "#EXTINF:${len},${aud[0]},${aud[2]},${aud[5]},${art}${alb}${tit}" >> ${flist}
				echo "${ffile}" >> ${flist}
			;;
		esac

		if [[ $len && ! -s "${mpptmp}/wxmpon" ]]; then
			echo "${c}" > "${mpptmp}/wxmpon"
			if [[ ! -e "$ct_info" ]]; then
				echo ""${ffile##*/}":::"${tit}":::${len}:::${aud[5]}:::${aud[0]}:::${aud[2]}:::0:::0" > "$ct_info"
			fi
			"$run_mplayer" "${c}::${ffile// /\\ }" &
		fi

	done
}

sanitize_playlist(){
	local cc=-1
	while IFS= read -ru 3 mLine
	do
		case "$mLine" in
		['#']*)
			continue
			;;
		*)
			if [[ -e "${mLine}" ]]; then
				mLine="${mLine}"
			else
				mLine=$(echo "${plpath}/${mLine}" | sed 's/\r$//')
			fi
			;;
		esac
		if [[ -e "${mLine}" ]]; then
			cc=$((cc+1))
			make_playlist "${mLine}::$cc"
		else
			errMsg="Error :( - please check the input file(s). wx media player cannot play media"
			show_noti "$errMsg" 1
			msgout "\n$errMsg\n"
			exit 1
		fi
	done 3< "$1"
}

get_playlist(){

	rm -rf "${mpptmp}/tpt"

	orgifs="$IFS"; IFS=$'\n'
	inxlist=( "$@" )
	# IFS="$orgifs"

	if [[ $myterm ]]; then
		shopt -s nullglob
	fi

	for f in ${inxlist[@]}
	do
		if [[ -d "${f}" ]]; then continue ; fi
		mcnt=$((mcnt+1))
		case "${f,,}" in
		*.m3u)
			local cfile=$(perl -ne '
				if ($. <= 2) {
					# s/[^[:ascii:]]/+/g;
					if (m{^#EXTINF:(\d+),(\d+),.*$}) { print $1 }
					else { print 'NA' }
				} else { last }' "${f}" )
			# msgout "$cfile"
			case "$cfile" in
			[0-9]*)		# if number then playlist in wxmp format
				xlist="$f"
				# this if will trigger mplayer in slave mode with 1st track
				if [[ $(pidof mplayer) == '' ]]; then
					# set -x
					echo "#EXTM3U" > ${flist}
					x=3		# 1st playlist track
					# loop - check if track exists and play from original file
					while true
					do
						local pfile=$(sed ''"$x"'q;d' "${xlist}" | sed 's/\r$//')
						# local pfile=$(tail -n +"$x" "${xlist}" | head -n 1)
						if [[ -s "$pfile" ]]; then		# file exists check
							y=$((x+1))
							tail -n +"$((x-1))" "${xlist}" | head -n 2 >> "${flist}"
							rm -rf "${mpptmp}/tpt"
							trackno=$(((x-1)/2))
							pline=$((x-1))
							pline=$(sed ''"$((x-1))"'q;d' "${xlist}")
							# ct_info format = file name, title[4], track time[0], kbps[3], Hz[1], channels[2], total playlist time, total tracks
							orgifs="$IFS"; IFS=$','; pline=($(echo "$pline")); IFS="$orgifs"
							# EXTINF:220[0] 44100[1] 2[2] 256.0[3] Shakira / Shakira Waka Waka[4]
							if [[ ${pline[4]} == '' ]]; then
								rm "${flist}"
								sanitize_playlist "${f}"
								exit 0
							fi
							echo "${pfile}:::${pline[4]}:::${pline[0]#*:}:::${pline[3]}:::${pline[1]}:::${pline[2]}:::0:::0" > "$ct_info"

							"$run_mplayer" "${trackno}::${pfile// /\\ }" &
							break
						else
							sanitize_playlist "${f}"
							break
						fi
						x=$((x+2))
					done
					# while the 1st track playes create temp playlist
					if [[ $(($(stat -c %s "${xlist}")/(1024**2))) -gt 27 ]]; then sleep 5 ; fi
					tail -n +"$y" "${xlist}" | sed '/./!d' >> "${flist}"
					rm -rf "${mpptmp}/tpt"
				else
					tail -n +2 "${xlist}" | sed '/./!d' >> "${flist}"
					rm -rf "${mpptmp}/tpt"
				fi
				;;
			*)
				sanitize_playlist "${f}"
			;;
			esac
			msgout "$plpath" > $plist_path
			;;
		*)
			printf "%s\n" "${f}" >> "$preflist"
			;;
		esac
	done
	IFS="$orgifs"
	rm -rf "${mpptmp}/tpt"
}

if [[ $( mplayer_test ) != 'false' ]]; then
	get_playlist "$@"
	if [[ -s "${preflist}" ]]; then sanitize_playlist "${preflist}"; fi
	rm -rf "$preflist"
else
	exit 1
fi

if [[ $debug ]]; then
	set +x
fi

exit 0



