package fixConfig;

=comments
    Parses bash config and messages file formats
    This version (c) 2018 Paramvir Likhari
=cut

$fixConfig::VERSION = '7';

require Exporter;
@fixConfig::ISA = qw(Exporter);
@fixConfig::EXPORT = qw(fixConf);

use strict;
use warnings;

my (%conf);

sub readConf {
    my $fixFile = shift;
    my $balanceSplit = sub {
        if ( not $_[0] ) { return '' }
        $_[0] =~ s/
                    (?: \s* "([^"]+)" (\s*) )
                   |
                    (?: \s* '([^']+)' (\s*) )
                   |
                    (?: \s* (\S+) (\s*))
                  /
                   ($1||$3||$5) . ($2||$4||$6 ? '\',\'' : '')
                  /exg;
        return($_[0]);
    };

    if ( -s "$fixFile" ) {

        my $constr;
        {
        open (in_data, '<', "$fixFile") or die "missing conkywx config file: $!";
            local $/ = undef;
            $constr = <in_data> ;
        close (in_data);
        }
        $constr =~ s/(?<=[\s])#.*\n/\n/g;
        $constr =~ s/\s*\n/\n/g;


        ## generic bash array to perl variable
        my ($startTag, $endTag, $offset, $st1);
        $startTag = '=(';
        $endTag = ')';
        $offset = 0;
        $st1 = index( $constr, $startTag ) ;
        while ($st1 != -1) {
            $offset = $st1 + 1;
            my $nd1 = index($constr, $endTag, $offset);

            if ($nd1 != -1) {
                ## format bash array to string
                ## $st1-3 getting last 3 characters before '=(' to match array name
                my $retstr = &$balanceSplit(substr($constr, $st1-3, ($nd1+3) - $st1));

                ## get array name
                my ($fstr1) = $retstr =~ m{(.*?)=\(} ;
                my ($fstr2) = $constr =~ m{(.*?)$fstr1} ;
                $fstr1 = $fstr2 . $fstr1 ;

                ## assign name and clear the name marker
                # ($conf{$fstr1} = $retstr) =~ s/.*=\(_,//;
                ($conf{$fstr1} = $retstr) =~ s/.*=\(\',\'//;
            }

            ## look for next occurrence
            $st1 = index($constr, $startTag, $offset);
        }

        $constr =~ s/.*=\(.*\n//g;
        $constr =~ s/.*\).*\n//g;

        for (split(('\n'),$constr)) {
            if ( index($_,'=') != -1 ) {
                my ($k, $v) = split( /(?:=)/, $_,2 ) ;
                    $v =~ tr/\x27|\x22//d;
                    $conf{$k} = $v;
                    # msgout($k,$v);
            }
        }

        # Units_downloaded now hardcoded in conkywx.sh as well
        $conf{Units_downloaded}    = "IM"            ;
        # define some defaults
        $conf{today_date_format}   ||= '%A %b %d %Y' ;
        $conf{minimum_fetch_time}  ||= 900           ;
        $conf{Time_format}         ||= '%R'          ;
        $conf{show_degsym_fc_temp} ||= 5             ;
        $conf{moon_outline}        ||= 0             ;
        $conf{moon_time}           ||= '%b %d'       ;
        $conf{use_short_date}      ||= 0             ;

    }

}

sub readMsgs {
    my $fixFile = shift;

    if ( -s $fixFile ) {

        my $Output;
        {
        open (in_data, '<', "$fixFile") or die "unable to open language message file: $!";
            local $/ = undef;
            $Output = <in_data> ;
        close (in_data);
        }

        $Output =~ s/(?<=[\s])#.*\n/\n/g;
        $Output =~ s/\s*\n/\n/g;
        $Output =~ tr/\x27|\x22//d;
        $Output =~ s/\x0a\x0a/\x0a/g;

        for (split /\n/, $Output) {
            my ($k,$v) = split('=',$_,2);
            if ($v) {
                $v =~ s/\x5c\x6e/\x0a/g ;
                $v =~ s/\x5c\x74/\x09/g ;
                $v =~ s/\x5b/\x5c\x5b/g ;
                $v =~ s/\x5d/\x5c\x5d/g ;
                $conf{$k} = $v;
            }
        }
    }
}

sub fixConf {
    my $fixFile = shift;
    my $opts = shift;
    return undef unless defined $fixFile;

    if (index($fixFile, '.msgs') != -1) {
        &readMsgs($fixFile);
    }
    else {
        &readConf($fixFile);
    }

    if ( not $opts ) {
        while ( my ($key1, $value1) = each %conf ) {
            if ($value1) {
                if ($key1 eq 'pcondition_blacklist' or $key1 eq 'template_cleanup') {
                    $value1 =~ s/\,\'$//;
                    print("$key1={\'$value1}\n");
                }
                else {
                    print("$key1=[[$value1]]\n");
                }
            }
            else {
                print("$key1=[[NA]]\n")
            }
        }
    }
    else { return %conf }
}

1;
