--[[
    Paramvir Likhari - conkywx 2011 - 2017 - GPL3
    common library for use with conkywx lua implementation
--]]

local w = {true}

local PI        = 3.14159265358979323846
local TORADIANS = 3.14159265358979323846/180
local TODEGREES = 180/3.14159265358979323846
local font               = "Mono"
local Default_Font_Name  = 'Mono'
local Default_Font_Style = 'NORMAL'
local Default_Font_Size  = 12

local MT         = require('math')
local lfloor    = MT.floor
local sin       = MT.sin
local cos       = MT.cos

local tonum     = tonumber

local TB = require('table')
local tconcat   = TB.concat

local OS = require('os')
local lexecute  = OS.execute
local lexit     = OS.exit

local IO = require('io')
local lopen     = IO.open
local lpopen    = IO.popen
local lstderr   = IO.stderr

local ST = require('string')
local supper    = ST.upper
local lfind     = ST.find
local lsub      = ST.sub
local sprintf   = ST.format
local lmatch    = ST.match
local lgsub     = ST.gsub

local myPackage = package
local lpairs    = pairs



local lcollectgarbage = collectgarbage

local lcairo_move_to                       = cairo_move_to
local lcairo_line_to                       = cairo_line_to
local lcairo_curve_to                      = cairo_curve_to
local lcairo_rotate                        = cairo_rotate
local lcairo_show_text                     = cairo_show_text
local lcairo_stroke                        = cairo_stroke
local lcairo_stroke_preserve               = cairo_stroke_preserve
local lcairo_fill                          = cairo_fill
local lcairo_fill_preserve                 = cairo_fill_preserve
local lcairo_save                          = cairo_save
local lcairo_restore                       = cairo_restore
local lcairo_select_font_face              = cairo_select_font_face
local lcairo_set_font_size                 = cairo_set_font_size
local lcairo_set_source_rgba               = cairo_set_source_rgba
local lcairo_set_source                    = cairo_set_source
local lcairo_arc                           = cairo_arc
local lcairo_arc_negative                  = cairo_arc_negative
local lcairo_close_path                    = cairo_close_path
local lcairo_new_sub_path                  = cairo_new_sub_path
local lcairo_transform                     = cairo_transform
local lcairo_translate                     = cairo_translate
local lcairo_matrix_init                   = cairo_matrix_init
local lcairo_set_line_width                = cairo_set_line_width
local lcairo_pattern_create_radial         = cairo_pattern_create_radial
local lcairo_pattern_add_color_stop_rgba   = cairo_pattern_add_color_stop_rgba
local lcairo_pattern_destroy               = cairo_pattern_destroy
local lcairo_text_extents                  = cairo_text_extents
local lcairo_set_line_cap                  = cairo_set_line_cap
local lcairo_set_fill_rule                 = cairo_set_fill_rule
local lcairo_surface_destroy               = cairo_surface_destroy
local lcairo_set_operator                  = cairo_set_operator
local lcairo_clip                          = cairo_clip
local lcairo_mask_surface                  = cairo_mask_surface
local lcairo_mask                          = cairo_mask
local lcairo_paint                         = cairo_paint
local lcairo_set_source_surface            = cairo_set_source_surface
local lcairo_scale                         = cairo_scale
local lcairo_image_surface_get_height      = cairo_image_surface_get_height
local lcairo_image_surface_get_width       = cairo_image_surface_get_width
local lcairo_image_surface_create_from_png = cairo_image_surface_create_from_png

local limlib_load_image                   = imlib_load_image
local limlib_context_set_image            = imlib_context_set_image
local limlib_context_set_color            = imlib_context_set_color
local limlib_create_image                 = imlib_create_image
local limlib_create_cropped_scaled_image  = imlib_create_cropped_scaled_image
local limlib_image_get_width              = imlib_image_get_width
local limlib_image_get_height             = imlib_image_get_height
local limlib_image_fill_rectangle         = imlib_image_fill_rectangle
local limlib_image_set_has_alpha          = imlib_image_set_has_alpha
local limlib_image_copy_alpha_to_image    = imlib_image_copy_alpha_to_image
local limlib_image_set_format             = imlib_image_set_format
local limlib_free_image                   = imlib_free_image
local limlib_render_image_on_drawable     = imlib_render_image_on_drawable
local limlib_save_image_with_error_return = imlib_save_image_with_error_return


function w.msgout(...)
    local n = select("#",...)
    for i = 1, n do
        local v = tostring(select(i,...))
        lstderr:write(v)
        if i~=n then lstderr:write('\t') end
    end
    lstderr:write('\n')
end

function w.basename(str)
    if not str then return end
    return string.gsub(str, "(.*/)(.*)", "%2")
end

function w.round(num, idp)
    return tonum(sprintf("%." .. (idp or 0) .. "f", num))
end
-- I always want to capture your enthusiasm and energy

function w.getDatafile(dataFile)
    -- extension = extension or 'lua'
    if w.FileExists(dataFile) then
        local dataPath, ext
        dataPath = lmatch(dataFile, '^(.+)%/.*$')
        ext = lmatch(dataFile, '^.*%.(.+)$')

        if lmatch(dataFile, '.'..ext) then
            dataFile = lmatch(dataFile, '^.*%/(.-)%.' .. ext)
        end

        dataPath = tconcat({dataPath,'/?.',ext,';'})

        -- need testPath for lmatch to understand dataPath
        local testPath = dataPath
        testPath = lgsub(testPath, '%.','%%.')
        testPath = lgsub(testPath, '%?','%%?')
        -- add path only if it is missing
        if ( not lmatch(myPackage.path, testPath)) then
            myPackage.path = dataPath .. myPackage.path
        end
        return require(dataFile)
    else
        return nil
    end
end

function w.wxsplit(string,delimiter,wxnum,keepSpace)
    if not string then return nil end
    if type(string) == 'table' then return string end
    local result = {}
    local from   = 1
    local self   = string .. ''
    --- keepSpace flag skips following line as it removes spaces
    --- default has been to remove leading and trailing spaces
    --- so if a line has only spaces - that element will not exist in the
    --- resulting table - which will affect the number of elements in table
    if not keepSpace then self = lmatch(self, "^%s*(.-)%s*$" ) end
    local delim_from, delim_to = lfind( self, delimiter, 1 )
    if delim_to then
        if wxnum ~= 1 then
            while delim_from do
                result[#result+1] = lsub( self, from, delim_from-1 )
                from  = delim_to + 1
                delim_from, delim_to = lfind( self, delimiter, from  )
            end
            result[#result+1] = lsub( self, from )
        else
            if delimiter == "°" then acase = 2 else acase = 1 end
            result[#result+1] = lsub( self, 1, delim_to - acase )
            result[#result+1] = lsub( self, delim_from + 1, -1  )
        end
    else
        result[#result+1] = lsub( self, 1 )
    end
    return result
end

function w.getopts(...)
    -- 2016 - 2017 command line getopts paramvir
    local inStr = tconcat({'',...},' ')
    local p,t = {},{}
    -- if lmatch(inStr, 'f 9') then
    --     -- w.msgout(lgsub(inStr,"%s+%-(%l)(.-)",":|:%1:|:%2"))
    --     w.msgout(inStr)
    -- end
    t = w.wxsplit(lgsub(inStr,"%s+%-(%l)(.-)",":|:%1:|:%2"),':|:')
    for x = 1, #t do
        if x % 2 == 0 then
            if t[x] and t[x+1] ~= '' then
                local v = lmatch(t[x+1], "^%s*(.-)%s*$")
                if tonum(v) then
                    p[t[x]] = tonum(v)
                else
                    p[t[x]] = v
                end
            else
                w.msgout("\n\nError :( \n"..inStr.."\nNot valid options on command line conkywx lua!!!\n")
            end
        end
    end
    local ip11 = lsub(debug.getinfo(1).source, 2, -16)
    p['imgpath'] = tconcat({ip11,'images'})

    if p.p then
        local p1 = tostring(p.p)
        p1 = lgsub(p1, "%,", "|")
        if lmatch(p1, '%|') then
            local ptable = w.wxsplit(p1,'%|')
            p.lwx, p.lwy = ptable[1], ptable[2]
            if not p.lwx or not p.lwy then
                w.msgout("\n\nError :( \n-p "..p.p.."\n"..inStr.."\nNot valid position option on command line conkywx lua!!!\n")
            else
                p.lwx = tonum(p.lwx)
                p.lwy = tonum(p.lwy)
            end
        end
    end
    return p
end


function w.strip_ext_codes( str )
    local s = ""
    for i = 1, str:len() do
        if str:byte(i) >= 32 and str:byte(i) <= 126 then
            s = tconcat({s,str:sub(i,i)})
        end
    end
    return s
end

function w.set_font(s)
    --- paramvir 2013 - 2016
    local cr      = s[1]
    lcairo_restore(cr)
    -- w.msgout(p[2])
    local p       = s[2]
    local wxsplit = w.wxsplit
    local fontname, fontsize, fontstyle, fontslant
    local function parse_ft(aa)
        if aa[1] == "style" then
            if aa[2] ~= "" then
                return( supper(aa[2]))
            else
                return('NORMAL')
            end
        elseif aa[1] == "size" then
            if aa[2] then
                return(tonum(aa[2]))
            else
                return(Default_Font_Size)
            end
        end
    end

    local ftT = { BOLD = true, NORMAL = true }
    if tonum(p) then
        fontsize = p
    elseif ftT[supper(p)] then
        fontstyle = supper(p)
    elseif supper(p) == 'ITALIC' then
        fontslant = supper(p)
    elseif lmatch(p, ':') then
        local ft = wxsplit(p,":")
        for x = 1, #ft do
            if lmatch(ft[x], "=") then
                -- w.msgout(ft[x])
                local fst = wxsplit(ft, "=")
                local fontw = parse_ft(fst)
                if tonum(fontw) then
                    fontsize  = fontw
                else
                    fontstyle = fontw
                end
            elseif tonum(ft[x]) then
                fontsize = tonum(ft[x])
            elseif ftT[supper(ft[x])] then
                fontstyle = supper(ft[x])
            elseif supper(ft[x]) == 'ITALIC' then
                fontslant = supper(ft[x])
            else
                fontname = ft[x]
            end
        end
    else
        fontname = p
    end

    fontname  = fontname  or Default_Font_Name
    fontstyle = fontstyle or Default_Font_Style
    fontsize  = fontsize  or Default_Font_Size
    fontslant = fontslant or 0

    if fontstyle == 'BOLD'   then fontstyle = 1 else fontstyle = 0 end -- 0 is normal
    if fontslant == 'ITALIC' then fontslant = 1 else fontslant = 0 end -- 0 is normal

    lcairo_select_font_face(cr, fontname, fontslant, fontstyle)
    lcairo_set_font_size(cr, fontsize)
    lcairo_save(cr)

    return fontsize
end

function w.put_image(p)

    local cr         = p.cr
    local to_rgba    = w.to_rgba

    local get_points = w.get_points
    local wxsplit    = w.wxsplit
    local msgout     = w.msgout

    local bgcolour11 = "0xFF0030:1"
    local bgcolour11 = wxsplit(bgcolour11,":")

    local scale      = p.scale or 1
    local ppx1, ppy1

    if p.offcen then
        ppx1, ppy1 = get_points(p.start_angle,p.end_angle,0,-scale*126)
        scale      = scale*1.8
        p.x, p.y   = p.x + ppx1, p.y + ppy1
    end

    local file_tp, opt2
    if p.ip then
        file_tp = tconcat({p.ip, p.file})
        opt2    = "1"
    else
        file_tp = p.file
    end
    local image = lcairo_image_surface_create_from_png(file_tp)
    local w     = lcairo_image_surface_get_width(image)
    local h     = lcairo_image_surface_get_height(image)
    lcairo_save(cr)

    --- make changes here to x,y for panning image
    -- direction left +0, right -0, up 0+, down 0-
    local pan = {}
    if p.pan then
        pan = wxsplit(p.pan, ':')
    else
        pan[1] = 0
        pan[2] = 0
    end

    lcairo_translate(cr, p.x + pan[1], p.y + pan[2] )

    if not opt2 then
        local nw,nh
        nw = (scale*100)/w
        nh = (scale*100)/h
        lcairo_scale(cr, nw, nh)
    else
        lcairo_scale(cr, scale, scale)
    end

    if p.rotate then lcairo_rotate(cr, p.theta) end

    lcairo_translate(cr, -0.5*w, -0.5*h)
    lcairo_set_source_surface(cr, image, 0, 0)

    -- msgout(p.dial_colour)

    if p.clip then lcairo_clip(cr) end
    if p.dial_colour then
        lcairo_set_source_rgba(cr, to_rgba(p.dial_colour))
        lcairo_set_operator(cr, CAIRO_OPERATOR_SOURCE)
        lcairo_mask_surface(cr, image, 0, 0)
        -- cairo_paint_with_alpha(cr,0.25)
    else
        -- if p.clip then lcairo_clip(cr) end
        lcairo_paint(cr)
    end

    lcairo_surface_destroy(image)
    lcairo_restore(cr)
end


function w.put_image_imlib(p)

    local wx   = p.wx or 0
    local wy   = p.wy or 0
    local file = p.rfile or error('put_image_imlib error: input file name with path requred')

    local cr = p.cr
    local put_image = w.put_image
    local FileExists = w.FileExists
    local alpha = p.a or 255

    local saveFile = p.sfile


        local pan = {}
        if p.b then
            pan = w.wxsplit(p.b, ':')
        else
            pan[1] = 0
            pan[2] = 0
        end

        local wsize, xsize, ysize, px, py
        if p.s then
            wsize = w.wxsplit(p.s,",")
            xsize = lmatch(wsize[1], "^%s*(.-)%s*$" )
            ysize = lmatch(wsize[2], "^%s*(.-)%s*$" )
        else
            xsize,ysize=0,0
        end

        -- local bgcolour = p.c or "0x000000:0.2"
        -- bgcolour       = wxsplit(bgcolour,":")
        -----------------------------------------
        local width    = tonum(xsize) or conky_window.width
        local height   = tonum(ysize) or conky_window.height


    if not FileExists(saveFile) then

        local scale, scalex, scaley
        -- local scaleval = p.e or 1
        if lmatch(p.e, ':') then
            scale = w.wxsplit(p.e,':')
            scalex = scale[1] * 0.3
            scaley = scale[2] * 0.3
        else
            scalex = (p.e or 1) * 0.3
            scaley = scalex * 1.8
        end

        imlib_set_cache_size(0)


        --- convert all formats to png files
        local primaryImage = limlib_load_image(file)
        limlib_context_set_image(primaryImage)
        limlib_image_set_format('png')
        local retval = limlib_save_image_with_error_return(saveFile)

        primaryImage = limlib_load_image(saveFile)

        if not primaryImage then return end

        limlib_context_set_image(primaryImage)

        local w1 = limlib_image_get_width()
        local h1 = limlib_image_get_height()

        local scaled = limlib_create_cropped_scaled_image( pan[1], pan[2], w1/scalex, h1/scaley, width, height)

        limlib_context_set_image(scaled)
        local w2 = limlib_image_get_width()
        local h2 = limlib_image_get_height()

        local maskImage = limlib_create_image(w2,h2)
        limlib_context_set_image( maskImage )
        limlib_image_set_has_alpha( 1 )
        limlib_context_set_color( 0, 0, 0, alpha )
        limlib_image_fill_rectangle( 0, 0, w2, h2 )

        limlib_context_set_image(scaled)
        limlib_image_set_has_alpha( 1 )
        limlib_image_copy_alpha_to_image( maskImage, 0, 0 )

        limlib_image_set_format('png')
        limlib_save_image_with_error_return(saveFile)

        limlib_free_image()
        primaryImage = nil
        maskImage    = nil
        scaled       = nil
    else
        local image = lcairo_image_surface_create_from_png(saveFile)
        local w1    = lcairo_image_surface_get_width(image)
        local h1    = lcairo_image_surface_get_height(image)

        lcairo_translate(cr, wx, wy )
        lcairo_translate( cr, 0.001 * w1, 0.001 * h1 )
        lcairo_set_source_surface(cr, image, 0, 0)
        lcairo_paint(cr)
        lcairo_surface_destroy(image)
    end

end --function image ##################


function w.FindLuma(p)
    local luma = (((((p[1] / 0x10000) % 0x100)/255) * 0.2126) +
                   ((((p[1] / 0x100  ) % 0x100)/255) * 0.7152) +
                      ((p[1] % 0x100  )       /255) * 0.0722)
    if luma < 0.5 then return true else return nil end
end

function w.to_rgba(p)
    return ((p[1] / 0x10000) % 0x100) / 255,
           ((p[1] / 0x100) % 0x100)   / 255,
            (p[1] % 0x100)            / 255, p[2]
end

function w.to_rgba_contrast(p)
    if w.FindLuma(p) then
        return 1 - (((p[1] / 0x10000) % 0x100) / 255),
               1 - (((p[1] / 0x100) % 0x100) / 255),
               1 - ((p[1] % 0x100) / 255), 1
    else
        return 0,0,0,1
    end

end

function w.colour_blend2(value,midvalue,guageheight)
    local colourFact
    colourFact = ((value - midvalue) / (guageheight - midvalue)) * 3.182525
    return ((1 - colourFact) * colourFact) + 1,
           1 - colourFact,
           (colourFact - 1) * colourFact, 1
end

function w.colour_blend(value,midvalue,guageheight,test)
    local colourFact
    colourFact = ((value - midvalue) / guageheight) * 3
    -- if test then print(colourFact, value, midvalue) end
    return ((1 - colourFact) * colourFact) + 1,
           0.5 - colourFact,
           (colourFact - 1) * colourFact, 1
end

function w.trim(p)
    if not p.text then return nil end
    return lmatch(p.text,'^%s*(.-)%s*$')
    -- return lgsub(p.text,'%s+','')
end

function w.get_points(startpt,degval,degfact,sizefact,opsign)
    -- startpt, degval, degfactor, sizefactor
    if opsign == 1 then sizefact = 0 - sizefact end
    local arc
    arc = TORADIANS * ( startpt + (degval or 0) + degfact )
    return (0 + ( sizefact * sin(arc) )), (0 - ( sizefact * cos(arc) ))
end

function w.getAngleInRadians(start_angle, current_angle)
    return (TORADIANS * ((current_angle + start_angle) - 90))
end


function w.split_newlines(s)
    local ts = {}
    local posa = 1
    while 1 do
        local pos, chars = lmatch(s, '()([\r\n].?)', posa)
        if pos then
            if chars == '\r\n' then pos = pos + 1 end
            local line = s:sub(posa, pos)
            ts[#ts+1] = line
            posa = pos + 1
        else
            local line = s:sub(posa)
            if line ~= '' then ts[#ts+1] = line end
            break
        end
    end
    return ts
end

function w.rPrint(s, l, i) -- recursive Print (structure, limit, indent)
    l = (l) or 1000; i = i or "";  -- default item limit, indent string
    if (l<1) then w.msgout("ERROR: Item limit reached."); return l-1 end;
    local ts = type(s);
    -- if (ts ~= "table") then w.msgout(i,ts,s); return l-1 end
    if (ts ~= "table") then w.msgout(i,s); return l-1 end
    -- print (i,ts);           -- print "table"
    w.msgout(i,ts);           -- print "table"
    for k,v in lpairs(s) do  -- print "[KEY] VALUE"
        -- l = w.rPrint(v, l, i.."\t["..tostring(k).."]");
        l = w.rPrint(v, l, i.." / ["..tostring(k).."]");
        if (l < 0) then break end
    end
    return l
end

-- function w.memory(p)
--     -- local memTable, memStr
--     -- local memStr = lexecute('cat /proc/meminfo')
--     local memStr = w.ExecInfo({rfile = '/proc/meminfo'})
--     -- memTable = w.wxsplit(memStr, '\n')

--     if p.mem then
--         print(memStr[1])
--     end

-- end

function w.ExecInfo(p)
    local f1
    if p.cmd then                                    -- run a command
        -- zap shell errors to /dev/null
        f1 = lpopen(p.cmd .. ' 2> /dev/null')
        -- f1 = lpopen(p.cmd)
    elseif p.rfile then                              -- read a file
        f1 = lopen(p.rfile,'r')
    elseif p.wfile or p.afile and p.wtext then       -- write to a file
        if     p.wfile then f1 = lopen(p.wfile,'w')
        elseif p.afile then f1 = lopen(p.afile,'a')
        end
        if f1 then
            if type(p.wtext) == 'table' then
                for d = 1, #p.wtext
                do
                    f1:write(p.wtext[d]..'\n')
                end
            else
                f1:write(p.wtext)
            end
            f1:close()
        end
        return nil
    end
    if f1 then                            -- return what ever
        local rinfo = f1:read("*a")
        f1:close()
        -- if lmatch(rinfo,'Timestamp') then print(rinfo) end

        -- rinfo = lmatch(rinfo, "^%s*(.-)%s*$" )
        return lmatch(rinfo, "^%s*(.-)%s*$" )
    else
        return nil
    end
end


function w.firstToUpper(str)
    if str then return (lgsub(str, "^%l", string.upper)) end
    return ''
end

function w.FileStat(t)
    local outstat = w.ExecInfo({ cmd = tconcat({'stat -c %s ',t.ifile}) })
    local instat = w.ExecInfo({ rfile = t.ofile })
    if instat then
        if tonum(instat) ~= tonum(outstat) then
            w.ExecInfo({ wfile = t.ofile, wtext = outstat })
            return nil
        else
            return true
        end
    else
        w.ExecInfo({ wfile = t.ofile, wtext = outstat })
        return nil
    end
end

function w.FileExists(fname)
    if not fname then
        return error("\n\nError :( no file name defined to test for or the file does not exist. \n")
    end
    -- local f = w.ExecInfo({ cmd = [[if [ -e "]] .. fname .. [[" ]; then echo "true"; fi]] })
    -- if f == 'true' then return true else return nil end
    local f = lopen(fname,"r")
    if f ~= nil then io.close(f) return true else return nil end
end

function w.DirExists(dpath)
    -- local f = w.ExecInfo({ cmd = tconcat({ 'if [ -d "',dpath,'" ]; then echo "true"; fi'})})
    local f = w.ExecInfo({ cmd = [[if [ -d "]] .. dpath .. [[" ]; then echo "true"; fi]]})
    if f == 'true' then return true else return nil end
end

function w.stat(p)
    local wT, retval
    if p.opts then wT = w.wxsplit(p.opts,'%%') end

    retval = w.ExecInfo({cmd = tconcat({[[stat -c "]], p.opts,'" ', p.file}) })

    if #wT > 2 then
        return w.wxsplit(retval,' ')
    else
        return retval
    end
end


function w.mkdir(dpath)
    if not w.DirExists( dpath ) then
        w.ExecInfo({ cmd = tconcat({ 'mkdir ', dpath }) })
    end
end

function w.copy(p)
    w.ExecInfo({ cmd = tconcat({ 'cp ', p.from, ' ', p.to }) })
end

function w.move(p)
    -- print(tconcat({ 'mv ', p.from, ' ', p.to }))
    w.ExecInfo({ cmd = tconcat({ 'mv ', p.from, ' ', p.to }) })
end

function w.replaceText(p)
    if w.FileExists( p.rfile ) then
        local str = w.wxsplit(w.ExecInfo({ rfile = p.rfile }), '\n')
        for i = 1, #str do
            str[i] = lgsub(str[i], p.stext, p.rtext)
        end
        w.ExecInfo({ wtext = str, wfile = p.rfile })
    else
        local rfile = p.rfile or 'No file Name defined!!'
        error('\n\nError :(\nFile to replace text in does not exist.' .. rfile)
    end
end

function w.grep(p)
    if w.FileExists( p.rfile ) then
        -- print(tconcat({'grep -m 1 ', p.stext,' ', p.rfile }))
        -- local str = w.wxsplit(w.ExecInfo({ rfile = p.rfile }), '\n')
        -- for i = 1, #str do
        --     if str[i]:lmatch(p.stext) then
        --         return str[i]:lmatch(p.stext)
        --     end
        -- end
        local retval = w.ExecInfo({ cmd = tconcat({'grep -m1 ', p.stext,' ', p.rfile })})
        if not retval or retval == '' then
            return nil
        end
    else
        local rfile = p.rfile or 'No file Name defined!!'
        error('\n\nError :(\nFile to search/grep in does not exist.' .. rfile)
    end
    return ''
end


function w.background(p)

--[[
    Paramvir 2015
    This script draws a background on the Conky window or can be
    used to place shaded squares anywhere within the conky window.

    Use -s w,h to state size - will fill window if NA,NA given
    Use -p x,y for position - will use 0,0 if no position given
    This allows you to have multiple backgrounds per conky window
    code based on instrument_sqr - perhaps will unify one day ;-)

]]

    local cr      = p.cr
    local wxsplit = w.wxsplit
    local to_rgba = w.to_rgba

    local trim = w.trim

    local xsize, ysize, px, py
    if p.s then
        local wsize = wxsplit(p.s,",")
        xsize = lmatch(wsize[1], "^%s*(.-)%s*$" )
        ysize = lmatch(wsize[2], "^%s*(.-)%s*$" )
    else
        xsize,ysize=0,0
    end
    if p.p then
        local wsize = wxsplit(p.p,",")
        px = tonum(lmatch(wsize[1], "^%s*(.-)%s*$" ))
        py = tonum(lmatch(wsize[2], "^%s*(.-)%s*$" ))
    else
        px,py=0,0
    end

    local bgcolour = p.c or "0x000000:0.2"
    bgcolour       = wxsplit(bgcolour,":")
    -----------------------------------------
    local width    = tonum(xsize) or conky_window.width
    local height   = tonum(ysize) or conky_window.height

    if not width or width == "NA" then
        width = conky_window.width
    end
    if not height or height == "NA" then
        height = conky_window.height
    end

    -- print('AA',xsize,ysize, width, height)

    local cnr  = p.r or 15

    --- define corner values
    local wcnr = width  - cnr
    local hcnr = height - cnr

    local rad = { tl = 0, tr = 0, bl = 0, br = 0 }
    if p.q then
        if rad[p.q] then
            rad[p.q] = cnr
        elseif lmatch(p.q, ':') then
            local noCurve = {}
            noCurve = w.wxsplit(p.q, ':')
            for x = 1, #noCurve do
                rad[noCurve[x]] = cnr
            end
        end
    end

    local function makeBackground(cr,p)
        local TORADIANS = p.TORADIANS
        lcairo_new_sub_path(cr)
        local px = p.px
        local py = p.py
        local cnr = p.cnr
        -- tr
        local rad = p.rad.tr
        lcairo_arc(cr,
            px + wcnr + rad,
            py + cnr  - rad,
            cnr - rad,
            -90 * TORADIANS,
              0 * TORADIANS
            )
        -- br
        local rad = p.rad.br
        lcairo_arc(cr,
            px + wcnr + rad,
            py + hcnr + rad,
            cnr - rad,
             0 * TORADIANS,
            90 * TORADIANS
            )
        -- bl
        local rad = p.rad.bl
        lcairo_arc(cr,
            px + cnr  - rad,
            py + hcnr + rad,
            cnr - rad,
             90 * TORADIANS,
            180 * TORADIANS
            )
        -- tl
        local rad = p.rad.tl
        lcairo_arc(cr,
            px + cnr - rad,
            py + cnr - rad,
            cnr - rad,
            180 * TORADIANS,
            270 * TORADIANS
            )
        lcairo_close_path(cr)
    end

    local mbg = {
        px        = px,
        py        = py,
        cnr       = cnr,
        hcnr      = hcnr,
        wcnr      = wcnr,
        TORADIANS = TORADIANS,
        rad       = rad,
    }

    if p.i then
        ---- background image
        makeBackground(cr, mbg)
        local piData = {
            cr    = cr,
            x     = px + (width  / 2),
            y     = py + (height / 2),
            file  = p.i,
            scale = p.e,
            clip  = 1,
            pan   = p.b,
        }
        w.put_image(piData)
        -- w.msgout(piData.x, piData.y, piData.file, piData.scale, piData.clip, piData.pan)
    end

    makeBackground(cr, mbg)
    lcairo_set_source_rgba(cr, to_rgba(bgcolour))
    lcairo_fill(cr)

    return ""
end

local function tableLength(T)
    local count = 0
    -- for _ in pairs(T) do count = count + 1 end
    for x,v in lpairs(T) do
        count = count + 1
        -- print(x,v)
    end
    return count
end

function w.make_bezel(p)
    local cr     = p.cr
    lcairo_restore(cr)
    local bzl    = {}
    bzl.b_rad    = p.bezel_size*0.81
    bzl.r_rad    = p.bezel_size
    bzl.bezT     = p.style             or { 2, 2, 2.5 }
    bzl.bezStyle = tonum(bzl.bezT[2])  or 2
    -- print(bzl.bezStyle,bzl.bezT[1],bzl.bezT[2],bzl.bezT[3],cr)
    bzl.thick    = tonum(bzl.bezT[3])  or 2.5
    bzl.thick    = bzl.thick * 0.01
    bzl.s_rad    = bzl.b_rad * 0.2
    bzl.i_rad    = (bzl.r_rad * 0.482) + (bzl.r_rad * bzl.thick*0.4)
    bzl.o_rad    = (bzl.r_rad * 0.482) + (bzl.r_rad * bzl.thick*0.4)*2.8
    --print(bzl.bezT,p.style)
    lcairo_set_line_width(cr, bzl.b_rad * bzl.thick)

    function bzl.outer(p)
        local cro = p.cr
        local pat
        -- lcairo_save(cro)
        pat = lcairo_pattern_create_radial(p.px-70,p.py-39,bzl.s_rad,p.px-24,p.py-24,bzl.b_rad)
        lcairo_pattern_add_color_stop_rgba(pat, 0, 1, 1, 1, 1)
        lcairo_pattern_add_color_stop_rgba(pat, 1, 0, 0, 0, 1)
        lcairo_set_source(cro, pat)

        -- lcairo_save(cro)
        if p.tilt then
            local matrix0 = cairo_matrix_t:create()
            tolua.takeownership(matrix0)
            local t1 = w.wxsplit(p.tilt,':')
            -- w.msgout(t1[1],t1[2],t1[3],t1[4],0,0)
            lcairo_matrix_init(matrix0, t1[1],t1[2],t1[3],t1[4],0,0)
            lcairo_translate(cro,p.px,p.py)
            lcairo_transform(cro,matrix0)
            lcairo_translate(cro,-p.px,-p.py)
        end
        -- lcairo_restore(cro)

        -- lcairo_restore(cro)
        lcairo_arc(cr, p.px, p.py, p.o_rad, 0, 2 * PI)
        if p.InStyle and p.InStyle ~= 99 then lcairo_fill_preserve(cro) end
        lcairo_stroke(cro)
        lcairo_pattern_destroy(pat)
        -- lcairo_save(cro)
        -- lcairo_restore(cro)
    end

    function bzl.inner(p)
        local cri = p.cr
        local pat
        -- lcairo_restore(cri)
        -- lcairo_save(cri)
        pat = lcairo_pattern_create_radial(p.px+70,p.py+39,bzl.s_rad,p.px+24,p.py+24,bzl.b_rad)
        lcairo_pattern_add_color_stop_rgba(pat, 0, 1, 1, 1, 1)
        lcairo_pattern_add_color_stop_rgba(pat, 1, 0, 0, 0, 1)
        lcairo_set_source(cri, pat)

        -- lcairo_restore(cri)
        if p.tilt and bzl.bezStyle ~= 3 and bzl.bezStyle ~= 6 then
            local matrix0 = cairo_matrix_t:create()
            tolua.takeownership(matrix0)
            local t1 = w.wxsplit(p.tilt,':')
            lcairo_matrix_init(matrix0, t1[1],t1[2],t1[3],t1[4],0,0)
            lcairo_translate(cri,p.px,p.py)
            lcairo_transform(cri,matrix0)
            lcairo_translate(cri,-p.px,-p.py)
        end
        -- lcairo_save(cri)

        lcairo_arc(cr, p.px, p.py, p.i_rad, 0, 2 * PI)
        if p.InStyle then lcairo_fill_preserve(cri) end
        lcairo_stroke(cri)
        lcairo_pattern_destroy(pat)
        -- lcairo_save(cri)
    end

    bzl.bezTable = {
        [1] = { inout = {'inner'}; Ir = bzl.i_rad; Outr = bzl.o_rad },
        [2] = { inout = {'outer'}; Ir = bzl.o_rad; Outr = bzl.i_rad },
        [3] = { inout = {'outer','inner'}; Ir = bzl.i_rad; Outr = bzl.o_rad },
        [4] = { inout = {'inner'}; Ir = bzl.i_rad; Outr = bzl.o_rad; InStyle = true },
        [5] = { inout = {'outer'}; Ir = bzl.o_rad; Outr = bzl.i_rad; InStyle = true },
        [6] = { inout = {'outer','inner'}; Ir = bzl.i_rad; Outr = bzl.o_rad; InStyle=99},
    }

    -- lcairo_restore(cr)
    for i = 1, #bzl.bezTable[bzl.bezStyle].inout
    do
        bzl[bzl.bezTable[bzl.bezStyle].inout[i]]({
            cr      = cr,
            px      = p.px,
            py      = p.py,
            o_rad   = bzl.bezTable[bzl.bezStyle].Outr,
            i_rad   = bzl.bezTable[bzl.bezStyle].Ir,
            tilt    = p.tilt,
            InStyle = bzl.bezTable[bzl.bezStyle].InStyle,
        })
    end
    lcairo_save(cr)
end

function w.instrument_sqr(p)
-- instrument_sqr by paramvir 2014
    local cr          = p.cr
    lcairo_restore(cr)
    --------------------------------------
    local wx,wy       = p.wx+3,p.wy+3
    local rout        = p.routI * 0.49
    local cnr         = 15
    local width       = rout * 2
    local main_colour = {0x404040,1}
    --print(wx,wy)
    -----------------------------------------
    -- p.tilt = '1:0:-0.3:0.9'
    local matrix0 = cairo_matrix_t:create()
    tolua.takeownership(matrix0)

    if p.tilt then
        local t1 = w.wxsplit(p.tilt,':')
        lcairo_matrix_init(matrix0, t1[1],t1[2],t1[3],t1[4],0,0)
        lcairo_translate(cr,p.wx,p.wy)
        lcairo_transform(cr,matrix0)
        lcairo_translate(cr,-p.wx,-p.wy)
    end
    lcairo_new_sub_path(cr)
    lcairo_arc(cr, wx + width - cnr, wy + cnr        , cnr, -90 * TORADIANS,   0 * TORADIANS)
    lcairo_arc(cr, wx + width - cnr, wy + width - cnr, cnr,   0 * TORADIANS,  90 * TORADIANS)
    lcairo_arc(cr, wx + cnr        , wy + width - cnr, cnr,  90 * TORADIANS, 180 * TORADIANS)
    lcairo_arc(cr, wx + cnr        , wy + cnr        , cnr, 180 * TORADIANS, 270 * TORADIANS)
    lcairo_close_path(cr)
    lcairo_new_sub_path(cr)
    lcairo_arc(cr,wx+rout,wy+rout,rout*0.933,0,PI*2)
    lcairo_set_source_rgba(cr,w.to_rgba(main_colour))
    lcairo_stroke_preserve(cr)
    lcairo_set_fill_rule(cr,CAIRO_FILL_RULE_EVEN_ODD)
    lcairo_fill(cr)
    -----------------------------------------
    ----- panel fitting background
    local bolthd  = rout * 0.15
    local xymove  = rout * 0.2
    local scrhdx  = wx + xymove
    local scrhdy  = wy + xymove
    local w       = width + wx
    local h       = width + wy
    local blthdmv = xymove*0.3
    lcairo_arc(cr,wx+xymove,wy+xymove,bolthd,TORADIANS,0)
    lcairo_set_source_rgba(cr,0.35,0.35,0.35,1)
    lcairo_fill_preserve(cr)
    lcairo_set_line_width(cr, xymove*.15)
    lcairo_set_source_rgba(cr,0.1,0.1,0.1,1)
    lcairo_stroke(cr)
    lcairo_set_source_rgba(cr,0.7,0.7,0.7,1)
    lcairo_move_to(cr, scrhdx-blthdmv, scrhdy-blthdmv)
    lcairo_line_to(cr, scrhdx+blthdmv, scrhdy+blthdmv)
    lcairo_move_to(cr, scrhdx+blthdmv, scrhdy-blthdmv)
    lcairo_line_to(cr, scrhdx-blthdmv, scrhdy+blthdmv)
    lcairo_stroke(cr)
    lcairo_arc(cr,w-xymove,wy+xymove,bolthd,TORADIANS,0)
    lcairo_set_source_rgba(cr,0.35,0.35,0.35,1)
    lcairo_fill_preserve(cr)
    lcairo_set_source_rgba(cr,0.1,0.1,0.1,1)
    lcairo_stroke(cr)
    lcairo_set_source_rgba(cr,0.7,0.7,0.7,1)
    lcairo_move_to(cr, w-xymove-blthdmv, scrhdy-blthdmv)
    lcairo_line_to(cr, w-xymove+blthdmv, scrhdy+blthdmv)
    lcairo_move_to(cr, w-xymove+blthdmv, scrhdy-blthdmv)
    lcairo_line_to(cr, w-xymove-blthdmv, scrhdy+blthdmv)
    lcairo_stroke(cr)
    lcairo_arc(cr,w-xymove,h-xymove,bolthd,TORADIANS,0)
    lcairo_set_source_rgba(cr,0.35,0.35,0.35,1)
    lcairo_fill_preserve(cr)
    lcairo_set_source_rgba(cr,0.1,0.1,0.1,1)
    lcairo_stroke(cr)
    lcairo_set_source_rgba(cr,0.7,0.7,0.7,1)
    lcairo_move_to(cr, w-xymove-blthdmv, h-xymove-blthdmv)
    lcairo_line_to(cr, w-xymove+blthdmv, h-xymove+blthdmv)
    lcairo_move_to(cr, w-xymove+blthdmv, h-xymove-blthdmv)
    lcairo_line_to(cr, w-xymove-blthdmv, h-xymove+blthdmv)
    lcairo_stroke(cr)
    lcairo_arc(cr,wx+xymove,h-xymove,bolthd,TORADIANS,0)
    lcairo_set_source_rgba(cr,0.35,0.35,0.35,1)
    lcairo_fill_preserve(cr)
    lcairo_set_source_rgba(cr,0.1,0.1,0.1,1)
    lcairo_stroke(cr)
    lcairo_set_source_rgba(cr,0.7,0.7,0.7,1)
    lcairo_move_to(cr, scrhdx-blthdmv, h-xymove-blthdmv)
    lcairo_line_to(cr, scrhdx+blthdmv, h-xymove+blthdmv)
    lcairo_move_to(cr, scrhdx+blthdmv, h-xymove-blthdmv)
    lcairo_line_to(cr, scrhdx-blthdmv, h-xymove+blthdmv)
    lcairo_stroke(cr)
    -----------------------------------------

    lcairo_save(cr)
end

function w.make_txt(p)
    -- pre-declare font, size, lcairo_select_font_face at calling function head
    local cr      = p.cr
    lcairo_restore(cr)
    local align   = p.align or "c"
    local mtxt    = w.trim({ text = p.text })
    -- local mtxt    = p.text
    local extents = p.ext or nil
    local awidth  = p.wid or nil

    local function alignText(cr,align,alignWidth)
        lcairo_text_extents(cr,mtxt,extents)
        if alignWidth then
            alignWidth = alignWidth - 30
            extents.width = -(alignWidth - extents.width)
        end
        if align == "r" then      -- make right align
            lcairo_move_to(cr,p.px-extents.width,p.py)
        elseif align == "br" then -- make right align
            lcairo_move_to(cr,p.px-extents.width,p.py+extents.height)
        elseif align == "c" then  -- make centered
            lcairo_move_to(cr,p.px-(extents.width/2),p.py)
        elseif align == "bc" then -- make centered
            lcairo_move_to(cr,p.px-(extents.width/2),p.py+extents.height)
        elseif align == "l" then  -- keep left aligned
            lcairo_move_to(cr,p.px,p.py)
        elseif align == "bl" then -- keep left aligned
            lcairo_move_to(cr,p.px,p.py+extents.height)
        end
    end

    -- alignText(cr,align)

    if p.icolour then
        lcairo_set_source_rgba(cr,w.to_rgba(p.icolour))
    elseif p.colourContrast then
        lcairo_set_source_rgba(cr,w.to_rgba_contrast(p.colourContrast))
    end

    local labelStyle, labelFont, labelSize, labelAlign
    if p.style then
        if not tonum(p.style) and lmatch(p.style,':') then
            local labelTable = {}
            labelTable = w.wxsplit(p.style,':')
            if not tonum(labelTable[1]) then
                labelFont  = labelTable[2] or 'Mono'
                labelSize  = tonum(labelTable[3])
                labelStyle = tonum(labelTable[4])
                labelAlign = labelTable[5]
            else
                labelFont  = 'Mono'
                labelSize  = tonum(labelTable[1])
                labelStyle = tonum(labelTable[2])
                -- labelAlign = 'l'
            end
            local b,i
            if     labelStyle == 1 then b = 1; i = 0
            elseif labelStyle == 2 then b = 0; i = 1
            elseif labelStyle == 3 then b = 1; i = 1
            end
            lcairo_select_font_face(cr, labelFont, i, b)
            lcairo_set_font_size(cr, labelSize)
        else
            lcairo_select_font_face(cr, 'Mono', 0, 0)
            lcairo_set_font_size(cr, tonum(p.style))
        end
    end

    if labelAlign then
        alignText(cr,labelAlign,conky_window.width)
    elseif awidth then
        alignText(cr,align,awidth)
    else
        alignText(cr,align)
    end
    -- w.msgout(mtxt)

    lcairo_show_text(cr,mtxt)
    lcairo_stroke(cr)
    lcairo_save(cr)
end

function w.make_scale_txt2(p)
    -- pre-declare font, size, lcairo_select_font_face at calling function head
    local cr = p.cr
    lcairo_restore(cr)
    local extents = p.ext or nil
    local get_points = w.get_points
    local getAngleInRadians = w.getAngleInRadians
    p.lineThickness = p.lineThickness or 0.5
    if not p.ifact then
        p.ifact=1
    end
    if p.trout then
        if p.i % p.ifact == 0 then
            local ppx2,ppy2 = get_points(p.startscale,(p.i*(p.endscale/p.maxscale)),0,p.trout)

            lcairo_text_extents(cr,p.text,extents)
            local width  = extents.width
            local height = extents.height
            if not p.squeeze then  -- do the squeeze if nil
                lcairo_move_to(cr,p.xval+ppx2-(width/2),p.yval+(ppy2/1.02)+(height/2))
            else
                lcairo_move_to(cr,p.xval+ppx2-(width/2),p.yval+ppy2+(height/2))
            end
            lcairo_show_text(cr,p.text)
            lcairo_stroke(cr)
        end
    end
    if not p.dotext then  -- output marks if nil
        if p.rout then
            local ppx1,ppy1 = w.get_points(p.startscale,(p.i*(p.endscale/p.maxscale)),0,p.rout)
            local pix1,piy1 = w.get_points(p.startscale,(p.i*(p.endscale/p.maxscale)),0,p.rin)
            lcairo_move_to(cr,p.xval+ppx1,p.yval+ppy1)
            lcairo_line_to(cr,p.xval+pix1,p.yval+piy1)
        else
            lcairo_set_line_width( cr, p.lineLength or 5 )
            local unitAngle  = p.i * ( p.endscale / p.maxscale )
            local lineOffset = p.lineThickness / (p.rin * 0.05)
            local start_arc  = unitAngle - lineOffset
            local stop_arc   = unitAngle + lineOffset
            lcairo_arc(cr, p.xval, p.yval, p.rin, getAngleInRadians(p.startscale, start_arc), getAngleInRadians(p.startscale, stop_arc))
        end
        lcairo_stroke(cr)
    end
    lcairo_save(cr)
end


function w.circlewriting(p)
    local cr = p.cr
    -- text, radi, horiz, verti, start, finish
    lcairo_restore(cr)
    local inum      = string.len(p.text)
    local deg       = ( p.finish - p.start )/( inum - 1 )
    local textcut   = lgsub(p.text, ".", "%1|")
    local texttable = w.wxsplit(textcut,"|")
    for i=1, inum do
        local txs, tys
        interval = ( TORADIANS * ( p.start + ( deg * ( i - 1 ))))
        txs      = 0 + p.radi * ( sin(interval))
        tys      = 0 - p.radi * ( cos(interval))
        -- lcairo_move_to(cr, txs + p.horiz, tys + p.verti )
        lcairo_move_to(cr, txs + p.horiz, tys + p.verti )
        lcairo_rotate(cr, interval)
        lcairo_show_text(cr, (texttable[i]))
        lcairo_stroke(cr)
        lcairo_rotate(cr, -interval)
    end
    lcairo_save(cr)
end --circlewriting

function w.choose_indicator(p)
    -- indicator functions by paramvir 2014

    local cr         = p.cr
    local to_rgba    = w.to_rgba
    local get_points = w.get_points
    local wxsplit    = w.wxsplit
    local FindLuma   = w.FindLuma
    local chin       = p.ch_indi
    local ptr        = {true}

    if p.ch_indi < 0 then
        p.scale = p.scale + 180
        chin = chin * -1
    end

    function ptr.BaseRing(pb)
        lcairo_save(cr)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_set_line_width(cr, pb.rout1*.07)
        lcairo_arc(cr, pb.px, pb.py, pb.rout1*.05, TORADIANS, 0)
        lcairo_stroke(cr)
        lcairo_restore(cr)
    end

    function ptr.make_indicator1(pb)
        lcairo_restore(cr)
        local rout1     = pb.rout1*0.78
        local px,py     = pb.px,pb.py
        local ppx1,ppy1 = get_points(pb.startval,pb.scale,10,rout1)
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,-10,rout1)
        local ppx3,ppy3 = get_points(pb.startval,pb.scale,0,(rout1*0.2))
        local ppx4,ppy4 = get_points(pb.startval,pb.scale,0,(rout1*0.98))
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_move_to(cr,px+ppx3,py+ppy3)
        lcairo_line_to(cr,px+ppx1,py+ppy1)
        lcairo_line_to(cr,px+ppx2,py+ppy2)
        lcairo_line_to(cr,px+ppx3,py+ppy3)
        lcairo_fill(cr)
        lcairo_arc(cr,px+ppx4,py+ppy4,(rout1*0.17),TORADIANS,0)
        lcairo_fill(cr)
        lcairo_save(cr)
    end ---- }}}

    function ptr.make_indicator2(pb)
        lcairo_restore(cr)
        local px,py            = pb.px,pb.py
        local center_thickness = pb.cthick or pb.rout1*0.14
        local center_dot       = center_thickness*0.6
        --------------------------------
        local ppx,ppy   = get_points(pb.startval,pb.scale,0,pb.rout1)
        local pilx,pily = get_points(pb.startval,pb.scale,-90,center_thickness)
        local pirx,piry = get_points(pb.startval,pb.scale,90,center_thickness)
        --------------------------------
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_arc(cr,px,py,center_thickness,TORADIANS,0)
        lcairo_move_to(cr,px+pilx,py+pily)
        lcairo_line_to(cr,px+ppx, py+ppy)
        lcairo_line_to(cr,px+pirx,py+piry)
        lcairo_line_to(cr,px+pilx,py+pily)
        lcairo_fill(cr)
        -----------------------------------
        lcairo_set_source_rgba(cr,0,0,0,1)
        lcairo_arc(cr,px,py,center_dot,TORADIANS,0)
        lcairo_fill(cr)
        lcairo_save(cr)
    end

    function ptr.make_indicator3(pb)
        lcairo_restore(cr)
        local px,py            = pb.px,pb.py
        local center_thickness = pb.cthick or pb.rout1*0.1
        local center_dot       = center_thickness*0.7
        --------------------------------
        local ppx,ppy   = get_points(pb.startval,pb.scale,0,pb.rout1,1)
        local ppox,ppoy = get_points(pb.startval,pb.scale,180,pb.rout1,1)
        --------------------------------
        local pilx,pily = get_points(pb.startval,pb.scale,-90,center_thickness)
        local pirx,piry = get_points(pb.startval,pb.scale,90,center_thickness)
        --white------------------------------
        lcairo_set_line_width(cr,pb.rout1*0.004)
        lcairo_move_to(cr,px+pilx,py+pily)
        lcairo_line_to(cr,px+ppx,py+ppy)
        lcairo_line_to(cr,px+pirx,py+piry)
        lcairo_set_source_rgba(cr,1,1,1,1)
        lcairo_fill_preserve(cr)
        lcairo_set_source_rgba(cr,0,0,0,1)
        lcairo_stroke(cr)
        --red------------------------------
        lcairo_move_to(cr,px+pirx,py+piry)
        lcairo_line_to(cr,px+ppox,py+ppoy)
        lcairo_line_to(cr,px+pilx,py+pily)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_fill_preserve(cr)
        lcairo_set_source_rgba(cr,0,0,0,1)
        lcairo_stroke(cr)
        -- black dot
        lcairo_arc(cr,px,py,center_dot,TORADIANS,0)
        --lcairo_set_source_rgba(cr,0,0,0,1)
        lcairo_fill(cr)
        lcairo_save(cr)
    end

    function ptr.make_indicator4(pb)
        lcairo_restore(cr)
        local px,py            = pb.px,pb.py
        local center_thickness = pb.cthick or pb.rout1*0.04
        --------------------------------
        local ppx,ppy   = get_points(pb.startval,pb.scale,0,   pb.rout1)
        local ppx1,ppy1 = get_points(pb.startval,pb.scale,180,(pb.rout1*0.5))
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,-7, (pb.rout1*0.81))
        local ppx3,ppy3 = get_points(pb.startval,pb.scale,7,  (pb.rout1*0.81))
        --------------------------------
        lcairo_set_line_width(cr,center_thickness)
        lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_move_to(cr,px+ppx,py+ppy)
        lcairo_line_to(cr,px+ppx1,py+ppy1)
        lcairo_move_to(cr,px+ppx2,py+ppy2)
        lcairo_line_to(cr,px+ppx3,py+ppy3)
        lcairo_stroke(cr)
        local ppx4,ppy4=get_points(pb.startval,pb.scale,180,(pb.rout1*0.65))
        local arc1=TORADIANS*(pb.startval+pb.scale-150)
        local arc2=TORADIANS*(pb.startval+pb.scale-30)
        lcairo_arc(cr,px+ppx4,py+ppy4,(pb.rout1*0.14705882),arc1,arc2)
        lcairo_set_line_width(cr,center_thickness)
        lcairo_stroke(cr)
        -----------------------------------
        lcairo_set_source_rgba(cr,1,1,0,1)
        lcairo_set_line_width(cr,center_thickness) -- center dot
        lcairo_arc(cr,px,py,center_thickness,TORADIANS,0)
        lcairo_stroke(cr)
        lcairo_save(cr)
    end

    function ptr.make_indicator5(pb)
        lcairo_restore(cr)
        local px,py            = pb.px,pb.py
        local center_thickness = pb.cthick or pb.rout1*0.04
        --------------------------------
        local ppx,ppy   = get_points(pb.startval,pb.scale,0,pb.rout1)
        local ppox,ppoy = get_points(pb.startval,pb.scale,180,(pb.rout1*0.8))
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,-10,(pb.rout1*0.8))
        local ppx3,ppy3 = get_points(pb.startval,pb.scale,10,(pb.rout1*0.8))
        --------------------------------
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_set_line_width(cr,center_thickness)
        lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
        lcairo_move_to(cr,px+ppox,py+ppoy)
        lcairo_line_to(cr,px+ppx,py+ppy)
        lcairo_line_to(cr,px+ppx3,py+ppy3)
        lcairo_line_to(cr,px+ppx2,py+ppy2)
        lcairo_line_to(cr,px+ppx,py+ppy)
        lcairo_stroke(cr)
        lcairo_set_line_width(cr,center_thickness)
        local ppox1,ppoy1 = get_points(pb.startval,pb.scale,180,(pb.rout1*0.65))
        local arc1        = TORADIANS*(pb.startval+pb.scale-150)
        local arc2        = TORADIANS*(pb.startval+pb.scale-30)
        lcairo_arc(cr,px+ppox1,py+ppoy1,(pb.rout1*0.14705882),arc1,arc2)
        lcairo_stroke(cr)
        lcairo_arc(cr,px+ppox,py+ppoy,(pb.rout1*0.14705882),arc1,arc2)
        lcairo_stroke(cr)
        -----------------------------------
        lcairo_set_source_rgba(cr,1,1,0,1)
        lcairo_set_line_width(cr,center_thickness)
        lcairo_arc(cr,px,py,center_thickness,TORADIANS,0)
        lcairo_stroke(cr)
        lcairo_save(cr)
    end

    function ptr.make_indicator6(pb)
        lcairo_restore(cr)
        local px, py           = pb.px, pb.py
        local center_thickness = pb.cthick or pb.rout1*0.1
        local center_dot       = pb.rout1 * 0.10
        if pb.chands == 1 then
            center_dot = pb.rout1 * 0.08
        end
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        local ppx1,ppy1 = get_points(pb.startval,pb.scale,0,pb.rout1)
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,180,(pb.rout1*0.30))
        ------------------------------
        lcairo_set_line_width(cr, center_thickness)
        lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
        lcairo_move_to(cr,px+ppx1,py+ppy1)
        lcairo_line_to(cr,px+ppx2,py+ppy2)
        lcairo_stroke(cr)
        -----------------------------------
        lcairo_arc(cr,px,py, center_dot, TORADIANS, 0)
        lcairo_set_source_rgba(cr,1,1,0,1)
        lcairo_fill_preserve(cr)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_set_line_width(cr, center_dot*0.5)
        lcairo_stroke(cr)
        lcairo_save(cr)
    end

    function ptr.make_indicator7(pb)
        lcairo_restore(cr)
        local sfact            = 1
        local rout1            = pb.rout1*sfact
        local scale            = pb.scale -- end angle, pb.startval is start angle value
        local center_thickness = pb.cthick or rout1*0.1

        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        local ppx,ppy   = get_points(pb.startval, scale, 0, rout1)
        local ppox,ppoy = get_points(pb.startval, scale, 0, (rout1*(pb.length or 0.5)))
        --------------------------------
        lcairo_set_line_width(cr, center_thickness)
        lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
        lcairo_move_to(cr,pb.px+ppx,pb.py+ppy)
        lcairo_line_to(cr,pb.px+ppox,pb.py+ppoy)
        lcairo_stroke(cr)
        lcairo_save(cr)
    end

    function ptr.make_indicator8(pb)
        lcairo_restore(cr)
        local px,py            = pb.px,pb.py
        local center_thickness = pb.cthick or pb.rout1*0.05
        local center_dot       = pb.rout1*0.1
        if pb.chands then center_dot = pb.rout1*0.08 end
        --------------------------------
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        local ppx0,ppy0 = get_points(pb.startval,pb.scale,0,pb.rout1*0.82)
        local ppx1,ppy1 = get_points(pb.startval,pb.scale,0,pb.rout1)
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,-7.5,(pb.rout1*0.81))
        local ppx3,ppy3 = get_points(pb.startval,pb.scale,7.5,(pb.rout1*0.81))
        --------------------------------
        lcairo_set_line_width(cr,center_thickness)
        lcairo_move_to(cr,px,py)
        lcairo_line_to(cr,px+ppx0,py+ppy0)
        lcairo_stroke_preserve(cr)
        lcairo_move_to(cr,px+ppx1,py+ppy1)
        lcairo_line_to(cr,px+ppx3,py+ppy3)
        lcairo_line_to(cr,px+ppx2,py+ppy2)
        lcairo_line_to(cr,px+ppx1,py+ppy1)
        lcairo_fill(cr)
        -----------------------------------
        lcairo_arc(cr,px,py,center_dot,TORADIANS,0)
        lcairo_set_source_rgba(cr,1,1,0,1)
        lcairo_fill_preserve(cr)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_set_line_width(cr,center_dot*0.3) -- center dot
        lcairo_stroke(cr)
        lcairo_save(cr)
    end

    function ptr.make_indicator9(pb)
        lcairo_restore(cr)
        local px,py            = pb.px,pb.py
        local center_thickness = pb.cthick or pb.rout1*0.05
        --------------------------------
        local ppx0,ppy0 = get_points(pb.startval,pb.scale,0,pb.rout1,1)
        local ppx1,ppy1 = get_points(pb.startval,pb.scale,180,(pb.rout1*-0.39),1)
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,-7.8,(pb.rout1*0.81),1)
        local ppx3,ppy3 = get_points(pb.startval,pb.scale,7.8,(pb.rout1*0.81),1)
        --------------------------------
        lcairo_set_line_width(cr,center_thickness)
        lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND)
        lcairo_move_to(cr,px+ppx0,py+ppy0)
        lcairo_line_to(cr,px+ppx1,py+ppy1)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_stroke(cr)
        lcairo_move_to(cr,px+ppx2,py+ppy2)
        lcairo_line_to(cr,px+ppx1,py+ppy1)
        lcairo_line_to(cr,px+ppx3,py+ppy3)
        lcairo_fill(cr)
        -----------------------------------
        lcairo_save(cr)
    end

    function ptr.make_indicator10(pb)
        lcairo_restore(cr)
        local px,py            = pb.px,pb.py
        local center_thickness = pb.cthick or pb.rout1*0.05
        --------------------------------
        local ppx0, ppy0 = get_points(pb.startval,pb.scale,0,pb.rout1,1)
        local ppx1, ppy1 = get_points(pb.startval,pb.scale,180,(pb.rout1*-1.1),1)
        local ppix, ppiy = get_points(pb.startval,pb.scale,180,(pb.rout1*-0.5),1)
        local ppx2, ppy2 = get_points(pb.startval,pb.scale,-9.9,(pb.rout1*0.78),1)
        local ppx3, ppy3 = get_points(pb.startval,pb.scale,9.9,(pb.rout1*0.78),1)
        --------------------------------
        lcairo_set_line_width(cr,center_thickness)
        lcairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
        lcairo_move_to(cr,px+ppx0,py+ppy0)
        lcairo_line_to(cr,px+ppx1,py+ppy1)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_stroke(cr)
        lcairo_move_to(cr,px+ppx2,py+ppy2)
        lcairo_line_to(cr,px+ppx3,py+ppy3)
        lcairo_line_to(cr,px+ppix,py+ppiy)
        lcairo_fill(cr)
        -----------------------------------
        lcairo_save(cr)
    end

    function ptr.make_indicator11(pb)
        lcairo_restore(cr)
        local fileN = tconcat({pb.ip,'/lua_windvane/arrow-',pb.arrow,'.png'})
        w.put_image({
            cr          = cr,
            x           = pb.px,
            y           = pb.py,
            file        = fileN,
            scale       = pb.rout1*0.02,
            theta       = (pb.scale+90) * TORADIANS,
            dial_colour = pb.icolour,
            rotate      = true,
            })
        -----------------------------------
        lcairo_save(cr)
    end ---- }}}

    function ptr.make_indicator50(pb) ---- {{{
        lcairo_restore(cr)
        local px,py     = pb.px,pb.py
        local ppx1,ppy1 = get_points(pb.startval,pb.scale, 5,pb.rout1*0.66)
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,-5,pb.rout1*0.66)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_move_to(cr,px,py)
        lcairo_line_to(cr,px+ppx1,py+ppy1)
        lcairo_line_to(cr,px+ppx2,py+ppy2)
        lcairo_close_path(cr)
        local ppx4,ppy4 = get_points(pb.startval,pb.scale,0,pb.rout1*0.90)
        local ppx5,ppy5 = get_points(pb.startval,pb.scale,-10,pb.rout1*0.72)
        local ppx6,ppy6 = get_points(pb.startval,pb.scale,10,pb.rout1*0.72)
        lcairo_move_to(cr,px+ppx4,py+ppy4)
        lcairo_line_to(cr,px+ppx5,py+ppy5)
        lcairo_line_to(cr,px+ppx6,py+ppy6)
        lcairo_close_path(cr)
        lcairo_fill(cr)
        ptr.BaseRing({ px=px, py=py, rout1=pb.rout1, icolour=pb.icolour })
        lcairo_save(cr)
    end

    function ptr.make_indicator51(pb)
        lcairo_restore(cr)
        local px,py     = pb.px,pb.py
        local ppx1,ppy1 = get_points(pb.startval,pb.scale,5,pb.rout1*0.66)
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,-5,pb.rout1*0.66)
        lcairo_set_line_width(cr, 3)
        lcairo_set_source_rgba(cr, to_rgba({pb.arrow_in_colour[1],0.36}))
        lcairo_move_to(cr,px,py)
        lcairo_line_to(cr,px+ppx1,py+ppy1)
        lcairo_line_to(cr,px+ppx2,py+ppy2)
        lcairo_close_path(cr)
        lcairo_fill_preserve(cr)
        lcairo_set_source_rgba(cr,to_rgba({pb.arrow_in_colour[1],1}))
        lcairo_stroke(cr)
        local ppx4,ppy4 = get_points(pb.startval,pb.scale,0,pb.rout1*0.93)
        local ppx5,ppy5 = get_points(pb.startval,pb.scale,-10,pb.rout1*0.75)
        local ppx6,ppy6 = get_points(pb.startval,pb.scale,10,pb.rout1*0.75)
        lcairo_set_source_rgba(cr,to_rgba({pb.arrow_in_colour[1],1}))
        lcairo_move_to(cr,px+ppx4,py+ppy4)
        lcairo_line_to(cr,px+ppx5,py+ppy5)
        lcairo_line_to(cr,px+ppx6,py+ppy6)
        lcairo_close_path(cr)
        lcairo_fill_preserve(cr)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_stroke(cr)
        ptr.BaseRing({ px=px, py=py, rout1=pb.rout1, icolour=pb.icolour })
        lcairo_save(cr)
    end

    function ptr.make_indicator52(pb)
        lcairo_restore(cr)
        local px,py   = pb.px,pb.py
        local mpoint  = 1
        local f2point
        if pb.intype then f2point = pb.rout1*0.45 else f2point = pb.rout1*0.27 end
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,mpoint,pb.rout1*0.54)
        local ppx3,ppy3 = get_points(pb.startval,pb.scale,f2point,(pb.rout1*0.36))
        local ppx4,ppy4 = get_points(pb.startval,pb.scale,mpoint,(pb.rout1*0.18))
        local ppx9,ppy9 = get_points(pb.startval,pb.scale,0,(pb.rout1*0.936))
        lcairo_set_source_rgba(cr,to_rgba(pb.arrow_in_colour))
        lcairo_curve_to(cr, px+ppx9, py+ppy9, px+ppx2, py+ppy2, px+ppx3, py+ppy3)
        lcairo_curve_to(cr, px+ppx3, py+ppy3, px+ppx4, py+ppy4, px, py)
        lcairo_close_path(cr)
        lcairo_fill_preserve(cr)
        lcairo_set_line_width(cr, 3)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_stroke(cr)
        ptr.BaseRing({ px=px, py=py, rout1=pb.rout1, icolour=pb.icolour })
        lcairo_save(cr)
    end

    function ptr.make_indicator53(pb)
        lcairo_restore(cr)
        local px,py = pb.px,pb.py
        local fpoint
        if pb.intype then fpoint = pb.rout1*0.3 else fpoint = pb.rout1*0.2 end
        local mpoint = -0.5
        local ppx1,ppy1 = get_points(pb.startval,pb.scale,-mpoint,(pb.rout1*0.18))
        local ppx2,ppy2 = get_points(pb.startval,pb.scale,-fpoint,pb.rout1*0.36)
        local ppx3,ppy3 = get_points(pb.startval,pb.scale,-mpoint,pb.rout1*0.54)
        local ppx9,ppy9 = get_points(pb.startval,pb.scale,0,(pb.rout1*0.936))
        local ppx4,ppy4 = get_points(pb.startval,pb.scale,mpoint,pb.rout1*0.54)
        local ppx5,ppy5 = get_points(pb.startval,pb.scale,fpoint,(pb.rout1*0.36))
        local ppx6,ppy6 = get_points(pb.startval,pb.scale,mpoint,(pb.rout1*0.18))
        lcairo_set_source_rgba(cr,to_rgba(pb.arrow_in_colour))
        lcairo_curve_to(cr, px,      py,      px+ppx1, py+ppy1, px+ppx2, py+ppy2)
        lcairo_curve_to(cr, px+ppx2, py+ppy2, px+ppx3, py+ppy3, px+ppx9, py+ppy9)
        lcairo_curve_to(cr, px+ppx9, py+ppy9, px+ppx4, py+ppy4, px+ppx5, py+ppy5)
        lcairo_curve_to(cr, px+ppx5, py+ppy5, px+ppx6, py+ppy6, px, py)
        lcairo_close_path(cr)
        lcairo_fill_preserve(cr)
        lcairo_set_line_width(cr, 3)
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_stroke(cr)
        ptr.BaseRing({ px=px, py=py, rout1=pb.rout1, icolour=pb.icolour })
        lcairo_save(cr)
    end

    function ptr.make_indicator54(pb)
        lcairo_restore(cr)
        local rout1 = pb.rout1*0.75
        local px,py = pb.px,pb.py
        local ppx1,ppy1   = get_points(pb.startval,pb.scale,0,(rout1*0.17))
        local ppx4,ppy4   = get_points(pb.startval,pb.scale,18,rout1*0.33)
        local ppx5,ppy5   = get_points(pb.startval,pb.scale,-18,rout1*0.33)

        local ppx12,ppy12 = get_points(pb.startval,pb.scale,0,rout1*0.68)
        local ppx13,ppy13 = get_points(pb.startval,pb.scale,0,rout1*0.75)
        local ppx14,ppy14 = get_points(pb.startval,pb.scale,-8,rout1*0.72)
        local ppx15,ppy15 = get_points(pb.startval,pb.scale,0,rout1*0.80)
        local ppx16,ppy16 = get_points(pb.startval,pb.scale,8,rout1*0.72)
        local ppx20,ppy20 = get_points(pb.startval,pb.scale,0,rout1*1.3)

        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        local LineThick = rout1*.036
        lcairo_set_line_width(cr, LineThick)
        local rad1 = rout1*.10
        local rad2 = rout1*.036
        lcairo_arc(cr, px+ppx1, py+ppy1, rad1*.7, 0, 2*PI)
        lcairo_stroke(cr)
        lcairo_arc(cr, px+ppx4, py+ppy4, rad1, PI+(TORADIANS*pb.scale), PI*2+(TORADIANS*pb.scale))
        lcairo_stroke(cr)
        lcairo_arc_negative(cr, px+ppx5, py+ppy5, rad1, PI+(TORADIANS*pb.scale), PI*2+(TORADIANS*pb.scale))
        lcairo_stroke(cr)
        if pb.intype then
            local ppx2,ppy2   = get_points(pb.startval,pb.scale,-12,rout1*0.53)
            local ppx3,ppy3   = get_points(pb.startval,pb.scale,12,rout1*0.53)
            local ppx6,ppy6   = get_points(pb.startval,pb.scale,20,rout1*0.42)
            local ppx7,ppy7   = get_points(pb.startval,pb.scale,15,rout1*0.47)
            local ppx8,ppy8   = get_points(pb.startval,pb.scale,-20,rout1*0.42)
            local ppx9,ppy9   = get_points(pb.startval,pb.scale,-15,rout1*0.47)
            local ppx10,ppy10 = get_points(pb.startval,pb.scale,-23,rout1*0.6)
            local ppx11,ppy11 = get_points(pb.startval,pb.scale,23,rout1*0.6)
            lcairo_arc(cr, px+ppx6,  py+ppy6,  rad2, 0, 2*PI)
            lcairo_arc(cr, px+ppx7,  py+ppy7,  rad2, 0, 2*PI)
            lcairo_fill(cr)
            lcairo_arc(cr, px+ppx8,  py+ppy8,  rad2, 0, 2*PI)
            lcairo_arc(cr, px+ppx9,  py+ppy9,  rad2, 0, 2*PI)
            lcairo_fill(cr)
            lcairo_arc(cr, px+ppx10, py+ppy10, rad2, 0, 2*PI)
            lcairo_fill(cr)
            lcairo_arc(cr, px+ppx11, py+ppy11, rad2, 0, 2*PI)
            lcairo_fill(cr)
            lcairo_arc(cr, px+ppx2, py+ppy2, rad1, 0, 2*PI)
            lcairo_stroke(cr)
            lcairo_arc(cr, px+ppx3, py+ppy3, rad1, 0, 2*PI)
            lcairo_stroke(cr)
            lcairo_arc(cr, px+ppx12, py+ppy12, rad1, 0, 2*PI)
            lcairo_stroke(cr)
        else
            local ppx12a,ppy12a=get_points(pb.startval,pb.scale,0,rout1*0.50)
            lcairo_arc(cr, px+ppx12a, py+ppy12a, rad1*0.78, 0, 2*PI)
            lcairo_stroke(cr)
            lcairo_arc(cr, px+ppx12, py+ppy12, rad1*0.76, 0, 2*PI)
            lcairo_stroke(cr)
        end
        lcairo_arc(cr, px+ppx13, py+ppy13, rad2, 0, 2*PI)
        lcairo_fill(cr)

        lcairo_curve_to(cr, px+ppx14, py+ppy14, px+ppx15, py+ppy15, px+ppx20, py+ppy20)
        lcairo_curve_to(cr, px+ppx20, py+ppy20, px+ppx15, py+ppy15, px+ppx16, py+ppy16)
        lcairo_stroke(cr)
        ptr.BaseRing({ px=px, py=py, rout1=rout1, icolour=pb.icolour })
        lcairo_save(cr)
    end

    function ptr.make_indicator99(pb)
        lcairo_restore(cr)
        local px,py = pb.px,pb.py
        local bcolour, luma
        if lmatch(pb.bcolour, ":") then bcolour = wxsplit(pb.bcolour,":") end
        if FindLuma(bcolour) then luma = 0.5 else luma = 0.7 end
        local ppx1,ppy1      = get_points(pb.startval,pb.scale,-25,pb.rout1*0.9)
        local b_rad          = pb.rout1*2
        local s_rad          = b_rad*0
        local sangle, eangle = 30, 90
        local pat            = lcairo_pattern_create_radial(px+ppx1,py+ppy1,s_rad,px+ppx1,py+ppy1,b_rad)
        lcairo_pattern_add_color_stop_rgba(pat, 0, to_rgba({pb.icolour[1],1}))
        lcairo_pattern_add_color_stop_rgba(pat, luma, to_rgba({bcolour[1],0}))
        lcairo_set_source(cr, pat)
        local ppx2,ppy2      = get_points(0,0,0,pb.rout1)
        lcairo_line_to(cr,px+ppx2,py+ppy2)
        local RingNumbers    = 1/tonum(pb.Rings)
        for i=0,pb.rout1,pb.rout1*RingNumbers do
            lcairo_set_line_width(cr, 2)
            local angleA = PI+(TORADIANS*pb.scale)-(TORADIANS*sangle)
            local angleB = PI*2+(TORADIANS*pb.scale)-(TORADIANS*eangle)
            lcairo_arc(cr, px, py, i, angleA, angleB)
            lcairo_line_to(cr,px,py)
            lcairo_stroke(cr)
        end
        lcairo_set_source_rgba(cr,to_rgba(pb.icolour))
        lcairo_set_line_width(cr, 0.3)
        for i=0,pb.rout1,pb.rout1*RingNumbers do
            lcairo_arc(cr, px, py, i, 0+(TORADIANS*-90), PI*2+(TORADIANS*-90))
        end
        lcairo_stroke(cr)
        lcairo_pattern_destroy(pat)
        lcairo_save(cr)
    end

    if chin then
        ptr[tconcat({'make_indicator', chin})](p)
    else
        ptr.make_indicator3(p)
    end
end

lcollectgarbage()

return w
