package metar;

=comments
    Parses METAR
    This version (c) 2018 Paramvir Likhari
=cut

$metar::VERSION = '7';

require Exporter;
@metar::ISA = qw(Exporter);
@metar::EXPORT = qw(metarParser); # function to export

use strict;
use warnings;

# sub msgout  { print STDERR "@_ \n" }


## any reference to functions in the main program can be called like main::calltofunction()

sub metar_sky {

    my ($inval, $otr1) = @_ ;
    # - Light
    # + Heavy
    my %msky = (
        BC     => "Patches",          BL  => "Blowing",       BR   => "Mist",
        DR     => "Low Drifting",     DS  => "Duststorm",     DU   => "Dust",
        DZ     => "Drizzle",          FC  => "Funnel Cloud",  FG   => "Fog",
        FU     => "Smoke",            FZ  => "Freezing",      GR   => "Hail",
        GS     => "Snow Pellets",     HZ  => "Haze",          IC   => "Ice Crystals",
        MI     => "Shallow",          NA  => "NA",            PL   => "Ice Pellets",
        PO     => "Dust/Sand Whirls", PR  => "Partial",       PY   => "Spray",
        RA     => "Rain",             SA  => "Sand",          SG   => "Snow Grains",
        SH     => "Showers",          SN  => "Snow",          SQ   => "Squalls",
        SS     => "Sandstorm",        TS  => "Thunderstorm",  UP   => "Unknown Precipitation",
        VA     => "Volcanic Ash",     VC  => "Nearby",        OVC  => "Overcast",
        BKN    => "Mostly Cloudy",    SCT => "Partly Cloudy", FEW  => "Clear",
        CLR    => "Clear",            SKC => "Clear",         CAVU => "Clear",
        CAVOK  => "Clear",
    );

    $otr1  ||= 0  ;
    if ( $inval eq '' ) { return $inval }

    if ( $otr1 == 1 ) {
        return( $msky{$inval} ? $msky{$inval} : "Unknown :(" );
    }
    else {
        my ( @metarr, @mwxcond, $mwxclen ) ;
        $mwxclen = length($inval);

        my $intensity = substr( $inval, 0, 1 ) ;
        if    ( $intensity eq '-' ) { $intensity = "Light";  $mwxclen -= 1  }
        elsif ( $intensity eq '+' ) { $intensity = "Heavy";  $mwxclen -= 1  }
        else                        { $intensity = ""                       }

        if ( $inval =~ /\s+/ ) {
            $inval = substr( $inval, -${mwxclen} ) ;
            @metarr = split(/\s+/, $inval) ;
            for (@metarr) { push( @mwxcond , $msky{$_} ) }
        }
        elsif ( $mwxclen > 2 ) {
            # $inval =~ tr/-//d;
            # $inval =~ tr/+//d;
            $inval =~ tr/\x2b\x2d//d;
            @metarr = unpack '(a2)*', $inval;
            for (@metarr) { push( @mwxcond , $msky{$_} ) if (length($_) == 2) }
        }
        else {
            $inval = substr( $inval, -${mwxclen} ) ;
            push( @mwxcond , $msky{$inval} )
        }

        my $mxr = join( " ", @mwxcond ) ;

        if ( $intensity ) { return $intensity. " " . $mxr ; }
        else { return $mxr ; }
    }
}

sub metarParser {

    # my ($x, $y) = @_ ;
    my ($cwx_magellan, $temppath, $x, $y, %wx ) = @_ ;

    my $metarwx = `$cwx_magellan --metarwx "$x"`;

    my @base_wx_list = qw{city name nameid latitude longitude elevation updated icon condition temperature dewpoint humidity pressure precip_today visibility snow uv_index pollen metar wind_speed wind_dir wind_dir_degrees wind_gust_speed cloud1h cloud1t cloud2h cloud2t cloud3h cloud3t cloud4h cloud4t};

    for ( split( /\n/, $metarwx )) {
        next if (/^\s+$/);
        next if (/^#.*$/);
        my ($var, $val) = split( /\s+:\s+/, $_, 2);
        $wx{$var} = $val if($var);
    }
    $wx{wind_gust_speed} = ($wx{wind_gust_speed}) ? $wx{wind_gust_speed} : 0;

    if ( $wx{temperature} and $wx{dewpoint} ) {
        $wx{humidity} = int(100*((112-(0.1*$wx{temperature})+$wx{dewpoint}) / (112+(0.9*$wx{temperature})))**8)
    }
    else { $wx{humidity} = 'NA' }

    $wx{wind_dir} = ($wx{wind_dir_var})
    ? "Var"
    : ($wx{wind_dir_degrees})
    ? main::find_wind_dir($wx{wind_dir_degrees})
    : 'NA';

    $wx{condition} = ($wx{condition})
    ? metar_sky($wx{condition})
    : ($wx{cloud4t})
    ? metar_sky($wx{cloud4t}, 1 )
    : ($wx{cloud3t})
    ? metar_sky($wx{cloud3t}, 1 )
    : ($wx{cloud2t})
    ? metar_sky($wx{cloud2t}, 1 )
    : ($wx{cloud1t})
    ? metar_sky($wx{cloud1t}, 1 )
    : "Clear" ;

    $wx{city} =~ s/(\w+)/\u\L$1/g;
    $wx{city} = $wx{city} . "$y Wx";
    $wx{name} ||= $wx{city};

    for (@base_wx_list) { if (! $wx{$_}) { $wx{$_} = 'NA'}  }

    return %wx
}


1;
