
--[[
    Paramvir Likhari - conkywx 2011 - 2017 - GPL3
    music sort library for use with conkywx lua implementation
--]]

local w = {true}

local Global_Font_Size
local tconcat = table.concat
local sprintf = string.format
local slower  = string.lower
local lmatch  = string.match
local tonum   = tonumber
local ssub    = string.sub
local exec    = os.execute
local floor   = math.floor

local Yellow  = 0xFFFF00
local Lime    = 0x00FF00

local lcairo_text_extents    = cairo_text_extents
local lcairo_save            = cairo_save
local lcairo_set_source_rgba = cairo_set_source_rgba

-- package.path = package.path .. ";./lib/?.lua;"

local lcollectgarbage = collectgarbage

local GF = require("bargraph")
local bargraph = GF.bargraph

local SC = require("scroller")
local scroller = SC.scroller

local FC = require("luafunc")
local wxsplit         = FC.wxsplit
local to_rgba         = FC.to_rgba
local do_set_font     = FC.set_font
local make_txt        = FC.make_txt
local FileExists      = FC.FileExists
local DirExists       = FC.DirExists
local ExecInfo        = FC.ExecInfo
local firstToUpper    = FC.firstToUpper
local strip_ext_codes = FC.strip_ext_codes
local msgout          = FC.msgout


local function set_font(p)
    if not Global_Font_Size then
        Global_Font_Size = do_set_font(p) or 12
    end
    do_set_font(p)
end

local werdone          -- required for mplayer stuff
local shfl_loop_label  -- required for mplayer stuff
local loop_times       -- required for mplayer stuff
local lr = lr or 0     -- required for mplayer stuff
local ctrack
local tpt
local total_lines
local curr_aud

local function wxsprintf (format,str)
    if format == 'time' then
        if str and tonum(str) then str = tonum(str) end
        if str then
            return(sprintf("%02d:%02d:%02d",floor(str/3600),floor((str/60)%60),floor(str%60)))
        else
            return('00:00:00')
        end
    elseif format and format:match('[sq]') and not tonum(str) then
        if str then
            return(sprintf(format,str))
        else
            return('')
        end
    elseif format then
        if str then
            if format:match('[diu]') then str = floor(str) end
            return(sprintf(format,str))
        else
            return(0)
        end
    end
end

function w.music_sort(p)


    local audnow = {}
    local cr = p.cr
    local player     = slower(p.m or 'noplayer')
    local time_to_go = tonum(p.t or 0)


    local mdata

    p.c = p.c or Yellow .. ':1' -- for scroller call
    local aud_tt = 0
    if player ~= 'mplayer' then
        ctrack          = nil
        werdone         = nil
        shfl_loop_label = nil
        loop_times      = nil
        lr              = nil
    elseif player ~= 'audacious' then
        curr_aud        = nil
    end

    if p.d and p.d ~= '0' then p.d = 1 else p.d = nil end

    local function getVolume()
        local pvt = { '\"Master\"', '\"Master Surround\"' }
        local pvol
        for p = 1,#pvt do
            pvol= ExecInfo({ cmd = 'amixer get '.. pvt[p] })
            if not pvol:match('Unable') then break end
        end
        local so = { on = true, off = true }
        local s,p
        if pvol then
            s,p = pvol:match( "Mono: Playback.*%[(.-)%%%] %[.*%] %[(.-)%]" )
            if not s then
                s,p = pvol:match( "Front Left: Playback.*%[(.-)%%%] %[(.-)%]" )
            end
            if not so[p] then
                s,p = pvol:match( "Front Left: Playback.*%[(.-)%%%] %[.*%] %[(.-)%]" )
            end
        end
        return s, p or 0, 0
    end

    p.f = p.f or '12'

    if player == 'audacious' then
        conky_set_update_interval(1)
        audnow = {
            -- c_min = "audtool --current-song-output-length",
            -- c_sec = "audtool --current-song-output-length-seconds",
            -- c_alb = "audtool --current-song-tuple-data album",
            -- c_art = "audtool --current-song-tuple-data artist",
            -- c_ttl = "audtool --current-song-tuple-data title",
            c_alb = nil,
            c_art = nil,
            c_ttl = nil,
            c_tta = "audtool --current-song",
            c_kbt = "audtool --current-song-bitrate-kbps",
            c_khz = "audtool --current-song-frequency-khz",
            c_chl = "audtool --current-song-channels",
            t_min = "audtool --current-song-length",
            t_sec = "audtool --current-song-length-seconds",
            t_fnm = "audtool --current-song-filename",
            p_ttl = "audtool --playlist-length",
            p_pos = "audtool --playlist-position",
            -- astat = "audtool --playback-status",
            g_vol = "audtool --get-volume",
        }

        local AudStatus = ExecInfo({ cmd = 'audtool --playback-status' })
        if AudStatus ~= 'stopped' then
            local dur = tonum(ExecInfo({ cmd = 'audtool --current-song-output-length-seconds' }))
            local astat = ExecInfo({ cmd = 'audtool --playback-status' })
            local t,count = {},1
            w.trackTime = dur
            if dur <= 1 then
                for c,d in pairs(audnow) do
                    if d then
                        audnow[c] = ExecInfo({ cmd = d })
                        if tonum(audnow[c]) then
                            audnow[c] = tonum(audnow[c])
                        end
                        if c == 'c_tta' then
                            local c_art, c_alb, c_ttl = lmatch(audnow[c],'^(.-)%s*%-%s*(.-)%s*%-%s*(.-)$')
                            t[count]     = 'c_art=' .. c_art
                            t[count + 1] = 'c_alb=' .. c_alb
                            t[count + 2] = 'c_ttl=' .. c_ttl
                            count = count + 2
                        else
                            t[count] = c .. '=' .. audnow[c]
                        end
                    end
                    count = count + 1
                end
                ExecInfo({ wfile = '/tmp/audData', wtext = t})
            else
                local inFile = ExecInfo({ rfile = '/tmp/audData' })
                local t11 = wxsplit(inFile, '\n')
                if not t11[1] then
                    t11 = {
                        p_pos=1,
                        c_ttl='NA',
                        c_khz=44,
                        c_art='audio',
                        t_min='00:00',
                        g_vol=1,
                        c_alb='audio',
                        t_sec=1,
                        c_kbt=128,
                        c_chl=2,
                        p_ttl=1,
                        t_fnm='NA'
                    }
                end
                for i = 1,#t11 do
                    local l,r = t11[i]:match('^(.-)%=(.-)$')
                    if tonum(r) then
                        audnow[l] = tonum(r)
                    else
                        audnow[l] = r
                    end
                end
            end
            audnow.c_sec = dur
            audnow.astat = astat

            local fname
            if audnow.t_fnm then
                fname = audnow.t_fnm:match('.*/(.-)%..*$') or audnow.t_fnm
                audnow.t_fnm = 'File name: ' .. fname
            end

            if audnow.c_ttl and fname then
                audnow.c_ttl = audnow.c_ttl:match('(.-)%..*$') or audnow.c_ttl
                if audnow.c_ttl ~= fname then
                    audnow.c_ttl = 'Title: ' .. audnow.c_ttl
                else
                    audnow.c_ttl = 'File name: ' .. audnow.c_ttl
                end
            end

            if audnow.p_ttl == 1 then
                audnow.aud_tt = audnow.t_sec
            end

            if not audnow.g_vol or audnow.g_vol == '0' then audnow.g_volmain = 'Mute'
            elseif audnow.g_vol then audnow.g_volmain = audnow.g_vol
            end
            audnow.progress = (audnow.c_sec/audnow.t_sec)*100
            if time_to_go == 1 then
                audnow.c_sec = audnow.t_sec - audnow.c_sec
            end
            audnow.c_min = wxsprintf('time',audnow.c_sec)
            audnow.t_min = wxsprintf('time',audnow.t_sec)
            audnow.etime=nil
        else
            audnow.t_fnm = nil
        end
    elseif player == 'vlc' then
        local JSON = require ("JSON")
        conky_set_update_interval(1)
        local vlcstats = [[wget --user= --password=]] .. p.g .. [[ -q -o /dev/null http://127.0.0.1:8080/requests/status.json -O -]]
        local vlcr = ExecInfo({ cmd = vlcstats })
        if vlcr then
            vlcr = vlcr:gsub('Stream 1','Stream_1')
            vlcr = vlcr:gsub('Stream 0','Stream_0')
            local vlcp = JSON:decode(vlcr)
            if vlcp then
                audnow.astat = vlcp.state     --playing or paused
                w.trackTime  = vlcp.time
                if time_to_go == 1 then
                    vlcp.time = vlcp.length - vlcp.time
                end
                audnow.c_min    = wxsprintf('time',vlcp.time)  --current track time elapsed
                audnow.progress = sprintf("%02d",floor(vlcp.position*100)) -- progress bas %
                audnow.g_vol    = sprintf("%02d",floor(vlcp.volume/2.56))     -- volume
                if vlcp.information then
                    audnow.t_fnm = vlcp.information.category.meta.filename or '' -- filename
                    audnow.c_alb = vlcp.information.category.meta.album or ''
                    audnow.c_art = vlcp.information.category.meta.artist or ''
                    if vlcp.information.category.meta.title then
                        audnow.c_ttl = 'Title: ' .. vlcp.information.category.meta.title
                    else
                        audnow.c_ttl = 'File name: ' .. vlcp.information.category.meta.filename
                    end
                    audnow.t_min = wxsprintf('time',vlcp.length)

                    if vlcp.information.category.Stream_0 and
                        vlcp.information.category.Stream_0.Type == 'Audio' then
                        audnow.c_kbt = vlcp.information.category.Stream_0.Bitrate or 0
                        audnow.c_khz = vlcp.information.category.Stream_0.Sample_rate:match("(%d+)")
                        audnow.c_chl = vlcp.information.category.Stream_0.Channels
                    elseif vlcp.information.category.Stream_1 and
                        vlcp.information.category.Stream_1.Type == 'Audio' then
                        audnow.c_kbt = vlcp.information.category.Stream_1.Bitrate or 0
                        audnow.c_khz = vlcp.information.category.Stream_1.Sample_rate:match("(%d+)")
                        audnow.c_chl = vlcp.information.category.Stream_1.Channels
                    end
                    if not tonum(audnow.c_kbt) then
                        audnow.c_kbt = audnow.c_kbt:match('(%d+)')
                    end

                    if vlcp.information.category.Stream_0 and
                        vlcp.information.category.Stream_0.Type == 'Video' then
                        audnow.v_cod = vlcp.information.category.Stream_0.Codec
                        audnow.v_asp = vlcp.information.category.Stream_0.Resolution
                        audnow.v_vbr = vlcp.information.category.Stream_0.Frame_rate
                        audnow.v_vbr = sprintf("%02.0f",audnow.v_vbr)
                    elseif vlcp.information.category.Stream_1 and
                        vlcp.information.category.Stream_1.Type == 'Video' then
                        audnow.v_cod = vlcp.information.category.Stream_1.Codec
                        audnow.v_asp = vlcp.information.category.Stream_1.Resolution
                        audnow.v_vbr = vlcp.information.category.Stream_1.Frame_rate
                        audnow.v_vbr = sprintf("%02.0f",audnow.v_vbr)
                    end
                    audnow.c_khz = sprintf("%02d",floor(audnow.c_khz/1000))

                    if not audnow.p_pos then audnow.p_pos = 1 end
                    if not audnow.p_ttl then audnow.p_ttl = 1 end
                end
            end
            local vlcstats = [[wget --user= --password=]] .. p.g .. [[ -q -o /dev/null http://127.0.0.1:8080/requests/playlist.json -O -]]
            local vlcr = ExecInfo({ cmd = vlcstats })
            if vlcr then
                local vlcp = JSON:decode(vlcr)
                local tpt = 0
                if vlcp then
                    for d = 1, #vlcp.children[1].children do
                        tpt = tpt + vlcp.children[1].children[d].duration
                        if vlcp.children[1].children[d].current then
                            audnow.p_pos = d
                        end
                    end
                    audnow.p_ttl=#vlcp.children[1].children
                    audnow.etime = wxsprintf('time',tpt)
                end
            end
        end
    elseif player == 'mplayer' and DirExists('/tmp/wxmptmp/') then

        audnow.astat = ExecInfo({ rfile = '/tmp/wxmptmp/ANS_pause' })
        if not audnow.astat or audnow.astat == '' then
            audnow.astat = 'yes'
        end

        audnow.c_sec = ExecInfo({ rfile = '/tmp/wxmptmp/ANS_time_pos' })
        -- audnow.c_sec = ExecInfo({ cmd = 'head /tmp/wxmptmp/ANS_time_pos' })
        audnow.c_sec = tonum(audnow.c_sec) or 0
        w.trackTime = audnow.c_sec
        -- print(w.trackTime)

        audnow.t_fnm = nil
        if not audnow.t_fnm or audnow.t_fnm:match('echo') then
            audnow.t_fnm = 'Buffering...'
        end

        if audnow.t_fnm then
            local trackno = ExecInfo({ rfile = '/tmp/wxmptmp/trackno' })
            trackno = tonum(trackno) or 1

            local ct_info = ExecInfo({ rfile = '/tmp/wxmptmp/ct_info' })
            if ct_info then
                local ctin   = {}
                ctin         = wxsplit(ct_info,':::')
                audnow.p_pos = trackno
                audnow.t_fnm = ctin[1]        or 'Buffering...'
                audnow.c_ttl = ctin[2]        or 'Buffering...'
                audnow.c_min = (tonum(ctin[3]) or 1)
                audnow.c_kbt = (tonum(ctin[4]) or 1)
                audnow.c_khz = ctin[5]
                audnow.c_chl = (tonum(ctin[6]) or 1)
                tpt          = (tonum(ctin[7]) or 1)
                audnow.p_ttl = (tonum(ctin[8]) or 1)
            end

            tpt = tpt or 0
            local tptHours = tpt / 3600
            if tptHours > 99 then tptHours = 99 end
            audnow.etime = wxsprintf('time',tpt)

            local lloop
            if FileExists('/tmp/wxmptmp/loop') then
                lloop = ExecInfo({ rfile = '/tmp/wxmptmp/loop' })
            end
            if lloop then
                if lloop:match('loop') then
                    loop_times = tonum(lloop:match("(%-?%d+)"))
                end
                if loop_times == 0 then
                    loop_times = 99
                end
            else
                loop_times = nil
            end

            local shuffling,ctrk
            if FileExists('/tmp/wxmptmp/shuffle') then shuffling = true end

            if loop_times then
                shfl_loop_label = 'Loop ' .. lr .. '/' .. loop_times
                local lcnt      = ExecInfo({ rfile = '/tmp/wxmptmp/lcnt' })
                if lcnt then lr = tonum(lcnt:match("(%d+)")) end
            elseif shuffling == true then
                shfl_loop_label = ' Shfl'
            else
                shfl_loop_label = ''
            end

            audnow.progress = (audnow.c_sec/audnow.c_min)*100

            if time_to_go == 1 then
                audnow.c_sec = audnow.c_min - audnow.c_sec
            end
            audnow.t_min = wxsprintf('time',audnow.c_min)
            audnow.c_min = wxsprintf('time',audnow.c_sec)

            audnow.etime = audnow.etime or audnow.t_min
            -- audnow.p_ttl = audnow.p_ttl or 1
            audnow.c_kbt = wxsprintf("%02d",(audnow.c_kbt or 1)/1000)
            audnow.c_khz = wxsprintf("%02d",(audnow.c_khz or 1)/1000)
        end
    end

    if not p.d and audnow.t_fnm then
        local aud_ch
        audnow.c_chl = audnow.c_chl or 0
        local chnl = {Stereo='St',Mono='Mo'}
        if type(audnow.c_chl) == 'string' then
            aud_ch = chnl[audnow.c_chl]
        elseif type(audnow.c_chl) ~= 'string' and audnow.c_chl >= 2 then
            aud_ch = 'St'
        else
            aud_ch = 'Mo'
        end

        local playstate = {yes='paused',paused='paused',no='playing',playing='playing'}
        audnow.pause = playstate[audnow.astat]
        if audnow.p_pos then
            local getlastnum = tonum(ssub(audnow.p_pos,-1))
        end

        local traxtxt
        if audnow.v_asp or audnow.v_vbr then
            aud_ch = tconcat({audnow.v_asp,':',audnow.v_vbr,'/',aud_ch})
            if audnow.p_ttl and audnow.p_ttl > 1 then
                traxtxt = tconcat({firstToUpper(audnow.pause),':',audnow.p_pos, '/',audnow.p_ttl})
            else
                traxtxt = tconcat({firstToUpper(audnow.pause),': 1'})
            end
        else
            if audnow.p_ttl and audnow.p_ttl > 1 then
                traxtxt = tconcat({firstToUpper(audnow.pause),': ',audnow.p_pos, '/',audnow.p_ttl,' tracks'})
            else
                traxtxt = tconcat({firstToUpper(audnow.pause),': Single track'})
            end
        end

        if time_to_go == 1 then
            audnow.c_min = '-'..audnow.c_min
        end


        if not audnow.c_ttl or audnow.c_ttl == '' then
            audnow.c_ttl = 'File name: ' .. audnow.t_fnm
        end

        if audnow.c_art and audnow.c_art ~= '' then
            audnow.c_art = 'Artist: ' .. audnow.c_art .. ' / '
            -- audnow.c_art = audnow.c_art .. ' / '
        else audnow.c_art = '' end
        if audnow.c_alb and audnow.c_alb ~= '' then
            audnow.c_alb = 'Album: ' .. audnow.c_alb .. ' / '
        else audnow.c_alb = '' end

        local titletxt = audnow.c_art .. audnow.c_alb .. audnow.c_ttl
        if w.trackTime >= 2 and w.trackTime <= 3 or
            not FileExists('/tmp/trackPlaying') then
            os.execute( [[echo "]].. titletxt .. [[" > /tmp/trackPlaying ]] )
            -- print(w.trackTime)
        end

        local infotxt1
        if not audnow.etime then
            infotxt1 = tconcat({
                audnow.c_min,'/',audnow.t_min,' ',shfl_loop_label,
            })
        else
            infotxt1 = tconcat({
                audnow.c_min,'/',audnow.t_min,'/',audnow.etime,' ',shfl_loop_label,
            })
        end

        local infotxt2 = tconcat({
                aud_ch,'/',
                audnow.c_khz,'kHz/',
                audnow.c_kbt,'kbps'
            })

        audnow.g_volmain, audnow.mute = getVolume()
        if audnow.mute == 'off' then
            audnow.g_volmain = 'Mute'
        end
        if player == 'vlc' then
            audnow.g_vol = tconcat({'Vol:',audnow.g_vol,'%/',audnow.g_volmain,'%'})
        else
            if audnow.g_volmain and audnow.g_volmain ~= 'Mute' then
                audnow.g_vol = tconcat({'Vol:',audnow.g_volmain,'%'})
            else
                audnow.g_vol = tconcat({'Vol:',audnow.g_volmain})
            end
        end
        local voltxt = audnow.g_vol

        -- bar colour
        local ecolour  = p.e or Lime..':1'
        ecolour        = wxsplit(ecolour,":")
        local aud_bar_style = {
            cr  = cr,
            f   = '',
            a   = audnow.progress or 0,
            lwx = p.lwx+3,
            lwy = p.lwy+2,
            w   = p.b or conky_window.width-10,
            s   = p.k or 5,
            c   = ecolour,
            e   = ecolour,
            d   = {{ecolour[1],ecolour[1],ecolour[1]}},
        }
        bargraph(aud_bar_style)

        -- text colour
        local ccolour = p.c
        ccolour = wxsplit(ccolour,":")
        lcairo_set_source_rgba(cr, to_rgba(ccolour))
        lcairo_save(cr)

        if not tonum(p.f) and p.f:match(':') then
            set_font({ cr, p.f })
        else
            set_font({ cr, 'normal:'.. p.f })
        end

        local extents = cairo_text_extents_t:create()
        tolua.takeownership(extents)
        lcairo_text_extents(cr,infotxt1,extents)
        extents.height = extents.height + 3

        if p.v then
            -- msgout('yeke',p.v,tonum(conky_parse("${updates}")))
            -- local upd1 = tonum(conky_parse("${updates}"))
            local scroller_data = {
                cr      = cr,
                l       = p.l       or 36,      -- scroll text length
                s       = p.s       or 5,       -- scroll text step
                w       = p.w       or "B",     -- direction of scroll
                lwx     = p.lwx     or 50,      -- x coordinate
                lwy     = p.lwy-(extents.height*0.8)-3 or 250,        -- y coordinate
                d       = titletxt .. ' <:> ',             -- text to scroll
                c       = p.c,                  -- scroll text colour
                a       = 'l',                  -- text align
                i       = p.i       or 10,      -- vertical scroll jump
                f       = p.f       or 12,      -- font size:name:style etc
                r       = p.r       or 0,       -- range of vertical scroll
                o       = p.o       or "NA",    -- file path/name w scroll text
            }
            scroller(scroller_data)
        end

        local topLineHeight = p.lwy + extents.height + 4
        local botLineHeight = p.lwy + (extents.height * 2.2) + 4.5
        local mtData = {
            {
                cr = cr, ext=extents, text=infotxt1, px=p.lwx, py=topLineHeight, align='l'
            },
            {
                cr = cr, ext=extents, text=voltxt,   px=conky_window.width-5, py=topLineHeight, align='r'
            },
            {
                cr = cr, ext=extents, text=traxtxt,  px=p.lwx, py=botLineHeight, align='l'
            },
            {
                cr = cr, ext=extents, text=infotxt2, px=conky_window.width-5, py=botLineHeight, align='r'
            },
        }
        for x in pairs(mtData) do
            make_txt(mtData[x])
        end
    end

end -- music_sort end

lcollectgarbage()

return w
