#!/bin/bash

# debug='true'

if [[ $debug ]]; then
	rm -rf "/tmp/DEBUG_rmp"
	export PS4='+ (${LINENO}): ${FUNCNAME[0]:+${FUNCNAME[0]}(): }'
	exec 4>>"/tmp/DEBUG_rmp"
	BASH_XTRACEFD=4
	set -x
fi

mpptmp="/tmp/wxmptmp"
wxfifo="${mpptmp}/mpfifo"

player_config_file="$HOME/.config/conkywx/wxmp.conf"
wxmpin="$HOME/.config/conkywx/wxmpin.conf"

rmppath="$( cd "$(dirname "$0")" && pwd )"

msgout() { printf "%b\n" "$1"; }

mpconfig(){
cat <<-mkconfig

# DO NOT REMOVE THIS LINE or your custom changes will be over written !!!
[wxmp_config]

# Loops media playback <number> times.  0 means forever
# comment this option to stop looping after media completion
# NOTE:: you require this enabled for internet radio etc
loop=0

# turns off LIRC Linux Infrared Remote Control
nolirc

# use the current file name as the title for the video window
use-filename-title

# video codecs for vdpau - only used with nvidia see Arch Mplayer wiki for more information
# vc=ffh264vdpau,ffmpeg12vdpau,ffwmv3vdpau,ffvc1vdpau,ffodivxvdpau,

# Video output driver Default is xv - use others as required
# vo=vdpau
vo=xv
# vo=x11

# Necessary to select the buttons in DVD menus
mouse-movements

# Colour settings for a better video experience
saturation=12
contrast=5
brightness=6

# sound output
ao=alsa

# Audio Video autosync - increase the factor to improve sync
autosync=0

# deactivate direct rendering
nodr

# enables double buffering - to remove any flickering
double

# fixes monitor pixel aspect ration
monitorpixelaspect=1

# Adjust matching fuzziness when searching for subtitles
# Loads all subs containing movie name
sub-fuzziness=1

# Sets the autoscale coefficient of the OSD elements Freetype only
subfont-osd-scale=3

# use libass to render subtitles
ass

# Enables  extraction  of  Matroska  embedded fonts
embeddedfonts

# line spacing for libass
ass-line-spacing=0

# Set the scale coefficient to be used for fonts  in  the libass renderer
ass-font-scale=1

# Turns OFF flipping subtitles using FriBiDi
noflip-hebrew

# specify code page for iconv for unicode subtitles
subcp=UTF-8

# in KiloBytes how much memory to use for precaching a file or URL
cache=2048

# audio stream to use
aid=0

# Specify the position of subtitles on the screen
subpos=100

# startup volume
volume=42

# On Screen Display level 0 to 3 options
osdlevel=2

# rebuild index if no index found
idx

# draws the whole frame in a single run
noslices

# number of channels request - higher number good for ac3
channels=2

# numerous audio filters to enhance audio experience
af-add=scaletempo
af-add=equalizer=5.2:3.4:2.8:0.5:1.4:2.3:3.5:4.1:4.0:2.6
af-add=extrastereo
af-add=volnorm=2

mkconfig
}

read_config(){
	while IFS= read -ru 5 cLine
	do
		case "$cLine" in
		['#']*|''|\[wxmp_config\])
			continue
			;;
		*=*)
			cOptL="$(echo ${cLine%%=*})"
			cOptR="$(echo ${cLine#*=})"
			mppconf+=( "-$cOptL $cOptR" )
			;;
		*)
			cOptL="$(echo ${cLine})"
			mppconf+=( "-$cOptL" )
			;;
		esac
	done 5< "$player_config_file"
}

if [[ ! -s "$player_config_file" ]]; then
	mpconfig > "$player_config_file"
	read_config
else
	if [[ $(grep -x '\[wxmp_config\]' "$player_config_file") == '' ]]; then
		mpconfig > "$player_config_file"
	fi
	read_config
fi

if [[ ! -s "$wxmpin" ]]; then
	cp /etc/mplayer/input.conf "$wxmpin"
fi

run_mplayer(){
	# if [[ $(pidof mplayer) == '' ]]; then

		ins="$1"
		tnum="${ins%::*}"
		tname="${ins#*::}"

		mpp="/usr/bin/mplayer -idle -quiet -slave -input file=${wxfifo} -input conf=${wxmpin}  ${mppconf[@]} "

		$mpp 2>&1 | {
		while IFS= read -r line
		do
			case "${line}" in
			ANS_time_pos*)
				echo "${line#*=}" > "${mpptmp}"/"${line%=*}"     2> /dev/null
				;;
			ANS_pause*)
				echo "${line#*=}" > "${mpptmp}"/"${line%=*}"     2> /dev/null
				;;
			ANS*)
				echo "${line#*=}" > "${mpptmp}"/"${line%=*}"     2> /dev/null
				;;
			*Quit*)
				killall mplayer wxmp.sh dompp.sh $wxmpsched_pid  2> /dev/null
				rm -rf $mpptmp                                   2> /dev/null
				exit 0
				;;
			esac
		done
		} &

		echo "loadfile '${tname//\'/\\\'}'" > "$wxfifo"
		echo "start of play" > "${mpptmp}/sop"
		echo "$tnum" > "${mpptmp}/trackno"
		echo "$tnum" > "${mpptmp}/wxmpon"
		# echo "$rmppath" > "${mpptmp}/rmppath"
		$(dirname $0)/wxmpsched.lua &  wxmpsched_pid=$!
		# xfce4-terminal -e "$(dirname $0)/wxmpsched.lua"
	# fi
}

run_mplayer "$1"

