package rssReader;

=comments
    Parses RSS
    This version (c) 2018 Paramvir Likhari
=cut

$rssReader::VERSION = '7';

require Exporter;
@rssReader::ISA = qw(Exporter);
@rssReader::EXPORT = qw(rssShow);

use strict;
use warnings;

use Encode;

## any reference to functions in the main program can be called like main::calltofunction()

my %asciiMap = (
    quot   => '"', apos   => "'", amp    => '&', lt     => '<', gt     => '>',
    nbsp   => '',  iexcl  => '¡', cent   => '¢', pound  => '£', curren => '¤',
    yen    => '¥', brvbar => '¦', sect   => '§', uml    => '¨', copy   => '©',
    ordf   => 'ª', laquo  => '«', not    => '¬', shy    => '­',  reg    => '®',
    macr   => '¯', deg    => '°', plusmn => '±', sup2   => '²', sup3   => '³',
    acute  => '´', micro  => 'µ', para   => '¶', middot => '·', cedil  => '¸',
    sup1   => '¹', ordm   => 'º', raquo  => '»', frac14 => '¼', frac12 => '½',
    frac34 => '¾', iquest => '¿', times  => '×', divide => '÷', Agrave => 'À',
    Aacute => 'Á', Acirc  => 'Â', Atilde => 'Ã', Auml   => 'Ä', Aring  => 'Å',
    AElig  => 'Æ', Ccedil => 'Ç', Egrave => 'È', Eacute => 'É', Ecirc  => 'Ê',
    Euml   => 'Ë', Igrave => 'Ì', Iacute => 'Í', Icirc  => 'Î', Iuml   => 'Ï',
    ETH    => 'Ð', Ntilde => 'Ñ', Ograve => 'Ò', Oacute => 'Ó', Ocirc  => 'Ô',
    Otilde => 'Õ', Ouml   => 'Ö', Oslash => 'Ø', Ugrave => 'Ù', Uacute => 'Ú',
    Ucirc  => 'Û', Uuml   => 'Ü', Yacute => 'Ý', THORN  => 'Þ', szlig  => 'ß',
    agrave => 'à', aacute => 'á', acirc  => 'â', atilde => 'ã', auml   => 'ä',
    aring  => 'å', aelig  => 'æ', ccedil => 'ç', egrave => 'è', eacute => 'é',
    ecirc  => 'ê', euml   => 'ë', igrave => 'ì', iacute => 'í', icirc  => 'î',
    iuml   => 'ï', eth    => 'ð', ntilde => 'ñ', ograve => 'ò', oacute => 'ó',
    ocirc  => 'ô', otilde => 'õ', ouml   => 'ö', oslash => 'ø', ugrave => 'ù',
    uacute => 'ú', ucirc  => 'û', uuml   => 'ü', yacute => 'ý', thorn  => 'þ',
    yuml   => 'ÿ',
);
my $asciiMapKeyList = join("|", keys %asciiMap) ;



sub wrapText {

    # from module Text:Wrap - simple n sleek - now with 3 times the speed

    my $break = '(?=\s)\X';
    my $huge = 'wrap'; # alternatively: 'die' or 'overflow'
    my $separator = "\n";
    my $separator2 = undef;

    my $CHUNK = qr/\X/;

    my $xlen = sub { scalar(() = $_[0] =~ /$CHUNK/g) };

    my $ip = '';
    my $xp = ' ';

    my ($columns, @t) = @_;
    $columns //= 45;  # <= screen width

    my $r = "";
    my $tail = pop(@t);
    my $t = join("", (map { /\s+\z/ ? ( $_ ) : ($_, ' ') } @t), $tail);
    my $lead = $ip;
    my $nll = $columns - &$xlen($xp) - 1;
    if ($nll <= 0 && $xp ne '') {
        my $nc = &$xlen($xp) + 2;
        $columns = $nc;
        $nll = 1;
    }
    my $ll = $columns - &$xlen($ip) - 1;
    $ll = 0 if $ll < 0;
    my $nl = "";
    my $remainder = "";

    pos($t) = 0;
    while ($t !~ /\G(?:$break)*\Z/gc) {
        if ($t =~ /\G((?:(?=[^\n])\X){0,$ll})($break|\n+|\z)/xmgc) {
            $r .= $nl . $lead . $1;
            $remainder = $2;
        }
        elsif ($huge eq 'wrap' && $t =~ /\G((?:(?=[^\n])\X){$ll})/gc) {
            $r .= $nl . $lead . $1;
            $remainder = defined($separator2) ? $separator2 : $separator;
        }
        elsif ($huge eq 'overflow' && $t =~ /\G((?:(?=[^\n])\X)*?)($break|\n+|\z)/xmgc) {
            $r .= $nl . $lead . $1;
            $remainder = $2;
        }
        elsif ($huge eq 'die') {
            die "couldn't wrap '$t'";
        }
        else {
            die "This shouldn't happen";
        }

        $lead = $xp;
        $ll   = $nll;
        $nl   = defined($separator2)
            ? ($remainder eq "\n" ? "\n" : $separator2)
            : $separator;
    }
    $r .= $remainder;

    my($opos) = pos($t);

    $r .= $lead . substr($t, pos($t), length($t) - pos($t))
        if pos($t) ne length($t);

    return $r;

}

sub rssShow {
    # gdacs information - RSS link - http://www.gdacs.org/xml/RSS.xml
    # Near real-time alerts about natural disaster with a potential humanitarian impact

    my ( $temppath, $nowtime, $noFileMsg, $config_file, %conf, $ainfo  ) = @_;

    return if not $temppath;

    my $gdacsData = "${temppath}/gdacsData";
    my $newsData  = "${temppath}/newsData";

    my (%rssConf);

    my $whatToSee = sub {
        my ($text) = @_ ;
        return 'true' if not $rssConf{notReqType} and not $rssConf{notReqLevel};
        return '' if not $text ;


        my ($notReqType, $notReqLevel);
        if (defined $rssConf{notReqType} ) { $notReqType  = lc $rssConf{notReqType} }
        if (defined $rssConf{notReqLevel}) {
            ($notReqLevel = $rssConf{notReqLevel}) =~ s/(\w+)/\u\L$1/g
        }
        $notReqType  ||= 'nil';
        $notReqLevel ||= 'nil';

        my @excWords  = ('tropical cyclone', 'earthquake', 'tsunami', 'flood', 'volcano');
        my @excColour = ('Red', 'Orange', 'Green');
        for (@excWords) {
            if ($_ ne $notReqType) {
                if (index($text, $notReqType) == -1 ) {
                    for (@excColour) {
                        if ($_ ne $notReqLevel) {
                            if (index($text, $notReqLevel) == -1 ) {
                                return('true');
                            }
                        }
                    }
                }
            }
        }
    };

    my $gic = sub {
        my ($opt, $inval1, $inval2, $inval3 ) = @_ ;

        if ( $opt eq 'u' ) {
            return( sprintf("%.1f", $inval1 * 0.62137119 ) );
        }
        elsif ( $opt eq 'p' ) { #Fri, 25 Nov 2016 14:24:29 GMT
            if ( not $rssConf{pastEventDays} ) { return 1 }
            elsif ($inval1) {
                return ($nowtime - main::str2time($inval1)) < (86400 * $rssConf{pastEventDays}) ? 1 : 0 ;
            }
            else { return 1 }
        }
        elsif ( $opt eq 't' ) {
            if ( $inval3 eq 'UTC') {
                return main::time2str("%o %b, %Y %R","$inval1 $inval2 $inval3 DMY") . " LT" ;
            }
            elsif ( $inval3 eq 'AM' or $inval3 eq 'PM' ) {
                return main::time2str("%o %b, %Y %I:%M %p","$inval1 $inval2 $inval3 MDY") ;
            }
        }
        elsif ( $opt eq 'd' ) {
            return main::time2str("%o %b, %Y","$inval1",'dmy') ;
        }
    };

    my $getString = sub {
        my ($str, $startTag, $endTag, $from) = @_;

        return '' if not $$str;

        my $idx1 = index($$str, $startTag);

        if ( $idx1 != -1 ) {
            my $st1    = $idx1 + length($startTag);
            my $offset = $st1 + 1;
            my $nd1    = index($$str, $endTag, $offset);
            my $retval = substr($$str, $st1, $nd1 - $st1);
            if (index($retval,'<![CDATA[') != -1 ) {
                $retval =~ s/<!\[CDATA\[//;
                $retval =~ s/\]\]>//;
            }
            return $retval ;
        }
    };

    my $rss_unescape;
    $rss_unescape = sub {
        my $code_to_ascii = sub {
            if ( $_[0] <= 0x007F ) { return pack( "C" => $_[0] ) }
            return encode("utf-8", chr($_[0]));
        };
        my $fixStr = sub {
            $_[0] =~ s{
                    (&(?:\#(\d{1,4})|\#x([0-9a-fA-F]{1,2})|($asciiMapKeyList));)
                }{
                    $4 ? $asciiMap{$4} : &$code_to_ascii( $3 ? hex($3) : $2, $1 )
                }gex ;
            return $_[0]
        };
        $_[0] = &$fixStr($_[0]);
        $_[0] = &$fixStr($_[0]); # do this again to fix strings completely n return
        return  $_[0];
    };

    my $rssParse = sub {
        my ( $rssStr, %rs, @rss, @headText ) ;
        {
            open (FILE, "<:crlf", $rssConf{rssIn}) or
                        die "$noFileMsg $rssConf{rssIn} data file $!";
                local $/ = undef;
                $rssStr = <FILE> ;
            close (FILE);
        }

        ### parse rss =====================================
        $rssStr =~ s/<!\[CDATA\[//g;
        $rssStr =~ s/\]\]>//g;
        $rssStr =~ tr/\x0a\x0a//d;

        if ( index($rssConf{rssOut}, 'news') != -1 ) {
            while ( $rssStr =~ m{<channel>.*?<title>(.*?)</title>}g ) {
                push(@headText, $1) if $1 and $1 ne '';
            }
        }

        ## get number of items
        my $c = () = $rssStr =~ /<item>/g;
        if ($rssConf{numberOfItemsToView} and
            $rssConf{numberOfItemsToView} <= $c) {
            $c = $rssConf{numberOfItemsToView}
        }

        my $cnt = 0;
        my @rssT;
        $cnt = 0;
        while ( $rssStr =~ m{<item>.*?<pub[dD]ate>(.*?)</pub[dD]ate>}g ) {
            $rssT[$cnt++][0] = $1 ;
        }
        $cnt = 0;
        while ( $rssStr =~ m{<item>.*?<title>(.*?)</title>}g ) {
            $rssT[$cnt++][1] = $1  ;
        }
        $cnt = 0;
        while ( $rssStr =~ m{<item>.*?<description>(.*?)</description>}g ) {
            $rssT[$cnt][2] = ($1) ? $1 : '' ;

            if (index($rssT[$cnt][2],'&lt;') != -1 ) { $rssT[$cnt][2] =~ s/&lt;/</g }

            if (index($rssT[$cnt][2],'&gt;') != -1 ) { $rssT[$cnt][2] =~ s/&gt;/>/g }

            $rssT[$cnt][2] =~ s/<[^>]*>/\x20/g;

            if (index($rssT[$cnt][2],'http') != -1 ) { $rssT[$cnt][2] =~ s/http.*$//g }

            if (index($rssT[$cnt][2],'To read this article') != -1 ) {
                $rssT[$cnt][2] =~ s/To read this article.*$//g
            }

            if (index($rssT[$cnt][2],'mailto:') != -1 ) { $rssT[$cnt][2] =~ s/mailto:.*$//g }

            $rssT[$cnt][2] =~ tr/\x20//s;

            $rssT[$cnt][2] =~ s/^\s+//;

            $cnt++;
        }

        my $numItems = $#{rssT};

        my ($atype);
        my $aHeader = &$rss_unescape($headText[0]);

        my $aHeaderLine=sprintf("%s",'-' x $rssConf{textWrapColumn});

        my @printArray;
        my $pubCount = 0;
        if ($numItems > 0) {

            open(SFILE, ">", $rssConf{rssOut} ) or
                        die "$noFileMsg $rssConf{rssOut} file $!";

            push @printArray, "\#gdacs - rss data\n@@|@@|@@|\n";

            my $talert  = ($#{headText} > 1) ? $numItems - $#{headText} : $numItems ;

            my $articleCount = 1;

            my @goodOnes; $goodOnes[0] = 0;
            for my $i (0..$numItems) {
                if ( &$whatToSee($rssT[$i][1]) eq 'true' and
                                &$gic('p',$rssT[$i][0]) == 1 ) {
                    $pubCount++;
                    $goodOnes[$i] = 1
                }
                else {
                    $goodOnes[$i] = undef
                }
            }
            $pubCount = $pubCount - $#{headText} if $#{headText} > 1;

            push @printArray, ":: Showing data for past $rssConf{pastEventDays} day(s) :: \n@@|@@|@@|\n" if $rssConf{pastEventDays};
            push @printArray,  ":: $aHeader :: \n";

            ## spaces required between new line for lua sm scroller to handle empty lines
            push @printArray, "$aHeaderLine\n@@|@@|@@|\n";
            my ($cntNo);
            for my $i (0..$numItems) {

                if ( $goodOnes[$i] ) {

                    if ($rssConf{showImperial} == 1) {
                        $rssT[$i][1] =~ s/\s*([\.\d]+)\s*km\/h\)/" ".&$gic('u',$1)." mph \)"/eg ;
                        $rssT[$i][2]  =~ s/\s*([\.\d]+)\s*km\/h\)/" ".&$gic('u',$1)." mph \)"/eg ;
                        $rssT[$i][1] =~ s/\s*([\.\d]+)\s*km/" ".&$gic('u',$1)." mi"/eg ;
                        $rssT[$i][2]  =~ s/\s*([\.\d]+)\s*km/" ".&$gic('u',$1)." mi"/eg ;
                    }

                    $rssT[$i][1] = &$rss_unescape($rssT[$i][1]);
                    $rssT[$i][2]  = &$rss_unescape($rssT[$i][2]);
                    $rssT[$i][1] =~ s/\s*([\/\d]+)\s+([\:\d]+)\s+(.*?),/" ".&$gic('t',$1,$2,$3)/eg;
                    $rssT[$i][2]  =~ s/\s*([\/\d]+)\s+([\:\d]+)\s+(.*?),/" ".&$gic('t',$1,$2,$3)/eg;
                    $rssT[$i][2]  =~ s/\s*(\d{2}\/\d{2}\/\d{4})/" ".&$gic('d',$1)/eg;

                    { # start of block
                        no warnings; # ignore pesky Unicode wide character warnings

                        if (index($rssT[$i][1],'@@@') != -1 ) {

                            ($cntNo) = $rssT[$i][1] =~ /(\d+)/;

                            push @printArray,  ":: $headText[$cntNo] :: \n";
                            push @printArray,  "$aHeaderLine\n@@|@@|@@|\n";
                            next
                        }
                        else {
                            if (not $cntNo) {
                                if ( index(lc($aHeader), 'gdacs') != -1 ) {
                                    $atype = 'Alert' }
                                else { $atype = 'News' }
                            }
                            else {
                                if ( index(lc($headText[$cntNo]), 'gdacs') != -1 ) {
                                    $atype = 'Alert' }
                                else { $atype = 'News' }
                            }
                            # $ainfo = substr($headText[$cntNo], 0, 8);
                            # $ainfo = ( $ainfo =~ m{\b([A-Za-z.-]+)\b} ) ? $1 : 0 ;
                            $ainfo = ( $headText[$cntNo] =~ m{\b([A-Za-z.-]+)\b} ) ? $1 : 'Service' ;
                            $ainfo .=  $cntNo + 1;

                            # main::msgout($headText[$cntNo],$ainfo,'ddd');
                            push @printArray, ":: $atype :: $articleCount < $pubCount < $talert :: $ainfo \n";
                        }

                        push @printArray, main::time2str("Published on:: $rssConf{PublishDateFormat}",
                            $rssT[$i][0])."\n" if $rssT[$i][0];

                        if ( not $rssConf{reqTitleOrDesc} or $rssConf{reqTitleOrDesc} == 1 ) {
                            push @printArray, wrapText($rssConf{textWrapColumn}, $rssT[$i][1])."\n";
                        }

                        if ( not $rssConf{reqTitleOrDesc} or $rssConf{reqTitleOrDesc} == 2 ) {
                            push @printArray, wrapText($rssConf{textWrapColumn}, $rssT[$i][2])."\n";
                        }

                        push @printArray, "@@|@@|@@|\n";

                        $articleCount++

                    } # end of block
                }
            }
            print SFILE @printArray;
        }
        else { print SFILE "\n\nNo rss data to show\n\n"}

        $numItems = ($numItems > 0) ? $numItems + 1 : 0 ;
        my $noEventsMsg1 = "No $atype to show for the past $rssConf{pastEventDays} day(s).\nThere are a total of $numItems $atype items.\n";
        my $noEventsMsg2 = "To see more, change \"pastEventDays\" option for rssd in conkywx config file:: ${config_file} ";

        if ($pubCount == 0) {
            print SFILE wrapText($rssConf{textWrapColumn},$noEventsMsg1);
            print SFILE wrapText($rssConf{textWrapColumn},$noEventsMsg2);
        }
        close(SFILE);
    };

    if ( -e $gdacsData and $conf{gdacsUrl} ) {
        undef %rssConf;
        $rssConf{rssUrl}            = $conf{gdacsUrl};
        $rssConf{rssIn}             = $gdacsData;
        $rssConf{rssOut}            = "${temppath}/gdacs";
        $rssConf{textWrapColumn}    = $conf{gtextWrapColumn} // 45;
        $rssConf{showImperial}      = $conf{gshowImperial} // 0 ;
        $rssConf{pastEventDays}     = $conf{gpastEventDays} // 0 ;
        $rssConf{notReqType}        = $conf{gnotReqType};
        $rssConf{notReqLevel}       = $conf{gnotReqLevel};
        $rssConf{reqTitleOrDesc}    = $conf{greqTitleOrDesc};
        $rssConf{PublishDateFormat} = $conf{gPublishDateFormat} // '%o %b, %Y %I:%M %p';
        &$rssParse();
    }
    if ( -e $newsData and $conf{newsUrl} ) {
        undef %rssConf;
        $rssConf{rssUrl}              = $conf{newsUrl};
        $rssConf{rssIn}               = $newsData;
        $rssConf{rssOut}              = "${temppath}/news";
        $rssConf{textWrapColumn}      = $conf{ntextWrapColumn} // 45;
        $rssConf{showImperial}        = $conf{nshowImperial} // 0 ;
        $rssConf{pastEventDays}       = $conf{npastEventDays} // 0 ;
        $rssConf{reqTitleOrDesc}      = $conf{nreqTitleOrDesc};
        $rssConf{numberOfItemsToView} = $conf{nNumberOfItemsToView};
        $rssConf{PublishDateFormat}   = $conf{nPublishDateFormat} // '%o %b, %Y %I:%M %p';
        $rssConf{notReqType}          = $conf{gnotReqType};
        $rssConf{notReqLevel}         = $conf{gnotReqLevel};
        &$rssParse();
    }
}


1;
