package seismic;

=comments
    Parses SEISMIC
    This version (c) 2018 Paramvir Likhari
=cut

$seismic::VERSION = '7';

require Exporter;
@seismic::ISA = qw(Exporter);
@seismic::EXPORT = qw(doSeismic);

use strict;
use warnings;

# sub msgout  { print STDERR "@_ \n" }

sub doSeismic {

    my ($temppath, $noFileMsg, %conf ) = @_ ;

    return if not $temppath;

    # if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my $seismic_dataA = "$temppath/seismic_dataA";
    my $seismic_dataB = "$temppath/seismic_dataB";

    if ( -e $seismic_dataA and -e $seismic_dataB ) {

        my ( $ineqfileA, $ineqfileB, %eq, %eqB, @eqsetA, %seen, $cntr, $cntr2, $cntr3, @sorted2,%hdr,%ftr,%mid,%top, $globalCount, $alertData );

        ## USGS data
        {
        # open(FILE, "<:crlf", "$seismic_dataB") or die "$LMsg{no_open_file} seismic_dataB data file $!";
        open(FILE, "<:crlf", "$seismic_dataB") or die "$noFileMsg seismic_dataB data file $!";
            local $/ = undef;
            $ineqfileB = <FILE> ;
        close(FILE);
        }
        $ineqfileB =~ s/time,lat.*\n//;
        $ineqfileB =~ s/\.\d+Z//g;
        foreach (split('\n',$ineqfileB)) {
            ($eq{time},$eq{lat},$eq{long},$eq{depth},$eq{mag},$eq{loc}) = (split(',', $_))[0,1,2,3,4,13];
            $seen{$eq{time}} = 1;
            push( @eqsetA, {%eq} );
        }
        ## Euro data
        {
        open(FILE, "<:crlf", "$seismic_dataA") or die "$noFileMsg seismic_dataA data file $!";
            local $/ = undef;
            $ineqfileA = <FILE> ;
        close(FILE);
        }
        $ineqfileA =~ s/#.*\n//;
        $ineqfileA =~ s/\.\d+Z//g;
        $ineqfileA =~ s/\n/\#\n/g;
        foreach (split('\n',$ineqfileA)) {
            ($eq{time},$eq{lat},$eq{long},$eq{depth},$eq{mag},$eq{loc}) = (split('\|', $_))[1,2,3,4,10,12];
            unless ($seen{$eq{time}}) { push( @eqsetA, {%eq} ) }
        }

        $conf{num_of_events}     ||= 30;
        $conf{minimum_magnitude} ||= 4.5;
        $conf{alarm_magnitude}   ||= 5;
        $conf{alarm_flash}       ||= 7;

        my $choice = "rpt3";


        # for my $i ( 0 .. $#eqsetA ) {
        #   for my $role ( keys %{ $eqsetA[$i] } ) { msgout($i,$eqsetA[$i]{$role}) }
        # }

        my (%eqf, @scwatch, $scwatch_re );

        if ($conf{loc_width}) {
            if ( $conf{loc_width} < 24 ) { $eqf{all} = 24 }
            else { $eqf{all} = $conf{loc_width} }
        }
        else { $eqf{all} = 45 }

        if ($conf{loc_watch}) {
            # msgout($conf{loc_watch});
            if (index($conf{loc_watch}, ':') != -1) {
                ($scwatch_re = $conf{loc_watch}) =~ tr/:/|/;
            }
            else {
                $scwatch_re = $conf{loc_watch}
            }
        }

        $eqf{txt}  = $eqf{all} - 2 ;
        $eqf{txt1} = $eqf{all} - 1 ;
        $eqf{txt2} = $eqf{all} + 1 ;
        $eqf{hd}   = $eqf{all} - 1 ;

        my ($hLine, $vLine, $xJoin, $tlCorn, $trCorn, $blCorn, $brCorn, $bJoin, $tJoin, $lJoin, $rJoin, $box_char, $hvset, $jset);
        $conf{box_char} ||= '1:1';
        my %hvsetl = (
            '1' => '┄ |',
            '2' => '╴ ╷',
            '3' => '- |',
            '4' => '~ ║',
            '5' => '═ ║',
        );
        my %jsetl = (
            '1' => '╬ ╩ ╦ ╠ ╣ ╔ ╗ ╚ ╝',
            '2' => '┼ ┴ ┬ ├ ┤ ╭ ╮ ╰ ╯',
        );
        my $count = ($conf{box_char} =~ tr/://);
        if ($conf{box_char}) {
            if (length($conf{box_char}) eq 1 or $count == 0 ) {
                ($hLine, $vLine, $xJoin, $bJoin, $tJoin, $lJoin, $rJoin, $tlCorn, $trCorn, $blCorn, $brCorn ) = ($conf{box_char}) x 11;
            }
            elsif ($count eq 1 ) {
                my ($l, $r) = split(/:/,$conf{box_char});
                $l = ($hvsetl{$l}) ? $l : 1;
                $r = ($jsetl{$r})  ? $r : 1;
                ($hLine, $vLine, $xJoin, $bJoin, $tJoin, $lJoin, $rJoin, $tlCorn, $trCorn, $blCorn, $brCorn ) = split(/\s/, $hvsetl{$l} .' '. $jsetl{$r});
            }
            elsif ($count eq 10 ) {
                ($hLine, $vLine, $xJoin, $bJoin, $tJoin, $lJoin, $rJoin, $tlCorn, $trCorn, $blCorn, $brCorn ) = split(/:/, $conf{box_char});
            }
            else {
                ($hLine, $vLine, $xJoin, $bJoin, $tJoin, $lJoin, $rJoin, $tlCorn, $trCorn, $blCorn, $brCorn ) = split(/\s/, $hvsetl{1} .' '. $jsetl{1});
            }
        }

        my @sorted =  sort { $b->{time} cmp $a->{time} } @eqsetA ;

        foreach my $i (0..$#sorted) {

            if ($sorted[$i]{mag} >= $conf{minimum_magnitude} ) {
                $cntr++;
                if ($cntr <= $conf{num_of_events} ) {
                    $sorted[$i]{lat}  = ( $sorted[$i]{lat}  < 0 ) ? (sprintf("%5.2f S",$sorted[$i]{lat}*-1 ))
                    : (sprintf("%5.2f N",$sorted[$i]{lat} ));
                    $sorted[$i]{long} = ( $sorted[$i]{long} < 0 ) ? (sprintf("%6.2f W",$sorted[$i]{long}*-1))
                    : (sprintf("%6.2f E",$sorted[$i]{long}));

                    $sorted[$i]{time} = main::time2str($conf{seismic_time_format},$sorted[$i]{time}.'Z') ;

                    $eqf{time} = length($sorted[$i]{time});

                    $sorted[$i]{depth} = main::convert_unit($sorted[$i]{depth}, 'Depth');
                    $sorted[$i]{mag}  = sprintf("%4.1f",$sorted[$i]{mag});
                    $sorted[$i]{loc} =~ tr/"//d;
                    if ($scwatch_re) {
                        if ( lc($sorted[$i]{loc}) =~ /$scwatch_re/o ) { $sorted[$i]{loc} = "*". $sorted[$i]{loc} }
                    }

                    if ($sorted[$i]{mag} >= $conf{alarm_flash} ) {
                        $alertData .= "Seismic alert:  $sorted[$i]{mag}Mag. $sorted[$i]{loc}, Posn:$sorted[$i]{lat} $sorted[$i]{long} | ";
                        $sorted[$i]{loc} = '@'. $sorted[$i]{loc}
                    }

                    if (index($sorted[$i]{loc},'#') != -1) {
                        $sorted[$i]{loc} =~ tr/#//d;
                        $sorted[$i]{loc} =~ s/(\w+)/\u\L$1/g;
                    }
                    $globalCount = $cntr;
                    $eqf{hd1} = $eqf{time} + 0 if $eqf{time};

                    my $sprt;

                    if ($sorted[$i]{mag} >= $conf{alarm_magnitude} and
                        $sorted[$i]{mag} < $conf{alarm_flash} ) {
                        $cntr2++;
                        if ( index($sorted[$i]{loc}, '*') != -1 ) {
                            $sprt = "$vLine> %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{all}.$eqf{txt}s$vLine"
                        }
                        else {
                            $sprt = "$vLine> %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine %-$eqf{txt1}.$eqf{txt}s$vLine"
                        }
                    }
                    elsif ($sorted[$i]{mag} >= $conf{alarm_flash} ) {
                        if ( index($sorted[$i]{loc}, '@*') != -1 ) {
                                $sprt = "$vLine>>%-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{txt2}.$eqf{txt}s$vLine"
                        }
                        elsif ( index($sorted[$i]{loc}, '@') != -1 ) {
                                $sprt = "$vLine>>%-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{all}.$eqf{txt}s$vLine"
                        }
                        else {
                            $sprt = "$vLine>>%-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine %-$eqf{txt1}.$eqf{txt}s$vLine"
                        }
                    }
                    else {
                        if ( index($sorted[$i]{loc}, '*') != -1 ) {
                            $sprt = "$vLine  %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{all}.$eqf{txt}s$vLine"
                        }
                        elsif ( index($sorted[$i]{loc}, '@') != -1 ) {
                                    $sprt = "$vLine  %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{all}.$eqf{txt}s$vLine"
                        }
                        else {
                            $sprt = "$vLine  %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine %-$eqf{txt1}.$eqf{txt}s$vLine"
                        }
                    }
                    $sorted[$i]{rpt3} = sprintf($sprt,$sorted[$i]{time},$sorted[$i]{mag},$sorted[$i]{depth},$sorted[$i]{lat}, $sorted[$i]{long}, $sorted[$i]{loc});
                    push( @sorted2, $sorted[$i] );
                    # msgout($sorted[$i]);
                }
            }
        }

        my $mag_alarm;
        if ( $eqf{all} < 33 ) {
            $mag_alarm = ($cntr2)
            ? "Loc alarm > |$conf{alarm_magnitude}| = $cntr2/$globalCount "
            : "Location";
        }
        else {
            $mag_alarm = ($cntr2)
            ? "Location alarm > |$conf{alarm_magnitude}| = $cntr2/$globalCount "
            : "Location";
        }

        if ( $eqf{time} ) {
            $eqf{hd1} = $eqf{time} + 1;
            $hdr{rpt3} = sprintf("$vLine %-$eqf{hd1}s $vLine%4s $vLine %-8s $vLine %-7s $vLine %-8s $vLine %-$eqf{hd}s$vLine", "Time","Mag","Depth","Lat", "Long", $mag_alarm);

            $eqf{hd1} = $eqf{time} + 3;
            $mid{rpt3} = sprintf("$lJoin%s$xJoin%s$xJoin%s$xJoin%s$xJoin%s$xJoin%s$rJoin",$hLine x $eqf{hd1},$hLine x 5,$hLine x 10,$hLine x 9,$hLine x 10,$hLine x $eqf{all} );
            $top{rpt3} = sprintf("$tlCorn%s$tJoin%s$tJoin%s$tJoin%s$tJoin%s$tJoin%s$trCorn",$hLine x $eqf{hd1},$hLine x 5,$hLine x 10,$hLine x 9,$hLine x 10,$hLine x $eqf{all} );
            $ftr{rpt3} = sprintf("$blCorn%s$bJoin%s$bJoin%s$bJoin%s$bJoin%s$bJoin%s$brCorn",$hLine x $eqf{hd1},$hLine x 5,$hLine x 10,$hLine x 9,$hLine x 10,$hLine x $eqf{all} );

            open(SFILE, '>', "$temppath/seismic" ) or die "$noFileMsg seismic file $!";
                # print SFILE "Wx Seismic Table\n";
                print SFILE "$top{$choice}\n";
                print SFILE "$hdr{$choice}\n";
                print SFILE "$mid{$choice}";
                for my $i ( 0 .. $#sorted2 ) {
                    for my $role ( keys %{ $sorted2[$i] } ) {
                        print SFILE "\n$sorted2[$i]{$role}" if ( $role eq $choice);
                    }
                }
                print SFILE "\n$ftr{$choice}\n";
            close(SFILE);

            return $alertData if $alertData ;
        }
        else {
            main::show_noti('Warning :| no Seismic events match the minimum_magnitude settings in conkywx config file',1);
        }
    }
}


1;

