#!/usr/bin/lua

--------- smooth scroller by Paramvir (2017)

local w = {true}

----- to debug assign 1 to debug
local debug

local tconcat  = table.concat
local tonum    = tonumber
local lgsub    = string.gsub
local lsub     = string.sub
local sprintf  = string.format
local llower   = string.lower
local lmatch   = string.match
local lfind    = string.find
local lrep     = string.rep
local llen     = string.len
local lfloor   = math.floor
local llog10   = math.log10
local lexecute = os.execute
local lremove  = os.remove
local lgetenv  = os.getenv

local lcollectgarbage = collectgarbage

local lcairo_set_source_rgba  = cairo_set_source_rgba
local lcairo_set_line_width   = cairo_set_line_width
local lcairo_line_to          = cairo_line_to
local lcairo_move_to          = cairo_move_to
local lcairo_show_text        = cairo_show_text
local lcairo_stroke           = cairo_stroke
local lcairo_stroke_preserve  = cairo_stroke_preserve
local lcairo_fill             = cairo_fill
local lcairo_fill_preserve    = cairo_fill_preserve
local lcairo_save             = cairo_save
local lcairo_restore          = cairo_restore
local lcairo_set_source_rgba  = cairo_set_source_rgba
local lcairo_text_extents     = cairo_text_extents
local lcairo_rectangle        = cairo_rectangle
local lcairo_clip             = cairo_clip
local lcairo_select_font_face = cairo_select_font_face
local lcairo_transform        = cairo_transform
local lcairo_translate        = cairo_translate
local lcairo_matrix_init      = cairo_matrix_init

local lconky_set_update_interval = conky_set_update_interval

-- local myPackage = package
-- myPackage.path = myPackage.path .. ";./lib/?.lua;"

local FC = require("luafunc")

local wxsplit     = FC.wxsplit
local to_rgba     = FC.to_rgba
local ExecInfo    = FC.ExecInfo
local FileExists  = FC.FileExists
local set_font    = FC.set_font
local msgout      = FC.msgout

local function round(num, idp)
    return tonum(sprintf("%." .. (idp or 0) .. "f", num))
end

local cnTable = {}
local cn = 1
local function getCounter(infile)
    if not cnTable[infile] then
        cnTable[infile] = cn
        cn = cn + 1
    end
    for k, v in pairs(cnTable) do
        if k == infile then
            return v
        end
    end
end

local function fixSeismic (p)
    local cr          = p.cr
    local sccolour    = p.mcolour or '0x00c0ff:1'
    local mcolour     = wxsplit(sccolour, ":")
    local Yellow      = 0xFFFF00
    local Orchid1     = 0xFF83FA
    local h_light     = tconcat({Yellow, ':1'})
    local alt_h_light = tconcat({Orchid1,':1'})
    local px          = p.px
    local py          = p.py
    local style
    if h_light and h_light ~= '0' and p.data then
        mcolour = nil
        if lfind(p.data,'@',1,true) then
            if f_check == 0 then
                mcolour = wxsplit(h_light,":")
                mcolour = {mcolour[1],0.45}
            else
                mcolour = wxsplit(h_light,":")
            end
            p.data = lgsub(p.data, '%@%*', '*')
            p.data = lgsub(p.data, '%@', ' ')
            style = 1
        elseif lfind(p.data,'>',1,true) then
            mcolour = wxsplit(h_light,":")
            style = 1
        elseif lfind(p.data,'*',1,true) then
            mcolour = wxsplit(alt_h_light,":")
            style = 1
        elseif not mcolour then
            mcolour = wxsplit(sccolour,":")
            style = nil
        end
    end
    local b,i
    if     style == 1 then b = 1; i = 0
    elseif style == 2 then b = 0; i = 1
    elseif style == 3 then b = 1; i = 1
    end

    lcairo_select_font_face(cr, 'Mono', i, b)
    lcairo_set_source_rgba(cr,to_rgba(mcolour))
    lcairo_move_to(cr, px, py)
    lcairo_show_text(cr, p.data)
end

local function showData(p)

    local cr = p.cr

    for x = 1, #p.table do
        if not p.table[x] then p.table[x] = ' ' end

        local cnt3, py1
        if tonum(p.offset) == 3 then
            cnt3 = (p.startNum + x - p.offset) * p.htOfLine
            py1  = p.direction * (p.inc + p.py + cnt3)
        else
            py1 = p.direction * (p.inc + p.py + ((x - p.offset) * p.htOfLine) + p.htOfBlock)
        end

        if p.seismic then
            fixSeismic({
                cr      = cr,
                mcolour = p.colour,
                px      = p.px,
                py      = py1,
                data    = p.table[x],
            })
        else
            lcairo_move_to(cr, p.px, py1)
            lcairo_show_text(cr, p.table[x])
        end
    end
end


local function showProgress(p)
    local cr = p.cr
    lcairo_save(cr)

    local colour = {}
    colour = wxsplit(p.colour, ':')

    lcairo_set_line_width(cr, 5)

    local pxPosn = p.px + conky_window.width - 7

    if p.currentPosn > p.htOfWidget then
        p.currentPosn = p.currentPosn - p.htOfWidget
    end

    lcairo_set_source_rgba(cr, to_rgba({(colour[3] or 0x555555), (colour[4] or 0.3)}))
    lcairo_move_to(cr, pxPosn, p.py)
    lcairo_line_to(cr, pxPosn, p.py + p.htOfWidget)
    lcairo_stroke(cr)

    lcairo_set_source_rgba(cr, to_rgba({(colour[1] or 0x00ff00), (colour[2] or 1)}))
    lcairo_move_to(cr, pxPosn, p.py + (p.currentPosn))
    lcairo_line_to(cr, pxPosn, p.py + (p.currentPosn + 10))
    lcairo_stroke(cr)

    lcairo_restore(cr)
end

local function makeListData(p)

    local listData  = p.fpath .. (p.data  or 'listData')
    local listIndex = p.fpath .. (p.index or 'listIndex')

    lexecute( tconcat({[[ perl -ne 'next if /@@.*/;/::.*?::/ and print "@@|@@|@@|\n$_---------------------------------------\n" if not /::.*::.*::.*$/; /::.*::.*::.*$/ and do{$p=$.;$c++};$.==$p+2 and printf("%3d %s", $c, $_)' ]],
            p.file, [[ > ]], listData}) )

    lexecute( tconcat({[[ perl -ne 'next if not /::.*::.*::.*$/; print "$.\n";
        END{print "$.\n"}' ]], p.file, [[ > ]], listIndex}) )

    local maxRecords
    maxRecords = lmatch(ExecInfo({ cmd = "wc -l " .. listIndex }),'([%d]+)%s+')

    return listIndex, listData, (tonum(maxRecords) or 1) - 1

end

local function textFind(p)
    -- p.find = find terms, p.dfile = data, p.ifile = index file
    if not p.find then return nil end

    -- get a gsub solution with stopWords hash
    local stopWords = {'I', 'a', 'about', 'an', 'are', 'as', 'at', 'be', 'by',
                        'com', 'for', 'from', 'how', 'in', 'is', 'it', 'of',
                        'on', 'or', 'that', 'the', 'this', 'to', 'was', 'what',
                        'when', 'where', 'who', 'will', 'with', 'the', 'www'}

    local splitFind = {}
    if not lmatch(p.find, '%[.*%]') then
        splitFind = wxsplit(p.find,'%s+')
    end

    for x = 1, #splitFind do
        for y = 1, #stopWords do
            if llower(splitFind[x]) == stopWords[y] then
                p.find = lgsub(p.find,'(%w+)',
                    function (c)
                        if llower(c) == stopWords[y] then return '' end
                    end )
                break
            end
        end
    end
    -- print(p.find)

    local notFind,notFindOrg = "", nil
    if lmatch(p.find, 'NOT') then
        p.find, notFind = lmatch(p.find, '(.*)%s*NOT%s*(.*)')

        notFind = lmatch(notFind,'^%s*(.*)%s*$')

        if lmatch(notFind, '[%.]+') and not lmatch(notFind, '%.%*') then
            notFind = lgsub(notFind, '%.','\\.')
        end

        if lmatch(notFind, '%[.*%]') then
            notFind = lmatch(notFind, '%[(.*)%]')
        else
            notFind = lgsub(notFind,'%s+','|')
        end
        notFindOrg = notFind
        notFind = '/'..notFind..[[/i and print "$.NOT:" or ]]
    end

    p.find = lmatch(p.find,'^%s*(.*)%s*$')

    if lmatch(p.find, '[%.]+') and not lmatch(p.find, '%.%*') then
        p.find = lgsub(p.find, '%.','\\.')
    end

    local sep = ''
    if lmatch(p.find, '%[.*%]') then
        p.find = lmatch(p.find, '%[(.*)%]')
        sep = '|'
    else
        p.find = lgsub(p.find,'%s+','|')
    end

    if notFindOrg then p.find = p.find .. sep .. notFindOrg end

    -- print(p.find)

    local foundStr
    foundStr = ExecInfo({ cmd = tconcat({[[perl -ne 'if (/]], p.find,
                [[/i) { ]],notFind,[[ print "$.:" }' ]], p.dfile})})

    local foundIndex, loadIndex = {},{}
    loadIndex = wxsplit(ExecInfo({ rfile = p.ifile }),'\n')

    if foundStr then
        foundIndex = wxsplit(foundStr,':')
        foundStr = ''
        local fs,fn,lastVal = 0,0,0
        local notStr, seen
        for x = 1, #foundIndex do
            if lmatch(foundIndex[x],'NOT') then
                foundIndex[x] = lmatch(foundIndex[x],'(.*)NOT')
                notStr = 1
            end
            local f1 = tonum(foundIndex[x]) or nil
            for y = 1, #loadIndex do
                local l1 = tonum(loadIndex[y])
                if f1 and (f1 - l1) < 0 then
                    if notStr then fn = y; notStr = nil end
                    if lastVal == fn and not seen and lastVal ~= 0 then
                        local ntd = lfloor(llog10(lastVal)+1)
                        foundStr = lsub(foundStr, 0, -(ntd+2))
                        seen = 1
                    end
                    if fs ~= y then
                        if fn and fn == y then
                            break
                        else
                            fs = y
                            lastVal = y
                            seen = nil
                        end
                        if fs ~= 0 then foundStr = foundStr .. fs - 1 .. ',' end
                    end
                    break
                end
            end
        end
    end
    return foundStr
end

local function getIdx(p)
    return lmatch( ExecInfo({
                cmd = tconcat({'tail -n +',p.num,' ',p.file,' | head -n 2'})
            }),'([%d]+)\n([%d]+)' )
end

local function getLines(p)
    local retval, cmdLine
    retval =  ExecInfo({ cmd =
        tconcat({'tail -n +', (p.from or 1),' ', p.file,
            ' | head -n ', (p.to - p.from) + 1 })
        })
    if p.seismic then retval = "@@|@@|@@|\n" .. retval end
    if p.rtype then
        retval = lgsub(retval,'@@|@@|@@|','   ')
        return wxsplit(retval,'\n',0,1)
    else
        return retval
    end
end

local function checkLimits(p)
    if not tonum(p.num)   then return nil end
    if not tonum(p.limit) then return nil end
    if tonum(p.num) > tonum(p.limit) then
        print('\nWarning - :| No news item for '..p.num..' index number. Showing last news instead at '..p.limit..' index number.\n')
        return p.limit
    else
        return p.num
    end
end

-----------------------------------------------------------
local wxtmpDir = '/tmp/conkywx_' .. lgetenv("USER")

local ts,sm,ostat,cstat,txt,lv = {},{},{},{},{},{}
local sData, eData, extents, cnt = 1,1,0,0
local sNo, firstStart, testVal
local lscrlTable = {}
local cnt2 = 1
-- local listViewData = {}


function w.smscroller(p)

    local cr        = p.cr
    local px        = p.lwx  or 55
    local py        = p.lwy  or 20
    local width     = p.w    or 300
    local bcolour   = p.b    or nil
    local tcolour   = p.t    or '0xffff55:1'
    local speed     = p.e    or 15
    local updateVal = p.u    or nil
    local data      = p.i    or [[This is a sample scroller text | Another sample :)]]
    local fontSize  = p.f    or 12
    local goRight   = p.r    or 'l'
    local listView  = p.l    or nil

    local ldata
    ldata = data .. lmatch(conky_config,"([^/]+)$")
    -- msgout(ldata)
    if updateVal and updateVal == 0 then updateVal = nil end
    if listView  and listView  == 0 then listView  = nil end

    -- define timers
    sNo = getCounter(data)
    local tc = { [sNo] = sNo }

    local gr = { l = true, r = true, b = true }
    local gu = { d = true, u = true }

    if gr[goRight] then
        if updateVal then speed = 1.5 end
    end

    if not tonum(width) then
        local x, y = lmatch(width,"^%s*(.*)%s*,%s*(.*)%s*$")
        if x and y then
            width          = tonum(x)
            w.widgetHeight = tonum(y)
        end
    end

    w.widgetWidth = width  - 14
    px            = (px/2) + 5
    py            = py     + 10

    if not w.widgetHeight and gu[goRight] then
        error("\n\nError :( in lua smooth scroller command line option -w \n")
    end

    local checkFrequency = 60
    if lmatch(data,'Playing') then
        checkFrequency = 60
    else
        checkFrequency = 120
    end


    cnt = cnt + 1
    local listViewFile = wxtmpDir .. '/listView'
    if not listView or cnt % 30 < (#ts or 1) then
        if lmatch(data,'news') and FileExists(listViewFile) then
            listView = 1
            newnews = 1
        end
    end

    local wxnews = wxtmpDir .. '/wxnews'
    local newnews
    if listView then
        if not lv[wxnews] or cnt % 30 < (#ts or 1) then
            if FileExists(wxnews) then
                ostat[wxnews] = ExecInfo({ cmd = [[stat -c %Z ]] .. wxnews })
                if not cstat[wxnews] or cstat[wxnews] ~= ostat[wxnews] then
                    lv[wxnews] = ExecInfo({ cmd = 'head -n 1 ' .. wxnews})
                    newnews = 1
                    if tonum(lv[wxnews]) == 0 then
                        lv[wxnews] = nil
                        lremove(wxnews)
                    end
                end
                cstat[wxnews] = ostat[wxnews]
            end
        end
    end

    -- msgout(conky_info.update_interval,lmatch(conky_config,"([^/]+)$"))

    if not sm[ldata] or cnt % checkFrequency < (#ts or 1) or newnews then
        if lmatch(data,'/') and FileExists(data) or newnews then
            ostat[ldata] = ExecInfo({ cmd = [[stat -c %Z ]] .. data })
            if not cstat[ldata] or cstat[ldata] ~= ostat[ldata]  or newnews then
                local listData, listIndex, currData, filePath, maxRecords

                filePath = lmatch(data, '(.*)/') .. '/'

                if goRight == 'r' then
                    lexecute( [[tac ]]..data..' > '..data..'R' )
                    sm[ldata] = ExecInfo({ rfile = data..'R' })
                elseif gr[goRight] then
                    sm[ldata] = ExecInfo({ rfile = data })
                else
                    if lv[wxnews] and lv[wxnews] == 'listviewoff' then
                        listView = nil
                        lv[wxnews] = nil
                        currData = data
                        lremove(listViewFile)
                        lremove(wxnews)
                    end

                    if listView then
                        listIndex, listData, maxRecords = makeListData({
                            file  = data,
                            fpath = filePath,
                            data  = 'listData',
                            index = 'listIndex',
                            })
                        currData = listData
                    else
                        lremove(filePath..'newsView')
                        lremove(filePath..'listData')
                        lremove(filePath..'listIndex')
                        currData = data
                    end
                    sm[ldata] = getLines({from = 1, to = 72, file = currData})
                    -- -- on file update - reset main counters
                    cnt2 = 1; ts[tc[sNo]] = nil
                end

                if not sm[ldata] then return 'No Data' end

                if lv[wxnews] then
                    local viewTable, viewDataTable = {},{}

                    local searchItems = lv[wxnews]

                    if lmatch(lv[wxnews],'[Ss]:.*') then
                            lv[wxnews] = lmatch(lv[wxnews],'[Ss]:%s*(.*)')
                            lv[wxnews] = textFind({
                               find  = lv[wxnews],
                               dfile = data,
                               ifile = listIndex,
                               })
                    else
                        lv[wxnews] = lgsub(lv[wxnews],'%s+','')
                    end
                    viewTable  = wxsplit(lv[wxnews],',')

                    local numberOfItems = 0
                    for x = 1, #viewTable do
                        local idx1, idx2, k, num, dum, vt

                        if lmatch(viewTable[x],'%.%.') then
                            local llim, ulim, dum

                            llim, ulim = lmatch(viewTable[x], '([%d]+)%.%.([%d]+)')

                            if tonum(ulim) < tonum(llim) then llim, ulim = ulim, llim end

                            llim = checkLimits({ num = llim, limit = maxRecords })
                            ulim = checkLimits({ num = ulim, limit = maxRecords })

                            if llim then
                                idx1, dum = getIdx({ file = listIndex, num = llim })
                                dum, idx2 = getIdx({ file = listIndex, num = ulim })
                                numberOfItems = numberOfItems + (tonum(ulim) - tonum(llim)) + 1
                            end
                        elseif lmatch(viewTable[x],'[%d]+' ) then
                            viewTable[x] = checkLimits({ num = viewTable[x],
                                limit = maxRecords })
                            if viewTable[x] then
                                idx1, idx2 = getIdx({ file = listIndex,
                                    num = viewTable[x] })
                                numberOfItems = numberOfItems + 1
                            end
                        end

                        if idx1 then
                            vt = getLines({from = tonum(idx1), to = tonum(idx2),
                                file = data})
                            if lmatch(vt,'@@%|.*\n%s*%------.*$') then
                                vt = lmatch(vt,'(.*)\n@@%|.*\n%s*.*%------.*$')
                            end
                            if x == 1 then
                                vt = tconcat({'@@|@@|@@|\n',':: ::: results found ::\n',
                                    ':: Search :: ',searchItems ,
                                    ' ::\n---------------------------------------\n\n',
                                    vt,'\n'})
                            else
                                vt = '@@|@@|@@|\n' .. vt .. '\n'
                            end
                            viewDataTable[(x-1) + 1] = vt
                        end
                    end

                    if viewDataTable[1] then
                        viewDataTable[1] = lgsub(viewDataTable[1],':::',numberOfItems)
                        sm[ldata] = tconcat(viewDataTable)
                        local newsView = filePath .. 'newsView'
                        ExecInfo({wfile = newsView, wtext = sm[ldata]})
                        currData = newsView
                    else
                        lv[wxnews] = nil
                    end
                end

                local quake
                if lmatch(data,'seismic') then
                    quake = 1
                    sm[ldata] = "@@|@@|@@|\n" .. sm[ldata]
                end
                if tonum(speed) > 67 then
                    speed = 0.01
                else
                    speed = round(1/speed, 2)
                end

                lconky_set_update_interval(speed)

                if bcolour then w.b2colour = {to_rgba(wxsplit(bcolour,":"))} end

                w.t2colour = {to_rgba(wxsplit(tcolour,":"))}

                extents = cairo_text_extents_t:create()
                tolua.takeownership(extents)
                lcairo_text_extents(cr,sm[ldata],extents)

                -- -- reset counter on data change for music display basically
                if lmatch(data,'Playing') then
                    if goRight == 'r' then
                        if ts[tc[sNo]] then ts[tc[sNo]] = -extents.width end
                    elseif goRight == 'l' then
                        if ts[tc[sNo]] then ts[tc[sNo]] = -w.widgetWidth end
                    end
                end

                if gr[goRight] then
                    local k, num = lgsub(sm[ldata],"\n"," | ")
                    w.num        = num + 1
                    txt[ldata] = k .. lrep(' <:> ' .. k, lfloor(w.widgetWidth/extents.width))
                elseif gu[goRight] then
                    local _, num = lgsub(sm[ldata],"\n","")
                    if num * (extents.height + 4) < w.widgetHeight then
                        local rF
                        rF = lfloor((w.widgetHeight / ((extents.height + 4) * num)) + 0.99)
                        sm[ldata] = lrep(sm[ldata], (rF or 3) + 2)
                    end
                    txt[ldata]    = wxsplit(sm[ldata], '\n')
                    for x = 1, #txt[ldata] do
                        txt[ldata][x] = lgsub(txt[ldata][x], '@@|@@|@@|', '  ')
                    end
                    txt[ldata][1] = ' '

                    local lineCount
                    lineCount = lmatch(ExecInfo({cmd="wc -l "..currData}),'([%d]+)%s+')
                    txt[ldata]['lineCount'] = tonum(lineCount) - 0

                    txt[ldata].seismic  = quake
                    txt[ldata].currData = currData
                    txt[ldata].filePath = filePath
                    if listView then
                        txt[ldata].listIndex = listIndex
                    end
                end
            end
            cstat[ldata] = ostat[ldata]
        -- elseif lmatch(data,'/') and not FileExists(data) and
        --     ldata == data .. lmatch(conky_config,"([^/]+)$") then
        --     sm[ldata]  = nil
        --     txt[ldata] = nil
        end
    end

    lcairo_save(cr)
    lcairo_set_source_rgba(cr, w.t2colour[1],w.t2colour[2],w.t2colour[3],w.t2colour[4])
    lcairo_save(cr)
    fontSize = set_font({cr, fontSize})
    lcairo_restore(cr)

    lcairo_save(cr)
    if bcolour then
        lcairo_set_source_rgba(cr, w.b2colour[1],w.b2colour[2],w.b2colour[3],w.b2colour[4])
        lcairo_rectangle(cr, px*2, py-3, w.widgetWidth+14, -(fontSize*2))
        lcairo_fill(cr)
    end
    lcairo_restore(cr)

    lcairo_text_extents(cr,sm[ldata],extents)
    if extents.width + 25 >= w.widgetWidth and goRight == 'b' then goRight = 'l' end

    -- -- initialise
    local txtData, dd1, dd2, iDirectionFactor
    if gr[goRight] then
        lcairo_rectangle(cr, px*2, py-3, w.widgetWidth+14, -(fontSize*2))

        -- intialise counters
        if goRight == 'r' then
            if not ts[tc[sNo]] then ts[tc[sNo]] = 0 end
        else
            if not ts[tc[sNo]] then ts[tc[sNo]] = -w.widgetWidth end
        end
        if lmatch(data,'wxalert') then
            if ts[tc[sNo]] then ts[tc[sNo]] = ts[tc[sNo]] + (updateVal or 1) end
        else
            if ts[tc[sNo]] then ts[tc[sNo]] = ts[tc[sNo]] + (updateVal or 2) end
        end
    elseif gu[goRight] then
        lcairo_rectangle(cr, px, py+5, w.widgetWidth+14, w.widgetHeight)

        -- intialise counters
        if goRight == 'u' then
            iDirectionFactor = -w.widgetHeight
            if not ts[tc[sNo]] then ts[tc[sNo]] = iDirectionFactor end   -- up
        elseif goRight == 'd' then
            iDirectionFactor = lfloor(w.widgetHeight * 0.25)
            if not ts[tc[sNo]] then ts[tc[sNo]] = iDirectionFactor end  -- down
        end
        if ts[tc[sNo]] then ts[tc[sNo]] = ts[tc[sNo]] + (updateVal or 1) end
    end
    lcairo_clip(cr)

    -- execute
    local direction, preset, i, rfact

    if ts[tc[sNo]] then i = (updateVal or 1) - ts[tc[sNo]] end

    if gu[goRight] then

        txtData = txt[ldata]
        local numOfRecords = (txt[ldata].lineCount or 1)

        local ht = extents.height + 4

        local htOfText = ht * numOfRecords

        local txtLimit = lfloor((w.widgetHeight * 2) / ht)
        local lastNum

        if goRight == 'u' then
            direction =  1; preset = 2; rfact = 1
        elseif goRight == 'd' then
            direction = -1; preset = htOfText + 5; rfact = 2
        end

        ----- new data point threshold marker
        local iMod = i % -(lfloor(txtLimit * ht))

        ---- workout the index of data to pull
        local getData
        if i == -(iDirectionFactor) or i < -(htOfText) then
            sData = 1; eData = txtLimit * 2; getData = 1
            if i == w.widgetHeight then firstStart = 1 end
        elseif txt[ldata].mod and txt[ldata].mod < iMod then
            lastNum = cnt2
            cnt2    = cnt2 + txtLimit

            if firstStart then
                firstStart = nil
            elseif eData ~= numOfRecords then
                getData = 1
                sData   = lastNum
                eData   = sData + (txtLimit * 2)
                if eData > numOfRecords then eData = numOfRecords end
            end
        end

        -- if txt[ldata].mod and txt[ldata].mod < iMod then
        --     print(i,i % -(lfloor(txtLimit * ht)),iMod,txt[ldata].mod,'<<<<<<<<<<<<<<<<')
        -- end
        txt[ldata].mod = iMod

        --- get data from file
        if getData then
            lscrlTable[ldata] = {}
            lscrlTable[ldata] = getLines({from = sData, to = eData,
                file = txt[ldata].currData, rtype = 'table',
                seismic = txt[ldata].seismic })

            if sData == 1 then lscrlTable[ldata][1] = '   ' end

            if debug then
                if #lscrlTable[ldata] == #lscrlTable[ldata] - 1 then print('####################') end
                print(i,sData,eData,(eData-sData),#lscrlTable[ldata],txtLimit, ht,extents.height)
            end
        end

        -- if not listView then
            showProgress({
                cr          = cr,
                px          = px,
                py          = py,
                colour      = (p.g or '0x00ff00:1:0x555555:0.3'),
                htOfWidget  = w.widgetHeight,
                currentPosn = lfloor(((w.widgetHeight-i)/htOfText)*w.widgetHeight),
                })
        -- end

        showData({
            cr        = cr,
            px        = px,
            py        = py,
            startNum  = sData,
            htOfLine  = ht,
            htOfBlock = htOfText,
            inc       = i,
            offset    = 3,
            direction = direction,
            table     = lscrlTable[ldata],
            colour    = p.c,
            seismic   = txt[ldata].seismic,
        })

        if i >= -(htOfText + 1) and i <= -(htOfText - w.widgetHeight) then
            showData({
                cr        = cr,
                px        = px,
                py        = py,
                startNum  = sData,
                htOfLine  = ht,
                htOfBlock = htOfText,
                inc       = i,
                offset    = 2,
                direction = direction,
                table     = txtData,
                colour    = p.c,
                seismic   = txt[ldata].seismic,
            })
        end

        if i < -(htOfText * rfact) then
            ts[tc[sNo]]   = preset
            sData         = 1
            eData         = txtLimit * 2
            cnt2          = txtLimit - 1
            txt[ldata].mod = 0
        end

        -- local tilt = "1:0:-0.3:0.9"
        -- local matrix0 = cairo_matrix_t:create()
        -- tolua.takeownership(matrix0)
        -- local t1 = wxsplit(tilt,':')
        -- lcairo_matrix_init(matrix0, tonum(t1[1]),tonum(t1[2]),tonum(t1[3]),tonum(t1[4]),0,0)
        -- lcairo_translate(cr,px,py)
        -- lcairo_transform(cr,matrix0)
        -- lcairo_translate(cr,-px,-py)

    elseif gr[goRight] then
        if goRight == 'b' then
            direction = 1
            lcairo_text_extents(cr,sm[ldata],extents)
            txtData = sm[ldata]
            if i < px then i = -(i - (px*2)) end
            if i > (w.widgetWidth-extents.width)+px+14 then
                if ts[tc[sNo]] then
                    ts[tc[sNo]] = -((w.widgetWidth-extents.width)+px+14)
                end
            end
        else
            txtData = txt[ldata]
            if goRight == 'r' then
                txtData = ' <:> '..txtData;
                direction = -1; preset = 30
                lcairo_text_extents(cr,txtData,extents)
            elseif goRight == 'l' then
                lcairo_text_extents(cr,txtData,extents)
                direction = 1; preset = (i + extents.width + 23 )
            end
            if i < px then
                local txtData1 = ' <:> '..txtData
                lcairo_text_extents(cr,txtData1,extents)
                lcairo_move_to(cr,
                    direction * (i + extents.width + px - 28 ),
                    py-15)
                if goRight == 'r' then
                    lcairo_show_text(cr, txtData)
                elseif goRight == 'l' then
                    lcairo_show_text(cr, txtData1)
                end
            end
            if i <= -(extents.width) then
                if ts[tc[sNo]] then ts[tc[sNo]] = preset end
            end
        end
        lcairo_move_to(cr, direction * (i + px ), py-15)
        lcairo_show_text(cr, txtData)
    end
    return ''
end

lcollectgarbage()

return w

--[[ Notes etc...

options for NOT
    s:some text NOT some text --> s:king kong NOT kung kang
    range NOT some text       --> 2..67 NOT king kong
    indexes and or ranges NOT some text --> 2,4,7..19 NOT some text


]]
