--[[
	Paramvir Likhari - conkywx 2011 - 2016 - GPL3
	common library for use with conkywx lua implementation
--]]

local tconcat = table.concat
local slower  = string.lower
local supper  = string.upper
local tonum   = tonumber
local sprintf = string.format
local lmatch  = string.match
local lgsub   = string.gsub
local floor   = math.floor

-- package.path = package.path .. ";./lib/?.lua;"


local FC = require("luafunc")

local wxsplit  = FC.wxsplit
local ExecInfo = FC.ExecInfo
local getopts  = FC.getopts
local msgout   = FC.msgout

local w, sensdata = {}, {}
local lastUpdate  = 0

local function getCpuCount()
    local cpuNum = ExecInfo({ rfile = "/sys/devices/system/cpu/present" })
    if cpuNum then
        local cpuTable = wxsplit(cpuNum,'-')
        return cpuTable[2] + 1
    end
    return ''
end


function w.sysinfo(...)
    -- by paramvir 2015 - 2018 conkywx

    local p = {}
    local clnp
    if type(...) == 'table' then
        p = {...}
    elseif type(...) == 'string' then
        if lmatch(...,'\\') then
            clnp = lgsub(...,'\\','')
        else
            clnp = (...)
        end
        p = getopts(clnp)
    else
        error( "\nNo lua command line options given to sysinfo lua!!!\n")
    end

    local platform   = p.p
    local fahrenheit = p.f  or 0
    local wanted     = p.w

    -- msgout(...,p.w,'aaaaaaaaa')

    if tonum(fahrenheit) == 0 then fahrenheit = nil end

    local updates = tonum(conky_parse("${updates}"))

    local retval
    if wanted then
        local opt
        if platform then
            local monCall
            if wanted:match('pch') or wanted:match('fan') then
                if wanted:match('pch') then wanted = 'temp4' end
                monCall = '/sys/bus/platform/devices/'
            elseif wanted:match('cpu') then
                wanted = 'temp' .. wanted:match('%d+') + 1
                monCall = '/sys/class/hwmon/hwmon0'
                platform = ''
            end

            platform = monCall .. platform .. '/' .. wanted .. '_input'

            local sysStatus = ExecInfo({ rfile = platform }):match('%d+')

            if not sysStatus then
                error("\nError :(\nsysinfo wrong arguments")
            else
                if wanted:match('fan') then
                    retval = sysStatus
                elseif fahrenheit then
                    retval = sprintf("%03d",((tonum(sysStatus)/1000) * 1.8) + 32)
                else
                    retval = tonum(sysStatus)/1000
                end
            end
        else
            -- local wantTest = { 'cpu' = 1, 'fan'=1, 'vcore'=2, 'in'=1, '3vsb'=3, 'vbat'=2, 'systin'=3, 'cputin'=3, 'peci'=3, 'pch'=3, }
            if wanted then wanted = supper(wanted) end
            local to_run = 'sensors'
            local pu
            if wanted:match('CPU0') then wanted = 'Package id ' .. wanted:match('%d+') end
            if wanted:match('CPU') then wanted  = 'Core ' .. floor(wanted:match('%d+') - 1) end
            if wanted:match('PCH') then wanted  = 'PCH_CHIP' end

            if fahrenheit then to_run = to_run .. ' -f' end


            if lastUpdate ~= updates then
                local line = ExecInfo({ cmd = to_run })
                sensdata = wxsplit(line,'\n')
            end
            for k = 1, #sensdata do
                if wanted:match('FAN') then wanted = slower(wanted) end
                if sensdata[k]:match(wanted) then
                    retval=sensdata[k]:match('^.*:%s*+?(%d+)')
                end
            end
            lastUpdate = updates
        end
    else
        -- error("\nError :(\nsysinfo expects argument(s)")
        return 'ErrSysinfo'
    end

    return retval or ''
end

collectgarbage()
return w

