
--[[
    Paramvir Likhari - conkywx 2011 - 2018 - GPL3
    better image display with transparency
--]]

local w = { true }

local lcollectgarbage     = collectgarbage
local lcairo_text_extents = cairo_text_extents
local lcairo_save         = cairo_save
local lcairo_restore      = cairo_restore
local lconky_parse        = conky_parse
local lconky_window       = conky_window

local PC         = require('pre_exec')
local pre_exec   = PC.pre_exec

local FC         = require('luafunc')
-- local background = FC.background
local FileExists = FC.FileExists
-- local msgout     = FC.msgout
local ExecInfo   = FC.ExecInfo
local wxsplit    = FC.wxsplit
local make_txt   = FC.make_txt
local set_font   = FC.set_font
local put_image  = FC.put_image
local getopts    = FC.getopts

local OS         = require('os')
local lexecute   = OS.execute
-- local lremove    = OS.remove
-- local lexit      = OS.exit

local ST         = require('string')
local lmatch     = ST.match
local lgsub      = ST.gsub
local lrep       = ST.rep
local supper     = ST.upper

local TB         = require('table')
local tconcat    = TB.concat
local tsort      = TB.sort

local GF         = require("sysinfo")
local sysinfo    = GF.sysinfo

local floor  = math.floor

local tonum      = tonumber
-- local ts = {}
local sNo, extents, width, swidth
-- local wxspacer, wyspacer = 0,0

local cnTable = {}
local cn = 1
local function getCounter(infile)
    if not cnTable[infile] then
        cnTable[infile] = cn
        cn = cn + 1
    end
    for k, v in pairs(cnTable) do
        if k == infile then
            return v
        end
    end
end

local function nvidia(p)

    local opts = p.opts or 'd'
    local req  = p.req or ''

    if ExecInfo({ cmd = [[ type -p nvidia-smi ]] }) ~= '' then
        local nstr
        if lmatch(opts,'[rR]1') then
            nstr = pre_exec( 'nvidia-smi -q' )
        else
            nstr = ExecInfo({ cmd = 'nvidia-smi -q' })
        end

        local ntbl = wxsplit(nstr, '\n')

        for x = 1, #ntbl
        do
            if lmatch(supper(ntbl[x]), supper(req)) then
                if lmatch(opts, '[sS]') then
                    -- return NA1 if string required but not possible
                    return (lmatch(ntbl[x], ':%s+([%w. ]+)%s*') or 'NA1')
                elseif lmatch(opts, '[dD]') then
                    local retval
                    -- return NA2 if digit required but not possible
                    retval = (lmatch(ntbl[x], ':%s+([%d.]+)%s*') or 'NA2')
                    if lmatch(opts, '[fF]') then
                        -- 0.5 added to get closest degree
                        return floor((retval * 1.8) + 32 + 0.5)
                    else
                        return retval
                    end
                else
                    -- return NA3 if nothing possible - check to see if required
                    return 'NA3'
                end
            end
        end
    else
        -- return NA4 if nvidia-smi program is missing
        return 'NA4'
    end

    return 'NA'

end

local function fixcli(p)

    local lhs = p.opts or ''
    local cr = (p.cr or nil)

    if lmatch(lhs,'@nvidia') then
        if  not FileExists([[/tmp/yesHaveNvidia]]) and
            not FileExists([[/tmp/noHaveNvidia]]) then
            local haveNvidia
            haveNvidia = ExecInfo({ cmd = [[lspci | grep -im1 'nvidia' ]]})
            if not haveNvidia or haveNvidia == '' then
                haveNvidia = ExecInfo({ cmd = [[dmesg | grep -im1 'nvidia' ]]})
            end
            if haveNvidia ~= '' then
                lexecute([[touch /tmp/yesHaveNvidia]])
            else
                lexecute([[touch /tmp/noHaveNvidia]])
            end
        end
        if FileExists([[/tmp/yesHaveNvidia]]) then
            local sh, lh1, lh2, eh = lmatch(lhs, '^(.-)@nvidia%s+(.-)@(.-)@(.-)$')
            local nvVal = nvidia({ opts = (lh1 or ''), req = (lh2 or '')})
            if lmatch(nvVal, 'NA') then
                return (sh or '') .. nvVal
            else
                return (sh or '') .. nvVal .. (eh or '')
            end
        else
            return ''
        end
    end

    if lmatch(lhs,'@conky') then
        local val, val1 = lmatch(lhs, '@(.-)%s+\\(.-)@')
        local valt = wxsplit(ExecInfo({cmd = val .. ' ' .. val1}),'\n')
        if lmatch(lhs,'conkywx') then
            return valt[1]
        elseif lmatch(lhs,'conky ') then
            valt = wxsplit(valt[1],'%s+')
            return valt[2]
        end
    end

    if lhs and lmatch(lhs, '%$%{.*') then
        if lmatch(lhs, 'lua%s+sysinfo') then
            local lh1, rh = lmatch(lhs,'sysinfo%s+\\(.-)%}(.-)$')
            return sysinfo( lh1 ) .. rh
        elseif lmatch(lhs, 'lua%s+main') or
           lmatch(lhs, '\\%-n%s+')   then
            local progname, progopts = lmatch(lhs,'\\%-n%s+(.-)%s+(.*)%}')
            if progopts then
                progopts = lgsub(progopts,'\\%-','-')
                -- if lmatch(lhs,'|') then msgout(progname,progopts) end
                local opts     = getopts(progopts)
                local PSL      = require(progname)
                lcairo_save(cr)
                opts.cr = p.cr
                PSL[progname](opts)
                lcairo_restore(cr)
            end
            return ''
        else
            return lconky_parse(lhs)
        end
    end

    return lhs

end


----------------------------------------------------------------------
function w.textformat(p)

    p.l = p.l or p.e

    sNo = getCounter((p.l or p.e))
    local var = {}
    local wday = {
        -- ['Sunday']    = 'Sun',
        -- ['Monday']    = 'Mon',
        -- ['Tuesday']   = 'Tue',
        -- ['Wednesday'] = 'Wed',
        -- ['Thursday']  = 'Thu',
        -- ['Friday']    = 'Fri',
        -- ['Saturday']  = 'Sat',
        ['zzz']       = ''
    }

    if p.e and FileExists(p.e) then
        local fileI = ExecInfo({ rfile = p.e })
        if fileI then
            local L1, ii, it, dayFactor
            if p.b then
                local daysPerColumn = p.b or 1
                if daysPerColumn == 1 then
                    dayFactor = 0
                else
                    dayFactor = 1
                end
                L1 = lgsub(fileI, "[^\n]+",
                    function(pa)
                        it = (it or 0) + 1
                        -- if lmatch(pa, 'day') then
                        if lmatch(pa, 'zzz') then
                            ii = (ii or 0) + 1
                            if ii % daysPerColumn == dayFactor and it > 3 then
                                return ';,,' .. pa .. ','
                            else
                                return pa .. ','
                            end
                        else
                            return pa .. ','
                        end
                    end
                )
            else
                L1 = lgsub(fileI,'\n',',')
            end
            p.l = lgsub(L1, "%w+",
                            function(pb)
                                -- if lmatch(pb, 'zzz') then
                                --     print(pb)
                                -- end
                                return wday[pb] or pb
                            end
                        )
        end
    end

    local cr = p.cr

    local rows, cols = {}, {}
    local _
    local numofcols = 0
    local numofrows = 0
    -- local columnsToDisplay = 0
    local colsep = p.q or ','
    local rowsep = p.t or ';'

    if lmatch(p.l, rowsep) then
        _, numofcols = lgsub(p.l,rowsep,1)
        rows = wxsplit(p.l, rowsep)
        _, numofrows = lgsub(rows[1],colsep,1)
        for x = 1, #rows
        do
            cols[x] = wxsplit(rows[x], colsep)
        end
    elseif not lmatch(p.l, ';') then
        _, numofcols = lgsub(p.l,colsep,1)
        cols[1] = wxsplit(p.l, colsep)
    end

    -- print(numofcols,cols[1])

    local conkywidthDoubleColumn = lconky_window.width - (p.lwx or 0)
    local conkywidth = lconky_window.width

    -- local orgnumofcols = numofcols
    -- local orgnumofrows = numofrows
    numofcols = numofcols + 1
    numofrows = numofrows + 1

    if p.g then
        numofrows = numofcols
    end

    extents  = cairo_text_extents_t:create()
    tolua.takeownership(extents)

    local alignFactor = tonum(p.a) or 0

    var.sNo = {
        ['st1']       = cols,
        ['lcolour']   = wxsplit((p.c or '0xffffff:1'), ':'),
        ['rcolour']   = wxsplit((p.d or '0xffffff:1'), ':'),
        ['numofcols'] = numofcols,
        ['gotoCol']   = (( p.w or conkywidth ) / numofcols),
        ['gotoColImage'] = ( p.w or conkywidthDoubleColumn ) / numofcols,
        ['gotoRow']   = (( tonum(p.h) or 30 ) / numofrows),
        ['yoffset']   = (( tonum(p.h) or 30 ) / (numofrows ^ 2)) * 0.75,
        ['wx']        = (p.lwx or 0),
        ['wy']        = (p.lwy or 0),
        ['font']      = p.f or 'Mono',
        ['valign']    = p.a or 'l',
        ['halign']    = p.a or 'c',
        ['sep']       = p.r or ':',
    }

    local font
    if var.sNo.font then
        font = var.sNo.font
    else
        font = 'DejaVu Sans:10'
    end

    local columnsToColour
    if p.k == 1 then
        columnsToColour = tonum(p.k) + 1
    else
        columnsToColour = 1 + numofcols
    end

    -- msgout(var.sNo.updates, os.time())

    lcairo_save(cr)
    set_font({ cr, font })
    lcairo_restore(cr)

    local orginalColour = var.sNo.lcolour
    local orginalAlign  = var.sNo.valign

    for i = 1, #var.sNo.st1
    do
        local st2t,st3t,stst = {},{},{}
        local wxdir, wxspacer, flag, wydir, wide, yfact

        for j = 1, #var.sNo.st1[i]
        do
            local lhs, rhs, val, wxj, wyj, prelhs
            val = var.sNo.st1[i][j]
            if lmatch( val, var.sNo.sep ) and not lmatch( val, '0x' ) then
                lhs, rhs = lmatch(val, "^(.-)" .. var.sNo.sep .. "(.-)$")
            else
                lhs = val
            end

            wxj = var.sNo.wx + (var.sNo.gotoCol * (j - 1))
            wyj = var.sNo.wy + (var.sNo.gotoRow * (i - 1))

            if lmatch(lhs, '@[hHvV]') then
                wxdir, wxspacer = lmatch(lhs,'@([hH])([%d.-]+)@')
                wydir, wide, yfact = lmatch(lhs,'@([vV])([Ww]?)([%d.-]+)@' )
                wyj = wyj + (tonum(yfact) or 0)
                if wxdir and lmatch(wxdir, '[hH]') then
                    wxj = wxj + tonum(wxspacer)
                end
            end

            local localWidth
            local widgetWidth = (p.z or 100)
            if p.a == 'c' then
                localWidth = ((conkywidth - 5) / 2) - (widgetWidth / 2)
                wxj = wxj + localWidth
            elseif p.a == 'r' then
                localWidth = (conkywidth - 10) - widgetWidth
                wxj = wxj + localWidth
            end

            if lmatch(lhs, 'addpinfo') then
                if lmatch(lhs, 'wxrings') then
                    local offset = {}
                    if p.u and lmatch(p.u, ':') then
                        offset = wxsplit(p.u,':')
                    end
                    local m2, r2 = 312 / 580, 520 / 580
                    local ringData = tconcat{
                        ' -c ',
                        wxj + (offset[1] or 0),
                        ' -m ',
                        (conkywidth * m2) + (offset[2] or 0),
                        ' -b ',
                        (conkywidth * r2) + (offset[3] or 0),
                        ' -p ', wxj,',', wyj,
                    }
                    prelhs = lgsub(lhs, 'addpinfo', ringData )
                else
                    prelhs = lgsub(lhs, 'addpinfo', ' -p ' .. wxj .. ',' .. wyj )
                end
            -- elseif lmatch(lhs, 'cpubar') then
            --     -- prelhs = tconcat({ '${voffset ',tonum(wxj),'}','${offset ',tonum(wyj),'}', lhs })
            --     prelhs = tconcat({ '${voffset ',500,'}','${offset ',15,'}', lhs })
            --     -- print(prelhs)
            --     -- prelhs = lhs
            else
                prelhs = lhs
            end

            if  lmatch(lhs, '%$%{.*')  or
                lmatch(lhs, '@nvidia') or
                lmatch(lhs, '@conky')  then
                lhs = fixcli({cr = p.cr, opts = prelhs})
            end

            if not stst[i] then stst[i] = {} end

            if not st2t[i] then st2t[i] = {} end
            st2t[i][j] = (lhs or nil)
            if not st3t[i] then st3t[i] = {} end
            st3t[i][j] = (rhs or nil)

            if lhs and rhs then
                if lmatch(lhs, '@.-@') then
                    lhs = lmatch(lhs, '@.-@(.-)$')
                end
                stst[i][j] = (lhs or 'NA')
            else
                stst[i][j] = ''
            end

        end

        if not p.s or stst[i] and stst[i] ~= 'NA' then
            tsort(stst[i], function(a,b) return #a<#b end)
            local colonWidth = stst[i][#stst[i]]

            lcairo_text_extents(cr,colonWidth,extents)
            width   = (extents.width or 0)

            local offset = {}
            if p.o and lmatch(p.o, ':') then
                offset = wxsplit(p.o,':')
            else
                offset[1] = p.o or 0
            end
            width = (offset[i] or 0) + width
        end

        local soffset = {}
        if p.u and lmatch(p.u, ':') then
            soffset = wxsplit(p.u,':')
        else
            soffset[1] = p.u or 0
        end
        -- local offset = (p.u or 0)
        swidth = (soffset[i] or 0)

        lcairo_save(cr)
        local wxx, wyx
        wxdir, wxspacer, wydir, wide, yfact = nil,nil,nil,nil,nil
        flag = width

        for x = 1, #st2t[i]
        do
            if p.g then
                wxx = var.sNo.wx
                wyx = var.sNo.wy + (var.sNo.gotoRow * (x - 1)) + var.sNo.yoffset * (i - 1)
            else
                if st3t[i][x] then
                    wxx = var.sNo.wx + (var.sNo.gotoColImage * (i - 1))
                else
                    wxx = var.sNo.wx + (var.sNo.gotoCol * (i - 1))
                end
                wyx = var.sNo.wy + (var.sNo.gotoRow * (x - 1))
            end

            if lmatch(st2t[i][x], '@[hHvV]') then
                wxdir, wxspacer = lmatch(st2t[i][x],'@([hH])([%d.-]+)@')
                wydir, wide, yfact = lmatch(st2t[i][x],'@([vV])([Ww]?)([%d.-]+)@' )
                if wydir and lmatch(wydir,'[Ww]') then
                    st2t[i][x] = lgsub(rlhs,'[%w+ ]','%1 ')
                end
            end

            if wxdir and lmatch(wxdir, '[hH]') then
                wxx = wxx + tonum(wxspacer)
                width = flag - tonum(wxspacer)
            end

            if wydir and lmatch(wydir, '[vV]') then
                wyx = wyx + (tonum(yfact) or 0)
            end

            if lmatch(st2t[i][x], '@.-') then
                st2t[i][x] = lmatch(st2t[i][x], '@.*@(.-)$')
            end

            if p.e and lmatch(st2t[i][x], 'LT') then
                var.sNo.lcolour = var.sNo.rcolour
            elseif p.e and lmatch(st2t[i][x], 'HT') then
                var.sNo.lcolour = var.sNo.rcolour
            else
                var.sNo.lcolour = orginalColour
            end

            local wid
            local widgetWidth = (p.z or 20)
            if tonum(p.a) and i == 1 then
                var.sNo.valign = 'l'
            elseif p.a == 'c' and p.k then
                var.sNo.valign = 'c'
                wid = var.sNo.gotoCol
            elseif p.a == 'c' and i >= 1 then
                var.sNo.valign = 'c'
                wid = ((conkywidth - 5) / 2) - (widgetWidth / 2)
            elseif p.a == 'r' and i == 1 then
                var.sNo.valign = 'r'
            else
                if p.g then
                    var.sNo.valign = (p.a or 'l')
                elseif not stst[i][x] then
                    var.sNo.valign = 'r'
                    wxx = wxx + alignFactor
                elseif stst[i][x] then
                    var.sNo.valign = 'l'
                else
                    var.sNo.valign = orginalAlign
                end
            end

            if i == i % columnsToColour then
                var.sNo.lcolour = var.sNo.lcolour
                -- wid = var.sNo.gotoCol
            elseif i ~= i % columnsToColour then
                var.sNo.lcolour = var.sNo.rcolour
                var.sNo.valign  = 'r'
                wid = var.sNo.gotoCol
            else
                var.sNo.lcolour = orginalColour
            end

            if p.k and tonum(p.k) == 2 then
                if x ~= x % 2 then
                    var.sNo.lcolour = var.sNo.lcolour
                elseif x == x % 2 then
                    var.sNo.lcolour = var.sNo.rcolour
                else
                    var.sNo.lcolour = orginalColour
                end
            end

            if st2t[i][x] and lmatch(st2t[i][x], 'png$') then

                if p.i then
                    local iwidth = tonum(lmatch(p.i,'([%d.-]+)')) or 0
                    wxx = wxx + (tonum(iwidth) or 0)
                end

                local piData = {
                    cr    = cr,
                    x     = wxx,
                    y     = wyx,
                    file  = st2t[i][x],
                    scale = tonum(p.s)*.01,
                }
                put_image(piData)
            else
                local mkData = {
                    cr      = cr,
                    text    = st2t[i][x],
                    px      = wxx + swidth,
                    py      = wyx,
                    align   = var.sNo.valign,
                    icolour = var.sNo.lcolour,
                    ext     = extents,
                    wid     = wid,
                }
                make_txt(mkData)
                if st3t[i][x] then
                    if  lmatch(st3t[i][x], '%$%{.*')  or
                        lmatch(st3t[i][x], '@nvidia') or
                        lmatch(st3t[i][x], '@conky')  then
                        st3t[i][x] = fixcli({opts = st3t[i][x]})
                    end
                    local mkData = {
                        cr      = cr,
                        text    = st3t[i][x],
                        px      = wxx + width + 9,
                        py      = wyx,
                        align   = var.sNo.valign,
                        icolour = var.sNo.rcolour,
                        ext     = extents,
                        wid     = wid,
                    }
                    make_txt(mkData)
                    local mkData = {
                        cr      = cr,
                        text    = ':',
                        px      = wxx + width + 3,
                        py      = wyx,
                        align   = var.sNo.valign,
                        icolour = var.sNo.lcolour,
                        ext     = extents,
                        wid     = wid,
                    }
                    make_txt(mkData)
                end
            end
        end
        lcairo_restore(cr)
    end
end

lcollectgarbage()

return w

