
--[[
    Paramvir Likhari - conkywx 2011 - 2017 - GPL3
    Thermometer library for use with conkywx lua implementation
--]]

local w = {true}

local PI        = 3.14159265358979323846
local TORADIANS = 3.14159265358979323846/180

local sprintf = string.format
local tconcat = table.concat
local tonum   = tonumber
local supper  = string.upper
local find    = string.find
local match   = string.match

local lcollectgarbage = collectgarbage

local lcairo_move_to         = cairo_move_to
local lcairo_line_to         = cairo_line_to
local lcairo_rel_line_to     = cairo_rel_line_to
local lcairo_rotate          = cairo_rotate
local lcairo_show_text       = cairo_show_text
local lcairo_stroke          = cairo_stroke
local lcairo_fill            = cairo_fill
local lcairo_save            = cairo_save
local lcairo_restore         = cairo_restore
local lcairo_set_source_rgba = cairo_set_source_rgba
local lcairo_arc             = cairo_arc
local lcairo_set_line_width  = cairo_set_line_width
local lcairo_scale           = cairo_scale

-- package.path = package.path .. ";./lib/?.lua;"

local FC = require("luafunc")

local wxsplit      = FC.wxsplit
local to_rgba      = FC.to_rgba
local get_points   = FC.get_points
local do_set_font  = FC.set_font
local make_txt     = FC.make_txt
local colour_blend = FC.colour_blend

local function set_font(p)
    do_set_font(p)
end


function w.thermometer(p)
    local cr = p.cr
    local units = p.u or print("Thermometer Units missing")
    local scale = p.s or 1
    local temp  = p.t or 0
    local label = p.l or nil

    local scale_colour        = p.c or '0xcd950c:1'
    local scale_lines_colour  = p.d or '0xffd700:0.5'
    local scale_text_colour   = p.e or '0x4169e1:1'
    local text_readout_colour = p.f or nil --'0x4169e1:1'

    scale_colour        = wxsplit(scale_colour,":")
    scale_lines_colour  = wxsplit(scale_lines_colour,":")
    scale_text_colour   = wxsplit(scale_text_colour,":")
    text_readout_colour = wxsplit(text_readout_colour,":")

    temp  = (tonum(match(temp,'(%-?[%d%.]+)')) or 0)
    -- temp  = -20

    local gradHeight, scaleFactor, scaleSpacer, tVals = 150, 207, 7, {true}
    tVals = {
        F = {
            mid          = 75.2,
            ScFactor     = 137 / 160,
            tFactor      = 28,
            num          = {-20,"0°"..units,20,40,60,80,100,120,140},
        },
        C = {
            mid          = 24,
            ScFactor     = 137 / 80,
            tFactor      = 24,
            num          = {-20,-10,"0°"..units,10,20,30,40,50,60},
        },
    }

    local tempUsed
    if units == 'C' then
        if     temp < -20 then tempUsed = -20
        elseif temp >  60 then tempUsed =  60
        else   tempUsed = temp
        end
    else
        if     temp < -20 then tempUsed = -20
        elseif temp > 140 then tempUsed = 140
        else   tempUsed = temp
        end
    end
    local UnitFacT, UnitFacM
    UnitFacT = (tempUsed + tVals[units].tFactor) * tVals[units].ScFactor

    if scale > 10 then scale = scale / scaleFactor end
    local mx = (p.lwx * (1 / tonum(scale)) + 30)
    local my = (p.lwy * (1 / tonum(scale)) - 25) + 207

    local fsize = 10
    lcairo_scale(cr, scale, scale)
    lcairo_set_line_width(cr, 1.5)
    lcairo_set_source_rgba(cr, to_rgba(scale_colour))

    --graphics outer
    --bottom circle
    local radiusOuter, leftAngleOuter, rightAngleOuter, heightOuter
    radiusOuter     = 25
    leftAngleOuter  = 335
    rightAngleOuter = 360 - leftAngleOuter
    heightOuter     = gradHeight - 4
    lcairo_arc(cr, mx, my, radiusOuter, TORADIANS*(rightAngleOuter-90),
        TORADIANS*(leftAngleOuter-90))
    local lxo, lyo, rxo, ryo
    lxo, lyo = get_points(leftAngleOuter, 0, 0, radiusOuter)
    rxo, ryo = get_points(rightAngleOuter, 0, 0, radiusOuter)
    lcairo_arc(cr, mx+lxo+((rxo-lxo)*0.5), my+lyo-heightOuter, (rxo-lxo)*0.5, TORADIANS*180, 0)
    lcairo_line_to(cr,mx+lxo+((rxo-lxo)),my+lyo)
    lcairo_stroke(cr)

    ----------------------------------------------
    --graphics inner
    local colr, colg, colb, cola
    colr, colg, colb, cola = colour_blend(tempUsed, tVals[units].mid, gradHeight)
    lcairo_set_source_rgba(cr, colr, colg, colb, 1)

    --bottom circle
    local radiusInner, leftAngleInner, rightAngleInner, hIinner
    radiusInner     = radiusOuter-6
    leftAngleInner  = leftAngleOuter+9
    rightAngleInner = 0+(360-leftAngleInner)
    hIinner         = UnitFacT
    lcairo_arc(cr, mx, my, radiusInner, TORADIANS * (rightAngleInner-90), TORADIANS*(leftAngleInner-90))
    local lxi, lyi, rxi, ryi
    lxi, lyi = get_points(leftAngleInner,0,0,radiusInner)
    rxi, ryi = get_points(rightAngleInner,0,0,radiusInner)
    lcairo_arc(cr, mx+lxi+((rxi-lxi)*0.5),my+lyi-hIinner,(rxi-lxi)*0.5, TORADIANS*180, 0)
    lcairo_fill(cr)

    ----------------------------
    lcairo_set_line_width(cr,1)
    lcairo_set_source_rgba(cr, to_rgba(scale_lines_colour))
    local grad, lnx, lny, lnw
    grad = 8.5
    lnx  = mx  + lxo
    lnw  = rxo - lxo
    for i = 1, (#tVals[units].num*2)-1 do
        lny = my - radiusInner - (scaleSpacer + ((i-1) *grad)) - ((rxi-lxi) / 2)
        lcairo_move_to(cr,lnx,lny)
        lcairo_rel_line_to(cr,lnw,0)
        lcairo_stroke(cr)
    end
    --numbers
    lcairo_set_source_rgba(cr, to_rgba(scale_text_colour))
    lcairo_save(cr)
    set_font({ cr, tconcat({'Mono:',fsize,':normal'}) })
    lcairo_restore(cr)
    local extents = cairo_text_extents_t:create()
    tolua.takeownership(extents)

    grad=17
    lnx = mx + lxo + (rxo - lxo) + 4
    for i = 1, #tVals[units].num do
        lny = my - radiusInner - (scaleSpacer + (( i - 1 ) *grad)) - ((rxi - lxi)/2)+(fsize/3)
        lcairo_move_to(cr,lnx,lny)
        lcairo_show_text(cr,tVals[units].num[i])
        lcairo_stroke(cr)
    end

    fsize=12.5
    lcairo_save(cr)
    set_font({ cr, tconcat({'Mono:',fsize,':bold'})})
    lcairo_move_to(cr, mx+lxo-5, my-radiusInner-10-((rxi-lxi)/2))
    lcairo_rotate(cr,TORADIANS*(-90))
    lcairo_show_text(cr,label)
    lcairo_restore(cr)
    if not text_readout_colour then
        lcairo_set_source_rgba(cr, 0.5-colr, -colg, 0.5-colb, 1)
    else
        lcairo_set_source_rgba(cr, to_rgba(text_readout_colour))
    end
    lcairo_save(cr)
    local mkData = { cr = cr, ext=extents, text=tconcat({temp,"°",units}), px=mx, py=my-3, align='bc' }
    make_txt(mkData)
    lcairo_restore(cr)
    ------------------------------------
    lcairo_scale(cr,1/scale,1/scale)
end

lcollectgarbage()

return w
