package trends;

=comments
    Parses trend graphs and arrows
    This version (c) 2018 Paramvir Likhari
=cut

$trends::VERSION = '7';

require Exporter;
@trends::ISA = qw(Exporter);
@trends::EXPORT = qw( trendGraphs trendArrows ); # function to export

use strict;
use warnings;

sub msgout  { print STDERR "@_ \n" }

## any reference to functions in the main program can be called like main::calltofunction()

# -----------------------------------------------------------------------
# get forecast trends and arrow fonts
# -----------------------------------------------------------------------

my %naList = qw(-999.00 1 -999.0 1 -999 1 -9999.00 1 -9999.0 1 -9999 1 N/A 1 NA 1 null 1 "" 1 -- 1 CALM 1 Calm 1 calm 1 0NA 1 NA0 1 ) ;

sub normalize {

    my @arr002 = @_;
    my ( @arr001, $nval, $intval, $min, $max);
    my $fval = "";

    # control limits
    my $a1 = 0;
    my $b1 = 5;

    for (@arr002) {
        $_ = ( $naList{$_} or $_ == 0 ) ? 0.1 : $_ ;
        $min = $_ if !$min || $_ < $min;
        $max = $_ if !$max || $_ > $max;
    }
    for (@arr002) {
        if ( $max != $min ) {
            $nval = $a1+($_-$min)*(($b1-$a1)/($max-$min));
        }
        else {
            $nval = $a1+($_-$min)*(($b1-$a1)/1);
        }
        $intval = $nval - int($nval);
        push @arr001, ($intval>=0.5) ? int($nval + (($intval)+0.5)) : int($nval) ;
    }

    return join ',', @arr001 ;

}


sub trendArrows {

    my ( $nowtime, $forecast_trends, $noFileMsg, $fcttemp, $uvNow, $ftNow, $ftNow24, $conf ) = @_ ;

    my @confArr = split(/\|/, $conf );

    my $steadytrend  = $confArr[0];
    my $fallingtrend = $confArr[1];
    my $risingtrend  = $confArr[2];

    my ( $trend1, $trend_arrow1, @trend, @trend_arrow );

    my %today_condF = (
        Rising  => 'Today will be WARMER than yesterday.',
        Falling => 'Today will be COOLER than yesterday.',
        Steady  => 'Today will be the SAME as yesterday.',
        NA      => 'Forecast data is not available.',
    );
    my %today_condS = (
        Rising  => 'Warmer',
        Falling => 'Cooler',
        Steady  => 'Same',
        # Rising  => 'WARMER',
        # Falling => 'COOLER',
        # Steady  => 'SAME',
        NA      => 'NA',
    );

    my @ft_now = split( /\|/ , $ftNow);
    my @ft_now_24 = split(/\|/, $ftNow24);

    my @full_fc_data = ( "$nowtime @ft_now", "\n$nowtime @ft_now", "\n$nowtime @ft_now_24", "\n$nowtime @ft_now_24" );

    unless ( -e $forecast_trends ) {
        open (out_data, '>', "$forecast_trends") or die "$noFileMsg forecast save : $!";
            print out_data "@full_fc_data";
        close (out_data);
    }

    open (fc_read, '<', "$forecast_trends") or die "$noFileMsg forecast read : $!";
        my @fcarray = <fc_read>;
    close (fc_read);

    my @trends_now      = split(" ", $fcarray[0]);
    my @trends_store    = split(" ", $fcarray[1]); # we check time to this
    my @trends_now_24   = split(" ", $fcarray[2]);
    my @trends_store_24 = split(" ", $fcarray[3]); # we check time for this

    ### reset forecast data if old data
    if ( scalar( @trends_store ) < 9 or scalar( @trends_store_24 ) < 9 ) {
        unlink( $forecast_trends );
        exec($^X, $0, @ARGV);
    }

    # handle moon reaching 0 or 100 % illumination
    if (( $ft_now_24[6] == 100 ) and ( $trends_now_24[7] != 100 )) {
        $trends_now_24[7]     =   100;
        $trends_store_24[7]   =   100;
    }
    elsif (( $ft_now_24[6] == 0 ) and ( $trends_now_24[7] != 0 )) {
        $trends_now_24[7]     =   0;
        $trends_store_24[7]   =   0;
    }
    elsif (( $ft_now_24[6] eq "NA" ) or ( $trends_now_24[7] eq "NA" )) {
        unlink "$forecast_trends"
    }

    # Consolidate present and stored data
    my @nowdata    =   ( @ft_now, @ft_now_24 );
    my @nowstored  =   ( @trends_now[1..$#trends_now], @trends_now_24[1..$#trends_now_24] );

    ### limits for the 2 data sets
    my $hrLimit  = $#ft_now ;
    my $dayLimit = $hrLimit + 4;
    # msgout($hrLimit,$dayLimit,@nowstored);

    foreach my $l (0..$#nowdata) {
        # msgout($l,$nowdata[$l],$nowstored[$l]);
        if (( $nowdata[$l] eq "NA" ) or ( $nowstored[$l] eq "NA" )) {
            $trend1         =   "-";
            $trend_arrow1   =   "-";
        }
        else {
            if ( $nowdata[$l] == $nowstored[$l] ) {
                $trend1         =   "Steady" ;
                $trend_arrow1   =   $steadytrend ;
            }
            elsif ( $nowdata[$l] < $nowstored[$l] ) {
                if (( $l > $hrLimit ) && ( $l < $dayLimit )) {
                    $trend1     =   "Earlier" ;
                }
                else {
                    $trend1     =   "Falling" ;
                }
                $trend_arrow1   =   $fallingtrend ;
            }
            elsif ( $nowdata[$l] > $nowstored[$l] ) {
                if (( $l > $hrLimit ) && ( $l < $dayLimit )) {
                    $trend1     =   "Later" ;
                }
                else {
                    $trend1     =   "Rising" ;
                }
                $trend_arrow1   =   $risingtrend ;
            }
            else {
                $trend1         =   "-";
                $trend_arrow1   =   "-";
            }
        }
        $trend_arrow1 = ($trend_arrow1) ? $trend_arrow1 : "-";
        # msgout($l,$nowdata[$l],$nowstored[$l],$trend1,$trend_arrow1);
        push @trend, $trend1 ;
        push @trend_arrow, $trend_arrow1 ;
    }

    my $time_diff2      =   $nowtime - $trends_store[0] ;
    my $time_diff_time  =   $nowtime - $trends_store_24[0] ;

    my @trendsOut = @trend ;
    push (@trendsOut, @trend_arrow);
    push (@trendsOut, ($today_condF{$trendsOut[14]}, $today_condS{$trendsOut[14]}) );

    # write after every half hour
    if ( $time_diff2 > 1800 ) {
        my @full_fc_data1 = ( "@trends_store", "\n$nowtime @ft_now",  "\n@trends_now_24", "\n@trends_store_24" );
        open (out_data, '>', "$forecast_trends") or die "$noFileMsg forecast 1800 : $!";
            print out_data "@full_fc_data1";
        close (out_data);
    }

    # write after every 24 hour
    # TODO: - check if this is valid or we need to increase time to 36 hours
    if ( $time_diff_time > 86400 ) {
        my @full_fc_data2=( "@trends_now", "\n@trends_store", "\n@trends_store_24", "\n$nowtime @ft_now_24" );
        open (out_data, '>', "$forecast_trends") or die "$noFileMsg forecast 86400 : $!";
            print out_data "@full_fc_data2";
        close (out_data);
    }

    return @trendsOut ;
}


sub trendGraphs {

    my ( $nowtime, $temppath, $noFileMsg, $ftNow, $wsg, $wxMainTitle ) = @_;

    my $orghistds = ( $wxMainTitle eq "darkSky" ) ?
        "$temppath/wxcurrdata.html" :
        "$temppath/solValueF" ;

    my ($hourlystr, %wxtrend ) ;

    if ( -s $orghistds ) {
        {
        open (FILE, "<:crlf", "$orghistds") or die "$noFileMsg trends file $!";
            local $/ = undef;
            $hourlystr = <FILE> ;
        close (FILE);
        }

        my $hourly = ${main::get_line_num( \$hourlystr, 'hours = [{','currently = {', 'str')};

        $hourly =~ s/hours = /{"data":/;
        $hourly =~ s/,\s*$/}/;

        my $wxhr = main::jsonParser( $hourly );

        # msgout($$wxhr{data}[0]{temperature});
        # msgout($$wxhr{data}[0]{pressure});
        # msgout($$wxhr{data}[0]{dewPoint});
        # msgout($$wxhr{data}[0]{humidity});
        # msgout($$wxhr{data}[0]{windSpeed});
        # msgout($$wxhr{data}[0]{windGust});
        # msgout($$wxhr{data}[0]{solar}{ghi});
        # msgout($$wxhr{data}[0]{windBearing});
        # msgout($$wxhr{data}[0]{visibility});
        # msgout($$wxhr{data}[0]{uvIndex});

        my $count = 1;
        for my $k (0..$#{$$wxhr{data}}) {
            if (($count <= 12) and ($nowtime < $$wxhr{data}[$k]{time})) {
                # msgout($count++,$k,$$wxhr{data}[$k]{time});
                $wxtrend{temp}  .= ($$wxhr{data}[$k]{temperature} ||= 0) .',';
                $wxtrend{atemp} .= ($$wxhr{data}[$k]{apparentTemperature} ||= 0) .',';
                $wxtrend{dp}    .= ($$wxhr{data}[$k]{dewPoint}    ||= 0) .',';
                $wxtrend{hum}   .= ($$wxhr{data}[$k]{humidity}    ||= 0) .',';
                $wxtrend{press} .= ($$wxhr{data}[$k]{pressure}    ||= 0) .',';
                $wxtrend{ws}    .= ($$wxhr{data}[$k]{windSpeed}   ||= 0) .',';
                $wxtrend{wsg}   .= ($$wxhr{data}[$k]{windGust}    ||= 0) .',';
                $wxtrend{sol}   .= ($$wxhr{data}[$k]{solar}{ghi}  ||= 0) .',';
                $wxtrend{vis}   .= ($$wxhr{data}[$k]{visibility}  ||= 0) .',';
                $wxtrend{uvi}   .= ($$wxhr{data}[$k]{uvIndex}     ||= 0) .',';
                $wxtrend{cor}   .= (($$wxhr{data}[$k]{precipProbability}     ||= 0) * 100) .',';
                # $wxtrend{wb}    .= $$wxhr{data}[$k]{windBearing} .',';
                $count++;
            }
        }

        # msgout($wxtrend{temp});

        $wxtrend{temp}  = &normalize( split(',', $wxtrend{temp} ) ) ;
        $wxtrend{atemp} = &normalize( split(',', $wxtrend{atemp}) ) ;
        $wxtrend{dp}    = &normalize( split(',', $wxtrend{dp}   ) ) ;
        $wxtrend{hum}   = &normalize( split(',', $wxtrend{hum}  ) ) ;
        $wxtrend{press} = &normalize( split(',', $wxtrend{press}) ) ;
        $wxtrend{ws}    = &normalize( split(',', $wxtrend{ws}   ) ) ;
        $wxtrend{wsg}   = &normalize( split(',', $wxtrend{wsg}  ) ) ;
        $wxtrend{sol}   = &normalize( split(',', $wxtrend{sol}  ) ) ;
        $wxtrend{vis}   = &normalize( split(',', $wxtrend{vis}  ) ) ;
        $wxtrend{uvi}   = &normalize( split(',', $wxtrend{uvi}  ) ) ;
        $wxtrend{cor}   = &normalize( split(',', $wxtrend{cor}  ) ) ;

        return %wxtrend
    }
}



1;
