package uvinfo;

=comments
    Parses Ultra Violet data
    This version (c) 2018 Paramvir Likhari
=cut

$uvinfo::VERSION = '7';

require Exporter;
@uvinfo::ISA = qw(Exporter);
@uvinfo::EXPORT = qw(getUVinfo); # function to export

use strict;
use warnings;

sub msgout  { print STDERR "@_ \n" }

## any reference to functions in the main program can be called like main::calltofunction()

sub getUVinfo {

    my ( $uvdate, $nowtime, $uvdata, $noFileMsg, $confElevation, $Lat, $Long, %wx ) = @_;

    my ( $uvMax, $uvNow );

    if ($Lat and $Long) {
        if ($Lat =~ /s|S/)  {
            ($Lat) = $Lat =~ m{(\d+(\.\d+)?)};
            $Lat *=  -1;
        }
        else {
            ($Lat)  = $Lat  =~ m{(-?\d+(\.\d+)?)};
        }
        if ($Long =~ /W|w/)  {
            ($Long) = $Long =~ m{(\d+(\.\d+)?)};
            $Long *=  -1
        }
        else {
            ($Long) = $Long =~ m{(-?\d+(\.\d+)?)}
        }
    }

    my $uv_url = 'http://www.temis.nl/uvradiation/nrt/uvindex.php?lon='. "$Long\\&lat=$Lat" ;

    # if ( $copytmp eq '__' ) {
        ### stat command 7th item is size and 9th is modify time
        if ( not -e $uvdata or (stat ($uvdata))[7] == 0 or ($nowtime - (stat ($uvdata))[9]) > 388800  ) {
            `wget -q  -o /dev/null --tries=99 --timeout=30 $uv_url -O $uvdata`;
        }
    # }

    my ( $uvstr  ) ;
    {
    open (FILE, "<", $uvdata) or die "$noFileMsg uvdata file $!";
        local $/ = undef;
        $uvstr = <FILE> ;
    close (FILE);
    }

    if (index($uvstr,'Invalid input; please try again') != -1) {
        unlink $uvdata;
        die "\nError :( get_uvdata: User position not correct!!\n\tPerhaps add one in the config file. $!"
    }

    my $uvtbl = ${main::get_line_num( \$uvstr, '<table border=2','</table>','str')};
    $uvtbl =~ tr/\n//d;
    my ($cloud_factor,$time_factor);
    for (split('</tr>',$uvtbl)) {
        my ($uvt,$uvn) = (m{;\s*(.*?)\s*</td.*rap>\s*(.*?)\s*</td.*</td})
        ? ($1,$2): ('NA','NA') ;
        $uvMax = $uvn if $uvt eq $uvdate ;
    }

    my $teststr = ($wx{cloud1t} and $wx{cloud1t} ne 'NA')
    ? $wx{cloud1t}
    : ($wx{uvwx} ne 'na')
    ? $wx{uvwx}
    : "Clear" ;
    $teststr = lc($teststr);

    my $cond1 = 'overcast|cloudy|ovc|tstorm|sleet|rain|snow';
    my $cond2 = 'broken clouds|bkn|mcloudy|fog|mist';
    my $cond3 = 'scattered clouds|sct|pcloudy|dust|smoke|haze';

    if    (index($cond1,$wx{uvwx}) != -1 ) { $cloud_factor = 0.31 }
    elsif (index($cond1,$teststr ) != -1 ) { $cloud_factor = 0.31 }
    elsif (index($cond2,$teststr ) != -1 ) { $cloud_factor = 0.71 }
    elsif (index($cond3,$teststr ) != -1 ) { $cloud_factor = 0.89 }
    else { $cloud_factor = 1 }

    if ( $wx{curMin} < $wx{srise} or $wx{curMin} > $wx{sset} )
        { $time_factor = 0 }
    elsif ( $wx{curMin} < $wx{strans} ) {
        $time_factor = ($wx{curMin} - $wx{srise})/($wx{strans} - $wx{srise})
    }
    elsif ( $wx{curMin} > $wx{strans} ) {
        $time_factor = ($wx{sset} - $wx{curMin})/($wx{sset} - $wx{strans})
    }
    elsif ( $wx{curMin} == $wx{strans} ) {
        $time_factor = 1;
    }

    my $elevation = ( $confElevation )
    ? $confElevation
    : ($wx{elevation})
    ? $wx{elevation}
    : 0 ;

    ($elevation) = ($elevation =~ m{(-?\d+(\.\d+)?)})
        ? ((((main::ft_mtr($1))/1000) * 0.06 ) + 1 ) : 1 ;


    if ( defined $uvMax ) {
        $uvNow = sprintf("%.1f",$uvMax * $cloud_factor * $time_factor * $elevation);
        $uvNow = main::uv_pollen($uvNow, 'UV');
    }
    else {
        $uvMax = 0;
        $uvNow = 0;
        msgout("\n\nWarning: The Max UV is not defined - please send a DEBUG report to the author.\n\n")
    }

    # msgout( $uvMax, $uvNow );
    # die;
    return ($uvMax, $uvNow)
}


1;
