package wxDate::wxDate;

=comments
	Parses date and time string to epoch time
    wxStrftime module is used to give user format to epoch time.
	This version (c) 2018 Paramvir Likhari
=cut

my $VERSION = "7";

require Exporter;
@wxDate::wxDate::ISA = qw(Exporter);
@wxDate::wxDate::EXPORT = qw(time2str str2time tz_offset strftime asctime);
@wxDate::wxDate::EXPORT_OK = qw(parse_date time2iso time2isoz);

use strict;
use Time::Local;
use Carp;

use FindBin;
use lib "$FindBin::RealBin" . "/lib";

use wxDate::wxStrftime;
use wxDate::wxZone;

use vars qw(@MoY %MoY);
@MoY = qw(JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC);
@MoY{@MoY} = (1..12);

sub msgout { print STDERR "@_ \n" }

# pre declare all regexes
my $y42 = '(?<y>\d\d\d?\d?(?![\:\-\+]|[A-Z]+))';
my $y4  = '(?<y>\d\d\d\d)';
my $ms  = '(?<ms>[JFMASOND][A-Z]+)';
my $md  = '(?<ms>\d\d?)';
my $md2 = '(?<ms>\d\d)';
my $d   = '(?<d>\d\d?)';
my $d2  = '(?<d>\d\d)';
my $h   = '(?<h>\d\d?)?';
my $h2  = '(?<h>\d\d)';
my $hh  = '(?<h>[\d\:\.\,]+)?';
my $m   = '(?<m>\d\d?)?';
my $m2  = '(?<m>\d\d)';
my $s2  = '(?<s>\d\d)';
my $s   = '(?<s>[\d\.]+)?';
my $a   = '(?<a>\s*[AP][M])?';
my $zn  = '(?<z>[-+]?\d\d?:?\d?\d?|(?![AP][M]\b|DMY|MDY)[A-Z]+)?';
my $o   = '(?<o>[-+]?\d\d?:?\d?\d?|(?![AP][M]\b|DMY|MDY)[A-Z]+)?';
my $ds  = '(\s*[-\/])';
my $dds = '(\s*|[\-\/])';
my $ts  = '(\s*[:.,](?![A-Z]+))';
my $tts = '(?:\s*|[:.,])';

my $datedmdy = qr/${d}${ds}${md}${ds}${y42}/o;       # day month num  year 4 2
my $datemddy = qr/${md}${ds}${d}${ds}${y42}/o;       # month num  day year 4 2
my $dateymdd = qr/${y4}${ds}?${md}?${ds}?${d}?/o;    # year 4 2 month num day
my $dateymns = qr/${y4}${md2}${d2}/o;                # year 4 2 month num day
my $ddms     = qr/${d}${dds}${ms}/o;                 # day month text
my $ddmsy    = qr/${d}${dds}${ms}${dds}${y42}/o;     # day month text
my $dmsd     = qr/${ms}${dds}${d}/o;                 # month text day
my $dmsdy    = qr/${ms}${dds}${d}${dds}${y42}/o;     # day month text
my $timehmns = qr/${h2}${m2}${s2}\s*${a}/o;          # hms
my $timehms  = qr/${h}${ts}?${m}${ts}?${s}\s*${a}/o; # hms
my $timehh   = qr/${hh}${a}/o;                       # hms 2
my $tzInfo   = qr/\s*${zn}\s*${o}\s*/o;              # timezone and offset

my @ta = (
    "(${dateymdd})\\s*[-T]?\\s*(${timehmns}|${timehh})(${tzInfo})",
    "(${h}${ts}${m}\s*${a}|$timehms)(${tzInfo})",
    "(${datedmdy})\\s*(${timehh})(${tzInfo})(?:DMY)?\\s*",
    "(${d}${ds}${md})\\s*(${timehh})\\s*(${y42})(${tzInfo})(?:DMY)?\\s*",
    "(${datemddy})\\s*${timehh}${tzInfo}(?:MDY)\\s*",
    "(${md}${ds}${d})\\s*(${timehh})\\s*(${y42})(${tzInfo})(?:MDY)\\s*",
    "(${ddmsy})\\s*(${timehh})(${tzInfo})",
    "(${ddmsy})(${tzInfo})\$",
    "(${ddms})\\s*(${timehh})\\s*(${y42}?)(${tzInfo})",
    "(${ddms})\\s*(${timehh})\\s*(${zn}?)\\s*(${y42}?)\\s*(${o}?)\\s*",
    "(${dmsdy})\\s*(${timehh})(${tzInfo})",
    "(${dmsdy})(${tzInfo})\$",
    "(${dmsd})\\s*(${timehh})?\\s*(${zn})?\\s+(${y42})?\\s*(${o})?\\s*",
);

# convert time string to formatted time
sub time2str {
	my $format = shift;
	my $time   = shift // time;
	my $opts   = shift;
    my $tEpoch;
    if ( index($format, '%') == -1 ) {
        if    ( not $time and length int($format) == 10 ) { $tEpoch = $format }
        elsif ( not $time and not $format ) { $tEpoch = time }
        else  { $tEpoch = str2time($format) }
        $format = '%a %b %d %T %Z %Y '
    }
    elsif (length int($time) != 10) {
        unless ( $tEpoch = str2time($time,$opts) )
        { carp "\nWarning :( time string \"$time\" to wxDate module not supported -" }
    }
    else { $tEpoch = $time }

    return strftime($format, localtime($tEpoch));
}


sub str2time {
    my $str  = shift ;
    my $opts = shift ;
    return undef unless defined $str;

    $str = ( $opts ) ? uc($str . " $opts") : uc $str;

    my $returnval = sub {
        my ($sec,$min,$hr,$ampm,$day,$mon,$yr,$tz,$tzoff) = @_;

        $hr = (substr($hr,0,1) eq ':') ? substr($hr,1,(length($hr)-1)) : $hr ;
        if ($hr =~ /[:.,]/) {
            ($hr,$min,$sec) = split(/[:.,]/,$hr);
        }

        my $tzTest = sub {
            my $zone = shift;
            if ($zone =~ /(\d\d\d?\d?)/) {
                my $factor = ($zone < 1) ? '-' : '+';
                if (length $1 == 2 or length $1 == 4 ) {
                    my $zone1 = substr($1,0,2);
                    if ($zone1 > 14 or $zone1 < -14) { return 'NA' }
                }
            }
            return $zone
        };
        if (&$tzTest($tz) eq 'NA' and not $yr) {$yr = $tz ; undef $tz}
        if (not $tz and $tzoff) { $tz = $tzoff; undef $tzoff }
        $tzoff = ( $tzoff ) ? tz_offset($tzoff) : 0;
        $tz    = ( $tz ) ? tz_offset($tz) : tz_offset() ;

        # Translate month name to number
        $mon = substr($mon,0,3);
        $mon = $MoY{$mon} ||
                     ($mon =~ /^\d\d?$/ && $mon >= 1 && $mon <= 12 && int($mon));
        # month correction after year fix if year fix required

        # If the year is missing, we assume first date before the current,
        # because of the formats we support such dates are mostly present
        # on "ls -l" listings.
        if (not $yr or not $mon or not $day) {
            my ($curMon, $curDay, $curYear);
            ($curDay, $curMon, $curYear) = (localtime)[3, 4, 5];
            $yr  = ($yr  eq '') ? $curYear + 1900 : $yr  ;
            $mon = ($mon eq '') ? $curMon  + 1    : $mon ;
            $day = ($day eq '') ? $curDay         : $day ;
        }
        elsif (length($yr) < 3) {
            # Find "obvious" year
            my $cur_yr = (localtime)[5] + 1900;
            my $m = $cur_yr % 100;
            my $tmp = $yr;
            $yr += $cur_yr - $m;
            $m  -= $tmp;
            $yr += ($m > 0) ? 100 : -100 if abs($m) > 50;
        }

        # month correction
        $mon--;

        # Make sure clock elements are defined
        $hr  = 0 unless $hr;
        $min = 0 unless $min;
        $sec = 0 unless $sec;

        # Compensate for AM/PM using index no worries re space b4 etc
        if ($ampm) {
            $hr  = 0  if $hr == 12 and index($ampm,'AM') != -1;
            $hr += 12 if $hr != 12 and index($ampm,'PM') != -1;
        }

        my $t = timegm($sec,$min,$hr,$day,$mon,$yr );
        return $t < 0 ? undef : ($t - $tz) - $tzoff ;
    };

    $str =~ s/^\s+//;                                        # remove leading space
    $str =~ s/\s+$//;                                        # remove trailing space
    $str =~ s/\(.*?\)//g;                                    # remove anything in brackets
    $str =~ s/(?:SUN|MON|TUE|WED|THU|FRI|SAT)[A-Z]*,?\s*//g; # Useless weekday
    $str =~ s/(?:LAST|UPDATED|ON|OFF)[A-Z]*,?\s*//g;         # Useless words
    $str =~ s/\,(?:\s+)/ /g;                                 # remove any commas


    my ($sec, $min, $hour, $day, $mon, $year, $ampm, $tz, $tzoff, $opts, $retv );
    {
        if ($str =~ /^\d\d\d\d.*$/ ) {
          if ($str =~ $ta[0]) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^\d\d?${ts}\d\d?(${ts}\d\d?)?.*$/ ) {
          if ($str =~ $ta[1]) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^\d\d?${ds}\d\d?.*DMY\s*$/ ) {
          my $dddd = join('|',(@ta)[2,3]);
          if ($str =~ $dddd) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^\d\d?${ds}\d\d?.*MDY\s*$/ ) {
          my $dddd = join('|',(@ta)[4,5]);
          if ($str =~ $dddd) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^\d\d?${dds}${ms}${dds}${y42}\s*\d\d?${ts}\d\d?(${ts}${s}?)?.*$/ ) {
          if ($str =~ $ta[6]) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^\d\d?${dds}${ms}${dds}${y42}.*$/ ) {
          if ($str =~ @ta[7]) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^\d\d?${dds}${ms}${tts}${timehh}.*$/ ) {
          my $dddd = join('|',(@ta)[8,9]);
          if ($str =~ $dddd) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^${ms}${dds}\d\d?${dds}${y42}\s*\d\d?${ts}\d\d?(${ts}${s}?)?.*$/ ) {
          if ($str =~ $ta[10]) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^${ms}${dds}\d\d?${dds}${y42}.*$/ ) {
          if ($str =~ @ta[11]) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^${ms}\s*\d\d?\s*$/ ) {
          if ($str =~ ${dmsd}) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        elsif ($str =~ /^${ms}${dds}\d\d?${tts}${h}(${tts}${m}${tts}${s})?${y4}?.*$/ ) {
          if ($str =~ @ta[12]) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
        else {
          my $dddd = qr/^${d}\s*${timehh}${tzInfo}/o;
          if ($str =~ $dddd) { return &$returnval(@+{qw(s m h a d ms y z o)}) }
        }
    }
    return undef;
}



sub time2iso (;$)
{
		my $time = shift;
		$time = time unless defined $time;
		my($sec,$min,$hour,$mday,$mon,$year) = localtime($time);
		sprintf("%04d-%02d-%02d %02d:%02d:%02d",
				$year+1900, $mon+1, $mday, $hour, $min, $sec);
}


sub time2isoz (;$)
{
		my $time = shift;
		$time = time unless defined $time;
		my($sec,$min,$hour,$mday,$mon,$year) = gmtime($time);
		sprintf("%04d-%02d-%02d %02d:%02d:%02dZ",
						$year+1900, $mon+1, $mday, $hour, $min, $sec);
}

1;

=comments

    The following formats do not have the day name like Tuesday etc -
    These get stripped during parsing as are not required for
    determining the date from date string.

    time zone and offset are supported - Numeric and AlphaNumeric but
    not the long format - GMT UTC CST +05:30 +0530

    Date with only digits can be ambiguous as we have day/month/year
    or month/day/year. So such dates have to be supplied with
    'DMY' or 'MDY' at the end of string to enforce the format.

    Supported formats
    '22/1/2017 4:44:52 dmy',
    '1/22/2017 4:44:52 pm GMT +5:30 mdy',
    '2:23 pm ',
    '2:23:45 pm ',
    '2017-01-09T05:23:52.410',
    '2017-01-09t05:00:10.4352 ',
    '2017-01-27T07:26:40Z' ,
    '1994-02-03-14:15:29 ' ,
    '1994-02-03 14:15:29 ' ,
    '19940203T141529 EST',
    '19940203T141529 ' ,
    '19940203141529 ' ,
    '19940203' ,
    '1994-02-03' ,
    '1999 10:02:18 GMT',
    '01-03-17,  5:30 pm mdy ',
    '01/03/2017 1:12:12 AM DMY',
    '01/03/2017 1:12:12 AM mdy',
    '01-09-2017 12:23:52  dmy',
    '01-09-1994 dmy',
    '09/01/2017 06:23:52 gmt DMY',
    '09/01 06:23:52 2017 DMY',
    '09/01/2017 6  gmt DMY',
    '09/01 6 pm 2017 dmy',
    '08 Feb 94 gmt' ,                        # old rfc850 HTTP format
    '09 Feb 1994' ,                          # proposed new HTTP format
    '03/Feb/1994' ,                          # common logfile format
    '03 Jan, 2017, 20:44',
    '14 Jan 2017 07:12:22',
    '15 January 12:23:12  2017',
    '15 January 12:23:12 GMT 2017',
    '15-January-17 12:23',
    '15-January-2017 2:23 ',
    '15-jan-2017 2:23 ',
    '08-Feb-94 14:15:29 ' ,                  # old rfc850 HTTP format
    '08-Feb-1994 14:15:29 ' ,                # broken rfc850 HTTP format
    '11 February 7:20:50 PM',
    '09 Feb 1994 22:23:32 ' ,                # HTTP format (no weekday)
    '08-Feb-94 14:15:36.75 ' ,               # rfc850 format (no weekday)
    '03/Feb/1994:17:03:55 ' ,                # common logfile format ok
    '08-Feb-1994 14:15:29 ' ,                # broken rfc850 format (no weekday)
    '09 Feb 1994 22:23:32 ' ,                # HTTP format
    'January 24, 2017, 10:58 GMT +5:30',
    'January 14, 12:42 PM',
    'January 14',
    'January   15 2017 12:23:52.55',
    'January 09, 2017, 06:00 pm  ',
    'January 9, 94, 06:00 pm est',
    'January 09, 2017, 15:30 ',
    'January, 03, 2017, 05:30:00 pm ',
    'Feb  03 5:03 pm Z -0500' ,               # Unix 'ls -l' format
    'Feb  3 5 AM 1994  ' ,                    # ctime(3) format NW
    'Feb  3 5 pm 94' ,                        # ctime(3) format NW
    'Feb  3  1994 ' ,                         # Unix 'ls -l' format
    'Feb  3 1:00:00 1994',                    # ANSI C asctime() format NW
    'Feb  3 7:03:55 am est 1994 ' ,           # ctime(3) format NW

=cut
