package wxDate::wxStrftime;


require 5.000;

require Exporter;
@wxDate::wxStrftime::ISA = qw(Exporter);
@wxDate::wxStrftime::EXPORT = qw(ctime asctime strftime);
@wxDate::wxStrftime::EXPORT_OK = qw(asctime_n ctime_n @DoW @MoY @DayOfWeek @MonthOfYear);

use strict;

use FindBin;
use lib "$FindBin::RealBin" . "/lib";
use wxDate::wxZone;


# constants
use vars qw(@DoW @DayOfWeek @MoY @MonthOfYear %strftime_conversion $VERSION);
use vars qw($template $sec $min $hour $mday $mon $year $wday $yday $isdst);

sub msgout { print STDERR "@_ \n" }

$VERSION = 6.0.0;

CONFIG: {
    @DoW =     qw(Sun Mon Tue Wed Thu Fri Sat);
    @DayOfWeek =   qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
    @MoY =     qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
    @MonthOfYear = qw(January February March April May June
              July August September October November December);

    %strftime_conversion = (
    '%',    sub { '%' },
    'a',    sub { $DoW[$wday] },
    'A',    sub { $DayOfWeek[$wday] },
    'b',    sub { $MoY[$mon] },
    'B',    sub { $MonthOfYear[$mon] },
    'c',    sub { asctime_n($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst, "") },
    'C',    sub { int(($year + 1900) / 100) },
    'd',    sub { sprintf("%02d", $mday) },
    'D',    sub { sprintf("%02d/%02d/%02d", $mon+1, $mday, $year%100) },
    'e',    sub { sprintf("%2d", $mday) },
    'f',    sub { fracprintf ("%3.3f", $sec) },
    'F',    sub { sprintf("%4d-%02d-%02d", $year+1900, $mon+1, $mday) },
    'h',    sub { $MoY[$mon] },
    'H',    sub { sprintf("%02d", $hour) },
    'I',    sub { sprintf("%02d", $hour % 12 || 12) },
    'j',    sub { sprintf("%03d", $yday + 1) },
    'k',    sub { sprintf("%2d", $hour) },
    'l',    sub { sprintf("%2d", $hour % 12 || 12) },
    'm',    sub { sprintf("%02d", $mon+1) },
    'M',    sub { sprintf("%02d", $min) },
    'n',    sub { "\n" },
    'o',    sub { sprintf("%d%s", $mday, (($mday < 20 && $mday > 3) ? 'th' : ($mday%10 == 1 ? "st" : ($mday%10 == 2 ? "nd" : ($mday%10 == 3 ? "rd" : "th"))))) },
    'p',    sub { $hour > 11 ? "PM" : "AM" },
    'P',    sub { $hour > 11 ? "pm" : "am" },
    'r',    sub { sprintf("%02d:%02d:%02d %s", $hour % 12 || 12, $min, $sec, $hour > 11 ? 'PM' : 'AM') },
    'R',    sub { sprintf("%02d:%02d", $hour, $min) },
    's',    sub { time },
    'S',    sub { sprintf("%02d", $sec) },
    't',    sub { "\t" },
    'T',    sub { sprintf("%02d:%02d:%02d", $hour, $min, $sec) },
    'u',    sub { $wday + 1 },
    'U',    sub { wkyr(0, $wday, $yday) },
    'v',    sub { sprintf("%2d-%s-%4d", $mday, $MoY[$mon], $year+1900) },
    'w',    sub { $wday },
    'W',    sub { wkyr(1, $wday, $yday) },
    'y',    sub { sprintf("%02d",$year%100) },
    'Y',    sub { $year + 1900 },
    'x',    sub { sprintf("%02d/%02d/%02d", $mon + 1, $mday, $year%100) },
    'X',    sub { sprintf("%02d:%02d:%02d", $hour, $min, $sec) },
    'Z',    sub { uc(&tz_name()) },
    'z',    sub { my $offset = &tz_local_offset();
                    sprintf("%+03d%02d", $offset / 3600, ($offset % 3600)/60) },
    );


}

sub fracprintf {
    my($t,$s) = @_;
    my($p) = sprintf($t, $s-int($s));
    $p=~s/^0+//;
    $p;
}

sub asctime_n {
    my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst, $TZname) = @_;
    ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst, $TZname) = localtime($sec) unless defined $min;
    $year += 1900;
    $TZname .= ' '
    if $TZname;
    sprintf("%s %s %2d %2d:%02d:%02d %s%4d",
      $DoW[$wday], $MoY[$mon], $mday, $hour, $min, $sec, $TZname, $year);
}

sub asctime
{
    return asctime_n(@_)."\n";
}

# is this formula right?
sub wkyr {
    my($wstart, $wday, $yday) = @_;
    $wday = ($wday + 7 - $wstart) % 7;
    return int(($yday - $wday + 13) / 7 - 1);
}

sub ctime {
    my($time) = @_;
    asctime(localtime($time), &tz2zone(undef,$time));
}

sub ctime_n {
    my($time) = @_;
    asctime_n(localtime($time), &tz2zone(undef,$time));
}

my $formats = join '', sort keys %strftime_conversion;

sub strftime {
    # unsupported strftime GNU options EGgOsV
    local ($template, $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = @_;
    my @t = ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);
    my $fmt = $template;

    my $strftime_modifier = sub {
        my ($prefix, $modifier, $format, @t) = @_;
        my $suffix = '';

        no warnings 'uninitialized';
        my $str = strftime("%$format", @t);

        for (;;) {
            if ($modifier eq '_' and $suffix !~ /0/ or $modifier eq '-' and $suffix !~ /0/ and $format =~ /[aAbBDFhnpPrRtTxXZ%]$/) {
                $str =~ s/^([+-])(0+)(\d:.*?|\d$)/' ' x length($2) . $1 . $3/ge;
                $str =~ s/^(0+)(.+?)$/' ' x length($1) . $2/ge;
            }
            elsif ($modifier eq '-' and $suffix !~ /0/ and $format =~ /[CdgGHIjmMNsSuUVwWyYz]$/) {
                $str =~ s/^([+-])(0+)(\d:.*?|\d$)/$1$3/g;
                $str =~ s/^(0+)(.+?)$/$2/g;
            }
            elsif ($modifier eq '-') {
                $str =~ s/^ +//ge;
            }
            elsif ($modifier eq '0' and $suffix !~ /_/) {
                $str =~ s/^( +)/'0' x length($1)/ge;
            }
            elsif ($modifier eq '^' and "$prefix$suffix" =~ /#/ and $format =~ /Z$/) {
                $str = lc($str);
            }
            elsif ($modifier eq '^' and $format !~ /[pP]$/) {
                $str = uc($str);
            }
            elsif ($modifier eq '#' and $format =~ /[aAbBh]$/) {
                $str = uc($str);
            }
            elsif ($modifier eq '#' and $format =~ /[pZ]$/) {
                $str = lc($str);
            };

            last unless $prefix =~ s/(.)$//;
            $suffix = "$modifier$suffix";
            $modifier = $1;
        };
        return $str;
    };

    my $strftime_0z = sub {
        my ($digits, $format, @t) = @_;
        $digits --;
        my $str = strftime($format, @t);
        $str =~ /^([+-])(.*)$/ or return $format;
        return $1 . sprintf "%0${digits}s", $2;
    };

    $fmt =~ s/%([_0\^#-]*)([_0\^#-])((?:[1-9][0-9]*)?:*[EO]?[a-zA-Z])/$strftime_modifier->($1, $2, $3, @t)/ge;
    $fmt =~ s/%([_0\^#-]*)([_0\^#-])((?:[1-9][0-9]*)?[%])/$strftime_modifier->($1, $2, $3, @t) . '%'/ge;
    $fmt =~ s/%(\d+)([aAbBDeFhklnpPrRtTxXZ])/sprintf("%$1s", strftime("%$2", @t))/ge;
    $fmt =~ s/%(\d+)([cCdHIjmMSuUwWyY])/sprintf("%0$1s", strftime("%$2", @t))/ge;
    $fmt =~ s/%(\d+)(:*[z])/$strftime_0z->($1, "%$2", @t)/ge;
    $fmt =~ s/%([$formats])/$strftime_conversion{$1}->(@t)/ge;
    $fmt =~ s/%([_0\^#-]+(?:[1-9][0-9]*)?|[_0\^#-]?(?:[1-9][0-9]*))([a-zA-Z%])/%%$1$2/;

    return $fmt;
};
1;

__END__

=head1 NAME

wxDate::wxStrftime -- format times ala POSIX asctime

=head1 SYNOPSIS

    use wxDate::wxStrftime
    print ctime(time);
    print asctime(localtime(time));
    print strftime(template, localtime(time));

=head2 strftime conversions

    %%  PERCENT
    %a  day of the week abbr
    %A  day of the week
    %b  month abbr
    %B  month
    %c  ctime format: Sat Nov 19 21:05:57 1994
    %C  The century number (year/100) as a 2-digit integer.
    %d  DD
    %D  MM/DD/YY
    %e  Like %d, the day of the month as a decimal number, but a leading
        zero is replaced by a space.
    %f  floating point seconds (milliseconds): .314
    %F  Equivalent to %Y-%m-%d (the ISO 8601 date format).
    %h  month abbr
    %H  hour, 24 hour clock, leading 0's)
    %I  hour, 12 hour clock, leading 0's)
    %j  day of the year
    %k  hour
    %l  hour, 12 hour clock
    %m  month number, starting with 1, leading 0's
    %M  minute, leading 0's
    %n  NEWLINE
    %o  ornate day of month -- "1st", "2nd", "25th", etc.
    %p  AM or PM
    %r  time format: 09:05:57 PM
    %R  time format: 21:05
    %S  seconds, leading 0's
    %t  TAB
    %T  time format: 21:05:57
    %U  week number, Sunday as first day of week
    %v  DD-Mon-Year
    %w  day of the week, numerically, Sunday == 0
    %W  week number, Monday as first day of week
    %x  date format: 11/19/94
    %X  time format: 21:05:57
    %y  year (2 digits)
    %Y  year (4 digits)
    %Z  timezone in ascii. eg: PST

=head1 DESCRIPTION

This module provides routines to format dates.  They correspond
to the libc routines.  &strftime() supports a pretty good set of
conversions -- more than most C libraries.

strftime supports a pretty good set of conversions.

The POSIX module has very similar functionality.  You should consider
using it instead if you do not have allergic reactions to system
libraries.

unsupported options GgV [ISO 8601 week dates] s+
 s is already being generated by str2time
 EO modifiers

Between the '%' character and the conversion specifier character,
an optional flag and field width may be specified.

The following flag characters are permitted:
_      (underscore) Pad a numeric result string with spaces.

-      (dash) Do not pad a numeric result string.

0      Pad  a  numeric  result string with zeros even if the conversion
      specifier character uses space-padding by default.

^      Convert alphabetic characters in result string to uppercase.

#      Swap the case of the result string.  (This flag works only  with
      certain  conversion  specifier  characters,  and of these, it is
      only really useful with %Z.)

An optional decimal width specifier may follow  the  (possibly  absent)
flag.   If  the  natural  size of the field is smaller than this width,
then the result string is padded (on the left) to the specified width.

=head1 GENESIS

Written by David Muir Sharnoff <muir@idiom.org>.

The starting point for this package was a posting by
Paul Foley <paul@ascent.com>

=head1 LICENSE

Copyright (C) 1996-2010 David Muir Sharnoff.
Copyright (C) 2011 Google, Inc.
Copyright (C) 2017 Paramvir Likhari
The original is quite a bit changed and formats to conkywx requirements

License hereby granted for anyone to use, modify or redistribute
this module at their own risk.

