
package wxDate::wxZone;


require 5.002;

require Exporter;
use Carp;
use strict;
use vars qw(@ISA @EXPORT $VERSION @tz_local);

@ISA     = qw(Exporter);
@EXPORT  = qw(tz2zone tz_local_offset tz_offset tz_name);
$VERSION = "2.24";

sub msgout { print STDERR "@_ \n" }

sub tz2zone (;$$$)
{
	my($TZ, $time, $isdst) = @_;

	use vars qw(%tzn_cache);

	$TZ = defined($ENV{'TZ'}) ? ( $ENV{'TZ'} ? $ENV{'TZ'} : 'GMT' ) : ''
	    unless $TZ;

	# Hack to deal with 'PST8PDT' format of TZ
	# Note that this can't deal with all the esoteric forms, but it
	# does recognize the most common: [:]STDoff[DST[off][,rule]]

	if (! defined $isdst) {
		my $j;
		$time = time() unless $time;
		($j, $j, $j, $j, $j, $j, $j, $j, $isdst) = localtime($time);
	}

	if (defined $tzn_cache{$TZ}->[$isdst]) {
		return $tzn_cache{$TZ}->[$isdst];
	}

	if ($TZ =~ /^
		    ( [^:\d+\-,] {3,} )
		    ( [+-] ?
		      \d {1,2}
		      ( : \d {1,2} ) {0,2}
		    )
		    ( [^\d+\-,] {3,} )?
		    /x
	    ) {
		my $dsttz = defined($4) ? $4 : $1;
		$TZ = $isdst ? $dsttz : $1;
		$tzn_cache{$TZ} = [ $1, $dsttz ];
	} else {
		$tzn_cache{$TZ} = [ $TZ, $TZ ];
	}
	return $TZ;
}

sub tz_local_offset (;$)
{
	my ($time) = @_;

	$time = time() unless $time;
	my (@l) = localtime($time);
	my $isdst = $l[8];

	if (defined($tz_local[$isdst])) {
		return $tz_local[$isdst];
	}

	$tz_local[$isdst] = &calc_off($time);

	return $tz_local[$isdst];
}

sub calc_off
{
	my ($time) = @_;

	my (@l) = localtime($time);
	my (@g) = gmtime($time);

	my $off;

	$off =     $l[0] - $g[0]
		+ ($l[1] - $g[1]) * 60
		+ ($l[2] - $g[2]) * 3600;

	# subscript 7 is yday.

	if ($l[7] == $g[7]) {
		# done
	} elsif ($l[7] == $g[7] + 1) {
		$off += 86400;
	} elsif ($l[7] == $g[7] - 1) {
		$off -= 86400;
	} elsif ($l[7] < $g[7]) {
		# crossed over a year boundry!
		# localtime is beginning of year, gmt is end
		# therefore local is ahead
		$off += 86400;
	} else {
		$off -= 86400;
	}

	return $off;
}

# constants

CONFIG: {
	use vars qw(%dstZone %zoneOff %dstZoneOff %Zone);

	my @dstZone = (
        "ndt"    =>   -2*3600-1800,	 # Newfoundland Daylight
        "brst"   =>   -2*3600,       # Brazil Summer Time (East Daylight)
        "adt"    =>   -3*3600,  	 # Atlantic Daylight
        "edt"    =>   -4*3600,  	 # Eastern Daylight
        "cdt"    =>   -5*3600,  	 # Central Daylight
        "mdt"    =>   -6*3600,  	 # Mountain Daylight
        "pdt"    =>   -7*3600,  	 # Pacific Daylight
        "akdt"   =>   -8*3600,       # Alaska Daylight
        "ydt"    =>   -8*3600,  	 # Yukon Daylight
        "hdt"    =>   -9*3600,  	 # Hawaii Daylight
        "bst"    =>   +1*3600,  	 # British Summer
        "mest"   =>   +2*3600,  	 # Middle European Summer
        "metdst" =>   +2*3600, 	     # Middle European DST
        "sst"    =>   +2*3600,  	 # Swedish Summer
        "fst"    =>   +2*3600,  	 # French Summer
        "cest"   =>   +2*3600,       # Central European Daylight
        "eest"   =>   +3*3600,       # Eastern European Summer
        "msd"    =>   +4*3600,       # Moscow Daylight
        "wadt"   =>   +8*3600,  	 # West Australian Daylight
        "kdt"    =>  +10*3600,	     # Korean Daylight
        "cadt"   =>  +10*3600+1800,	 # Central Australian Daylight
        "aedt"   =>  +11*3600,  	 # Eastern Australian Daylight
        "eadt"   =>  +11*3600,  	 # Eastern Australian Daylight
        "nzd"    =>  +13*3600,  	 # New Zealand Daylight
        "nzdt"   =>  +13*3600,  	 # New Zealand Daylight
	);

	my @Zone = (
        'a'      => +1*3600,       'acdt'   => +10*3600+1800, 'acst'   => +9*3600+1800,
        'adt'    => -4*3600,       'aedt'   => +11*3600,      'aes'    => +10*3600,
        'aest'   => +10*3600,      'aft'    => +4*3600+1800,  'ahdt'   => -9*3600,
        'ahst'   => -10*3600,      'akdt'   => -8*3600,       'akst'   => -9*3600,
        'amst'   => +4*3600,       'amt'    => +4*3600,       'anast'  => +13*3600,
        'anat'   => +12*3600,      'art'    => -3*3600,       'ast'    => -4*3600,
        'at'     => -1*3600,       'awst'   => +8*3600,       'azost'  => +0,
        'azot'   => -1*3600,       'azst'   => +5*3600,       'azt'    => +4*3600,
        'b'      => +2*3600,       'badt'   => +4*3600,       'bat'    => +6*3600,
        'bdst'   => +2*3600,       'bdt'    => +6*3600,       'bet'    => -11*3600,
        'bnt'    => +8*3600,       'bort'   => +8*3600,       'bot'    => -4*3600,
        'brt'    => -3*3600,       'bst'    => 0,             'bt'     => 0,
        'btt'    => +6*3600,       'c'      => +3*3600,       'cast'   => +9*3600+1800,
        'cat'    => +2*3600,       'cct'    => +6*3600+1800,  'cdt'    => -6*3600,
        'cest'   => +2*3600,       'cet'    => +1*3600,       'cetdst' => +2*3600,
        'chadt'  => +13*3600+2700, 'chast'  => +12*3600+2700, 'ckt'    => -10*3600,
        'clst'   => -3*3600,       'clt'    => -4*3600,       'cot'    => -5*3600,
        'cst'    => -6*3600,       'csut'   => +10*3600+1800, 'cut'    => +0,
        'cvt'    => -1*3600,       'cxt'    => +7*3600,       'chst'   => +10*3600,
        'd'      => +4*3600,       'davt'   => +7*3600,       'ddut'   => +10*3600,
        'dnt'    => +1*3600,       'dst'    => +2*3600,       'e'      => +5*3600,
        'easst'  => -5*3600,       'east'   => -6*3600,       'eat'    => +3*3600,
        'ect'    => -5*3600,       'edt'    => -4*3600,       'eest'   => +3*3600,
        'eet'    => +2*3600,       'eetdst' => +3*3600,       'egst'   => +0,
        'egt'    => -1*3600,       'emt'    => +1*3600,       'est'    => -5*3600,
        'esut'   => +11*3600,      'f'      => +6*3600,       'fdt'    => undef,
        'fjst'   => +13*3600,      'fjt'    => +12*3600,      'fkst'   => -3*3600,
        'fkt'    => -4*3600,       'fst'    => undef,         'fwt'    => +1*3600,
        'g'      => +7*3600,       'galt'   => -6*3600,       'gamt'   => -9*3600,
        'gest'   => +5*3600,       'get'    => +4*3600,       'gft'    => -3*3600,
        'gilt'   => +12*3600,      'gmt'    => +0,            'gst'    => +4*3600,
        'gt'     => +0,            'gyt'    => -4*3600,       'gz'     => +0,
        'h'      => +8*3600,       'haa'    => -3*3600,       'hac'    => -5*3600,
        'hae'    => -4*3600,       'hap'    => -7*3600,       'har'    => -6*3600,
        'hat'    => -2*3600+1800,  'hay'    => -8*3600,       'hdt'    => -9*3600+1800,
        'hfe'    => +2*3600,       'hfh'    => +1*3600,       'hg'     => +0,
        'hkt'    => +8*3600,       'hl'     => 'local',       'hna'    => -4*3600,
        'hnc'    => -6*3600,       'hne'    => -5*3600,       'hnp'    => -8*3600,
        'hnr'    => -7*3600,       'hnt'    => -3*3600+1800,  'hny'    => -9*3600,
        'hoe'    => +1*3600,       'hst'    => -10*3600,      'i'      => +9*3600,
        'ict'    => +7*3600,       'idle'   => +12*3600,      'idlw'   => -12*3600,
        'idt'    => +3*3600,       'iot'    => +5*3600,       'irdt'   => +4*3600+1800,
        'irkst'  => +9*3600,       'irkt'   => +8*3600,       'irst'   => +4*3600+1800,
        'irt'    => +3*3600+1800,  'ist'    => +5*3600+1800,  'it'     => +3*3600+1800,
        'ita'    => +1*3600,       'javt'   => +7*3600,       'jayt'   => +9*3600,
        'jst'    => +9*3600,       'jt'     => +7*3600,       'k'      => +10*3600,
        'kdt'    => +10*3600,      'kgst'   => +6*3600,       'kgt'    => +5*3600,
        'kost'   => +12*3600,      'krast'  => +8*3600,       'krat'   => +7*3600,
        'kst'    => +9*3600,       'l'      => +11*3600,      'lhdt'   => +11*3600,
        'lhst'   => +10*3600+1800, 'ligt'   => +10*3600,      'lint'   => +14*3600,
        'lkt'    => +6*3600,       'lst'    => 'local',       'lt'     => 'local',
        'm'      => +12*3600,      'magst'  => +12*3600,      'magt'   => +11*3600,
        'mal'    => +8*3600,       'mart'   => -9*3600+1800,  'mat'    => +3*3600,
        'mawt'   => +6*3600,       'mdt'    => -6*3600,       'med'    => +2*3600,
        'medst'  => +2*3600,       'mest'   => +2*3600,       'mesz'   => +2*3600,
        'met'    => +1*3600,       'mewt'   => +1*3600,       'mex'    => -6*3600,
        'mez'    => +1*3600,       'mht'    => +12*3600,      'mmt'    => +6*3600+1800,
        'mpt'    => +10*3600,      'msd'    => +4*3600,       'msk'    => +3*3600,
        'msks'   => +4*3600,       'mst'    => -7*3600,       'mt'     => +8*3600+1800,
        'mut'    => +4*3600,       'mvt'    => +5*3600,       'myt'    => +8*3600,
        'n'      => -1*3600,       'nct'    => +11*3600,      'ndt'    => -2*3600+1800,
        'nft'    => +11*3600,      'nor'    => +1*3600,       'novst'  => +7*3600,
        'novt'   => +6*3600,       'npt'    => +5*3600+2700,  'nrt'    => +12*3600,
        'nst'    => -3*3600+1800,  'nsut'   => +6*3600+1800,  'nt'     => -11*3600,
        'nut'    => -11*3600,      'nzdt'   => +13*3600,      'nzst'   => +12*3600,
        'nzt'    => +12*3600,      'o'      => -2*3600,       'oesz'   => +3*3600,
        'oez'    => +2*3600,       'omsst'  => +7*3600,       'omst'   => +6*3600,
        'oz'     => 'local',       'p'      => -3*3600,       'pdt'    => -7*3600,
        'pet'    => -5*3600,       'petst'  => +13*3600,      'pett'   => +12*3600,
        'pgt'    => +10*3600,      'phot'   => +13*3600,      'pht'    => +8*3600,
        'pkt'    => +5*3600,       'pmdt'   => -2*3600,       'pmt'    => -3*3600,
        'pnt'    => -8*3600+1800,  'pont'   => +11*3600,      'pst'    => -8*3600,
        'pwt'    => +9*3600,       'pyst'   => -3*3600,       'pyt'    => -4*3600,
        'q'      => -4*3600,       'r'      => -5*3600,       'r1t'    => +2*3600,
        'r2t'    => +3*3600,       'ret'    => +4*3600,       'rok'    => +9*3600,
        's'      => -6*3600,       'sadt'   => +10*3600+1800, 'sast'   => +2*3600,
        'sbt'    => +11*3600,      'sct'    => +4*3600,       'set'    => +1*3600,
        'sgt'    => +8*3600,       'srt'    => -3*3600,       'sst'    => -12*3600,
        'swt'    => +1*3600,       't'      => -7*3600,       'tft'    => +5*3600,
        'tha'    => +7*3600,       'that'   => -10*3600,      'tjt'    => +5*3600,
        'tkt'    => -10*3600,      'tmt'    => +5*3600,       'tot'    => +13*3600,
        'trut'   => +10*3600,      'tst'    => +3*3600,       'tuc '   => +0,
        'tvt'    => +12*3600,      'u'      => -8*3600,       'ulast'  => +9*3600,
        'ulat'   => +8*3600,       'usz1'   => +2*3600,       'usz1s'  => +3*3600,
        'usz3'   => +4*3600,       'usz3s'  => +5*3600,       'usz4'   => +5*3600,
        'usz4s'  => +6*3600,       'usz5'   => +6*3600,       'usz5s'  => +7*3600,
        'usz6'   => +7*3600,       'usz6s'  => +8*3600,       'usz7'   => +8*3600,
        'usz7s'  => +9*3600,       'usz8'   => +9*3600,       'usz8s'  => +10*3600,
        'usz9'   => +10*3600,      'usz9s'  => +11*3600,      'utz'    => -3*3600,
        'uyt'    => -3*3600,       'uz10'   => +11*3600,      'uz10s'  => +12*3600,
        'uz11'   => +12*3600,      'uz11s'  => +13*3600,      'uz12'   => +12*3600,
        'uz12s'  => +13*3600,      'uzt'    => +5*3600,       'v'      => -9*3600,
        'vet'    => -4*3600,       'vlast'  => +11*3600,      'vlat'   => +10*3600,
        'vtz'    => -2*3600,       'vut'    => +11*3600,      'w'      => -10*3600,
        'wakt'   => +12*3600,      'wast'   => +2*3600,       'wat'    => +1*3600,
        'west'   => +1*3600,       'wesz'   => +1*3600,       'wet'    => +0,
        'wetdst' => +1*3600,       'wez'    => +0,            'wft'    => +12*3600,
        'wgst'   => -2*3600,       'wgt'    => -3*3600,       'wib'    => +7*3600,
        'wit'    => +9*3600,       'wita'   => +8*3600,       'wst'    => +1*3600,
        'wtz'    => -1*3600,       'wut'    => +1*3600,       'x'      => -11*3600,
        'y'      => -12*3600,      'yakst'  => +10*3600,      'yakt'   => +9*3600,
        'yapt'   => +10*3600,      'ydt'    => -8*3600,       'yekst'  => +6*3600,
        'yekt'   => +5*3600,       'yst'    => -9*3600,       'z'      => +0,
        'utc'    => +0,
	);

	%Zone = @Zone;
	%dstZone = @dstZone;
	%zoneOff = reverse(%Zone);
	%dstZoneOff = reverse(%dstZone);

}

sub tz_offset (;$$)
{
	my ($zone, $time) = @_;

    return &tz_local_offset($time) unless($zone);

    $time = time() unless $time;
    my(@l) = localtime($time);
    my $dst = $l[8];

    $zone = lc $zone;

    # correction for values like -730 > -7:30
    if (index($zone, ':') == -1 and $zone !~ /[a-z.]+/) {
        my $factor = ($zone < 1) ? '-' : '+';
        my ($zone1) = $zone =~ m{(\d+)};
        if (length $zone1 == 3 ) {
            $zone = $factor . substr($zone1,0,1) . ':' . substr($zone1,1,2);
        }
    }

    if ( $zone =~ /^([-+])?(\d\d?):?(\d?\d?)?$/ ) {
        my $v = ($3) ? $3 : 0 ;
        my $tzv = $2;
        if ($tzv > 14 or $tzv < -14) { return tz_offset() }
        return $1 . ($tzv * 3600 + $v * 60);
    } elsif ( $zone =~ /^(([-+])?[\d\.]+)$/ ) {
        # if ($1 > 14 or $1 < -14) { tz_offset() }
        return $1 * 3600;
    } elsif (exists $dstZone{$zone} && ($dst || ! exists $Zone{$zone})) {
        return $dstZone{$zone};
    } elsif (exists $Zone{$zone}) {
		return $Zone{$zone};
	}
	undef;
}

sub tz_name (;$$)
{
	my ($off, $dst) = @_;

	$off = tz_offset()
		unless(defined $off);

	$dst = (localtime(time))[8]
		unless(defined $dst);

	if (exists $dstZoneOff{$off} && ($dst || !exists $zoneOff{$off})) {
		return $dstZoneOff{$off};
	} elsif (exists $zoneOff{$off}) {
		return $zoneOff{$off};
	}
	sprintf("%+5d", int($off / 60) * 100 + $off % 60);
}

1;
=head1 NAME

Date::Zone -- miscellaneous timezone manipulations routines

=head1 SYNOPSIS

    use Date::Zone;
    print tz2zone();
    print tz2zone($ENV{'TZ'});
    print tz2zone($ENV{'TZ'}, time());
    print tz2zone($ENV{'TZ'}, undef, $isdst);
    $offset = tz_local_offset();
    $offset = tz_offset($TZ);

=head1 DESCRIPTION

This is a collection of miscellaneous timezone manipulation routines.

C<tz2zone()> parses the TZ environment variable and returns a timezone
string suitable for inclusion in L<date(1)>-like output.  It opionally takes
a timezone string, a time, and a is-dst flag.

C<tz_local_offset()> determins the offset from GMT time in seconds.  It
only does the calculation once.

C<tz_offset()> determines the offset from GMT in seconds of a specified
timezone.

C<tz_name()> determines the name of the timezone based on its offset

=head1 AUTHORS

Paramvir Likhari - 2017 cleaned up added time zones - named it wxZone
Graham Barr <gbarr@pobox.com>
David Muir Sharnoff <muir@idiom.com>
Paul Foley <paul@ascent.com>

=cut
