
--[[
    Paramvir Likhari - conkywx 2011 - 2020 - GPL3
    wxclock library for use with conkywx lua implementation
    time zone code worked on by Freebird
--]]

local w = {}

local Global_Font_Size = 12
local PI           = 3.14159265358979323846
local TORADIANS    = 3.14159265358979323846/180
local tconcat      = table.concat
local tonum        = tonumber
local random       = math.random
local sin          = math.sin

local lcairo_stroke                      = cairo_stroke
local lcairo_stroke_preserve             = cairo_stroke_preserve
local lcairo_fill                        = cairo_fill
local lcairo_save                        = cairo_save
local lcairo_restore                     = cairo_restore
local lcairo_set_source_rgba             = cairo_set_source_rgba
local lcairo_set_source                  = cairo_set_source
local lcairo_arc                         = cairo_arc
local lcairo_transform                   = cairo_transform
local lcairo_translate                   = cairo_translate
local lcairo_matrix_init                 = cairo_matrix_init
local lcairo_set_line_width              = cairo_set_line_width
local lcairo_pattern_create_radial       = cairo_pattern_create_radial
local lcairo_pattern_add_color_stop_rgba = cairo_pattern_add_color_stop_rgba
local lcairo_pattern_destroy             = cairo_pattern_destroy

local Red          = 0xFF0000
local Blue         = 0x0000ff
local RoyalBlue    = 0x4169E1
local NavyBlue     = 0x000080
local MidnightBlue = 0x191970
local Cyan         = 0x00ffff
local Black        = 0x000000
local White        = 0xFFFFFF
local Yellow       = 0xFFFF00
local Magenta      = 0xff00ff
local Orchid1      = 0xFF83FA
local Gold         = 0xFFD700
local DarkGold     = 0xCD950C
local Orange       = 0xFF3300
local Orange3      = 0xFF5500
local SaddleBrown  = 0x8B4513
local Grey81       = 0xd0d0d0
local Lime         = 0x00FF00
local DarkLime     = 0x32B000


-- package.path = package.path .. ";./lib/?.lua;"


local FC = require("luafunc")

local wxsplit          = FC.wxsplit
local to_rgba          = FC.to_rgba
local get_points       = FC.get_points
local set_font      = FC.set_font
local make_txt         = FC.make_txt
local make_scale_txt2  = FC.make_scale_txt2
local make_bezel       = FC.make_bezel
local choose_indicator = FC.choose_indicator
local circlewriting    = FC.circlewriting
local put_image        = FC.put_image
local instrument_sqr   = FC.instrument_sqr
local msgout           = FC.msgout

local function make_ampm(p)
    local cr = p.cr
    lcairo_restore(cr)
    local ap_rad           = p.ampm_size*250
    local LineThicknessOut = ap_rad*0.15
    local LineThicknessIn  = ap_rad*0.12
    local s_rad            = ap_rad/5
    local px,py            = p.px,p.py
    --print(px,py)
    ---- top half
    lcairo_set_source_rgba(cr,1,1,1,1)
    lcairo_set_line_width(cr, LineThicknessOut)
    lcairo_arc(cr, px, py, ap_rad/1.75, PI, 2 * PI)
    lcairo_stroke_preserve(cr)
    lcairo_set_source_rgba(cr,0,0,0,1)
    lcairo_set_line_width(cr, LineThicknessIn)
    lcairo_stroke(cr)
    ----bottom half
    lcairo_set_source_rgba(cr,0,0,0,1)
    lcairo_set_line_width(cr, LineThicknessOut)
    lcairo_arc(cr, px, py, ap_rad/1.75, 2*PI, PI)
    lcairo_stroke_preserve(cr)
    lcairo_set_source_rgba(cr,1,1,1,1)
    lcairo_set_line_width(cr, LineThicknessIn)
    lcairo_stroke(cr)
    set_font({ cr, tconcat({"Mono:",ap_rad*0.5,":bold"}) })
    local extents = cairo_text_extents_t:create()
    tolua.takeownership(extents)
    local mktTable = {
        {
            cr = cr, ext=extents, text ='AM', px=px, py=py, align='c', icolour={0x000000,1}
        },
        {
            cr = cr, ext=extents, text ='AM', px=px-ap_rad*.05, py=py-ap_rad*0.0286, align='c', icolour={0xffffff,1}
        },
        {
            cr = cr, ext=extents, text ='PM', px=px-ap_rad*.02, py=py+ap_rad*0.06, align='bc', icolour={0x000000,1}
        },
        {
            cr = cr, ext=extents, text ='PM', px=px-ap_rad*.07, py=py+ap_rad*0.0286, align='bc', icolour={0xffffff,1}
        },
    }
    for x in pairs(mktTable) do
        make_txt(mktTable[x])
    end
    lcairo_save(cr)
end

-- -- not in use but still a good design - images work fine for gears
    -- local function make_gear(p)
    --     lcairo_restore(cr)
    --     local ap_rad          = p.gear_size * 50
    --     local max             = p.max or MAX
    --     local dir             = p.dir or 1
    --     local LineThickness   = ap_rad*0.075
    --     local LineThicknessIn = ap_rad*0.075
    --     local s_rad           = ap_rad/5

    --     value                 = tonum(conky_parse("${updates}")) % max
    --     local theta           = dir*value*6/max*60

    --     lcairo_set_source_rgba(cr,0,0,0,0.7)
    --     lcairo_set_line_width(cr, 2)
    --     local slim_step, star_step, gstep

    --     slim_step             = 9
    --     star_step             = 18
    --     gstep                 = 4

    --     if p.style ~= 99 then
    --         for i = 0, 360, star_step do
    --             local pxa1,pya1 = get_points(theta,i,slim_step,ap_rad)
    --             local pxa2,pya2 = get_points(theta,i,slim_step+gstep,ap_rad)
    --             local pxa3,pya3 = get_points(theta,i,star_step,ap_rad*0.81)
    --             local pxa4,pya4 = get_points(theta,i,star_step+gstep,ap_rad*0.81)
    --             lcairo_line_to(cr, p.px+pxa1, p.py+pya1)
    --             lcairo_line_to(cr, p.px+pxa2, p.py+pya2)
    --             lcairo_line_to(cr, p.px+pxa3, p.py+pya3)
    --             lcairo_line_to(cr, p.px+pxa4, p.py+pya4)
    --         end
    --     else
    --         lcairo_arc(cr, p.px, p.py, ap_rad, 0, 2*PI)
    --     end
    --     lcairo_stroke(cr)

    --     local RadAng = {}; local steps
    --     if      p.style == 1  then steps = 60    -- 6 dots
    --     elseif  p.style == 2  then steps = 90    -- 4 dots
    --     elseif  p.style == 3  then steps = 120   -- 3 dots
    --     elseif  p.style == 4  then steps = 180   -- 2 dots
    --     elseif  p.style == 5  then steps = 70    -- 5 dots
    --     elseif  p.style == 99 then steps = 70    -- 5 dots
    --     else    steps   = 90
    --     end

    --     lcairo_set_source_rgba(cr,0,0,0,0.4)
    --     for i=0,360,steps do
    --         local jx,jy = get_points(0, theta , i, ap_rad/2.0)
    --         lcairo_arc(cr, p.px+jx, p.py+jy, ap_rad/5, 0, 2*PI)
    --         lcairo_fill(cr)
    --     end

    --     lcairo_set_source_rgba(cr,0,0,0,1)
    --     lcairo_set_line_width(cr, 2)
    --     lcairo_arc(cr, p.px, p.py, ap_rad*0.2, 0, 2*PI)
    --     lcairo_stroke(cr)
    --     lcairo_save(cr)
    -- end


function w.clock(p)
    --[[
        fancyclock.lua - lua script for the mechanical clock
        Sunday, 20 October 2013 13:55 - original source from written by easysid
        :: This widget has been totally modified to comply with conkywx
        2017 - Paramvir
    ]]

    local cr          = p.cr
    local tnig        = 125
    local clock_style = tonum(p.k)  or 25
    local MAX         = 60

    if clock_style < 1 or clock_style > 41 then
        clock_style = 25
    end

    local function draw_sweep(t)
        lcairo_save(cr)
        lcairo_set_source_rgba(cr, to_rgba(t.scol))
        lcairo_set_line_width(cr, t.arc_thickness)
        lcairo_arc(cr, t.x, t.y, t.R, t.start_angle * TORADIANS, t.end_angle * TORADIANS)
        lcairo_stroke(cr)
        lcairo_restore(cr)
    end

    local function draw_clock_hands(t)

        local hsecs, hours, apx, apy

        local csize    = t.ds.csize
        local m_scale  = t.m_scale or 0.5
        local h_scale  = t.h_scale or 0.5
        local ap_scale = csize*.20 or 0.25

        -- show ampm at the center if size less than 150 pixels - 0.5
        apx      = t.ds.x
        if csize < 0.419 then
            apy      = t.ds.y
            ap_scale = csize*.41 or 0.25
        else
            apy      = t.ds.y - ( 50 * csize )
        end
        local extents = cairo_text_extents_t:create()
        tolua.takeownership(extents)
        --calculate the seconds for each
        local msecs = t.ds.time.min*60 + t.ds.time.sec
        local start_sweep
        local hours = t.ds.time.hour
        --local hours = 8.25
        if  not t.ds.thc then
            -- 12 hour clock
            local hours1
            if hours > 12 then hours1 = hours - 12 else hours1 = hours end
            hsecs = (hours1*3600 + msecs)
            start_sweep = (0-90)
        else
            -- 24 hour clock
            if t.ds.thc == 2 then
                hsecs = ((hours+12)*3600 + msecs)/2
                start_sweep = (0+90)
            else
                hsecs = (hours*3600 + msecs)/2
                start_sweep = (0-90)
            end
        end
        local m_theta = TORADIANS*((msecs*0.1)-90)
        local h_theta = TORADIANS*((hsecs/120)-90)

        -- Show am/pm  #########################################
        lcairo_save(cr)
        if not t.ds.thc and t.show_ap ~= 0 then
            local ap_file = 'ampm.png'
            local ap_hsecs = (hours*3600) + msecs
            local ampmData = { cr = cr, ampm_size = ap_scale, px = apx, py = apy }
            make_ampm(ampmData)
            local ciData = {
                cr       = cr,
                px       = apx,
                py       = apy,
                startval = 0-90,
                scale    = ap_hsecs*(360/86400),
                rout1    = 142*ap_scale,
                ch_indi  = 8,
                cthick   = t.ds.line_width,
                csize    = csize,
                icolour  = {0xff00ff,1},
            }
            choose_indicator(ciData)
        end
        lcairo_restore(cr)

        if t.show_dd and t.show_dd ~= 0 then
            -- draw weekdays  #########################################
            local R     = t.length
            local r     = R * 0.2 -- the radius of the small circle of seconds hand
            local xfact = R * 2.3

            lcairo_save(cr)
            if t.arrow_colour and csize >= 0.419 then
                set_font({ cr, 'Mono:' .. R * 0.6 .. ':bold' })
                local wdays = {'S','M','T','W','T','F','S'}
                lcairo_save(cr)
                lcairo_set_source_rgba(cr, to_rgba({"0x666666",0.7}) )
                lcairo_arc(cr, t.ds.x+xfact, t.ds.y, R*1.3, TORADIANS, 0)
                lcairo_fill(cr)
                lcairo_set_source_rgba(cr, to_rgba({Lime,1}))
                lcairo_save(cr)
                for i = 1, 7 do
                    local mtData = {
                        cr          = cr,
                        xval        = t.ds.x + xfact,
                        yval        = t.ds.y,
                        startscale  = 0,
                        endscale    = 360,
                        maxscale    = 7,       -- number of weekdays
                        trout       = R-1,
                        text        = wdays[i],
                        ext         = extents,
                        dotext      = 1,
                        i           = i-1,
                    }
                    make_scale_txt2(mtData)
                end
                lcairo_save(cr)
                local ciData = {
                    cr       = cr,
                    px       = t.ds.x+xfact,
                    py       = t.ds.y,
                    startval = 0,
                    scale    = (360/7)*(t.ds.time.wday-1),
                    rout1    = R,
                    ch_indi  = 6,
                    cthick   = t.ds.line_width,
                    csize    = csize,
                    icolour  = {0xffff00,1},
                }
                choose_indicator(ciData)

            -- draw months and date #########################################
                lcairo_restore(cr)
                set_font({ cr, 'Mono:' .. R * 0.9 .. ':bold' })
                local mon = {'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'}
                local mt = mon[t.ds.time.month]
                local dt = t.ds.time.day
                lcairo_set_source_rgba(cr, to_rgba({"0x666666",0.7}) )
                lcairo_arc(cr, t.ds.x-xfact, t.ds.y, R*1.3, TORADIANS, 0)
                lcairo_fill(cr)
                local mktTable = {
                    {
                        cr = cr, ext=extents, text=mt, px=t.ds.x-xfact, py=t.ds.y,
                    },
                    {
                        cr = cr, ext=extents, text=dt, px=t.ds.x-xfact, py=t.ds.y+(R * 0.9),
                    },
                }
                for x = 1, 2 do
                    make_txt(mktTable[x])
                end
                lcairo_save(cr)
            end
        end

    -- draw the clock arrows hours and minutes #########################################
        if t.ds.hm_arrows > 6 and t.ds.hm_arrows < 11 and t.ds.style ~= 3 then
            local h_hand = tconcat({'hr',t.ds.hm_arrows,'.png'})
            local m_hand = tconcat({'mn',t.ds.hm_arrows,'.png'})

            local offcen, scaleFactor

            if tonum(t.ds.hm_arrows) == 10 then
                offcen = true;  scaleFactor = 1.07
            else
                offcen = false; scaleFactor = 3
            end
            local start_angle = start_sweep
            local hend_angle  = (((hsecs-(12*3600))/120) - 90)
            local mend_angle  = ((msecs*0.1)-90)
            local piData = {
                {
                    cr          = cr,
                    x           = t.ds.x,
                    y           = t.ds.y,
                    file        = m_hand,
                    scale       = m_scale*scaleFactor,
                    theta       = m_theta,
                    offcen      = offcen,
                    rotate      = true,
                    start_angle = start_angle,
                    end_angle   = mend_angle,
                    dial_colour = t.arrow_colour,
                    ip          = t.ds.ip,
                },
                {
                    cr          = cr,
                    x           = t.ds.x,
                    y           = t.ds.y,
                    file        = h_hand,
                    scale       = h_scale*scaleFactor,
                    theta       = h_theta,
                    offcen      = offcen,
                    rotate      = true,
                    start_angle = start_angle,
                    end_angle   = hend_angle,
                    dial_colour = t.arrow_colour,
                    ip          = t.ds.ip,
                },
            }
            for x = 1, 2 do
                put_image(piData[x])
            end
        end
        local hfactor = 155
        local mfactor = 160
        -- print(m_scale,h_scale,csize,csize*mfactor,m_scale*123.6)

        local chands
        -- 12 hour ring
        -- hour sweep  #########################################
        if  t.ds.style == 3 or
            t.ds.style == 4 then
            lcairo_save(cr)
            local start_angle = start_sweep
            local end_angle   = (((hsecs-(12*3600))/120) - 90)
            local dsData = {
                x             = t.ds.x,
                y             = t.ds.y,
                arc_thickness = t.ds.arc_thickness,
                scol          = t.arrow_colour,
                R             = h_scale * hfactor * 1.21 ,
                start_angle   = start_angle,
                end_angle     = end_angle,
            }
            draw_sweep(dsData)
            if t.ds.style == 4 then chands = 1 end
        end
        local carrows = { 50,51,52,53,54, }
        local cav
        if  t.ds.style ~= 3 and t.ds.hm_arrows < 7 or t.ds.hm_arrows > 10 then
            if t.ds.hm_arrows > 10 and t.ds.hm_arrows < 16 then
                cav = carrows[t.ds.hm_arrows - 10]
            elseif t.ds.hm_arrows > 15 then
                cav = 51
            else
                cav = t.ds.hm_arrows
            end
            lcairo_save(cr)
            local ciData = {
                cr              = cr,
                px              = t.ds.x,
                py              = t.ds.y,
                startval        = 0+90,
                scale           = ((hsecs/120)-90),
                rout1           = h_scale*hfactor,
                ch_indi         = cav,
                intype          = true,
                cthick          = t.ds.arc_thickness,
                csize           = csize,
                chands          = 1,
                icolour         = t.arrow_colour,
                arrow_in_colour = t.arrow_in_colour,
            }
            choose_indicator(ciData)

            lcairo_restore(cr)
        end

        -- minute sweep  #########################################
        if  t.ds.style == 3 or
            t.ds.style == 4 then
            lcairo_save(cr)
            -- min sweep
            local start_angle = (0-90)
            local end_angle   = ((msecs*0.1)-90)
            local dsData = {
                x             = t.ds.x,
                y             = t.ds.y,
                arc_thickness = t.ds.arc_thickness,
                scol          = t.arrow_colour,
                R             = m_scale * mfactor * 1.05,
                start_angle   = start_angle,
                end_angle     = end_angle,
            }
            draw_sweep(dsData)

            if t.ds.style == 4 then chands = 1 end
        end
        if  t.ds.style ~= 3 and t.ds.hm_arrows < 7 or t.ds.hm_arrows > 10 then
            --print(cav,t.ds.hm_arrows)
            lcairo_save(cr)
            local ciData = {
                cr              = cr,
                px              = t.ds.x,
                py              = t.ds.y,
                startval        = 0+90,
                scale           = ((msecs*0.1)-90),
                rout1           = m_scale*mfactor,
                ch_indi         = cav,
                cthick          = t.ds.arc_thickness,
                csize           = csize,
                chands          = 1,
                icolour         = t.arrow_colour,
                arrow_in_colour = t.arrow_in_colour,
            }
            choose_indicator(ciData)
        end
    end     -- end of draw_clock_hands


    local function draw_seconds(t)

        local R = t.length
        local r = 5*t.ds.csize -- the radius of the small circle of seconds hand

        local secs = t.ds.time.sec
        local start_angle = (0 - 90)
        local end_angle   = ((secs*6) - 90)

        lcairo_set_source_rgba(cr, to_rgba(t.ds.colourSecondsHand))

        sweep_file = 'sweep3.png'

        local extents = cairo_text_extents_t:create()
        tolua.takeownership(extents)

        local lineLength, lineThickness, chands
        if t.ds.style == 1 then
            chands = nil
            lcairo_set_source_rgba(cr, to_rgba({Grey81,0.7}) )
            lcairo_arc(cr, t.ds.x, t.ds.y, R*1.3, TORADIANS, 0)
            lcairo_fill(cr)
            lcairo_set_source_rgba(cr, to_rgba(t.ds.colourSecondsRing))
            lcairo_save(cr)
            for i=1,60 do
                if (i%5==0) then
                    lineLength = 4
                    lineThickness = 1.5 * t.ds.csize
                    rini = (R + (4 * t.ds.csize)) + ((lineLength-3)/2)
                    -- lcairo_set_line_width(cr,(1.5*t.ds.csize))
                else
                    lineThickness = 0.5 * t.ds.csize
                    lineLength = 3
                    -- rini = R + (3 * t.ds.csize)
                    rini = R + (4 * t.ds.csize)
                    -- lcairo_set_line_width(cr,(0.5*t.ds.csize))
                end
                lcairo_save(cr)
                local mtData = {
                    cr            = cr,
                    startscale    = 0,
                    endscale      = 360,
                    maxscale      = 60,
                    -- rout          = R,
                    lineLength    = lineLength,
                    lineThickness = lineThickness,
                    rin           = rini,
                    xval          = t.ds.x,
                    yval          = t.ds.y,
                    i             = i,
                }
                make_scale_txt2(mtData)
            end
            lcairo_stroke(cr)
            if t.ds.csize < 0.8 then
                set_font({ cr, "Mono:8:bold" })
            else
                set_font({ cr, "Mono:11:bold" })
            end
            local degs1={"0","20","40"}
            for i=1,#degs1 do
                local mtData = {
                    cr         = cr,
                    startscale = 0,
                    endscale   = 360,
                    maxscale   = #degs1,
                    trout      = R-(r*1.85),
                    text       = degs1[i],
                    dotext     = 1,
                    ext        = extents,
                    xval       = t.ds.x,
                    yval       = t.ds.y,
                    i          = i-1,
                }
                make_scale_txt2(mtData)
            end
            lcairo_save(cr)
        elseif t.ds.style == 2 then
        -- draw seconds as a dot
            local jx,jy = get_points(0, secs*6 , 0 ,R)
            lcairo_arc(cr, t.ds.x+jx, t.ds.y+jy, t.ds.line_width*2, TORADIANS, 0)
            lcairo_fill(cr)
        elseif  t.ds.style == 3 or
                t.ds.style == 4 then
            local dsData = {
                x             = t.ds.x,
                y             = t.ds.y,
                arc_thickness = t.ds.arc_thickness,
                scol          = t.ds.colourSecondsHand,
                R             = R,
                start_angle   = start_angle,
                end_angle     = end_angle,
            }
            draw_sweep(dsData)

            if t.ds.style == 4 then chands=1 end
        elseif t.ds.style == 5 then
            chands=1
        elseif t.ds.style == 8 then
            chands=1
            t.ch_indi=8
        elseif t.ds.style == 6 or t.ds.style == 7 then
            local piData = {
                cr          = cr,
                x           = t.ds.x,
                y           = t.ds.y,
                file        = sweep_file,
                scale       = R/38.5,
                theta       = end_angle * TORADIANS,
                dial_colour = t.sweep_colour,
                rotate      = true,
                ip          = t.ds.ip,
            }
            put_image(piData)
            if t.ds.style == 6 then chands=1 end
            lcairo_save(cr)
            local ciData = {
                cr       = cr,
                px       = t.ds.x,
                py       = t.ds.y,
                theta    = end_angle * TORADIANS,
                icolour  = t.sweep_colour,
                bcolour  = t.ds.ColourBground,
                startval = 0,
                scale    = secs*6,
                rout1    = R*1.2875,
                ch_indi  = 99,
                cthick   = t.ds.line_width,
                Rings    = t.Rings,
            }
            choose_indicator(ciData)
            lcairo_restore(cr)
        end
        lcairo_save(cr)

        if  t.ds.style ~= 2 and
            t.ds.style ~= 3 and
            t.ds.style ~= 7 then
            local ciData = {
                cr       = cr,
                px       = t.ds.x,
                py       = t.ds.y,
                startval = 0,
                scale    = secs*6,
                rout1    = R,
                ch_indi  = t.ch_indi or 6,
                cthick   = t.ds.line_width,
                csize    = t.ds.csize,
                chands   = chands,
                icolour  = t.ds.colourSecondsHand,
            }
            choose_indicator(ciData)
        end
    end     -- end of draw_seconds

    local function run_gear(cr, t)
        if t.tick then
            value = t.arg or tonum(conky_parse("${updates}"))
        else
            value = tonum(conky_parse("${updates}")) % (t.max or MAX)
        end
        local theta = ((t.dir or 1) * value * (6 / (t.max or MAX))) - 90
        local piData = {
            cr     = cr,
            x      = t.x,
            y      = t.y,
            file   = t.file or 'gear1.png',
            theta  = theta,
            scale  = t.scale or 1,
            rotate = true,
            ip     = t.ip,
        }
        put_image(piData)
    end     -- end of run_gear

    local function draw_clock_face(t)
        lcairo_save(cr)
        local R = t.length*80
        local r = 5*t.ds.csize -- the radius of the small circle of seconds hand

        -- setup Face gradient
        local grad_colour
        if t.ColourBground:match("::") then  -- check for gradient 3 colours
            grad_colour    = wxsplit(t.ColourBground,"::")
            grad_colour[1] = wxsplit(grad_colour[1],":")
            grad_colour[2] = wxsplit(grad_colour[2],":")
            grad_colour[3] = wxsplit(grad_colour[3],":")
        else
            local bcolour = wxsplit(t.ColourBground,":")
            lcairo_set_source_rgba(cr, to_rgba(bcolour))
            lcairo_arc(cr, t.ds.x, t.ds.y, 149*t.ds.csize, TORADIANS,0)
            lcairo_fill(cr)
            lcairo_save(cr)
        end
        local cface
        -- msgout(t.file,t.ds.ip)
        if t.file >= 9 then
            local piData = {
                cr    =  cr,
                x     =  t.ds.x,
                y     =  t.ds.y,
                file  =  tconcat({'face', t.file, '.png'}),
                scale =  t.ds.csize,
                ip    =  t.ds.ip
            }
            put_image(piData)
        else
            cface = t.file
            if grad_colour then
                r1 = lcairo_pattern_create_radial(t.ds.x, t.ds.y, 75 * t.ds.csize, t.ds.x, t.ds.y, 150 * t.ds.csize);
                lcairo_pattern_add_color_stop_rgba(r1, 0,   to_rgba(grad_colour[1]))
                lcairo_pattern_add_color_stop_rgba(r1, 0.01, to_rgba(grad_colour[2]))
                lcairo_pattern_add_color_stop_rgba(r1, 1,   to_rgba(grad_colour[3]))
                lcairo_set_source(cr, r1);
                lcairo_arc(cr, t.ds.x, t.ds.y, 149*t.ds.csize, TORADIANS,0)
                lcairo_fill(cr);
                lcairo_pattern_destroy(r1)
            end

            local localhf, sweep_factor, show_num
            if     t.thc == 0 then localhf = 1
            elseif t.thc == 1 then localhf = 2
            elseif t.thc == 2 then localhf = 2; sweep_factor = 180
            else                   localhf = 1
            end

            if not sweep_factor then sweep_factor = 0 end

            if t.thc == 1 or t.thc == 2 then
                show_num = 1
            else
                show_num = nil
            end
            -- local cfs = {
            --     [1] = {a=1,      b=1,c=R*1.45,d=R*1.5,e=R*1.6,f=R*1.65}, -- roman circ out
            --     [2] = {a=1,      b=1,c=R*1.65,d=R*1.7,e=R*1.8,f=R*1.45}, -- roman circ in
            --     [3] = {a=localhf,b=2,c=R*1.45,d=R*1.5,e=R*1.6,f=R*1.73}, -- out
            --     [4] = {a=localhf,b=2,c=R*1.65,d=R*1.7,e=R*1.8,f=R*1.5},  -- in
            --     [5] = {a=localhf,b=3,c=R*1.45,d=R*1.5,e=R*1.6,f=R*1.64}, -- circ out
            --     [6] = {a=localhf,b=3,c=R*1.65,d=R*1.7,e=R*1.8,f=R*1.4},  -- circ in
            --     [7] = {a=localhf,b=3,c=R*1.7, d=R*1.6,e=R*1.7,f=R*1.6},  -- circ in marks
            --     [8] = {a=localhf,b=4,c=R*1.65,d=R*1.7,e=R*1.8,f=0},      -- marks only
            -- }

            cface = 5
            local cfs = {
                [1] = {a=1,      b=1,c=R*1.55,d=R*1.5,e=R*1.6,f=R*1.65}, -- roman circ out
                [2] = {a=1,      b=1,c=R*1.82,d=R*1.7,e=R*1.8,f=R*1.45}, -- roman circ in
                [3] = {a=localhf,b=2,c=R*1.55,d=R*1.5,e=R*1.6,f=R*1.73}, -- out
                [4] = {a=localhf,b=2,c=R*1.82,d=R*1.7,e=R*1.8,f=R*1.5},  -- in
                [5] = {a=localhf,b=3,c=R*1.55,d=R*1.5,e=R*1.6,f=R*1.64}, -- circ out
                [6] = {a=localhf,b=3,c=R*1.82,d=R*1.7,e=R*1.8,f=R*1.4},  -- circ in
                [7] = {a=localhf,b=3,c=R*1.82,d=R*1.6,e=R*1.7,f=R*1.6},  -- circ in marks
                [8] = {a=localhf,b=4,c=R*1.82,d=R*1.7,e=R*1.8,f=0},      -- marks only
            }

            local hour_factor, roman, marks_main, marks_sec, radius_marks, radius_num
            hour_factor  = cfs[cface].a
            roman        = cfs[cface].b
            marks_main   = cfs[cface].c
            marks_sec    = cfs[cface].d
            radius_marks = cfs[cface].e
            radius_num   = cfs[cface].f
            -- print(R, R/9, R/6)

            --- fix factors n font sizes for different clock sizes
            local myffact
            if t.ds.csize < 0.56667 then
                if t.ds.csize < 0.35556 then
                    myffact      = (30*t.ds.csize)
                else
                    myffact      = tconcat({(25*t.ds.csize),":bold"})
                end
                if roman ~= 4 then
                    marks_main   = R*1.65
                    marks_sec    = R*1.55
                    radius_marks = marks_main
                    if roman == 2 then
                        radius_num   = R*1.65
                    else
                        radius_num   = R*1.52
                    end
                end
                if show_num == 1 then
                    marks_main   = R*1.6
                    marks_sec    = R*1.7
                    radius_marks = R*1.8
                    radius_num   = 0
                end
            else
                myffact = tconcat({(21*t.ds.csize),":bold"})
            end
            set_font({ cr, tconcat({"FreeSerif:",myffact}) })

            -- show markers clock face
            local lineLength, lineThickness
            local lineMin, lineSec = R/6, R/9
            lcairo_save(cr)
            for i=1,(60*hour_factor) do
                local rini
                lineLength = lineMin
                lineThickness = 2.5 * t.ds.csize
                rini = marks_main - ((lineLength - lineSec )/2)
                if (i%5==0) and not (i%10==0) then
                    lcairo_set_source_rgba(cr, to_rgba(t.ds.colourTextMarkers))
                elseif (i%10==0) then
                    lcairo_set_source_rgba(cr, to_rgba({Red,1}))
                else
                    lineLength = lineSec
                    lineThickness = 2 * t.ds.csize
                    rini = marks_main
                    lcairo_set_source_rgba(cr, to_rgba(t.ds.colourTextMarkers))
                end
                lcairo_save(cr)
                local mtData = {
                    cr            = cr,
                    startscale    = 0,
                    endscale      = 360,
                    maxscale      = (60*hour_factor),
                    lineLength    = lineLength,
                    lineThickness = lineThickness,
                    rin           = rini,
                    xval          = t.ds.x,
                    yval          = t.ds.y,
                    i             = i,
                }
                make_scale_txt2(mtData)

                -- if (i%10==0) then
                --     lcairo_set_source_rgba(cr, to_rgba({Red,0.9}))
                --     rini = marks_main - ((lineLength - lineSec )/2)
                --     -- rini = marks_sec
                -- else
                --     lcairo_set_source_rgba(cr, to_rgba(t.ds.colourTextMarkers))
                -- end
                -- lcairo_save(cr)
                -- local mtData = {
                --     cr            = cr,
                --     startscale    = 0,
                --     endscale      = 360,
                --     maxscale      = (60*hour_factor),
                --     lineLength    = lineLength,
                --     lineThickness = lineThickness,
                --     rin           = rini,
                --     xval          = t.ds.x,
                --     yval          = t.ds.y,
                --     i             = i,
                -- }
                -- make_scale_txt2(mtData)
            end
            lcairo_stroke(cr)

            local extents = cairo_text_extents_t:create()
            tolua.takeownership(extents)
            lcairo_set_source_rgba(cr, to_rgba(t.ds.colourTextMarkers))
            lcairo_save(cr)
            -- show numbers
            if roman == 1 then
                local degs1={"I ","II","III","IV","V ","VI","VII","VIII","IX","X ","XI","XII"}
                lcairo_save(cr)
                local horiz = t.ds.x
                local verti = t.ds.y
                local radi  = radius_num
                for i=1,(12*hour_factor) do
                    local text = degs1[i]
                    local start = (((30*i)/hour_factor)-5)
                    if i == 1  then start = start+4   end
                    if i >= 3  then start = start-2   end
                    if i == 5  then start = start+4   end
                    if i >= 6  then start = start+1   end
                    if i == 10 then start = start+3.5 end
                    local cwData = {
                        cr     = cr,
                        text   = text,
                        radi   = radi,
                        horiz  = horiz,
                        verti  = verti,
                        start  = start,
                        finish = (start-3)+((string.len(text))*4.5),
                        }
                    circlewriting(cwData)
                end
            elseif roman == 2 then
                local degs1={}
                for i=1,(12*hour_factor) do
                    degs1[i] = i
                end
                lcairo_save(cr)
                for i=1,#degs1 do
                    local mtData = {
                        cr          = cr,
                        startscale  = (sweep_factor+30/hour_factor),
                        endscale    = 360,
                        maxscale    = #degs1,
                        trout       = radius_num,
                        text        = degs1[i],
                        ext         = extents,
                        dotext      = 1,
                        squeeze     = 1,
                        xval        = t.ds.x,
                        yval        = t.ds.y,
                        i           = i-1,
                    }
                    make_scale_txt2(mtData)
                end
            elseif roman == 3 then
                local degs1={}
                for i=1,(12*hour_factor) do
                    degs1[i] = tconcat({i,' '})
                end
                local horiz = t.ds.x
                local verti = t.ds.y
                local radi  = radius_num
                for i=1,(12*hour_factor) do
                    local text  = degs1[i]
                    local start = ((((sweep_factor*2)+30*i)/hour_factor)-2)
                    if i >= 10 then start = start-2   end
                    local cwData = {
                        cr     = cr,
                        text   = text,
                        radi   = radi,
                        horiz  = horiz,
                        verti  = verti,
                        start  = start,
                        finish = (start-2)+((string.len(text))*4),
                    }
                    circlewriting(cwData)
                end
                lcairo_save(cr)
            end
        end
    end     -- end of draw_clock_face

    local confTable = {
        [1]  = { b = '0x00ff00:1', e = '0x0000ff:1', i = '0xff00ff:1', j = 0, h = 12, },
        [2]  = { b = '0x00ff00:0.45', e = '0x0000ff:1', i = '0xff00ff:1', h = 12, },
        [3]  = { b = '0x00ff00:1::0x00ff00:1::0x191970:1',i='0xff00ff:1',j=0,h=13,m=0.96,},
        [4]  = { b = '0x00ff00:0.45::0x00ff00:1::0x191970:1',i='0xff00ff:1',h=13,m=0.96,},
        [5]  = { b = '0x00ffff:1',e='0x0000ff:1',i='0xff00ff:1',j = 0, h = 14, m=0.96, },
        [6]  = { b = '0x00ffff:0.45', e = '0x0000ff:1', i = '0xff00ff:1', h = 14, m=0.96, },
        [7]  = { b = '0x00ffff:1::0x00ffff:1::0x191970:1', i = '0xff33ff:1', j = 0, h = 12, },
        [8]  = { b = '0x00ffff:0.45::0x00ffff:1::0x191970:1', i = '0xff00ff:1', h = 12, },
        [9]  = { b = '0x00004c:1',e='0xffff69:1',i='0x00ff00:1',a=7,h=10,q=6,x=0,y=0,j=0,},
        [10] = { b = '0x00004c:0.79',e='0xffff69:1',i='0x00ff00:1',a=7,h=10,q=6,x=0,y=0,},
        [11] = { b = '0xffff00:1',e='0x0000ff:1',i='0xff00ff:1',j=0,h=11,m=0.90,},
        [12] = { b = '0xffff00:0.45', e = '0x0000ff:1',i='0xff00ff:1',h=11,m=0.90, },
        [13] = { b = '0xffff00:1::0xffff00:1::0x8b4513:1',i='0xff00ff:1',j=0,h=13,m=0.96,},
        [14] = { b = '0xffff00:0.45::0xffff00:1::0x8b4513:1',i='0xff00ff:1',h=13,m=0.96,},
        [15] = { a = 7,b='0x8b4513:1',h=10,j=0, },
        [16] = { a = 7,b='0x8b4513:0.75',h=10, },
        [17] = { b = '0xffaaff:1',e='0x0000ff:1',i='0x005c00:1',j=0,a=2,h=14,l=123,m=0.96,},
        [18] = { b = '0xffaaff:0.45',e='0x0000ff:1',i='0x005c00:1',a=2,h=14,l=123,m=0.96,},
        [19] = { b = '0xffbbff:1::0xffbbff:1::0x810541:1',i='0x004c00:1',h=15,j=0,m=0.91,},
        [20] = { b = '0xffbbff:0.45::0xffbbff:1::0x810541:1',i='0x004c00:1',h=15,m=0.91,},
        [21] = { b = '0x810541:1', i = '0xffff00:1', j = 0, h=11, m=0.90 },
        [22] = { b = '0x810541:0.65', i = '0x00ff00:1', h = 11, m=0.90 },
        [23] = { b = '0xcd950c:1',e='0x0000ff:1',i='0x0000ff:1',j=0,h=14,m=0.96,},
        [24] = { b = '0xcd950c:0.65', e = '0x0000ff:1', i = '0x0000ff:1', h = 14, m=0.96, },
        [25] = { b = '0xcd950c:1::0xcd950c:1::0x000000:0.8', i = '0xffff00:1',
            o = '0xac2c00:0.6', j = 0, h = 14, m=0.96, },
        [26] = { b = '0xcd950c:0.45::0xcd950c:1::0x000000:0.8', i = '0xffff00:1',
            o = '0xac2c00:0.6', h = 14, m=0.96, },
        [27] = { b = '0x000000:1', e = '0x00ffff:1', c = '0xff00ff:1', i = '0xffd700:1',
            a = 3, l = 123, m = 0.72, j = 0, },
        [28] = { b = '0x000000:0.84', e = '0xffff99:1', c = '0xff00ff:1', i = '0x00ff33:1',
            a = 7, h = 10, x = 0, y = 0, },
        [29] = { b='0xffffff:1',e='0x000000:1',i='0x4169e1:1',a=5,h=15,m=0.9,},
        [30] = { f = 9, i = '0x8b4513:1', a = 2, h = 15, x = 0, l = 123, m=0.9 },
        [31] = { f = 10, b = '0xfff1e4:1', i = '0x8b4513:1', j = 0, l=135, },
        [32] = { f = 11, b = '0xffffff:1', i = '0x8b4513:1', j = 0, l=135, },
        [33] = { f = 12, b = '0x0d0d0d0:1', i = '0x8b4513:1', j = 0, l=138, },
        [34] = { f = 13, b = '0xe4e4e4:1', i = '0x8b4513:1', j = 0, l=136, },
        [35] = { f = 14, b = '0xe4e4e4:1', i = '0x8b4513:1', j = 0, },
        [36] = { f = 15, b = '0x000000:0.77', i = '0xffff00:1', u = 1, l = 72, m = 0.9, },
        [37] = { f = 16, b = '0xffffff:1', i = '0x8b4513:1', j = 0, u = 2, },
        [38] = { f = 17, b = '0xffffff:1', i = '0x8b4513:1', j = 0, u = 1, },
        [39] = { f = 18, b = '0xffffff:1', i = '0x8b4513:1', j = 0, u = 1, },
        [40] = { f = 19,i='0xff3300:1',c='0xff5a00:1',a=8,h=12,x=0,y=0,j=0,l=132,},
        [41] = { f = 20,i='0xff3300:1',c='0xff5a00:1',a=8,h=12,x=0,y=0,j=0,l=132,},
    }

    if clock_style > 30 and clock_style < 39 then
        local confTable2 = { a = 5, h = 6, x = 0, y = 0, }
        for k,v in pairs(confTable2) do confTable[clock_style][k] = v end
    end

--- define locals
    local pxo                 = p.lwx       or 0
    local pyo                 = p.lwy       or 0
    local base_size           = p.s         or 150
    local show_seconds        = p.a         or confTable[clock_style].a or 1
    local ipath               = tconcat({p.imgpath,'/lua_clock/'})
    local clock_arrows        = p.h         or confTable[clock_style].h or 6
    local arrow_colour        = p.i         or confTable[clock_style].i or '0x00ff00:1'
    local cface_image         = p.f         or confTable[clock_style].f or 5
    local dial_colour         = p.d         or nil
    local colourSecondsHand   = p.c         or confTable[clock_style].c or '0xff0000:1'
    local colourSecondsRing   = p.r         or '0x000000:1'
    local ColourBground       = p.b         or confTable[clock_style].b or '0xcd950c:0.45::0xcd950c:1::0x000000:1'
    local colourTextMarkers   = p.e         or confTable[clock_style].e or '0xffffff:1'
    local arrow_in_colour     = p.o         or confTable[clock_style].o or '0xffff00:0.6'
    -- 0 for 12 hr, 1 for 24 hr,2 for reverse 24 hr [0]
    local thc                 = p.u         or confTable[clock_style].u or nil
    local clock_hand_size     = p.m         or confTable[clock_style].m or 0.85
    local clock_sec_size      = p.l         or confTable[clock_style].l or 115
    local line_thickness      = p.w
    local show_ampm           = p.y         or confTable[clock_style].y or 1
    local show_dd             = p.x         or confTable[clock_style].x or 1
    local show_gears          = p.j         or confTable[clock_style].j or 1
    local tilt                = p.t         or nil
    local Rings               = p.q         or confTable[clock_style].q or 6
    local BezelType           = p.z         or 0
    local offset              = p.v         or 0

    local center_gear         = 'gear1.png'
    local top_left_gear       = 'gear2.png'
    local bottom_gear         = 'gear7.png'
    local top_right_gear      = 'gear8.png'
    local bottom_right_gear   = 'gear3.png'
    local spring_wheel        = 'gear9.png'

    if base_size <= 110 then
        line_thickness = line_thickness or base_size * 0.02
    elseif base_size <= 200 then
        line_thickness = line_thickness or base_size * 0.01
    else
        line_thickness = line_thickness or 2
    end

    if thc == 0 then thc = nil end

    colourSecondsHand = wxsplit(colourSecondsHand,   ":")
    colourSecondsRing = wxsplit(colourSecondsRing,  ":")
    colourTextMarkers = wxsplit(colourTextMarkers, ":")
    arrow_colour      = wxsplit(arrow_colour,   ":")
    arrow_in_colour   = wxsplit(arrow_in_colour,":")
    dial_colour       = wxsplit(dial_colour,":")

    local max_clock_size = 310
    local clock_size     = base_size / max_clock_size
    local px             = (pxo+(base_size*0.52))
    local py             = (pyo+(base_size*0.52))

    local bimageT        = wxsplit(BezelType,':')
    local bimage         = bimageT[1] or 0
    local bezStyle = {}
    if tonum(bimage) == 1 then          -- use instrument_sqr
        bezStyle[1] = 1
        clock_size = clock_size * 0.95
    elseif tonum(bimage) == 2 then      -- use bezel
        bezStyle = {2, bimageT[2] or 2, bimageT[3] or 2.5}
    end

    -- clock hands
    local secfact
    if clock_size < 0.5 then
        secfact = 4
    else
        secfact = 2
    end

    local line_thickness = secfact * clock_size * line_thickness
    local arc_thickness = arc_thickness1 or line_thickness * 1.25

    local time = os.date('*t')
-- common values for pointer options
    local ds = {
        x                 = px,
        y                 = py,
        time              = time,
        csize             = clock_size,
        colourSecondsHand = colourSecondsHand,
        colourSecondsRing = colourSecondsRing,
        colourTextMarkers = colourTextMarkers,
        ColourBground     = ColourBground,
        line_width        = line_thickness,
        ip                = ipath,
        arc_thickness     = arc_thickness,
        style             = show_seconds,
        thc               = thc,
        show_sweep        = 1,
        hm_arrows         = clock_arrows,
    }
    -- io.stderr:write(string.format("incoming offset %d \n",offset))

    local offsethr = 0
    local offsetmin = 0
    offset = tonum(offset)
    offset = offset / 30
    -- io.stderr:write(string.format("offset div 30 %d \n",offset))
     if (offset % 2) == 0 then
        offsethr = offset / 2
    else
        offsethr = (offset - 1) / 2
        offsetmin = 30
    end
    -- io.stderr:write(string.format("offset %d  offsethr %d  offsetmin %d\n",offset,offsethr,offsetmin))

    if (ds.time.hour + tonum(offsethr)) >= 24 then
        ds.time.hour = (ds.time.hour + tonum(offsethr)) - 24
    else
        ds.time.hour = ds.time.hour + tonum(offsethr)
        if ds.time.hour < 0 then ds.time.hour = ds.time.hour + 24 end
    end

    if (ds.time.min + tonum(offsetmin)) >= 60 then
        ds.time.min = (ds.time.min + tonum(offsetmin)) - 60
        ds.time.hour = ds.time.hour + 1
    else
        ds.time.min = ds.time.min + tonum(offsetmin)
    end
    -- io.stderr:write(string.format("offset %d  offsethr %d  offsetmin %d\n",offset,offsethr,offsetmin))
    -- io.stderr:write(string.format("ds.time.hour %d  ds.time.min %d\n\n",ds.time.hour,ds.time.min))

-- clock bezel ##############################################
    -- print('>>',bezStyle[1],bezStyle[2],bezStyle[3],bimage)
    if tonum(bezStyle[1]) == 1 then
        lcairo_save(cr)
        local Tv = {
            cr    = cr,
            tilt  = tilt,
            routI = base_size,
            wx    = px-(base_size*0.5),
            wy    = py-(base_size*0.5),
        }
        instrument_sqr(Tv)
        lcairo_restore(cr)
    elseif tonum(bezStyle[1]) == 2 then
        lcairo_save(cr)
        local Tv = {
            cr         = cr,
            bezel_size = base_size,
            px         = px,
            py         = py,
            tilt       = tilt,
            style      = bezStyle
        }
        make_bezel(Tv)
        lcairo_restore(cr)
    elseif tilt then
        local matrix0 = cairo_matrix_t:create()
        tolua.takeownership(matrix0)
        local t1 = wxsplit(tilt,':')
        lcairo_matrix_init(matrix0, t1[1],t1[2],t1[3],t1[4],0,0)
        lcairo_translate(cr,px,py)
        lcairo_transform(cr,matrix0)
        lcairo_translate(cr,-px,-py)
    end

-- show gears
    if show_gears == 1 then
        local gearTable = {
            -- bottom left spring wheel
            {
                x     =  px-(42*clock_size),
                y     =  py+(30*clock_size),
                file  =  spring_wheel,
                max   =  2,
                scale =  clock_size*.8,
                dir   =  1,
                ip    =  ipath,
            },
            -- top left gear
            {
                x     =  px-(39*clock_size),
                y     =  py-(25*clock_size),
                file  =  top_left_gear,
                max   =  60,
                scale =  clock_size,
                dir   =  -1,
                tick  =  true,
                ip    =  ipath,
            },
            -- top right gear
            {
                x     =  px+(33*clock_size),
                y     =  py-(27*clock_size),
                scale =  clock_size,
                file  =  top_right_gear,
                max   =  300,
                dir   =  -1,
                ip    =  ipath,
            },
            -- center gear
            {
                x     =  px,
                y     =  py,
                file  =  center_gear,
                scale =  clock_size,
                ip    =  ipath,
            },
            -- bottom gear
            {
                x     =  px+(0*clock_size),
                y     =  py+(49.5*clock_size),
                file  =  bottom_gear,
                max   =  60,
                scale =  clock_size*0.75,
                dir   =  1,
                ip    =  ipath,
            },
            -- bottom right gear
            {
                x     =  px+(35*clock_size),
                y     =  py+(29*clock_size),
                file  =  bottom_right_gear,
                dir   =  -1,
                scale =  clock_size,
                ip    =  ipath,
            },
        }
        for x in pairs(gearTable) do
            run_gear(cr, gearTable[x])
        end
    end

------ draw clock face
    local Tv = {
        ds            =  ds,
        thc           =  thc,
        ColourBground =  ColourBground,
        dial_colour   =  dial_colour,
        file          =  cface_image,
        length        =  clock_size,
    }
    draw_clock_face(Tv)

    lcairo_save(cr)
    local dSec = {}
    if show_seconds == 1 then
        ds.y        =  ds.y+(50*clock_size)   -- change ds.y show seconds at offcenter
        dSec = {
            ds      = ds,
            ch_indi = 6,
            length  = (clock_size * 25)
        }
        draw_seconds(dSec)
        ds.y        =  ds.y-(50*clock_size)   -- reset ds.y show seconds at offcenter
    elseif  show_seconds == 2 or show_seconds == 3  then
        dSec = {
            ds      = ds,
            length  =  (clock_size * (clock_sec_size or 120) )
        }
        draw_seconds(dSec)
    end
    lcairo_restore(cr)

------ draw clock hands
    lcairo_save(cr)
    local Tv = {
        ds              =  ds,
        m_scale         =  clock_size * 0.9 * clock_hand_size,
        h_scale         =  clock_size * 0.7 * clock_hand_size,
        arrow_colour    =  arrow_colour,
        arrow_in_colour =  arrow_in_colour,
        show_ap         =  show_ampm,
        length          =  clock_size * 25,
        show_dd         =  show_dd,
    }
    draw_clock_hands(Tv)
    lcairo_restore(cr)

    lcairo_save(cr)
    if  show_seconds > 3 then
        dSec = {
            ds            = ds,
            sweep_colour  = arrow_colour,
            length        = (clock_size * (clock_sec_size or 120)),
            Rings         = Rings,
        }
        draw_seconds(dSec)
    end
    lcairo_restore(cr)

    if ds.style ~= 3 then
        lcairo_set_source_rgba(cr,0.5,0.7,0,1)
        lcairo_arc(cr, px, py, (clock_size * 4), TORADIANS, 0)
        lcairo_stroke(cr)
    end

end

collectgarbage()

return w
