#!/bin/bash

 #bpath1="$( dirname "$1" )"
 #mpppath="$( cd "$bpath1" && pwd )"

# debug='true'
# This defines the output of set -x debugging xtrace
if [[ $debug ]]; then
    rm -rf '/tmp/wxmpdebug'
    export PS4='+ (${LINENO}): ${FUNCNAME[0]:+${FUNCNAME[0]}(): }'
    exec 4>>'/tmp/wxmpdebug'
    BASH_XTRACEFD=4
    set -x
fi

msgout() { printf "\n%b\n" "$1" >&2; }

version=200228

bpath2="$( dirname "$0" )"
mypath="$( cd "$bpath2" && pwd )"


mpptmp="/tmp/wxmptmp"
wxfifo="${mpptmp}/mpfifo"
plist_path="${mpptmp}/plist_path"
flist="${mpptmp}/tmpflist.m3u"
tmpflist='/tmp/tmpflist.m3u'
tmpflist2='/tmp/tmpflist2.m3u'
tolua="${mpptmp}/tolua"
shuffle="${mpptmp}/shuffle"
lloop="${mpptmp}/loop"
llcnt="${mpptmp}/lcnt"
random="${mpptmp}/random"
ctrk="${mpptmp}/ctrk"
CurrTrack="${mpptmp}/trackno"
CurPosnFile="${mpptmp}/ANS_time_pos"
PauseStatus="${mpptmp}/ANS_pause"
tempri="${mpptmp}/tempri"
preflist="/tmp/preflist.m3u"

run_mplayer="${mypath}/rmp.sh"

icp="$HOME/.config/conkywx/icp"
system_config_file="$HOME/.config/conkywx/system.conf"
player_config_file="$HOME/.config/conkywx/wxmp.conf"

wxloc=$(grep -x 'wxloc=.*' "$system_config_file")
if [[ ${wxloc} != '' ]]; then
    wxloc="${wxloc##*=}"
    wxloc=${wxloc//\"/}
    MakePlaylist="${wxloc}/lib/dompp.sh"
else
    MakePlaylist="${mypath}/lib/dompp.sh"
fi

fileTypeA='mp2 mp3 mp4 m4a mp4a mkv mpg mpeg ogg ogm wma wmv asf qt mov rm ra ram flac avi flv f4v swf mpa mpv mts rmvb vob ts webm weba 3gp2 3gp'

fileTypeB=$(grep -e 'wxmpFileTypes=.*' "$system_config_file")
if [[ $fileTypeB == '' ]]; then
  printf "%s\n" "# wx media player file types - you can add more to the line" >> "$system_config_file"
  printf "%s\n" "wxmpFileTypes=\"${fileTypeA}\"" >> "$system_config_file"
  fileTypeB=( $(echo ${fileTypeA}) )
else
  fileTypeB="${fileTypeB//[\"\']}"
  fileTypeB=( $(echo "${fileTypeB/*=}") )
fi

DefTerm=$(grep -x 'DefTerm=.*' "$system_config_file")
if [[ ${DefTerm} != '' ]]; then
    DefTerm="${DefTerm##*=}"
    DefTerm="${DefTerm//\"/}"
else
    DefTerm=$(bash ${wxloc}/lib/xdg-terminal.sh)
    #DefTerm=$(bash ./xdg-terminal.sh)
    if [[ $DefTerm -ne -1 && $DefTerm != '' ]]; then
        echo "DefTerm=$DefTerm" >> "$system_config_file"
        #echo "DefTerm=$DefTerm"
    else
        echo -e "\n\nError :( Houston we have a Problem. \nWe need a terminal.\n\n"
    fi
fi

TerminalTitle() {
    printf "%b" "\e]2;$1\a"
}

riUpdate(){
    if [[ -e ${tempri} ]]; then
        wxmpRefreshInterval=$(< "$tempri" )
    else
        wxmpRefreshInterval=1
    fi
}
riUpdate

save_error(){
cat <<-sverr

  You do not appear to have zenity installed. No worries - just copy
  the playlist:

      /tmp/wxmptmp/tmpflist.m3u

  to a directory of your choice and name it what ever you like. :-)

sverr
}

doclear(){

    riUpdate
    #clear
    clear && echo -en "\e[3J"
    printf "\033c"
}


show_help(){
cat <<-helpmsg
Program: wxmp - wx media player
Usage:   wxmp [options] [path/]filename

Fast Start::

    ffprobe is used to index files - part of ffmpeg package
    mplayer is used to index files if ffmpeg/ffprobe is not installed

    o  open any folder in file manager -
    o  select all tracks
    o  right click mouse and choose "wx media player"
       OR
    o  open a terminal in a folder or navigate to it
    o  type wxmp * - you will see files being index and hear music start
       if media files exist in that folder
    o  type wxmp -j to get wxmp user interface *jump*.

Basic options:
 -v|--version          - player version
 -q|--quit             - quit player
 -h|--help             - view this help screen

 -j|--jump             - view playlist - choose track or see options below

jump view screen logical commands ::
      e                - Edit Mode - User has 30 seconds to enter options
   ** Track Numbers    - Use 'e' to enter Edit Mode and enter number of track
                         to play and press Enter Key.
      n                - jump to next track
      p                - jump to previous track
      Page Up          - move 1 page back    - towards top of playlist
      Page Dn          - move 1 page forward - towards bottom of playlist
      Home             - move to first page
      End              - move to last page
      Arrow Keys::
      Right | Left     - seek +-10 seconds
      Up    | Down     - seek +-60 seconds
      Ctrl-Up|Ctrl-Down- seek +-600 seconds (10 mins)
      SpaceBar | pause - pause and resume player
      .                - Step forward. Pressing once will pause media, every
                         consecutive press will play one frame and then go into
                         pause mode again (any other key unpauses)
      +                - Increase Master Volume
      -                - Decrease Master Volume
   ** s | seek         - seek same as --seek below
      [                - Track speed - reduce
      ]                - Track speed - increase
      Delete Key       - Track speed - reset
      V | save         - save playlist (same --save option)
      u | shuffle      - play tracks in random order
      U | noshuffle    - play tracks in sequential order
      q                - quit jump view or help screen
      Q | quit         - quit wx media player
      F5|r|Enter Key   - refresh view + Go To view track No. - this will
                         do the page_jump on which the track exists
   ** d                - delete tracks from playlist:: d <from> <to>
                         for example, using 12 and 30 as track numbers::
                         delete single track : d 12
                         delete many tracks  : d 12 30
   ** l                - loop same as --loop below
      L                - stop loop of track
   ** gNo              - go to screen page
   ** gtNo             - go to the screen page which has the track number
   ** f                - search for tracks - use word or words
                         separate words with space for normal search
                         separate words with | character for multiple
                         search words
      F                - Return to original playlist
      b                - popup progress window
      B                - popup progress window inverse track time
      i                - Toggle to inverse Title bar progress time

Note::
   ** - These items require Edit Mode [press 'e' to enter edit mode]
   Rest of the commands are single key operation.

command line options passed to wxmp command ::
 --pause               - pause player
 -u|--shuffle          - play tracks in random order
 -U|--noshuffle        - play tracks in sequential order
 -V|--save             - save playlist
 -d|--delete           - delete tracks from playlist:: d <from> <to>
                         for example, using 12 and 30 as track numbers::
                         delete single track : d 12
                         delete many tracks  : d 12 30
 -f|--find             - search for tracks - use word or words
 -b|--progress         - popup progress window - add 1 to inverse track time
 -s|--seek             - seek <value> [type]
                         Seek to some place in the track.
                         0 - a relative seek of +/- <value> seconds (default)
                         1 - a seek to <value> % in the track
                         2 - a seek to an absolute position of <value> seconds
 -l|--loop             - set track repetition number
                         0 for repeat 99 times
 -L                    - Stop loop of track
 --load                - load files from the command line
                          -i  - search pattern types
                                default search pattern "mp2|mp3|mp4|m4a|mp4a|
                                mkv|mpg|mpeg|ogg|ogm|wma|wmv|asf|qt|mov|rm|ra|
                                ram|flac|avi|flv|f4v|swf|mpa|mpv|mts|rmvb|vob|
                                ts|webm|weba|3gp2|3gp"
                                This is stored in ~/config/conkywx/system.conf
                          -ui - alternate search pattern Eg: "flv|mp4"
                          -xd - exclude folder(s) "learn|abcd.*efgh|some thing"
                          -xn - exclude filename(s) "abcd efgh|learn|simple"
                          -xs - search for specific file types
                                Eg: "avi|mp4" thus ignoring the default pattern
                          -f  - Do not follow symbolic links - default: follow
                          -m  - Max depth into folder tree - default is 10
                                For example: "abcd efgh" are 2 words - if they
                                have any other number of characters between
                                them - then use "abcd.*efgh"
                                The pipe character | is used as separator.
                                Simple regular expressions can be used
                          -w  - path to search for files
                                For example: '/home/adamo/media' for seaching
                                in the media directory for audio files
 *                     - load files from current folder - uses default pattern
                         used in --load option above.
 -c|--custom           - send custom commands to mplayer
                         For custom commands to send to mplayer:
                         in a terminal refer to mplayer -input cmdlist
                         view http://www.mplayerhq.hu/DOCS/tech/slave.txt

Adding shortcuts to keys::
  o  /usr/bin/wxmp -n       map to XF86AudioNext
  o  /usr/bin/wxmp -p       map to XF86AudioPrev
  o  /usr/bin/wxmp --pause  map to XF86AudioPlay
  o  /usr/bin/wxmp -q       map to Any key to quit player
  Any of the commands for wxmp can be used to map to keyboard.

Adding Tracks to wx media player::
  m3u playlists or individual tracks
  o  File Browser / Manager
     Right click on the media / track and select "wx media player".
  o  Command Line
     Navigate to the Directory that contains the media tracks
     1) Use wxmp --load. To customize this operation see --load above.
     This can be used in the directory which contains the media and will
     media in sub-directories as well
     OR
     2) use wxmp "/path to directory/*" for all media in that directory
     OR
     3) use wxmp "/path to directory/*.mp3" for specific media type in that
     directory
     However if you are already in the directory in question - use wxmp "*"
     or wxmp "*.m4a" as required
     NOTE: You need to quote the path to directory as shown above for
     options 2) and 3)
  o  wxmp config file is located at ~/.config/conkywx/wxmp.conf
  o  mplayer key input config file is located at ~/.config/conkywx/wxmpin.conf

2015 - 2017 (c) Paramvir Likhari

helpmsg
}

save_list(){
    if [[ -e "$flist" ]]; then
        fname=$(zenity --file-selection --filename="$HOME/$USER" --file-filter='*.m3u' --confirm-overwrite --save --title="Save wx media player playlist" )
        case "$fname" in
        *.[mM]3[uU])              cp "${flist}" "${fname}" ;;
        *)  fname="${fname}.m3u"; cp "${flist}" "${fname}" ;;
        esac
    fi
}

shuffle(){

  trackTime=$(< '/tmp/wxmptmp/tpt' )
  tr1=${trackTime#*::}

  if [[ $1 == 'true' ]]; then
    seq 1 1 "$tr1" | shuf -o '/tmp/wxmptmp/random' &
    echo "0:$tr1" > '/tmp/wxmptmp/ctrk'
    echo "shuffle" > $shuffle
  else
    rm -rf $shuffle $random $ctrk
  fi
}

loops(){
  if [[ $2 == 'true' ]]; then
    echo "loop $1" > $lloop
  else
    echo "loop -1" > $lloop
    rm -rf $lloop $llcnt
    lptxt=
  fi
}

ProgressBar(){
  if [[ $1 -eq 3 ]]; then DefTerm= ; fi

  case $DefTerm in
  xterm)
    xterm +bc +vb +mb -fa 'Mono' -fs 11 -bg 'black' -fg 'green' -geometry 80x4 -e "${wxloc}/lib/barwxmp.sh $1" &
    ;;
  xfce4-terminal)
    $DefTerm --title "track progress - wx media player" --hide-menubar --hide-toolbar --geometry 80x4 -e "${wxloc}/lib/barwxmp.sh $1" &
    ;;
  *)
    if [[ $1 -eq 3 ]]; then
        "${wxloc}/lib/barwxmp.sh" &
    else
        $DefTerm --title "track progress - wx media player" --hide-menubar --geometry 80x4 -e "${wxloc}/lib/barwxmp.sh $1" &
    fi
    ;;
  esac
}


GetFileList(){

    gflist=$(printf "|%s" "${fileTypeB[@]}")
    # gflist="+(${gflist/\|/})"
    gflist="${gflist/\|/}"
    # gflist='mp3|m4a|ogg|wma|flac'

    FollowSymLink='-L'
    while [ $# -gt 0 ]
    do
        case "$1" in
        -i)   shift; addin="$1"
              ;;
        -ui)  shift; gflist=; addin="$1"
              ;;
        -xd)  shift                        # exclude directories "learn|abcd.*efgh|some thing"
              orgifs="$IFS"; IFS=$'\|'
              read -a excp1 <<<"$1"
              for d1 in "${excp1[@]}" ; do
                  excp+="\|\/${d1// /\\ }\/"
              done
              IFS="$orgifs"
              ;;
        -xn)  shift                        # exclude filename(s) "abcd efgh|learn|simple"
              orgifs="$IFS"; IFS=$'\|'
              read -a excn1 <<<"$1"
              IFS="$orgifs"
              for d2 in "${excn1[@]}" ; do
                  excp+="\|${d2// /\\ }"
              done
              ;;
        -xs)  shift                       #search for specific file types ignore default
              gflist="$1"
              ;;
        -f)   FollowSymLink=
              ;;
        -m)   shift; MaxDepth="$1"
              ;;
        -w)   shift; lookIn="$1"
              ;;
        esac
        shift
    done

    if [[ ! $lookIn ]]; then
        lookIn="${PWD}"
    fi
    excp="${excp#\\|*}"
    MaxDepth=${MaxDepth:-10}

    gflist+="|${addin}"

    case "$lookIn" in
      *\/\**) lookIn="${lookIn%\/\**}" ;;
      *\/)    lookIn="${lookIn%\/}"    ;;
    esac


    find ${FollowSymLink} "${lookIn}"/ -maxdepth ${MaxDepth} -type f -regextype posix-awk -iregex ".*\.("${gflist}")" -print | if [[ "${excp}" ]]; then sed '/'"${excp}"'/Id'; else cat - ; fi | sort > "$preflist"

}

SearchTerm(){

    SearchFor9="$@"
    SearchFor=${SearchFor9// /\.\*}

    declare -A seen
    FisrtTrack=

    orgifs="$IFS"; IFS=$'|'
    SearchArr=( $(echo "$SearchFor") )
    IFS="$orgifs"

    for s in "${SearchArr[@]}"
    do
        orgifs="$IFS"; IFS=$'\n'
            SearchArrayIn+=( $(awk 'BEGIN {IGNORECASE=1};/'"$s"'/{if (!seen[$0]++) print NR":"$0;}' "${flist}" ) )
        IFS="$orgifs"
    done

    if [[ "${SearchArrayIn[1]}" != '' ]]; then
        echo "${SearchFor9}" > "${mpptmp}/sf"
        printf "%s\n" "#EXTM3U" >  "${flist}AA"
        for f in "${SearchArrayIn[@]}"
        do
            count=$((count+1))
            TNum="${f%%:*}"
            TName="${f#*:}"
            case "$TName" in
            ['#']*)
                CNum=$((TNum+1))
                ;;
            *)
                CNum=$((TNum-1))
                ;;
            esac
            if [[ ${seen[@]} =~ $TNum ]]; then continue ; fi
            CName=$(tail -n +$CNum "${flist}" | head -n 1 | perl -ne 's/[^[:ascii:]]/+/g; print' )
            seen+=$CNum
            case "$CName" in
            ['#']*)
                printf "%s\n%s\n" "$CName" "$TName" >>  "${flist}AA"
                if [[ "$FisrtTrack" == '' ]]; then FisrtTrack="$TName"; fi
                ;;
            *)
                printf "%s\n%s\n" "$TName" "$CName" >>  "${flist}AA"
                if [[ "$FisrtTrack" == '' ]]; then FisrtTrack="$CName"; fi
                ;;
            esac
        done

        mv "${flist}" "${flist}-org"
        mv "${mpptmp}/tpt" "${mpptmp}/tpt-org"
        mv "${mpptmp}/trackno" "${mpptmp}/trackno-org"
        mv "${mpptmp}/ct_info" "${mpptmp}/ct_info-org"
        mv "${mpptmp}/fstat" "${mpptmp}/fstat-org"
        rm "${mpptmp}/shuffle"
        echo "$TracksPerPage" > "${mpptmp}/TracksPerPage-org"
        mv "${flist}AA" "${flist}"
        echo '1' > "${mpptmp}/trackno"
        echo "loadfile '${FisrtTrack//\'/\\\'}'" > "$wxfifo"
    fi
}


ShowMessage(){
    doclear
    printf "%b" "$Header"
    echo -e "\n\n\n\n\n${hspace}$1"
}

delete_track(){

    doclear

    local InVal=($( echo "$1" ))
    case "$1" in
    *d*)
        if [[ ${InVal[2]} ]]; then ToTrack="${InVal[2]}"; FromTrack="${InVal[1]}"
        else ToTrack="${InVal[1]}"; FromTrack="${InVal[1]}"
        fi
        ;;
    *)
        if [[ "${InVal[1]}" ]]; then ToTrack="${InVal[1]}"; FromTrack="${InVal[0]}"
        else ToTrack="${InVal[0]}"; FromTrack="${InVal[0]}"
        fi
    esac

    if [[ $FromTrack -gt $ToTrack ]]; then
        swap1=$FromTrack
        FromTrack=$ToTrack
        ToTrack=$swap1
    fi

    wh=$(stty size)
    dnrows=${wh% *}; dncols=${wh#* }

    local last_track=$(< '/tmp/wxmptmp/trackno' )

    if [[ $InVal ]]; then
        track_number1=$FromTrack
        dhline=$(printf '=%.0s' $(seq $dncols) )
        dheader='List of tracks to be deleted from the playlist'
        dhLen=${#dheader}
        dlhdstr=$(((ncols-dhLen)/2))
        dhspace=$(printf ' %.0s' $(seq $dlhdstr) )
        dheader="${dhspace}${dheader}"

        printf "%s\n" "$dheader"
        printf "%s\n" $dhline
        for d in $( seq $FromTrack $ToTrack )
        do
            local track_name=$(sed '/^$/d;/^#EXT/d' "${flist}" | sed ''"$d"'q;d' )
            TracksArr+=( "$track_name" )
            tLen=${#track_name}
            if [[ $tlen -lt 4 ]]; then tlen=4; fi
            tLen=$((dncols-(tlen+5)))
            printf "%4d. %.${tLen}s\n" "$d" "${track_name##*/}"
            c=$((c+1))
        done
        printf "%s\n" $dhline
        printf "%s" "Delete above $c track(s) from Playlist [Y/n]: "

        read -r ch1
        case $ch1 in
        [yY])
            local c=$FromTrack
            printf "%s\n" $dhline
            for d in "${TracksArr[@]}"
            do
                c=$((c+1))
                track_name="$c $d"
                tLen=${#track_name}
                if [[ $tlen -lt 4 ]]; then tlen=4; fi
                tLen=$((dncols-tlen))
                printf "%.${tLen}s\n" "Deleting: ${c}. ${track_name##*/}"

            done
            printf "%s\n" $dhline
            tt1=$(((ToTrack * 2) + 1 ))
            ft1=$((FromTrack * 2))
            sed -i ''"$ft1"','"$tt1"'d' "$flist"


            if [[ $track_number1 -lt $2 ]]; then last_track=$((last_track-1)); fi
            local last_track1=$(sed '/^$/d;/^#EXT/d' "${flist}" | sed ''"$last_track"'q;d' )
            echo "loadfile '${last_track1//\'/\\\'}'" > "$wxfifo"
            echo "${last_track}" > '/tmp/wxmptmp/trackno'
            rm -rf "${mpptmp}/tpt" "$mpptmp/firstrun"
            ;;
        *)    echo "Nothing... aborting delete operation... "
        esac
    else
        echo "No track number entered to delete ..."
    fi
}

lets_quit(){
    if [[ $(pidof mplayer) != '' ]]; then
        echo "quit"  > $wxfifo
        rm -rf $mpptmp                      2> /dev/null
        killall -SIGTERM mplayer wxmp.sh rmp.sh $BarPid    2> /dev/null
    fi
    exit 0
}


RefreshPage(){

    if [[ -s '/tmp/wxmptmp/tn' ]]; then
        trackno=$(< '/tmp/wxmptmp/tn' )
    elif [[ -s '/tmp/wxmptmp/trackno' ]]; then
        trackno=$(< '/tmp/wxmptmp/trackno' )
    fi
    if [[ $trackno -gt $TracksPerPage ]]; then
        inval=$((((trackno-ks)/TracksPerPage)+2))
    else
        inval=1
    fi
    page_jump=$(((inval*TracksPerPage)-ks))
    d=$page_jump
    x3=$((d+2))
    echo $((d+TracksPerPage)) > '/tmp/wxmptmp/mtrk'
    exec "$0" --jump && exit

}

AreWePaused() {
    # create a manual toggle file for pause - so if the player pauses and this file
    # does not exist - we jump to next track as all auto pauses are happening
    # at end of variable bitrate tracks
    if [[ -e '/tmp/wxmptmp/ipause' ]]; then
        rm '/tmp/wxmptmp/ipause'
    else
        if [[ $1 ]]; then echo "1" > '/tmp/wxmptmp/ipause' ; fi
    fi
}

TrackSeek(){
    SeekVal="$1"
    if [[ -e "$CurPosnFile" ]]; then
        CurrPosn=$(sed 's/\..*//' "$CurPosnFile")
    fi
    if [[ $TrackTime ]]; then
        CurrPosn=${CurrPosn%.*}
        SeekVal=${SeekVal%.*}
        SeekVal1=$(( CurrPosn + SeekVal ))
        if [[ "$SeekVal1" -lt 0 ]]; then SeekVal1=0 ; fi
        if [[ $SeekVal1 -lt $TrackTime ]]; then
            echo "seek $SeekVal1 2" > $wxfifo
        else
            SeekVal=
        fi
    fi
}

KeyTest() {

{   stty_state=$(stty -g)
    stty raw isig -echo
    keycode=$(dd bs=8 conv=sync count=1)
    stty "$stty_state"
} </dev/tty 2>/dev/null
printf %s "$keycode" | xxd

    exit 0

}

jump(){

    TerminalTitle "wx media player - Play List Control"

    w_h=$(stty size)
    nrows=${w_h% *}; ncols=${w_h#* }

    if [[ -s '/tmp/wxmptmp/tpt' ]]; then
        trackTime=$(< '/tmp/wxmptmp/tpt' )
        tr1=${trackTime#*::}          # get actual number of tracks
        trackTime=${trackTime%::*}

        #tr=$(((tr1-1)/2))
        tr1=${tr1%.*}
        TotalNumOfTracks=$tr1
        TotalTracks=$tr1

        trackTimeF=$(echo "$trackTime" | awk '{
            printf "%02dy %02dd %02d:%02d:%02d", ($1/(3600*24*365.25)), (($1/(3600*24))%365.25), (($1/3600)%24),(($1/60)%60),($1%60) ;
        }')
    else
        local tr1=$(wc -l "${flist}")      # reading from tmp playlist file
        tr1="${tr1% *}"
        tr1=$(((tr1-1)/2))                 # get actual number of tracks
        TotalNumOfTracks=$tr1              # get actual number of tracks
        TotalTracks=$tr1
    fi

    nrows=$(( nrows - 5 ))          #### Change the number 5 to adjust the TracksPerPage value
    TracksPerPage=$nrows            #### This is the lines for number of lines to show
    ks=$(( TracksPerPage + 1 ))

    if [[ $TotalNumOfTracks -gt $TracksPerPage ]]; then
        while [[ $mod != 0 ]]
        do
            mod=$(( TotalNumOfTracks % TracksPerPage ))
            if [[ $mod -ne 0 ]]; then TotalNumOfTracks=$((TotalNumOfTracks+1)); fi
        done
        TotalNumOfTracks=$((TotalNumOfTracks-1)); fact=$TracksPerPage
    else
        fact=$TotalNumOfTracks
    fi

    x3=1
    y=-1

    ldrstr="Options: PgUp|PgDn|p:<|n:>|[b|B]:progress|saVe|q:quit|Q:quit wxmp|h:help"
    ldrstr2="e:edit|F5:refresh : "
    lhdstr="${#ldrstr}"
    lhdstr=$(((ncols-lhdstr)/2))
    hspace=$(printf ' %.0s' $(seq $lhdstr) )
    ldrstr="${hspace}${ldrstr}\n${hspace}${ldrstr2}"

    hline=$(printf '=%.0s' $(seq $ncols) )

    if [[ "${trackTimeF}" == '' ]]; then trackTimeF='Refresh screen'; fi
    Header="wx media player playlist - Total Tracks: ${TotalTracks}  Time: ${trackTimeF}"
    hlen=${#Header}
    hlen=$(((ncols-hlen)/2))
    hspace1=$(printf ' %.0s' $(seq $hlen) )
    Header="${hspace1}${Header}\n$hline"
    doclear
    printf "%b" "$Header"

    while :
    do
        ESC=$(printf "\033")

        for ((d=0; d<=$TotalNumOfTracks; d++))
        do
            if [[ -s '/tmp/wxmptmp/mtrk' ]]; then
                local mtrk=$(< '/tmp/wxmptmp/mtrk' )
                rm -rf '/tmp/wxmptmp/mtrk'
                d=$((mtrk-TracksPerPage))
                x3=$((d+2))
                continue 1
            fi
            if [[ $d -ge $TotalNumOfTracks ]]; then
                x3=1
            fi

            ##==========================

            if [[ $((d%TracksPerPage)) -eq 0 ]]; then
                inarr=()
                t1=()
                local x2=$((x3*2))
                jt=$((TracksPerPage*2))
                if [[ $d -lt $((TotalTracks/2)) ]]; then
                    TrackNoToFind="+${x2}"
                else
                    TrackNoToFind=$(((((TotalTracks*2)+1)-x2)+1))
                fi
                orgifs="$IFS"; IFS=$'\n'
                    inarr=($(tail -n $TrackNoToFind "${flist}" | head -n $jt | perl -ne '
                        if ( ! /^#EXT/ ) {
                            # s/[^[:ascii:]]/+/g;
                            print "$_\n" }' ))
                    t1=($(tail -n $TrackNoToFind "${flist}"  | head -n $jt | perl -ne '
                        if ( /^#EXTINF/ ) {
                            # s/[^[:ascii:]]/+/g;
                            m{^#EXTINF:(\d+),.*$};
                            print "$1\n"  }' ))
                IFS="$orgifs"
                x3=$((x3+TracksPerPage))
                y=-1
            fi

            if [[ $d -ge $TotalNumOfTracks ]]; then
                x3=1
            fi
            x=$((d+1)); trv=$((x%fact)); y=$((y+1))
            local mn=; local sc=
            if [[ ${t1[y]} ]]; then
                mn=$((${t1[y]}/60))
                sc=$((${t1[y]}%60))
            fi

            local ttrk="${#inarr[@]}"
            local trk_name="${inarr[y]##*/}"
            if [[ -s '/tmp/wxmptmp/tn' ]]; then
                trackno=$(< '/tmp/wxmptmp/tn' )
            elif [[ -s '/tmp/wxmptmp/trackno' ]]; then
                trackno=$(< '/tmp/wxmptmp/trackno' )
            fi

            xlen=${#x}
            if [[ $xlen -lt 4 ]]; then xlen=4; fi
            xlen=$((xlen+12))
            if [[ ${#mn} -lt 3 ]]; then mns='  '; else mns=' '; fi
            tcols=$((ncols-xlen))
            trackCols="${#trk_name}"
            tput civis
            if [[ "${trk_name}" != '' ]]; then
              if [[ $x -eq $trackno ]]; then
                if [[ $trackCols -gt $tcols ]]; then
                  tcols=$(($tcols-2))
                  tput bold
                  printf "%4d.${mns}%02d:%02d<:>%.${tcols}s ┅\n" "$x" $mn $sc "${trk_name}"
                  tput sgr0
                else
                  tput bold
                  printf "%4d.${mns}%02d:%02d<:>%.${tcols}s\n" "$x" $mn $sc "${trk_name}"
                  tput sgr0
                fi
              else
                if [[ $trackCols -gt $tcols ]]; then
                  tcols=$(($tcols-2))
                  printf "%4d.${mns}%02d:%02d   %.${tcols}s ┄\n" "$x" $mn $sc "${trk_name}"
                else
                  printf "%4d.${mns}%02d:%02d   %.${tcols}s\n" "$x" $mn $sc "${trk_name}"
                fi
              fi
            fi

            ##==========================

            if [[ $trv -eq 0 ]]; then
                page=$(( x / TracksPerPage ))
                pages=$(( TotalNumOfTracks / TracksPerPage ))
                if [[ $(( TotalNumOfTracks % TracksPerPage )) -ne 0 ]]; then
                    pages=$((pages+1)) ;
                fi
                if [[ $TotalTracks -gt $TracksPerPage ]]; then
                    local info9="Page: ${page} / ${pages} =="
                else
                    local info9="Page: ${pages} =="
                fi

                if [[ -e '/tmp/wxmptmp/shuffle' ]]; then shfl='Shuffle On == '; else shfl= ; fi
                if [[ -e '/tmp/wxmptmp/loop' ]]; then lptxt='Loop On == '; else lptxt= ; fi

                if [[ -e "${mpptmp}/sf" ]]; then
                    fnd=$(< "${mpptmp}/sf" )
                    find1="find: $fnd == "
                else
                    find1=
                fi

                if [[ -e "$tempri" ]]; then
                    rintvl="Edit mode ${wxmpRefreshInterval}s == "
                fi
                info9="${find1}${lptxt}${shfl}${rintvl}${info9}"
                local inno="${#info9}"; inno=$((inno+2))
                fline=$(echo "$hline" | cut -c "${inno}"- )
                printf "%b" "$fline $info9"
                printf "%b" "${ldrstr}"
                tput cnorm

                rm -rf '/tmp/wxmptmp/tn'

                TrackToFind=$((trackno * 2))
                TInfo=$(tail -n +$TrackToFind "${flist}"  | head -n 1 | perl -ne '
                        if ( /^#EXTINF/ ) {
                            # s/[^[:ascii:]]/+/g;
                            m{^#EXTINF:(\d+),.*,(.*?)$};
                            print "$1:::$2" }' )
                TrackTime=${TInfo%:::*}
                TrackName=${TInfo#*:::}
                TrackTime=${TrackTime:-1}

                if [[ ! -e "$mpptmp/firstrun2" ]]; then
                    echo "firstrun" > "$mpptmp/firstrun"
                fi

                ### Read line here #####################################
                choice=; optinput=; set_move=
                while [[ "$choice" == '' ]]
                do
                  if [[ -e "$mpptmp/firstrun" ]]; then
                      echo "firstrun2" > "$mpptmp/firstrun2"
                      choice='r'; rm "$mpptmp/firstrun"; break
                  fi
                  if [[ -e "$CurPosnFile" ]]; then CurrPosn=$(< "$CurPosnFile"); fi
                  CurrPosn=${CurrPosn:-0}
                  if [[ -e "$icp" ]]; then InvertCurrPosn=$(< "$icp" ); fi
                  InvertCurrPosn=${InvertCurrPosn:-0}
                  Tdata=($( echo $CurrPosn $TrackTime $InvertCurrPosn | awk '{
                      ct=$1; if ( $3 == 1 ) { ct=$2-$1 };
                      ctm=sprintf("%02d:%02d:%02d", ((ct/3600)%24),((ct/60)%60),(ct%60));
                      ttm=sprintf("%02d:%02d:%02d", (($2/3600)%24),(($2/60)%60),($2%60));
                      val1=int(($1/$2)*100);
                      if ( $3 == 1 ) { ctm="-"ctm };
                      print val1, ctm, ttm ; }'))
                  TerminalTitle "|  ${Tdata[0]}%  ${Tdata[1]}  | ${TrackName##*/} - wx media player "
                  if [[ -e "$tempri" ]]; then
                      read -r -e -t $wxmpRefreshInterval choice
                      rm "$tempri" 2> /dev/null
                      rintvl=
                  else
                      IFS= read -s -r -n 1 -t 1 choice
                  fi

                  if [[ $? -lt 128 && -z $choice ]]; then
                      choice='r' ; break
                  fi

                  if [[ $choice == 'e' || $choice == '/' ]]; then
                      echo '30' > "$tempri"
                      optinput='true'
                      set_move='true'
                      choice="g${page}"
                      break
                  elif [[ $choice != '' ]]; then
                      read -sN1 -t 0.0001 k1
                      read -sN1 -t 0.0001 k2
                      read -sN1 -t 0.0001 k3
                      read -sN1 -t 0.0001 k4
                      read -sN1 -t 0.0001 k5
                      read -sN1 -t 0.0001 k6
                      choice+=${k1}${k2}${k3}${k4}${k5}${k6}
                      rm "$tempri" 2> /dev/null
                      break
                  fi
                  if [[ -s '/tmp/wxmptmp/trackno' ]]; then
                      trackplay=$(< '/tmp/wxmptmp/trackno' )
                      if [[ $trackno -ne $trackplay ]]; then
                          choice='r'
                      fi
                  fi
                  if [[ -e "$tempri" ]]; then
                      rm "$tempri" 2> /dev/null
                      choice='r'
                  fi
                  # if [[ ! -e '/tmp/wxmptmp/ipause' && -s "$PauseStatus" ]]; then
                  #     pyn=$(< "$PauseStatus" )
                  #     if [[ $pyn == 'yes' ]]; then choice='n'; fi
                  # fi
                done
                #####to see keycodes for this script ##############
                # echo "$choice" >> '/tmp/test'
                ###################################################
                case "$choice" in
                [0-9]*)
                    if [[ $choice -gt 0 && $choice -le $TotalTracks ]]; then
                        # AreWePaused
                        echo "jump $choice" >  $tolua
                        echo $choice > '/tmp/wxmptmp/trackno'
                        if [[ $choice -gt $TracksPerPage ]]; then
                            inval=$((((choice-ks)/TracksPerPage)+2))
                        else
                            inval=1
                        fi
                        page_jump=$(((inval*TracksPerPage)-ks))
                        if [[ $inval -le $pages ]]; then
                            d=$page_jump
                        fi
                    else
                        d=$((d-TracksPerPage))
                    fi
                    if [[ $d -le -1 ]]; then d=-1; fi
                    x3=$((d+2))
                    doclear; printf "%b" "$Header"
                    continue
                    ;;
                [gG]*)
                    choice=$(echo "${choice,,}" )
                    local inval=$(echo "${choice//[^0-9]/}" )
                    local intxt=$(echo "${choice//[^a-z]/}" )
                    case "$inval" in
                    [0-9]*)
                        case "$intxt" in
                        gt)
                            if [[ $inval -le $TotalTracks && $inval -gt $TracksPerPage ]]; then
                                inval=$((((inval-ks)/TracksPerPage)+2))
                            elif [[ $inval -le $TracksPerPage ]]; then
                                inval=1
                            fi
                            ;;
                        esac
                        if [[ $inval -le $pages ]]; then
                            page_jump=$(((inval*TracksPerPage)-ks))
                            if [[ $d -lt $TracksPerPage && $inval -gt $TotalTracks ]]; then
                                d=-1
                            else
                                d=$page_jump
                            fi
                            if [[ $d -le -1 ]]; then d=-1; fi
                            x3=$((d+2))
                            doclear; printf "%b" "$Header"
                            continue 1
                        else
                            if [[ $d -lt $TracksPerPage && $inval -gt $ttrk ]]; then
                                d=-1
                            else
                                d=$((d-TracksPerPage))
                            fi
                            if [[ $d -le -1 ]]; then d=-1; fi
                            x3=$((d+2))
                            doclear; printf "%b" "$Header"
                            continue 1
                        fi
                        ;;
                    *)
                        set_move='true'
                        ;;
                    esac
                    ;;
                i)
                    if [[ $InvertCurrPosn -eq 1 ]]; then printf 0 > "$icp";
                    else printf 1 > "$icp"; fi
                    set_move='true'
                    ;;
                \])
                    echo "speed_mult 1.1" > $wxfifo
                    set_move='true'
                    ;;
                \[)
                    echo "speed_mult 0.9091" > $wxfifo
                    set_move='true'
                    ;;
                "${ESC}[3~")   ## Delete Key
                    echo "speed_set 1.0" > $wxfifo
                    set_move='true'
                    ;;
                q)
                    rm -rf "$mpptmp/firstrun2"
                    echo -e "\n" && exit 0
                    ;;
                b)
                    ProgressBar
                    set_move='true'
                    ;;
                B)
                    ProgressBar "1"
                    set_move='true'
                    ;;
                [s][[:space:]]*|seek[[:space:]]*)
                    choi=$(echo $choice | sed 's/seek//gi;s/s//gi')
                    echo "seek $choi" > $wxfifo
                    set_move='true'
                    ;;
                [f][[:space:]]*|find[[:space:]]*)
                    if [[ ! -e "${flist}-org" ]]; then
                        choi=$(echo $choice | sed 's/^find//gi;s/^f//gi')
                        ShowMessage "  Please wait while we search for: $choi ... :))"
                        SearchTerm "$(echo $choi)"
                        if [[ -s '/tmp/wxmptmp/trackno' ]]; then
                            trackno=$(< '/tmp/wxmptmp/trackno' )
                        fi
                        if [[ $trackno -gt $TracksPerPage ]]; then
                            inval=$((((trackno-ks)/TracksPerPage)+2))
                        else
                            inval=1
                        fi
                        page_jump=$(((inval*TracksPerPage)-ks))
                        d=$page_jump
                        x3=$((d+2))
                        echo $((d+TracksPerPage)) > '/tmp/wxmptmp/mtrk'
                        exec "$0" --jump && exit
                    else
                        set_move='true'
                    fi
                    ;;
                F|fr|FR)
                    loops "-1" 'false'
                    mv -f "${flist}-org" "${flist}"
                    rm "${mpptmp}/shuffle" "${mpptmp}/sf" 2&> /dev/null
                    TracksPerPage=$(< "${mpptmp}/TracksPerPage-org")
                    rm "${mpptmp}/TracksPerPage-org" "${mpptmp}/tpt"
                    if [[ -s "${mpptmp}/trackno-org" ]]; then
                        trackno=$(< '/tmp/wxmptmp/trackno-org' )
                    fi
                    mv -f "${mpptmp}/fstat-org" "${mpptmp}/fstat"
                    echo "jump $trackno" >  $tolua
                    if [[ $trackno -gt $TracksPerPage ]]; then
                        inval=$((((trackno-ks)/TracksPerPage)+2))
                    else
                        inval=1
                    fi
                    page_jump=$(((inval*TracksPerPage)-ks))
                    d=$page_jump
                    x3=$((d+2))
                    echo $((d+TracksPerPage)) > '/tmp/wxmptmp/mtrk'
                    ShowMessage "  Returning back to main playlist... :))"
                    sleep 2
                    mv -f "${mpptmp}/trackno-org" "${mpptmp}/trackno"
                    mv -f "${mpptmp}/ct_info-org" "${mpptmp}/ct_info"
                    exec "$0" --jump && exit
                    ;;
                [l]*|loop*)
                    choi=$(echo "${choice//[^-0-9]/}" )
                    if [[ $choi ]]; then loops "$choi" 'true'; fi
                    set_move='true'
                    ;;
                L|noloop)
                    loops "-1" 'false'
                    set_move='true'
                    ;;
                V|save)
                    if [[ $(type -p zenity) != '' ]]; then
                        save_list
                    else
                        doclear; printf "%b" "$Header"
                        save_error
                        echo "$hline"
                        read -p "Press Enter key to continue..."
                    fi
                    set_move='true'
                    ;;
                u|shuffle)
                    shuffle 'true'
                    echo "shuffle"
                    set_move='true'
                    ;;
                U|noshuffle)
                    shuffle 'false'
                    set_move='true'
                    ;;
                ' '|pause)
                    # AreWePaused 99
                    echo "pause" > $wxfifo
                    set_move='true'
                    ;;
                [h]|help)
                    show_help | less
                    set_move='true'
                    ;;
                Q|quit)
                    echo "quit"  > $wxfifo
                    echo -e "\n"
                    exit 0
                    ;;
                \+)
                    amixer -q sset Master 4%+
                    ;;
                \-)
                    amixer -q sset Master 4%-
                    ;;
                \.)
                    # AreWePaused 99
                    echo 'key_down_event 46' > $wxfifo
                    # continue
                    ;;
                [n]|next|[\>])
                    if [[ -r '/tmp/wxmptmp/random' ]]; then
                        # AreWePaused
                        trackno=$(< '/tmp/wxmptmp/ctrk' )
                        trackno=${trackno%:*}
                        trackno=$((trackno+1))
                        next_track=$(tail -n +"$trackno" /tmp/wxmptmp/random | head -n 1)
                        if [[ $next_track -gt $TracksPerPage ]]; then
                            inval=$((((next_track-ks)/TracksPerPage)+2))
                        else
                            inval=1
                        fi
                        page_jump=$(((inval*TracksPerPage)-ks))
                        d=$page_jump
                        x3=$((d+2))
                        echo "$next_track" > '/tmp/wxmptmp/tn'
                        echo "next"  > $tolua
                        doclear; printf "%b" "$Header"
                        continue 1
                        #exec "$0" --jump && exit
                    else
                        # AreWePaused
                        trackno=$(< '/tmp/wxmptmp/trackno' )
                        next_track=$((trackno+1))
                        echo "$next_track" > '/tmp/wxmptmp/tn'
                        echo "next"  > $tolua
                        RefreshPage
                    fi
                    ;;
                [p]|previous|[\<])
                    if [[ -r '/tmp/wxmptmp/random' ]]; then
                        # AreWePaused
                        trackno=$(< '/tmp/wxmptmp/ctrk' )
                        trackno=${trackno%:*}
                        trackno=$((trackno-1))
                        next_track=$(tail -n +"$trackno" /tmp/wxmptmp/random | head -n 1)
                        if [[ $next_track -gt $TracksPerPage ]]; then
                            inval=$((((next_track-ks)/TracksPerPage)+2))
                        else
                            inval=1
                        fi
                        page_jump=$(((inval*TracksPerPage)-ks))
                        d=$page_jump
                        x3=$((d+2))
                        echo "$next_track" > '/tmp/wxmptmp/tn'
                        echo "prev"  > $tolua
                        doclear; printf "%b" "$Header"
                        continue 1
                    else
                        # AreWePaused
                        trackno=$(< '/tmp/wxmptmp/trackno' )
                        next_track=$((trackno-1))
                        echo "$next_track" > '/tmp/wxmptmp/tn'
                        echo "prev"  > $tolua
                        RefreshPage
                    fi
                    ;;
                [r]|refresh)
                    RefreshPage
                    ;;
                [d][[:space:]]*|[dD]elete[[:space:]]*)
                    delete_track "$choice"
                    if [[ $trackno -gt $TracksPerPage ]]; then
                        inval=$((((trackno-ks)/TracksPerPage)+2))
                    else
                        inval=1
                    fi
                    page_jump=$(((inval*TracksPerPage)-ks))
                    d=$page_jump
                    if [[ $d -gt $TracksPerPage ]]; then
                        echo $d > '/tmp/wxmptmp/mtrk'
                    fi
                    x3=$((d+2))
                    rm -rf "${mpptmp}/tpt" "$mpptmp/firstrun"
                    sleep 2
                    exec "$0" --jump && exit
                    ;;
                "${ESC}OH"|"${ESC}[H"|"${ESC}[1~"|"${ESC}[7~")   ## Home
                    d=-1
                    x3=$((d+2))
                    doclear; printf "%b" "$Header"
                    continue 1
                    ;;
                "${ESC}OF"|"${ESC}[F"|"${ESC}[8~"|"${ESC}[4~")   ## End
                    d=$(((pages * TracksPerPage)-ks))
                    x3=$((d+2))
                    doclear; printf "%b" "$Header"
                    continue 1
                    ;;
                "${ESC}[6~")   ## PgUp
                    if [[ $d -gt $(((pages * TracksPerPage)-ks)) ]]; then
                        d=-1
                        x3=$((d+2))
                    fi
                    doclear; printf "%b" "$Header"
                    continue 1
                    ;;
                "${ESC}[5~")   ## PgDn
                    d=$((d-(TracksPerPage*2)))
                    if [[ $d -le -1 ]]; then d=-1; fi
                    x3=$((d+2))
                    doclear; printf "%b" "$Header"
                    continue 1
                    ;;
                "[1;5A")   ## C-Up
                    TrackSeek '+600'
                    set_move='true'
                    ;;
                "[1;5B")   ## C-Dn
                    TrackSeek '-600'
                    set_move='true'
                    ;;
                "${ESC}[A")   ## Up
                    TrackSeek '+60'
                    set_move='true'
                    ;;
                "${ESC}[B")   ## Dn
                    TrackSeek '-60'
                    set_move='true'
                    ;;
                "${ESC}[C")   ## Rt
                    TrackSeek '+10'
                    set_move='true'
                    ;;
                "${ESC}[D")   ## Lt
                    #printf "seek -10" > $wxfifo
                    TrackSeek '-10'
                    set_move='true'
                    ;;
                "${ESC}[15~"|"${ESC}[[E")   ## F5
                    RefreshPage
                    ;;
                *)
                    # RefreshPage
                    set_move=
                    ;;
                esac

                doclear
                printf "%b" "$Header"

                if [[ $set_move ]]; then
                    if [[ $d -lt $TracksPerPage ]]; then
                        d=-1; x3=1
                    else
                        d=$((d-TracksPerPage)); x3=$((d+2))
                    fi
                    doclear; printf "%b" "$Header"
                    continue
                fi
            fi
        done
    done
}


if [[ $# -gt 0 ]]; then
    case "$1" in
    -h)             show_help | less && exit 0                    ;;
    --help)         show_help | less && exit 0                    ;;
    -v|--version)   printf "%s\n" "$version"; exit 0              ;;
    -q|--quit)      lets_quit                                     ;;
    -s|--seek)      shift; echo "seek $1 $2" > $wxfifo            ;;
    -l|--loop)      shift; loops "$1" 'true'                      ;;
    -L|--noloop)    shift; loops "-1" 'false'                     ;;
    -d|--delete)    shift; delete_track "$1 $2"                   ;;
    -n|next)        #AreWePaused
                    echo "next" > $tolua                          ;;
    -p|previous)    #AreWePaused
                    echo "prev" > $tolua                          ;;
    --pause)        #AreWePaused 99
                    echo "pause" > $wxfifo                        ;;
    -u|--shuffle)   shift; shuffle true                           ;;
    -U|--noshuffle) shift; shuffle false                          ;;
    --load)         shift
                    GetFileList "$@"
                    ${MakePlaylist} "${preflist}"                 ;;
                    # ${MakePlaylist} "${marrNRM[@]}"                 ;;
    -b)             ProgressBar                                   ;;
    -B)             ProgressBar "1"                               ;;
    -c|--custom)    shift;
                    case $1 in
                    *)   echo "pausing_keep_force $@" > $wxfifo
                         # "$run_mplayer" "pausing_keep_force $@" &
                         ;;
                    esac
                    ;;
    -V|--save)      save_list                                     ;;
    -j|--jump)      jump                                          ;;

    *)
        shopt -s extglob
        wxmpFileTypes=$(printf "|*.%s" "${fileTypeB[@]}")
        wxmpFileTypes="+(${wxmpFileTypes/\|/})"

        orgifs="$IFS"; IFS=$'\n'
        inxlist=( "$@" )
        cnt1=0; cnt2=0
        for f in ${inxlist[@]}
        do
            f2="${f}"
            case "${f2,,}" in
            *.m3u)
                marrM3U[$cnt1]="${f}"
                cnt1=$((cnt1 + 1))
                ;;
            ${wxmpFileTypes})
                marrNRM[$cnt2]="${f}"
                cnt2=$((cnt2 + 1))
                ;;
            *)
                continue
                ;;
            esac
        done
        IFS="$orgifs"

        if [[ ${marrNRM[0]} ]]; then
            msgout "Processing media tracks to playlist..."
            ${MakePlaylist} "${marrNRM[@]}"
            sleep 3
        fi
        if [[ ${marrM3U[0]} ]]; then
            msgout "Processing pre-compiled m3u playlist..."
            ${MakePlaylist} "${marrM3U[@]}"
        fi
        sleep 2
        if [[ -s "$tmpflist" ]]; then rm -rf "$tmpflist"; fi
        ;;
    esac
else
  if [[ -s /tmp/trackPlaying ]]; then
    jump
  else
    show_help | less && exit 0
 fi
fi

if [[ $debug ]]; then set +x ; fi

exit 0

:'
Notes::

7450ff

'
