package wxparser;

=comments
    Parses Weather current and forecast
    This version (c) 2018 Paramvir Likhari
=cut

$wxparser::VERSION = 180909;

require Exporter;
@wxparser::ISA = qw(Exporter);
@wxparser::EXPORT = qw(wxParserCurrent); # function to export

use strict;
use warnings;

sub msgout  { print STDERR "@_ \n" }

## any reference to functions in the main program can be called like main::calltofunction()

my ( %wx, %fctinfo, $temppath, $currdatam, $noFileMsg, $wxMainTitle, $wxFctTitle, $darkSkyLoc, $userLoc, $monAbb, $ltime, $year, $icao_id, $alreadyHaveForecast, $fctString, %wundFct, $waqidata, $wundoption, $fctBackUrl, %fctBackup ) ;

my @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);

my %msky = (
    BC     => "Patches",          BL  => "Blowing",       BR   => "Mist",
    DR     => "Low Drifting",     DS  => "Duststorm",     DU   => "Dust",
    DZ     => "Drizzle",          FC  => "Funnel Cloud",  FG   => "Fog",
    FU     => "Smoke",            FZ  => "Freezing",      GR   => "Hail",
    GS     => "Snow Pellets",     HZ  => "Haze",          IC   => "Ice Crystals",
    MI     => "Shallow",          NA  => "NA",            PL   => "Ice Pellets",
    PO     => "Dust/Sand Whirls", PR  => "Partial",       PY   => "Spray",
    RA     => "Rain",             SA  => "Sand",          SG   => "Snow Grains",
    SH     => "Showers",          SN  => "Snow",          SQ   => "Squalls",
    SS     => "Sandstorm",        TS  => "Thunderstorm",  UP   => "Unknown Precipitation",
    VA     => "Volcanic Ash",     VC  => "Nearby",        OVC  => "Overcast",
    BKN    => "Mostly Cloudy",    SCT => "Partly Cloudy", FEW  => "Clear",
    CLR    => "Clear",            SKC => "Clear",         CAVU => "Clear",
    CAVOK  => "Clear",
);


sub darkskyFctParser {

    my ( $fileToRead ) = @_ ;

    my $dsFct = ${main::get_line_num( $fileToRead, '<div class="dayDetails">','<div id="timeMachine"', 'str')};
    # my $dsFct2 = ${main::get_line_num( $fileToRead, '[{"time":','currently =', 'str')};

    # $dsFct2 =~ s/],/]/;
    # # msgout($dsFct2);
    # my $dsFct3 = jsonParser( $dsFct2 );

    # use Data::Dumper;
    # $Data::Dumper::Indent = 1;
    # # $Data::Dumper::Sortkeys = 1;
    # msgout Dumper($dsFct3);

    $dsFct =~ tr/\x0a\x20/ /s ;
    my $cntn = 1;
    while ($dsFct =~ m{<div class="summary">(.*?)</div>}g) {
        $fctinfo{"d_" . $cntn++ . "_font"} = main::do_get_DI(2, $1 ,$cntn);
    }
    while ($dsFct =~ m{<span class="num swip">(.*?)</span>}g) {
        # msgout($1);
        # $fctinfo{"d_" . $cntn . "_cor"} = ($1 and ($1 >= 10)) ? $1 * 100 :' 0';
        $fctinfo{"d_" . $cntn . "_cor"} = ( $1 ) ? $1 * 100 : ' 0';
    }
    $cntn = 1;
    while ($dsFct =~ m{<span class="highTemp.*?"temp">(.*?)˚</span>.*?="lowTemp.*?="temp">(.*?)˚</span>}g) {
        $fctinfo{"d_" . $cntn++ . "_temp"} = main::ch_fcunits("$2/$1");
        # msgout($fctinfo{"d_" . $cntn++ . "_temp"});
    }
    $cntn = 1;
    while ($dsFct =~ m{href="\/details\/.*?\/(.*?)\/us24\/en"}g) {
        $fctinfo{"d_" . $cntn++ . "_day" } = main::get_short_day(main::time2str("%a %d",$1));
    }

}

sub yahooFctParser {

    my ( $fileToRead ) = @_ ;

    my $yFct = ${main::get_line_num( $fileToRead,'"daily":','"precipitations":','str')};
    $yFct =~ s/^/{/;
    $yFct =~ s/,$//;

    # msgout($yFct);

    my $yhFct = main::jsonParser( $yFct );

    my %wkday = (
        '1' => 'Mon',
        '2' => 'Tue',
        '3' => 'Wed',
        '4' => 'Thu',
        '5' => 'Fri',
        '6' => 'Sat',
        '0' => 'Sun',
    );

    my $cntn = 1;
    for my $k (0..$#{$$yhFct{daily}}) {

        my $tMax = ($$yhFct{daily}[$k]{temperature}{high}) ? $$yhFct{daily}[$k]{temperature}{high} : 'NA';
        my $tMin = ($$yhFct{daily}[$k]{temperature}{low}) ? $$yhFct{daily}[$k]{temperature}{low} : 'NA';
        $fctinfo{"d_" . $cntn . "_temp"} = main::ch_fcunits("$tMax/$tMin");

        $fctinfo{"d_" . $cntn . "_cor" } = " $$yhFct{daily}[0]{precipitationProbability}" ? " $$yhFct{daily}[0]{precipitationProbability}" : 'NA';

        $fctinfo{"d_" . $cntn . "_hum" } = " $$yhFct{daily}[0]{humidity}" ? " $$yhFct{daily}[0]{humidity}" : 'NA';

        my $dy = ($$yhFct{daily}[$k]{observationTime}{day}) ? $$yhFct{daily}[$k]{observationTime}{day} : 'NA';
        my $wd = ($$yhFct{daily}[$k]{observationTime}{weekday}) ? $wkday{$$yhFct{daily}[$k]{observationTime}{weekday}} : 'Sun';
        $fctinfo{"d_" . $cntn . "_day" } = main::get_short_day("$dy $wd");

        my $desc = ($$yhFct{daily}[$k]{conditionDescription}) ? $$yhFct{daily}[$k]{conditionDescription} : 'NA';
        $fctinfo{"d_" . $cntn . "_font"} = main::do_get_DI(2, $desc ,$cntn++);

    }

}

sub weatherFctParser {

    my ( $fileToRead, $opt ) = @_ ;

    # data-testid="ExpandedDetailsCard"
    # data-testid="uvIndexSection"
    # "getSunV3DailyForecastUrlConfig": {
    # "getSunV3TileServerSeriesDynamicMapsUrlConfig": {

    my $xml1 = ${main::get_line_num( $fileToRead,'"getSunV3DailyForecastUrlConfig"','"getSunV3TileServerSeriesDynamicMapsUrlConfig"','str')};
    $xml1 =~ s/.*?"data":/{"data":/;
    $xml1 =~ s/,"status":200,.*$/}/;
    # msgout($xml1);

    my $wxfct = main::jsonParser( $xml1 );

    my $cntn = 1;
    for my $k (0..$#{$$wxfct{data}{dayOfWeek}}) {
        if ($$wxfct{data}{sunriseTimeLocal}[$k] =~ m{.*?-.*?-(.*?)T.*}) {
            $fctinfo{"d_" . $cntn . "_day" } = main::get_short_day("$$wxfct{data}{dayOfWeek}[$k] $1")
        }

        my $tMax = ($$wxfct{data}{temperatureMax}[$k]) ? $$wxfct{data}{temperatureMax}[$k] : 'NA';
        my $tMin = ($$wxfct{data}{temperatureMin}[$k]) ? $$wxfct{data}{temperatureMin}[$k] : 'NA';

        $fctinfo{"d_" . $cntn++ . "_temp"} = main::ch_fcunits("$tMax/$tMin");
    }

    $cntn = 1;
    for my $k (0..$#{$$wxfct{data}{daypart}[0]{precipChance}}) {
        if ($k % 2 == 0) {
            my $cor = ($$wxfct{data}{daypart}[0]{precipChance}[$k]) ? $$wxfct{data}{daypart}[0]{precipChance}[$k] : '0';
            $fctinfo{"d_" . $cntn . "_cor" } = " $cor";

            my $hum = ($$wxfct{data}{daypart}[0]{relativeHumidity}[$k]) ? $$wxfct{data}{daypart}[0]{relativeHumidity}[$k] : 'NA';
            $fctinfo{"d_" . $cntn . "_hum" } = $hum ;

            my $condL = ($$wxfct{data}{daypart}[0]{wxPhraseLong}[$k]) ? $$wxfct{data}{daypart}[0]{wxPhraseLong}[$k] : 'Sunny' ;
            $fctBackup{"d_" . $cntn . "_font"} = main::do_get_DI(2, $condL ,$cntn++)

        }
    }

}


sub wundergroundFctParser {

    my ( $fileToRead, $opt ) = @_ ;

    my $wxfct = main::jsonParser( $$fileToRead );

    my $cntn = 1;
    for my $k (0..$#{$$wxfct{forecasts}}) {
        if ($k+1 <= $#{$$wxfct{forecasts}})  {
            if ((($k == 0) and ($$wxfct{forecasts}[$k]{day_ind} eq 'N')) or ($$wxfct{forecasts}[$k]{day_ind} eq 'D')) {

                $fctinfo{"d_" . $cntn . "_font"}              =
                    ((not $fctinfo{"d_" . $cntn . "_font"}    or
                    $fctinfo{"d_" . $cntn . "_font"} eq 'NA') and
                    ($$wxfct{forecasts}[$k]{phrase_12char}))  ?
                    main::do_get_DI(2, $$wxfct{forecasts}[$k]{phrase_12char}, $cntn)                                    :
                    ($fctinfo{"d_" . $cntn . "_font"})        ?
                    $fctinfo{"d_" . $cntn . "_font"}          :
                    'NA'                                      ;

                my $tHi = ($$wxfct{forecasts}[$k]{temp}) ? $$wxfct{forecasts}[$k]{temp} : '--';

                my $tLo = ($$wxfct{forecasts}[$k+1]{temp}) ? $$wxfct{forecasts}[$k+1]{temp} : '--';

                my $temp = ($fctinfo{"d_" . $cntn . "_temp"}) ? $fctinfo{"d_" . $cntn . "_temp"} : 'NA';

                $fctinfo{"d_" . $cntn . "_temp"} = (($temp eq 'NA') and ("$tHi/$tLo")) ? main::ch_fcunits("$tHi/$tLo") : ($temp) ? $temp : 'NA' ;
                # msgout($fctinfo{"d_" . $cntn . "_temp"});

                $fctinfo{"d_" . $cntn . "_cor"}              =
                    ((not $fctinfo{"d_" . $cntn . "_cor"}    or
                    $fctinfo{"d_" . $cntn . "_cor"} eq 'NA') and
                    ($$wxfct{forecasts}[$k]{pop}))           ?
                    $$wxfct{forecasts}[$k]{pop}              :
                    ($fctinfo{"d_" . $cntn . "_cor"})        ?
                    $fctinfo{"d_" . $cntn . "_cor"}          :
                    'NA'                                     ;

                my ($date1) = $$wxfct{forecasts}[$k]{fcst_valid_local} =~ m{.*?-.*?-(.*?)T.*$};

                my $day1 = ($$wxfct{forecasts}[$k]{long_daypart_name}) ? $$wxfct{forecasts}[$k]{long_daypart_name} : 'NA';

                $fctinfo{"d_" . $cntn . "_day" } = main::get_short_day("$day1, $date1");

                $fctinfo{"d_" . $cntn . "_hum"}              =
                    (( not $fctinfo{"d_" . $cntn . "_hum"}   or
                    $fctinfo{"d_" . $cntn . "_hum"} eq 'NA') and
                    ($$wxfct{forecasts}[$k]{rh}))            ?
                    $$wxfct{forecasts}[$k]{rh}               :
                    ($fctinfo{"d_" . $cntn . "_hum"})        ?
                    $fctinfo{"d_" . $cntn . "_hum"}          :
                    'NA'                                     ;

                $fctinfo{"d_" . $cntn . "_corhum"}              =
                    (( not $fctinfo{"d_" . $cntn . "_corhum"}   or
                    $fctinfo{"d_" . $cntn . "_corhum"} eq 'NA') and
                    ($fctinfo{"d_" . $cntn . "_cor"})           and
                    ($fctinfo{"d_" . $cntn . "_hum"}) )         ?
                    $fctinfo{"d_" . $cntn . "_cor"}.'|'.$fctinfo{"d_" . $cntn . "_hum"}                                     :
                    ($fctinfo{"d_" . $cntn . "_corhum"})        ?
                    $fctinfo{"d_" . $cntn . "_corhum"}          :
                    'NA'                                        ;

                $cntn++;

            }
        }
        # msgout($k, $fctinfo{ "d_" . $k . "_day" });
        # msgout($k, $fctinfo{ "d_" . $k . "_temp" });
        # msgout($k, $fctinfo{ "d_" . $k . "_cor" });
        # msgout($k, $fctinfo{ "d_" . $k . "_font" });

    }

}


sub timeDateFctParser {

    my ( $fileToRead ) = @_ ;

    my $tndstr = ${main::get_line_num( $fileToRead,'var data={"','};','str')};

    $tndstr =~ s/var data=\{/\{"data":\{/;
    $tndstr =~ s/$/}}/;

    # msgout($tndstr);

    my $wxfct = main::jsonParser( $tndstr );

    my $cntn = 1;
    for my $k (0..$#{$$wxfct{data}{detail}}) {

        # my $num = $$wxfct{data}{detail}[$k]{date} ;
        # $num =~ s/[0]*$//;
        # $num .= '0' x (10 - length $num);
        # $fctinfo{"d_" . $cntn . "_day" } = ($$wxfct{data}{detail}[$k]{date}) ? main::get_short_day(main::time2str("%a %d",$num)) : 'NA';
        $fctinfo{"d_" . $cntn . "_day" } = ($$wxfct{data}{detail}[$k]{ts}) ? $$wxfct{data}{detail}[$k]{ts} : 'NA';

        $fctinfo{"d_" . $cntn . "_font" } = ($$wxfct{data}{detail}[$k]{desc}) ? main::do_get_DI(2, $$wxfct{data}{detail}[$k]{desc},$cntn) : 'NA';

        my $th = ($$wxfct{data}{detail}[$k]{temp}) ? $$wxfct{data}{detail}[$k]{temp} : 'NA';
        my $tl = ($$wxfct{data}{detail}[$k]{templow}) ? $$wxfct{data}{detail}[$k]{templow} : 'NA';
        $fctinfo{"d_" . $cntn . "_temp" } = main::ch_fcunits("$th/$tl");

        $fctinfo{"d_" . $cntn . "_cor" } = (" $$wxfct{data}{detail}[$k]{pc}") ? " $$wxfct{data}{detail}[$k]{pc}" : 'NA';

        $fctinfo{"d_" . $cntn++ . "_hum" } = ($$wxfct{data}{detail}[$k]{hum}) ? " $$wxfct{data}{detail}[$k]{hum}" : 'NA';

    }

}


sub accuweatherFctParser {

    my ( $fileToRead, $mainAlso ) = @_ ;

    $fctString =
    ${main::get_line_num( $fileToRead, '<div class="page-column-1">', '<div class="content-module bottom-forecast">', 'str', 1 )};

    $fctString =~ tr/\x0a\x20/\x20/s;

    # msgout($fctString);
    my $cntn = 1;
    while ($fctString =~ m{<span class="phrase">\s*(.*?)\s*</span>}g) {
        my $val = ($1) ? $1 : 'NA' ;
        $fctinfo{"d_" . $cntn++ . "_font"} = main::do_get_DI(2, $val ,$cntn);
        # msgout($fctinfo{"d_" . $cntn++ . "_font"});
    }
    $cntn = 1;
    while ($fctString =~ m{<div class="info precip">\s*<p>\s*Precip\s*</p>\s*<p>(.*?)%</p>}g) {
        my $val = ( $1 ) ? $1 : ' 0' ;
        $fctinfo{"d_" . $cntn++ . "_cor"} = $val;
    }
    $cntn = 1;
    while ($fctString =~ m{<div class="temps">\s*<span class="high"> (.*?)&#xB0; </span>\s*<span class="low"> / (.*?)&#xB0; </span>}g) {
        my ($ls,$rs) = ($1,$2);
        if (index($rs,'tonight') != -1 ) {
            $ls =~ tr/0-9.-//cd ;
            ($rs) = ($ls ne '') ? $ls : '--' ;
            $ls = '--';
        }
        else {
            $ls =~ tr/0-9.-//cd ;
            ($ls) = ($ls ne '') ? $ls : '--' ;
            $rs =~ tr/0-9.-//cd ;
            ($rs) = ($rs ne '') ? $rs : '--' ;
        }
        $fctinfo{"d_" . $cntn . "_temp"} = main::ch_fcunits("$ls/$rs");
        # msgout($cntn,$fctinfo{"d_" . $cntn . "_temp"});
        $cntn++;
    }
    $cntn = 1;
    while ($fctString =~ m{	<div class="date">\s*<p class="dow">\s*(.*?)\s*</p>\s*<p class="sub">\s*.*?/(.*?)\s*</p>\s*</div>}g) {
        # msgout($cntn, $1, $2);
        $fctinfo{ "d_" . $cntn . "_day" } = main::get_short_day("$1 $2");
        # msgout($cntn, $fctinfo{ "d_" . $cntn . "_day" });
        # msgout($cntn, $fctinfo{ "d_" . $cntn . "_temp" });
        # msgout($cntn, $fctinfo{ "d_" . $cntn . "_cor" });
        # msgout($cntn, $fctinfo{ "d_" . $cntn . "_font" });
        # msgout($cntn, $fctinfo{ "d_" . $cntn . "_cond" });
        $cntn++;
    }
}

sub msnFctParser {

    # if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

=comments
    the hash %wxta uses 4 parts - delimiter is ';;'
    1st = start context
    2nd = end context
    3rd = type of data expected
    4th = optional - use 1 to use rindex in place of index to search
=cut

    my ( $fileToRead ) = @_ ;

    my $fctdb = ${main::get_line_num( $fileToRead, '<ul class="forecast-list">',"normalsection", 'str')};
    my $fctl;
    for my $i (split('\n', $fctdb)) {
        # my $k = $$fctdb[$i];
        my $k = $i;
        if (index($k,'aria-label') != -1 )     {
            $fctl .= $k
        }
        if (index($k,'"image skyimg"') != -1 ) {
            $fctl .= $k
            }
        if (index($k,'<div title="Precipitation"') != -1) {
            $fctl .=$k.";;;"
        }
    }

    my $cntn = 0;
    my ($delim, %wxta);
    my @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);
    # if ( $wxFctTitle eq "msn"  ) {
        $delim = ';;;';
        %wxta = (
            day   => [ 'for ',            ' is',        'd'  ],
            cond  => [ '"image skyimg" ', ' data-icon', 'c'  ],
            temp  => [ 'High of ',        '<div title', 't' ],
            pop   => [ 'Precipitation',   '</span>',    'p'  ],
        );
    # }
    # $alreadyHaveForecast =~ tr/\x0a\x20/ /s ;


    # msgout($fstring);
    for ( split /$delim/, $fctl ) {
        # msgout('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'.$_);
        next if /Early AM/;
        $cntn++;
        for my $ky (keys %wxta) {
            # my $val;
            my $st     = $wxta{$ky}[0];
            my $nd     = $wxta{$ky}[1];
            my $type   = $wxta{$ky}[2];
            my $offset = $wxta{$ky}[3];
            $st  = index($_,$st) + length($st);;
            if ($offset) { $nd  = rindex($_,$nd) - $st }
            else { $nd  = index($_,$nd, $st) - $st }
            my $fnd = substr($_, $st, $nd );
            for ( $type ) {
                # msgout($ky,$st, $nd,$fnd);
                if    ( /d/ )  { # dates day
                    $fctinfo{"d_" . $cntn . "_day" } = main::get_short_day($fnd) ;
                }
                elsif ( /c/ )  { # alphanumeri with space cond
                    if ($fnd =~ m{title="(.*?)"}) {
                        $fctinfo{"d_" . $cntn . "_font"} = main::do_get_DI(2, $1,$cntn)
                    }
                }
                elsif ( /t/ ) { # numbers only - high low temps
                    if ($fnd =~ m{(--?|(-?([\.\d]+)?)).*(?:low of )(--?|(-?([\.\d]+)?))}) {
                        $fctinfo{"d_" . $cntn . "_temp"} = main::ch_fcunits("$1/$4")
                    }
                }
                elsif ( /p/ )  { # numbers only - pop
                    # if ($fnd =~ m{(\d+(\%)?)}) {
                    if ($fnd =~ m{(\d+)%}) {
                        $fctinfo{"d_" . $cntn . "_cor" } =  " $1"
                    }
                }
            }
            last if ( $cntn > 10 )
        }
    }
}


sub getCurrent {

    my ($instr, $wxMainTitle) = @_;
    # msgout($instr);

    my (%wxta,%wxtb);
    if ( $wxMainTitle eq 'msn' ) {
        %wxta = (
            city        => [ '"placeName" title=', '>', 'al' ],
            latitude    => [ ';lat=',       '&amp;l',        'n'  ],
            longitude   => [ ';long=',       '&amp;c',    'n' ],
            temperature => [ 'temperature is', 'degrees',  'n'  ],
            condition   => [ 'condition is ',  ' and the', 'a'  ],
            wind_dir    => [ 'class="winddir', '</li>',   'w'  ],
            # wind_speed  => [ 'winddir',        '</li>',    'n'  ],
            visibility  => [ 'Visibility',     '</li>',    'n'  ],
            pressure    => [ 'Barometer',      '</li>',    'n'  ],
            humidity    => [ 'Humidity',       '%',        'n'  ],
            dewpoint    => [ 'Dew Point',      '</li>',    'n'  ],
        );
    }
    if ( $wxMainTitle eq 'accu' ) {
        %wxta = (
            city        => [ '<title>',      ',',     'a'  ],
            latitude    => [ '<span class="latitude">',  '</span>',   'n'  ],
            longitude   => [ '<span class="longitude">', '</span>',   'n'  ],
            temperature => [ '<div classtemperatures><p classvalue>','xB0',   'n' ],
            condition   => [ '<div classphrase>',  '</div>',   'a'  ],
            wind_dir_degrees => [ 'transform: rotate', 'deg',  'n'  ],
            wind_speed  => [ 'Wind: ',    '</p', 'n'  ],
            wind_gust_speed  => ['<p>Gusts:',            '</p>', 'n'  ],
            visibility  => [ 'Visibility:',              '</p>', 'n'  ],
            pressure    => [ 'Pressure: ',               '</p>', 'n'  ],
            humidity    => [ 'Humidity',                 '</p>', 'n'  ],
            dewpoint    => [ 'Dew Point',                '</p>', 'n'  ],
            cloud1t     => [ 'Cloud Cover: ',             '</p>', 'n'  ],
            cloud1h     => [ 'Ceiling:',                 '</p>', 'n'  ],
        );
    }
    if ( $wxMainTitle eq 'timeDate' ) {
        %wxta = (
            updated          => ['<span id=wtct>',       '</span></p>', 'n1' ],
            city             => ['alt="Location of ',        '" style=', 'al'],
            station          => ['id=wt-loc title="',        '">',       'al'],
            condition        => ['class=mtt title="',        '." src="', 'al'],
            visibility       => ['Visibility: </span> ',     '&nbsp;',   'n' ],
            humidity         => ['Humidity: </span> ',       '%</p>',    'n' ],
            dewpoint         => ['Dew Point: </span> ',      '&nbsp;',   'n' ],
            temperature      => ['<div class=h1>Now ',       '&nbsp;',   'n' ],
            pressure         => ['Pressure: </span> ',       '</p>',     'n' ],
            wind_dir_degrees => ['title="Wind blowing from ','° ',       'n' ],
            wind_speed       => ['<br>Wind: ',               ' <span',   'n' ],
        );
    }

    my (%idxl, @idxarr, %idxl2);
    $instr =~ tr/\x0a\x20/ /s ;
    # $instr =~ s/&nbsp;/ /g;
    # msgout('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>',$instr);
    for my $ky ( keys %wxta) {
        my $st   = $wxta{$ky}[0];  # string where it starts
        my $nd   = $wxta{$ky}[1];  # string where it ends
        my $type = $wxta{$ky}[2];  # what to do with it
        my $st1  = index($instr, $st) + length($st); # get posn
        my $nd1  = index($instr, $nd, $st1);  # get number of char to pull
        my $fnd  = substr($instr, $st1, $nd1-$st1 );

        # msgout($ky,$st1, $nd1,'>>>'.$fnd.'<<<' );

        if    ($type eq 'n' ) {
            $wx{$ky} = ($fnd =~ m{(-?(\d+(\.\d+)?))}) ? $1 : 'NA'
            }
        elsif ($type eq 'd') {
            $wx{$ky} = ($fnd =~ m{,\s*(-?(\d+(\.\d+)?))}) ? $1 : 'NA'
            }
        elsif ($type eq 'n1') {
            $wx{$ky} = ($fnd =~ m{(.*?)}) ? $1 : 'NA'
            }
        elsif ($type eq 'a' ) {
            $wx{$ky} = ($fnd =~ m{(\w+((([,|\-|\s]+\w+))+)?)}) ? $1 : 'NA' ;
            if ($wx{wind_dir}) {
                ($wx{wind_dir_degrees},$wx{wind_dir}) = main::find_wind_dir($wx{wind_dir})
            }
        }
        elsif ($type eq 'al') {
            if ($wx{wind_dir_degrees} and not exists $wx{wind_dir} ) {
                $wx{wind_dir} = main::find_wind_dir($wx{wind_dir_degrees});
            }
            $wx{$ky} = $fnd ;
        }
        elsif ($type eq 'w' ) {
            ($wx{wind_dir},$wx{wind_speed}) = ($fnd =~ m{winddir.*?>(.*?)</div>(.*?)mph}) ? ($1,$2) : ("NA","NA")   ;
            if ($wx{wind_dir}) {
                ($wx{wind_dir_degrees},$wx{wind_dir}) = main::find_wind_dir($wx{wind_dir});
            }
        }

        # msgout($ky,$wx{$ky}) ;

    }
    if ( $wxMainTitle eq 'accu' ) {
        # msgout($wx{cloud1t});
        $wx{cloud1t} = ($wx{cloud1t}) ? $wx{cloud1t}.'%' : '0%'
    }

    return
}

sub wxParserForecast {

    my ($fctstr, $histCallStr);
    my $fct_data = "$temppath/fct_data.html" ;
    my $histcall = "$temppath/histcall" ;

    # msgout('ha ha ha in FCT') ;

    if ( -e $fct_data ) {
        {
        open (FILE, '<:crlf', "$fct_data") or die "$noFileMsg forecast data file $!";
            local $/ = undef;
            $fctstr = <FILE> ;
        close (FILE);
        }

        if ( $wxFctTitle eq "darkSky" ) {
            darkskyFctParser( \$fctstr );
        }
        elsif ( $wxFctTitle eq "yahoo" ) {
            yahooFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "timeDate" ) {
            timeDateFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "wund" ) {
            wundergroundFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "msn" ) {
            msnFctParser( \$fctstr );
        }
        elsif ( $wxFctTitle eq "accu" ) {
            accuweatherFctParser( \$fctstr )
        }
        elsif ( $wxFctTitle eq "wx" ) {
            weatherFctParser( \$fctstr )
        }
    }

}

sub msnWxParser {

    my ( $fileToRead ) = @_ ;

    my $msnwarn = ${main::get_line_num( $fileToRead, 'class="alerts','div','str')};
    if ($msnwarn) {
        my $c;
        foreach(split('</div>',$msnwarn)) {
            if ( m{<h2>(.*?)<span id="alertTime">(.*?)</span>} ){
                $c++; $wx{alertdata} .= "Weather alert $c: ".$1.$2.' | ';
            }
        }
    }
    my ( $msn1 ) = ${main::get_line_num( $fileToRead, 'weathermain','"DailyForecast"', 'str')};

    getCurrent($msn1, $wxMainTitle);
    $wx{name} = $wx{city};
    $wx{nameid}  = "MSNWX" ;
    $wx{updated} = $ltime;

}

sub darkskyWxParser {

    my ( $fileToRead, $opt ) = @_ ;

    my $wxMain = ${main::get_line_num( $fileToRead, 'currently =','map_info', 'str')};

    $wxMain =~ s/currently = /{"data":/;
    $wxMain =~ s/,\s*$/}/;
    # msgout($wxMain);
    my $wxM1 = main::jsonParser( $wxMain );

    my %wxBack = (
        updated          => 'time',
        cloud1t          => 'cloudCover',
        dewpoint         => 'dewPoint',
        condition        => 'summary',
        pressure         => 'pressure',
        wind_speed       => 'windSpeed',
        wind_gust_speed  => 'windGust',
        wind_dir_degrees => 'windBearing',
        icon             => 'icon',
        temperature      => 'temperature',
        humidity         => 'humidity',
        visibility       => 'visibility',
        solarRad         => 'ghi',
    );

    while ( my ($k1, $v1) = each %wxBack ) {

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$wxM1{data}{$v1})) ? $$wxM1{data}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$wxM1{data}{solar}{$v1})) ? $$wxM1{data}{solar}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        # msgout($k1, $wx{$k1});

    }

    if ( $wx{wind_dir_degrees} ) {
        $wx{wind_dir} = main::find_wind_dir($wx{wind_dir_degrees})
    }

    if ( not $wx{pressure} ) {
        $wx{pressure} = main::hPa_in($wx{pressure})
    }

    if ( $wx{humidity} and index($wx{humidity}, '0.') != -1 ) {
        $wx{humidity} = $wx{humidity} * 100
    }

    $wx{solarRad} = (($wx{solarRad}) and ($wx{solarRad} ne 'NA')) ? main::round($wx{solarRad}) : 'NA' ;

    if ($opt) {
        $wx{city} = $wx{name} = $opt;
        $wx{nameid} = 'darkSky'
    }
    else {
        $wx{city} = (not $wx{city}) ? $userLoc : $wx{city};
        $wx{name} = $wx{city};
    }

    $wx{cloud1t} = (($wx{cloud1t}) and (main::check_num($wx{cloud1t}) ne 'NA') and (index($wx{cloud1t}, '%') == -1)) ? ($wx{cloud1t} * 100) . '%' : ($wx{cloud1t} ) ? $wx{cloud1t} : 'NA';

}

sub wundergroundWxParser {

    my ( $fileToRead ) = @_ ;

    my ($wundwarn, %wundh);
    # my %wundhPws = (
    my %wundhPws = (
        city             => 'city',
        name             => 'adminDistrict',
        nameid           => 'stationID',
        latitude         => 'lat',
        longitude        => 'lon',
        elevation        => 'elev',
        updated          => 'epoch',
        condition        => 'wxPhraseLong',
        icon             => 'wxPhraseLong',
        temperature      => 'temp',
        dewpoint         => 'dewpt',
        humidity         => 'humidity',
        pressure         => 'pressure',
        precip_today     => 'precip24Hour',
        visibility       => 'visibility',
        snow             => 'snow24Hour',
        wind_speed       => 'windSpeed',
        wind_dir_degrees => 'winddir',
        wind_gust_speed  => 'windGust',
        cloud1h          => 'cloudCeiling',
        cloud1t          => 'cloudCoverPhrase',
        solarRad         => 'solarRadiation',
    );

    my %wundhAir = (
        city             => 'city',
        name             => 'adminDistrict',
        nameid           => 'displayName',
        latitude         => 'latitude',
        longitude        => 'longitude',
        elevation        => 'elevation',
        updated          => 'epoch',
        condition        => 'wxPhraseLong',
        icon             => 'wxPhraseLong',
        temperature      => 'temperature',
        dewpoint         => 'temperatureDewPoint',
        humidity         => 'relativeHumidity',
        pressure         => 'pressureAltimeter',
        precip_today     => 'precip24Hour',
        visibility       => 'visibility',
        snow             => 'snow24Hour',
        wind_speed       => 'windSpeed',
        wind_dir         => 'windDirectionCardinal',
        wind_dir_degrees => 'windDirection',
        wind_gust_speed  => 'windGust2',
        cloud1h          => 'cloudCeiling',
        cloud1t          => 'cloudCoverPhrase',
        solarRad         => 'solarradiation',
        updated          => 'validTimeUtc',
        solarRad         => 'solarRadiation',
    );

    if ($wundoption eq 'pws') { %wundh = %wundhPws }
    elsif ($wundoption eq 'air') { %wundh = %wundhAir }

    # msgout($wundoption);

    if (index($fileToRead,'class="alert-items">') != -1) { $wundwarn=1 }
    # @wundhash{@wundarr} = @base_wx_list;
    if ($wundwarn) {
        my $c;
        for my $k (split('<span>', ${main::get_line_num( $fileToRead,'<div class="alert-items">','</div>','str')})) {
            $k =~ s/<+[^>,]*>//g;
            next if $k =~ /^\s*$/;
            $c++;
            $wx{alertdata} .= "Weather alert $c: $k | ";
        }
    }

    my $wcData = ${main::get_line_num($fileToRead,'{&q;location&q;:',',&q;adminDistrictCode&q;','str')};
    $wcData .= '},';

    $wcData .= ${main::get_line_num($fileToRead,'&q;observations&q;','https://api.weather.com/v2/pws/observations/current?','str')};
    $wcData =~ s/},&q;expiresAt&q;:/,"expiresAt":/;
    $wcData =~ s/,&q;url&q;:&q;/,"airdata":/;

    $wcData .= ${main::get_line_num($fileToRead,'airport&a;subproduct=major&a',',&q;url&q;:&q;https://api.weather.com/v3/wx/observations/current?apiKey','str')};
    $wcData =~ s/airport&a;subproduct=major&a;.*?&q;value&q;://;
    $wcData =~ s/$/}/;
    $wcData =~ s/&q;/"/g;

    # msgout($wcData);
    my $wcData2 = main::jsonParser( $wcData );

    while ( my ($k1, $v1) = each %wundh ) {

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$wcData2{location}{$v1})) ? $$wcData2{location}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$wcData2{observations}[0]{$v1})) ? $$wcData2{observations}[0]{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$wcData2{observations}[0]{imperial}{$v1})) ? $$wcData2{observations}[0]{imperial}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$wcData2{airdata}{$v1})) ? $$wcData2{airdata}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

    }

    $wx{nameid} = ($wx{nameid} eq 'NA') ? 'wunderground.com' : $wx{nameid};

    if    ( $wx{name} ne 'null' and $wx{city} eq 'null' ) { $wx{city} = $wx{name} }
    elsif ( $wx{name} eq 'null' and $wx{city} ne 'null' ) { $wx{name} = $wx{city} }

    if ( not $wx{condition} ) {
        $wx{condition} = ($msky{$wx{cloud1t}}) ? $msky{$wx{cloud1t}} : "Clear" ;
        if ( $wx{icon} eq 'nt_' or $wx{icon} eq '' ) {
            $wx{icon} = $wx{condition}
        }
    }
}

sub weatherWxParser {

    my ( $fileToRead ) = @_ ;

    my %wxHash = (
        cloud1t          => 'cloudCoverPhrase',
        cloud1h          => 'cloudCeiling',
        pressure         => 'pressureAltimeter',
        humidity         => 'relativeHumidity',
        temperature      => 'temperature',
        dewpoint         => 'temperatureDewPoint',
        updated          => 'validTimeLocal',
        visibility       => 'visibility',
        wind_dir_degrees => 'windDirection',
        wind_dir         => 'windDirectionCardinal',
        wind_gust_speed  => 'windGust',
        wind_speed       => 'windSpeed',
        condition        => 'wxPhraseLong',
        icon             => 'wxPhraseLong',
    );

    my $xml1 = ${main::get_line_num($fileToRead,'getSunV3CurrentObservationsUrlConfig','getMukPageConfigUrlConfig','str')};

    my $city = ${main::get_line_num($fileToRead,'"displayName"','"dstEnd"','str')};

    $city =~ s/"displayName":"//;
    $city =~ s/",//;

    $xml1 =~ s/.*?"data":/{"data":/;
    $xml1 =~ s/,"status":200,.*$/}/;

    # msgout($xml1);
    my $wxMain = main::jsonParser( $xml1 );

    while ( my ($k1, $v1) = each %wxHash ) {

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$wxMain{data}{$v1})) ? $$wxMain{data}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

    }

    $wx{city}             = $wx{name} = $city;

    $wx{nameid} = 'weather.com';

    # $wx{updated} = $wx{updated} . ' MDY';

}


sub yahooWxParser {

    my ( $fileToRead ) = @_ ;

    my %yhHash = (
        nameid           => 'photoWoeid',
        city             => 'displayName',
        latitude         => 'latitude',
        longitude        => 'longitude',
        humidity         => 'humidity',
        pressure         => 'barometricPressure',
        condition        => 'conditionDescription',
        icon             => 'conditionDescription',
        visibility       => 'visibility',
        wind_speed       => 'windSpeed',
        temperature      => 'now',
        wind_dir_degrees => 'windDirection',
        updated          => 'timestamp',
    );

    my $yahooData = ${main::get_line_num( $fileToRead, '"WeatherStore":{','"forecasts"', 'str')};

    $yahooData =~ s/.*"imperial",/{/;
    $yahooData =~ s/,$/}/;

    # msgout($yahooData);
    my $yWx = main::jsonParser( $yahooData );

    while ( my ($k1, $v1) = each %yhHash ) {

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$yWx{location}{$v1})) ? $$yWx{location}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$yWx{observation}{temperature}{$v1})) ? $$yWx{observation}{temperature}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$yWx{observation}{observationTime}{$v1})) ? $$yWx{observation}{observationTime}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

        $wx{$k1} = ((not $wx{$k1} or $wx{$k1} eq 'NA') and ($$yWx{observation}{$v1})) ? $$yWx{observation}{$v1} : ($wx{$k1}) ? $wx{$k1} : 'NA';

    }

    $wx{name} = 'Yahoo';

    $wx{updated} =~ s/Z//;

}

sub timeDateWxParser {

    my ( $fileToRead, $opts ) = @_ ;

    my $y1 = 0;
    my $tndData = ${main::get_line_num( $fileToRead, '<section id=bk-focus class=bk-wt>','/section', 'str')};

    getCurrent($tndData, $wxMainTitle);
    # $wx{name} = $wx{nameid} = $wx{station};
    if ($opts) {
        $wx{city} = $wx{name} = $wx{station} = $opts
    }

    $wx{icon} = $wx{condition};

    $wx{nameid} = 'timeanddate.com';

    $wx{updated} =~ s/Z//;

}


sub metarWxParser {

    my ( $fileToRead ) = @_ ;

    if ($wx{ltxt} = ${main::get_line_num( $fileToRead, $icao_id,',','str' )}) {
        %wx = main::metar_parser($wx{ltxt}, 'METAR');
    }
    else {
        msgout(" \n\nError: :( \nChoose another METAR Station\nThis Station does not appear to be sending METAR reports.\n") ;
        die;
    }
    $wx{updated} = "${monAbb} $wx{updated} $year Z";

}


sub accuWxParser {

    my ( $fileToRead ) = @_ ;

    my $accustr =
    ${main::get_line_num($fileToRead,'<title>','</title>','str')};
    $accustr .=
    ${main::get_line_num($fileToRead,'<div class="current-conditions-card','<div class="card-button centered','str')};

    $accustr =~ tr/a-zA-Z0-9<_>,.\/: //cd;
    # msgout($accustr);

    getCurrent($accustr, $wxMainTitle);
    if ( $wx{wind_dir_degrees} or $wx{wind_dir_degrees} == 0  ) {
        $wx{wind_dir} = main::find_wind_dir($wx{wind_dir_degrees});
    }
    $wx{icon}    = $wx{condition};
    $wx{name}    = $wx{city};
    $wx{nameid}  = "AccuWX";
    $wx{updated} = $ltime;

    if ( -e "$temppath/accuwarn" ) {
        my ($warnstr, $c);
        {
            open (FILE, "<:crlf", "$temppath/accuwarn") or die "$noFileMsg accuwarn data file $!";
            local $/ = undef;
            $warnstr = <FILE> ;
            close (FILE);
        }
        my $accuwarn = main::get_line_num( \$warnstr,'<div class="info">','<div class="content awx-story">', 'arr');
        for (@{$accuwarn}) {
            if ( $_ =~ m{<h5 .*>(.*?)</h5>} ) {
                $c++;
                $wx{alertdata} .= "Weather alert $c: ".$1.' | '
            }
        }
    }

}


sub wxParserCurrent {

    ( $temppath, $currdatam, $noFileMsg, $wxMainTitle, $wxFctTitle, $darkSkyLoc, $userLoc, $monAbb, $ltime, $year, $icao_id, $waqidata, $wundoption, $fctBackUrl ) = @_ ;

    if ( -e $currdatam ) {

        my ( $htmlstr  ) ;
        {
        open (FILE, "<:crlf", $currdatam) or die "$noFileMsg main data file $!";
            local $/ = undef;
            $htmlstr = <FILE> ;
        close (FILE);
        }

        ### wunderground weather ################################################
        if ( $wxMainTitle eq "wund" ) {

            wundergroundWxParser( \$htmlstr );

        }

        ### weather.com weather ################################################
        elsif ( $wxMainTitle eq "wx" ) {

            weatherWxParser( \$htmlstr );

            ## weather forecast
            if ( $wxFctTitle eq "wx" ) {
                weatherFctParser( \$htmlstr );
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### darksky.net weather ################################################
        elsif ( $wxMainTitle eq "darkSky" ) {

            darkskyWxParser( \$htmlstr, $darkSkyLoc );

            if ( $wxFctTitle eq "darkSky") {
                darkskyFctParser( \$htmlstr ) ;
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### yahoo weather ################################################
        elsif ( $wxMainTitle eq "yahoo" ) {

            yahooWxParser( \$htmlstr );

            ### yahoo forecast
            if ( $wxFctTitle eq "yahoo" ) {
                yahooFctParser( \$htmlstr ) ;
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### timeanddate weather ################################################
        elsif ( $wxMainTitle eq "timeDate" ) {

            timeDateWxParser( \$htmlstr, $darkSkyLoc );

            ### yahoo forecast
            if ( $wxFctTitle eq "timeDate" ) {
                timeDateFctParser( \$htmlstr ) ;
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### msn.com weather ################################################
        elsif ( $wxMainTitle eq "msn" ) {

            msnWxParser( \$htmlstr );

            if ( $wxFctTitle eq "msn" ) {
                msnFctParser( \$htmlstr );
                $alreadyHaveForecast = 1; # required to disable forecast re calc
            }
        }

        ### METAR weather ################################################
        elsif ( $wxMainTitle eq "metar" ) {

            metarWxParser( \$htmlstr );

        }

        ### accuweather.com weather ################################################
        elsif ( $wxMainTitle eq "accu" ) {

            accuWxParser( \$htmlstr );

            ### accu forecast see below
        }
        ### end of wx section


        ### important do not delete - see conkywx.pl file end triggers
        $wx{dataIsThere} = 1 ;

        if ( not $alreadyHaveForecast ) {
            wxParserForecast();
        }

        my ($fctret, $wxret, $aqistr );

        ### air quality section
        if ( -e $waqidata ) {
            my ( $airstr  ) ;
            {
            open (FILE, "<:crlf", $waqidata) or die "$noFileMsg waqi data file $!";
                local $/ = undef;
                $airstr = <FILE> ;
            close (FILE);
            }
            my $airdata = main::jsonParser( $airstr );

            my $o3   = $$airdata{data}{iaqi}{o3}{v}   ||= 'NA';
            my $pm25 = $$airdata{data}{iaqi}{pm25}{v} ||= 'NA';
            my $pm10 = $$airdata{data}{iaqi}{pm10}{v} ||= 'NA';
            my $no2  = $$airdata{data}{iaqi}{no2}{v}  ||= 'NA';
            my $so2  = $$airdata{data}{iaqi}{so2}{v}  ||= 'NA';
            my $co   = $$airdata{data}{iaqi}{co}{v}   ||= 'NA';
            my $loc  = $$airdata{data}{city}{name}    ||= 'NA';

            $aqistr = "o3=>$o3\npm25=>$pm25\nno2=>$no2\nso2=>$so2\nco=>$co\npm10=>$pm10\nloc=>$loc\n";

        }

        ### current weather + solar data backup section using darkSky
        if ( -e "$temppath/solValueF" ) {
            my ( $altTD  ) ;
            {
            open (FILE, "<:crlf", "$temppath/solValueF" ) or die "$noFileMsg wind data file $!";
                local $/ = undef;
                $altTD = <FILE> ;
            close (FILE);
            }
            # msgout($wx{condition});
            darkskyWxParser( \$altTD );

        }

        while ( my ($k1, $v1) = each %wx ) {
            # msgout("${k1}=>${v1}");
            $wxret .= "${k1}=>${v1}\n"
        }

        ### forecast backup section - the fctBack uses wund forecast
        if (( -e "$temppath/fctBack.html") and ($wxFctTitle ne "wund" )) {
            my ( $fbstr  ) ;
            {
            open (FILE, "<:crlf", "$temppath/fctBack.html") or die "$noFileMsg fctBack data file $!";
                local $/ = undef;
                $fbstr = <FILE> ;
            close (FILE);
            }

            wundergroundFctParser( \$fbstr, 'yes' );

        }

        while ( my ($key2, $value2) = each %fctinfo ) {
            $fctret .= "${key2}=>${value2}\n"
        }

        # msgout($fctret);

        return( \$wxret, \$fctret, \$aqistr );
    }
}

1;
