
--[[
    modified Paramvir 2017 - added folder-size option
    du give values in kilobytes. the max_size will be the max size of folder you want - like a limit and based on that you will see a ring using percentage of du value off max_size

]]

local PI        = 3.14159265358979323846
local TORADIANS = 3.14159265358979323846/180
local floor     = math.floor
local cos       = math.cos
local sin       = math.sin
local abs       = math.abs
local tonum     = tonumber
local sprintf   = string.format
local len       = string.len
local lmatch    = string.match

local lpairs    = pairs
local lcollectgarbage = collectgarbage
local lconky_parse = conky_parse

local lcairo_move_to          = cairo_move_to
local lcairo_show_text        = cairo_show_text
local lcairo_stroke           = cairo_stroke
local lcairo_fill             = cairo_fill
local lcairo_select_font_face = cairo_select_font_face
local lcairo_set_font_size    = cairo_set_font_size
local lcairo_set_source_rgba  = cairo_set_source_rgba
local lcairo_arc              = cairo_arc
local lcairo_set_line_width   = cairo_set_line_width


local w, dt, gauge = {}, {}, {}
local wantImperialTemp
-- local myPackage = package
-- myPackage.path = myPackage.path .. ";./lib/?.lua;"

local FC = require("luafunc")
local to_rgba           = FC.to_rgba
local to_rgba_contrast  = FC.to_rgba_contrast
local get_points        = FC.get_points
local DirExists         = FC.DirExists
local getAngleInRadians = FC.getAngleInRadians
local colour_blend      = FC.colour_blend
local getDatafile       = FC.getDatafile
local msgout            = FC.msgout
local memory            = FC.memory

local GF = require("sysinfo")
local sysinfo = GF.sysinfo

local topVal, val, cnt = {},{},{}
local function draw_gauge_ring(cr, data, value, border)
    local x, y = data.x + border, data.y + border
    local graph_end_angle = (data.max_value * data.graph_unit_angle) % 360

    -- background ring
    lcairo_arc(cr, x, y, data.graph_radius,
        getAngleInRadians(data.graph_start_angle, 0),
        getAngleInRadians(data.graph_start_angle, graph_end_angle))
    lcairo_set_source_rgba(cr, to_rgba(data.graph_bg_colour))
    lcairo_set_line_width(cr, data.graph_thickness)
    lcairo_stroke(cr)

    -- arc of value
    val[data.id] = value % (data.max_value + 1)
    local start_arc = 0
    local stop_arc = 0
    local i = 1
    if dt.graph_fg_colour then
        lcairo_set_source_rgba(cr, to_rgba(dt.graph_fg_colour))
    else
        local colr, colg, colb, cola
        colr, colg, colb, cola = colour_blend(val[data.id], dt.blend_midpoint, graph_end_angle)
        lcairo_set_source_rgba(cr, colr, colg, colb, (dt.blend_alpha or 0.3))
    end
    while i <= val[data.id] do
        start_arc = (data.graph_unit_angle * i) - data.graph_unit_thickness
        stop_arc = (data.graph_unit_angle * i)
        lcairo_arc(cr, x, y, data.graph_radius,
            getAngleInRadians(data.graph_start_angle, start_arc),
            getAngleInRadians(data.graph_start_angle, stop_arc))
        lcairo_stroke(cr)
        i = i + 1
    end
    local angle = start_arc

    -- hand
----- max value achieved
    if topVal[data.id] and topVal[data.id] > val[data.id] + 3 then
        stop_arc  = (data.graph_unit_angle * topVal[data.id])
        start_arc = stop_arc - ((dt.hand_thickness/data.graph_radius)*100)
        lcairo_set_source_rgba(cr, to_rgba(dt.peak_fg_colour))
        if not dt.hand_type or dt.hand_type == 0 then
            lcairo_arc(cr, x, y, data.graph_radius,
                getAngleInRadians(data.graph_start_angle, start_arc),
                getAngleInRadians(data.graph_start_angle, stop_arc))
            lcairo_stroke(cr)
        elseif dt.hand_type == 1 then
            local jx,jy = get_points(dt.graph_start_angle, stop_arc , 0, data.graph_radius)
            lcairo_arc(cr, x+jx, y+jy, dt.hand_thickness, TORADIANS, 0)
            lcairo_fill(cr)
        end
    end
----- current value
    stop_arc  = (data.graph_unit_angle * val[data.id])
    start_arc = stop_arc - ((dt.hand_thickness/data.graph_radius)*100)
    lcairo_set_source_rgba(cr, to_rgba(data.hand_fg_colour))
    if not dt.hand_type or dt.hand_type == 0 then
        lcairo_arc(cr, x, y, data.graph_radius,
            getAngleInRadians(data.graph_start_angle, start_arc),
            getAngleInRadians(data.graph_start_angle, stop_arc))
        lcairo_stroke(cr)
    elseif dt.hand_type == 1 then
        local jx,jy = get_points(dt.graph_start_angle, stop_arc , 0, data.graph_radius)
        lcairo_arc(cr, x+jx, y+jy, dt.hand_thickness, TORADIANS, 0)
        lcairo_fill(cr)
    end
------ max values futures

    if not cnt[data.id] then cnt[data.id] = 0
    elseif cnt[data.id] > 100 then
        cnt[data.id] = 0
        topVal[data.id] = nil
        val[data.id] = nil
    end

    if not topVal[data.id] or
        val[data.id] > topVal[data.id] + 3 or
        cnt[data.id] % dt.peakDurationDisplay == 0
        then
        topVal[data.id] = val[data.id]
    end
    -- if cnt[data.id] % dt.peakDurationDisplay == 0 then
    --     topVal[data.id] = val[data.id]
    -- end
    cnt[data.id] = cnt[data.id] + 1

-- grads marks
    if  data.grad_radius > 0     and
        data.grad_length > 0     and
        data.grad_unit_angle > 0 then
        local nb_grad = graph_end_angle / data.grad_unit_angle
        local i = 0
        lcairo_set_source_rgba(cr, to_rgba(data.grad_fg_colour))
        local gradl = data.grad_length
        local gradr = data.grad_radius
        while i <= nb_grad do
            -- if i%5 == 0 then
            --     data.grad_length = 5
            --     data.grad_radius = data.grad_radius + ((data.grad_length-gradl)/2)
            --     -- data.grad_radius = data.grad_radius + gradl - 1
            -- else
            --     data.grad_length = gradl
            --     data.grad_radius = gradr
            -- end
            lcairo_set_line_width(cr, data.grad_length)
            start_arc = (data.grad_unit_angle * i) - (data.grad_mark_thickness * 0.5)
            stop_arc  = (data.grad_unit_angle * i) + (data.grad_mark_thickness * 0.5)
            lcairo_arc(cr, x, y, data.grad_radius,
                getAngleInRadians(data.graph_start_angle, start_arc),
                getAngleInRadians(data.graph_start_angle, stop_arc))
            lcairo_stroke(cr)
            -- lcairo_set_line_width(cr, data.graph_thickness)
            i = i + 1
        end
    end

    -- text
    if value > 10 then
        angle = start_arc - ((dt.textOffset / data.graph_radius) * 100)
    end

    local movex, movey = get_points(dt.graph_start_angle, angle, 0, data.graph_radius)
    lcairo_select_font_face(cr, dt.fontName, CAIRO_FONT_SLANT_NORMAL, data.txt_weight)
    lcairo_set_source_rgba(cr, to_rgba(data.txt_fg_colour))
    lcairo_set_font_size(cr, data.txt_size)

    if wantImperialTemp then
        value = floor((tonum(value) * 1.8) + 32.5)
    end

    if not data.txt_on_base and data.txt_radius > 0 then
        lcairo_move_to(cr, x + movex - (data.txt_size / 2), y + movey + 3)
        lcairo_show_text(cr, value)
    elseif data.txt_on_base then
        local xfac = 5
        lcairo_move_to(cr, x - (data.txt_size / 2) - (xfac * (len(value) - 1)), y + data.txt_radius + 3)
        lcairo_show_text(cr, value)
    end

    -- caption
    local tox, toy = get_points(dt.graph_start_angle, 0, 0, data.graph_radius)
    lcairo_select_font_face(cr, dt.fontName, CAIRO_FONT_SLANT_NORMAL, data.caption_weight);
    lcairo_set_font_size(cr, data.caption_size)
    lcairo_set_source_rgba(cr, to_rgba(data.caption_fg_colour))
    lcairo_move_to(cr, x + tox + 5, y + toy + 3)
    if data.graph_start_angle < 105 then
        local extents = cairo_text_extents_t:create()
        tolua.takeownership(extents)
        lcairo_text_extents(cr, data.caption, extents)
        lcairo_move_to(cr, x + tox - extents.width - 5, y + toy + 1)
    end
    lcairo_show_text(cr, data.caption)
end

local function fixPrint(num, max)
    return tonum(sprintf( "%2.0f", (num/max)*100))
end

-- other util functions
local function getFolderSize(str, max)
    if str then
        f1 = io.popen( [[ du -d 0 ]] .. str )
        if f1 then
            local rinfo = f1:read("*a")
            f1:close()
            rinfo = lmatch( rinfo, '^(%d+)%s*.*$' )
            -- local fsString = sprintf('${%s %s}', 'fs_free', str)
            -- local fsValue = floor((tonum(conky_parse(fsString)) or 0) + 0.5)
            -- msgout((fsValue/1024),rinfo, (rinfo/(fsValue/1024))*100)
            return fixPrint(rinfo,max)
        else
            return nil
        end
    else
        return nil
    end
end

-- memory('mem')

-- local function makeAverage(arg)
--     local ct1, ct2
--     if arg == 'cpu1' then
--         ct1 = lconky_parse('${cpu cpu1}')
--         ct2 = lconky_parse('${cpu cpu2}')
--     elseif arg == 'cpu2' then
--         ct1 = lconky_parse('${cpu cpu3}')
--         ct2 = lconky_parse('${cpu cpu4}')
--     end
--     return (ct1 + ct2) / 2
-- end

-- loads data and displays gauges
function w.wxrings(p)
    local cr = p.cr
    dt, gauge = {}, {}
    ----- data file defined with -d option
    if not p.d then
        error('Error :( Please define the data file for wxrings lua module.')
    end
    ----- pull in tables from data file
    if p.d then
        local RD = getDatafile(p.d)
        dt       = RD.dt
        gauge    = RD.gauge
    end

    local border = conky_window.border_outer_margin
    local function load_gauge_rings(cr, data)
        local str, value = '', 0
        if data.luaID and lmatch(data.luaID, 'net') then
            data.x = p.b   or data.x
            data.y = p.lwy or data.y
        end
        if data.luaID and lmatch(data.luaID, 'cpu') then
            data.x = p.c   or data.x
            data.y = p.lwy or data.y
        end
        if data.luaID and lmatch(data.luaID, 'mem') then
            data.x = p.m   or data.x
            data.y = p.lwy or data.y
        end
        -- msgout(dt.mem_x)
        wantImperialTemp = nil
        if data.conky_line then
            str = data.conky_line
        elseif data.name == 'folder-size' then
            str = getFolderSize(data.arg, data.max_size)
        elseif data.name == 'sysinfo' then
            if p.f then
                wantImperialTemp = 1
            end
            str = sysinfo('-w '..data.arg)
        else
            str = sprintf('${%s %s}', data.name, data.arg)
        end

        -- if data.name == 'cpu' and data.arg == 'cpu1' then
        --     value = makeAverage('cpu1')
        -- elseif data.name == 'cpu' and data.arg == 'cpu2' then
        --     value = makeAverage('cpu2')
        -- else
            value = tonum(lconky_parse(str)) or 0
        -- end

        if data.name == 'downspeedf' then
            value = fixPrint(value,dt.downloadMax)
        end
        if data.name == 'upspeedf' then
            value = fixPrint(value,dt.uploadMax)
        end
        draw_gauge_ring(cr, data, value, border)
    end

    for i in lpairs(gauge) do
        load_gauge_rings(cr, gauge[i])
    end

end

lcollectgarbage()

return w
